/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: abook-callbacks.h,v 1.6 2001/03/20 20:12:34 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _ABOOK_CALLBACKS_H_
#define _ABOOK_CALLBACKS_H_

#include "abook-main.h"


typedef enum {
	ABOOK_GROUP,
	ABOOK_ENTRY
} RowType;

typedef struct _Abook_rd Abook_rd;
typedef struct _UserData UserData;

struct _Abook_rd {
	RowType type;
	Abook_entry *entry;
	Abook_group *group;
};

struct _UserData {
	GtkSignalFunc func;
	gpointer data;
};
void entry_ctree_clicked_column (GtkWidget *clist, gint column, gpointer data);
void abook_widget_rehash_abook_clist (Abook *abook, GtkCTree *ctree);
void abook_widget_rehash_abook (Abook *abook);
void abook_widget_rehash_optionmenu (GtkOptionMenu *omenu, GtkSignalFunc func);

Abook *abook_get_current ();
void abook_widget_show (GtkSignalFunc kill_func);
void abook_widget_set (GtkWidget *widget);
GtkWidget *abook_widget_get ();
void abook_widget_hide ();
void abook_widget_kill (GtkWidget *widget, gpointer data);
void abook_widget_changed_abook (GtkMenuItem *item, gpointer data);

void abook_widget_select_row (GtkCTree *ctree, GtkCTreeNode *node, gint column);
void abook_widget_unselect_row (GtkCTree *ctree, GtkCTreeNode *node, gint column);
void abook_widget_destroy_row ();
void abook_widget_delete_entry ();

void abook_widget_add_entry (GtkWidget *widget, gpointer data);
void abook_widget_add_group (GtkWidget *widget, gpointer data);
void abook_widget_edit_entry (GtkWidget *widget, gpointer data);
void abook_widget_new_abook (GtkWidget *widget, gpointer data);
void abook_widget_delete_abook (GtkWidget *widget, gpointer data);

/* entry editor specific functions */
void abook_widget_entry_destroy (GtkWidget *widget, GdkEvent *event, Abook_entry *entry);
void abook_widget_entry_accept (GtkWidget *widget, Abook_entry *entry);
void abook_widget_entry_cancel (GtkWidget *widget, Abook_entry *entry);
void abook_widget_entry_email_activate (GtkEntry *widget, Abook_entry *entry);

void abook_widget_entry_email_selectrow (GtkWidget *widget, gint row, 
                   gint column, GdkEventButton *event, gpointer data);
void abook_widget_entry_email_unselectrow (GtkWidget *widget, gint row, 
                   gint column, GdkEventButton *event, gpointer data);
void abook_widget_entry_set_default_email (GtkWidget *widget, Abook_entry *entry);
void abook_widget_entry_remove_email (GtkWidget *widget, Abook_entry *entry);
void abook_widget_entry_add_email (GtkWidget *widget, gpointer data);

void abook_widget_entry_newgroup_sel (GtkWidget *widget, gint row, 
                   gint column, GdkEventButton *event, Abook_entry *entry);	
void abook_widget_entry_newgroup_unsel (GtkWidget *widget, gint row, 
                   gint column, GdkEventButton *event, Abook_entry *entry);
void abook_widget_entry_memgroup_sel (GtkWidget *widget, gint row, 
                   gint column, GdkEventButton *event, Abook_entry *entry);
void abook_widget_entry_memgroup_unsel (GtkWidget *widget, gint row, 
                   gint column, GdkEventButton *event, Abook_entry *entry);
void abook_widget_entry_addgroup (GtkWidget *widget, Abook_entry *entry);
void abook_widget_entry_delgroup (GtkWidget *widget, Abook_entry *entry);

/* group editor specific functions */
void abook_widget_group_accept (GtkWidget *widget, Abook_group *group);
void abook_widget_group_cancel (GtkWidget *widget, Abook_group *group);
void abook_widget_group_selentry (GtkWidget *widget, gint row);
void abook_widget_group_unselentry ();
void abook_widget_group_selmember (GtkWidget *widget, gint row);
void abook_widget_group_unselmember ();
void abook_widget_group_addmember (GtkWidget *widget, Abook_group *group);
void abook_widget_group_delmember (GtkWidget *widget, Abook_group *group);

/* new book callbacks */
void abook_widget_new_accept (GtkWidget *widget, gpointer data);
void abook_widget_new_cancel (GtkWidget *widget, gpointer data);

void abook_widget_find_entry_changed (GtkEditable *edit);

/* selector callbacks */
void abook_widget_selector_rsel (GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data);
void abook_widget_selector_runsel (GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data);
void abook_widget_selector_lsel (GtkCTree *ctree, GtkCTreeNode *node, gint column);
void abook_widget_selector_lunsel (GtkCTree *ctree, GtkCTreeNode *node, gint column);

void abook_widget_selector_add (GtkWidget *widget, gint type);
void abook_widget_selector_del (GtkWidget *widget);
void abook_widget_selector_changebook (GtkWidget *widget, Abook *abook);

void abook_widget_selector_accept (GtkWidget *widget, UserData *data);
void abook_widget_selector_cancel (GtkWidget *widget, gpointer data);
void abook_widget_selector_destroy (GtkWidget *widget, GdkEvent *event, gpointer data);

void abook_widget_set_types (GList *list);
void abook_widget_set_list (GList *l);
void abook_widget_selector_search_changed (GtkEditable *editable);

void abook_widget_selector_set_updator (GtkSignalFunc func, gpointer data);
void abook_widget_selector_search_custom (GtkWidget *widget, gpointer data);
void goto_webpage_callback (GtkWidget *, GtkWidget *);
void abook_entry_bpe(GtkWidget *widget, GdkEvent *event, gpointer data);
#endif
