/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: abook-opts.h,v 1.6 2001/03/20 20:12:34 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _ABOOK_OPTS_H_
#define _ABOOK_OPTS_H_

#include "abook-main.h"

typedef GtkWidget *(*QuestionFunc) (char *, char *, GtkSignalFunc, gpointer);

#ifndef _
#define _(stuff) (do_nothing (stuff))
#endif

typedef struct _Abook Abook;
typedef struct _Abook_entry Abook_entry;
typedef struct _Abook_group Abook_group;
typedef struct _Recipient Recipient;

struct _Recipient {
	char *email;
	int type;
};

struct _Abook {
	char *name;
	GList *entrys;
	GList *groups;
};

struct _Abook_entry {
	char *first, *middle, *last, *title, *nickname, *sex, *hphone, *wphone, *mphone;  // added *sex 

	char *pager, *fax, *street, *webpage, *company, *position, *cstreet, *ccity;

	char *cstate, *czip, *ccountry, *coffice, *cphone1, *cphone2, *cfax;

	char *cpager, *cwebpage, *cmanager, *cassistant, *def_email, *notes;

	char *dn;

	GList *email;
	GList *groups;
	GList *nodes;
	
	int dummy;
	int type;
};

struct _Abook_group {
	char *name;
	char *email;
	GList *members; //
	GtkCTreeNode *node;
	int type;
};

void abook_set_question_func (QuestionFunc func);
void abook_set_about_func (GtkSignalFunc func);
void abook_question (char *question, char *explanation, GtkSignalFunc func, gpointer data);

Abook *abook_new ();
GList *abook_first ();
void abook_destroy (Abook *abook);
void abook_destroy_all ();
Abook *abook_seek_by_name (char *name);
void abook_destroy_entries (Abook *abook);
void abook_destroy_groups (Abook *abook);
void abook_destroy_dummys ();

Abook_entry *abook_entry_new (Abook *abook);

Abook_entry *abook_entry_seek_in_part (char *partial);

void abook_entry_free (Abook_entry *entry);
void abook_entry_edit (Abook_entry *entry, char *first, char *middle, 
                       char *last, char *title, char *nickname, char *sex, char *hphone, 
                       char *wphone, char *mphone, char *pager, char *fax, 
                       char *street, char *webpage, char *notes, char *company, 
                       char *position, char *cstreet, char *ccity, char *cstate, 
                       char *czip, char *ccountry, char *coffice, char *cphone1, 
                       char *cphone2, char *cfax, char *cpager, char *cwebpage,
                       char *cmanager, char *cassistant);


void abook_entry_destroy (Abook *abook, Abook_entry *entry);
Abook_entry *abook_entry_seek_by_name (Abook *abook, char *name);
Abook_entry *abook_entry_seek_by_dname (Abook *abook, char *dname);
Abook_entry *abook_entry_seek_by_email (Abook *abook, char *email);
Abook_entry *abook_entry_seek_by_nickname (Abook *abook, char *nickname);
Abook *abook_entry_find_book (Abook_entry *entry);
char *abook_entry_add_email (Abook_entry *entry, char *email);
void abook_entry_remove_email (Abook_entry *entry, char *email);
char *abook_entry_edit_email (Abook_entry *entry, char *oldmail, char *newmail);
char *abook_entry_has_email (Abook_entry *entry, char *email);
void abook_entry_set_default (Abook_entry *entry, char *email);

Abook_group *abook_group_new (Abook *abook);
Abook_group *abook_group_seek (Abook *abook, char *name);
void abook_group_destroy (Abook *abook, Abook_group *group);

void abook_group_add_member (Abook_group *group, Abook_entry *entry);
void abook_group_remove_member (Abook_group *group, Abook_entry *entry);
GList *abook_group_get_members (Abook_group *group);

void abook_resolve_dnames (Abook *abook);
Abook_entry *abook_entry_seek_name_in_part (Abook *abook, char *partial);

Abook_entry *abook_create_dummy (char *email);
void abook_entry_dump (Abook_entry *entry);

#endif
