/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: calender-opts.h,v 1.9 2001/03/20 20:12:34 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _CALENDER_OPTS_H_
#define _CALENDER_OPTS_H_
#include "puma.h"

#define EVENT_APPOINTMENT      (1L<<0)
#define EVENT_MEETING_REQUEST  (1L<<1)
#define EVENT_ALL_DAY          (1L<<2)

#define EVENT_REMINDER         (1L<<3)
#define EVENT_RECUR            (1L<<4)
#define EVENT_INVITE_RECIP     (1L<<5)
#define EVENT_SEND_INVITES     (1L<<6)
#define EVENT_SPAWN_PROCESS    (1L<<7)  /* Spawn a process our self to schedule a process */
#define EVENT_SPAWN_AT         (1L<<8)  /* Use the at command to schedule a process */
#define EVENT_PREMINDED        (1L<<30) /* The pre-event has fired */
#define EVENT_REMINDED         (1L<<31) /* Event has fired and is done. */


typedef enum AttendeePriority {
	PRIORITY_NONE,
	PRIORITY_MANDATORY,
	PRIORITY_OPTIONAL,
	PRIORITY_RESOURCE,
	PRIORITY_ORGANIZER
} AttendeePriority;

typedef enum PersonalStatus {
	STATUS_BUSY,
	STATUS_FREE,
	STATUS_TENTATIVE,
	STATUS_OUT_OF_OFFICE
} PersonalStatus;

typedef struct EventAttendee {
	char *name;
	Abook_entry *entry; // if it can find it
	AttendeePriority priority;
} EventAttendee;

typedef struct EventTime {
	guint year, month, date, hour, minute;
	time_t time;
} EventTime;

typedef struct CalendarEvent {
	int flags;
	Llist *attendees;
	Llist *categories;
	PersonalStatus status;
	Abook_entry *entry;
	EventTime *start, *stop, *created;
	int reminder;
	unsigned long sequence;
	char *subject;
	char *location;
	char *information;
	char *uid;
	char *from;
	char *program;
} CalendarEvent;

typedef struct Todo {
	char *text;
	char *due;
	int complete;
	GtkWidget *treeitem;
} Todo;

void event_store_destroy_all ();
void event_store_rehash ();
GList *event_store_get ();
void event_store_add (CalendarEvent *event);
void event_store_delete (CalendarEvent *event);

void event_get_today (guint *year, guint *month, guint *day);
Llist *event_first ();
CalendarEvent *event_new ();
void event_add (CalendarEvent *event);
void event_destroy (CalendarEvent *event);
void calendar_rehash_daily ();
void calendar_rehash_weekly ();
CalendarEvent *event_find_conflict (time_t start, time_t end);
void event_display (CalendarEvent *event);
void event_new_needs_action (CalendarEvent *event);
void event_send_to_server (CalendarEvent *event);
int calendar_delete_from_server (CalendarEvent *event);

/* conversions */
void event_time_destroy (EventTime *time);
time_t event_time_to_time_t (EventTime *time);
EventTime *time_t_to_event_time (time_t time);
void event_time_dump (EventTime *time);
EventTime *gtk_to_event_time (GtkEntry *yeare, GtkEntry *monthe, GtkEntry *datee, GtkEntry *timee);

char *event_int_to_month (int month);
char *event_int_to_year (int year);
char *event_int_to_day (int day);
char *event_time_to_char (int hour, int min);

void calendar_rehash ();
int month_string_to_int (char *string);
int year_string_to_int (char *string);
void event_new_from_email (char *smonth, int syear, int sday, int shour, int smin, char *sampm,
                           char *emonth, int eyear, int eday, int ehour, int emin, char *eampm,
													 char *location, char *summary, char *subject, char *uid, char *from,
													 Llist *attendees);

void calendar_dump ();
void event_destroy_all ();
int event_validate (CalendarEvent *event);
Llist *event_get_categories ();
void event_add_category (char *category);
void event_remove_category (char *category);
char *event_categories_to_char (Llist *cats);
char *event_date_to_char (EventTime *time);

GList *todo_first ();
Todo *todo_new ();
void todo_delete (Todo *todo);
Todo *todo_find_from_tree (GtkWidget *treeitem);
void todo_destroy_all ();

#endif /* _CALENDER_OPTS_H_ */
