/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: globals.h,v 1.29 2001/03/20 20:12:34 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _GLOBALS_H_
#define _GLOBALS_H_

#include "pop3smtp.h"

#define ERROR_DIALOG			1
#define WARNING_DIALOG			2
#define MESSAGE_DIALOG			3
#define DEBUG_DIALOG			4

/* message flags */
#define MESSAGE_UNREAD		(1<<0)
#define MESSAGE_REPLY		(1<<1)
#define MESSAGE_FORWARD		(1<<2)
#define MESSAGE_EXPIRED		(1<<3)
#define MESSAGE_REPREQ		(1<<4)

/* global flags */
#define USE_SIGNATURE         (1<<0)
#define USE_UNIX_SIG          (1<<1)
#define HIDE_STATUS_WINDOW		(1<<2)
#define SHOW_SOME_HEADER      (1<<3)
#define SHOW_ALL_HEADER	      (1<<4)
#define HIDE_STATUS_WHEN_DONE (1<<5)
#define REPLY_NONE            (1<<6) 
//if this bit is set, stop
#define REPLY_QUOTED          (1<<7)
//if this bit is set, reply quoted, otherwise reply attached

typedef enum MessageStyle {
	STYLE_UNREAD_EVEN,
	STYLE_UNREAD_ODD,
	STYLE_READ_EVEN,
	STYLE_READ_ODD
} MessageStyle;

typedef struct Llist {
	int id;
	void *data;
	int type; // userdefinable, this thing doesn't care
	char *name;
	struct Llist *prev, *next;
} Llist;

typedef struct AddressBook_entry {
	char *first, *middle, *last, *title, *nickname;
	char *hphone, *wphone, *mphone, *pager, *fax;
	char *street, *webpage;
	char *def_email, *notes;
	Llist *email;
	unsigned long flags;
	int row_sel, row_def;
	int sendtype;
  struct AddressBook_entry *next, *prev;
} AddressBook_entry;


typedef struct AddressBook {
	char *name;
	AddressBook_entry *begin;
	struct AddressBook *next, *prev;
	int rowselected;
} AddressBook;

typedef struct Messages {
	unsigned long timestamp;
	int ID;
	char *fromname, *fromaddy;
	char *toaddy;
	char *subject;
	char *body;
	char *recieved;
	void *children;
  struct Messages *prev, *next;
	void *in_folder;
	int flags;
} Messages;



typedef struct Folders {
  struct Folders *next ;
  int id, pid;
  char *name;
  GtkCTreeNode *tree_item;
  GtkWidget *submenu;
  GtkWidget *ti;
  unsigned long flags;
  int read, unread;
  Messages *messages;
} Folders;

typedef struct _sizexy {
    int x, y;
} sizexy;

typedef struct _panedpos {
    int mnw_hp1, mnw_vp1, mnw_vp2;
    int msgw_vp1;
} panedpos;

typedef struct _window_sizes {
    sizexy main_window, new_message_window, message_window, calendar_window;
    sizexy msearch_window;
} window_sizes;

typedef struct _clistpos {
    int mw_fl[3], mw_msg[6], mw_mime[3];
    int msgw[3];
    int nmsgw[3];
    int ab[2];
    int msw[6];
} clistpos;

typedef struct TradeClientGlobs {

/* gtk widget handles */
	GtkWidget *main_window, *preferences, *newserver_type, *pop3_server_info;
	GtkWidget *new_messages[20], *about, *message_popup, *folder_popup, *create_folder;
	GtkWidget *abook_window, *sel_recp, *status, *filter_window, *mime_popup;
	GtkStyle *read_even, *read_odd, *unread_even, *unread_odd;
	GtkStyle *sens_u1, *sens_r1, *sens_u2, *sens_r2, *sens_u3, *sens_r3, *sens_u4, *sens_r4, *sens_u5, *sens_r5;
	GdkColor debug, message, warning, error;
/* global flags */
	unsigned long flags;
/* bliberous mimifications */
	int num_accounts, acct_selected, h1, h2, h3;
	int wsflag;
	int a1, a2, a3;
	void *visable;
	GList *accounts;
	void *msg_selected;
	int msg_row;
	char *default_account;
	char *create_folder_in;
	char *extcal;
	PopAccount *curaccount;
	Llist *mlist, *mimetypes;
/* addressbook stuff */	
	AddressBook *abooks;
	AddressBook *curbook;
	Llist *vsel, *rsel, *rlist;
/* child processes stuff */
	int dpipe[2];
	int pspipe[2];
	int cspipe[2];
	int pid;
	int getMsgTimeout ;
	int send_timeout;
	int eventLoopTimeout ;
	int eventTriggerTimeout ;
	unsigned long msgtotal, msgblocks, msgcur;
	int msg_count, msg_total;
	int force_fetch;
/* global settings */
	char *real_name, *email, *replyto, *organization, *signature;
	int language;
	gfloat wasted;
	int wwrap;
#ifdef WANTLDAP
	char *ldap_host ;
	char *ldap_username ;
	char *ldap_password ;
	short ldap_port ;
#ifdef WANTSSL
	short ldap_ssl;
#endif
	char *ldap_base ;
#endif
	window_sizes wsize;
	panedpos ppos;
	clistpos cpos;
	char *printer;
	MAILSTREAM *mailstream;
	GtkWidget *imap_window;
	GtkWidget *imap_node;
	GtkWidget *imap_list;
	GtkWidget *imap_list_node;
        ImapInfo *imap_tmp;
} TradeClientGlobs;

typedef struct account_data {
	int type;
	int id;
	int row;
} account_data;

extern TradeClientGlobs *tm_globs;
extern Folders *folders;

#endif /* _GLOBALS_H_ */
