/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: lang-support.h,v 1.4 2001/03/20 20:12:35 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <dirent.h>
#include <ctype.h>
										 

#ifndef _LANG_SUPPORT_H_
#define _LANG_SUPPORT_H_

#ifndef TRUE
#define TRUE (1==1)
#endif
#ifndef FALSE
#define FALSE (1==0)
#endif

#ifdef _
#undef _
#endif
#define _(s) (lang_xlate(__FILE__, __LINE__, __IFUNC__, s))

#define return_if_fail(expr) \
	if (!(expr)) { \
		return; \
	} 
	
#define return_val_if_fail(expr, val) \
	if (!(expr)) { \
		return val; \
	} 

#ifdef __GNUC__
# if __GNUC__ > 2 || \
     (__GNUC__ == 2 && __GNUC_MINOR__ >= (defined __cplusplus ? 6 : 4))
#  define __IFUNC__	__PRETTY_FUNCTION__
# else
#  define __IFUNC__	((__const char *) 0)
# endif
#else
# if defined __STDC_VERSION__ && __STDC_VERSION__ >= 199901L
#  define __IFUNC__	__func__
# else
#  define __IFUNC__	((__const char *) 0)
# endif
#endif

typedef struct _llist llist;
typedef int	(*CompareFunc) (void *a, void *b);

struct _llist {
	void *data;
	llist *next, *prev;
};

typedef struct LangNode {
	char *string;
	llist *locat;
	llist *trans;
} LangNode;

typedef struct LangLoc {
	char *file;
	char *func;
	unsigned int line;
} LangLoc;

typedef struct LangTrans {
	int langnum;
	char *string;
} LangTrans;


void lang_startup ();
char *lang_subst (char *string, char *search, char *replace);
void lang_generate_list( void ) ;
void lang_merge_glade (char *filename);
void lang_save_external (char *filename);
void lang_load_external (char *filename);
void lang_load_external_append_only (char *filename);
void lang_save_internal (char *filename);
char *lang_xlate (char *file, unsigned int line, char *func, char *string);
void lang_shutdown ();
char *lang_getlang ();
char *lang_find_nonliteral (char *string, char c);

#endif
