/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: mail-loadsave.h,v 1.9 2001/03/20 20:12:35 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _MAIL_LOADSAVE_H_
#define _MAIL_LOADSAVE_H_

#include "puma.h"

#define F_INDEX_MISSING			(1<<1)
#define F_INDEX_HEAD_CORRUPT		(1<<2)
#define F_INDEX_MESSAGE_CORRUPT		(1<<3)
#define F_NO_CACHE			(1<<4)
#define F_INDEX_DIRTY                   (1<<5)


typedef struct Message {
	int id;
	unsigned long seek;
	unsigned long size;
	unsigned long hsize;
	unsigned long flags;
	unsigned long time;
	char *folder;
	char *rfolder;
	char *from;
	char *subject;
	char *priority;
	char *replyto;
	char *sensitivity;
	char *expiry;
	char *replyby;
	char *keywords;
	char *date;
	char *text;
	char *uidl;
	time_t texttime;
  struct Message *prev, *next;
} Message;


typedef struct Folder {
	int id;
	unsigned long flags;
	unsigned long asize;
	char *name;
	char *parent;
	Message *msglst;
  struct Folder *prev, *next;
	GtkCTreeNode *tree_item;
	GtkWidget *submenu;
	GtkWidget *ti;
	int unread, total;
} Folder;

typedef struct msg_info {
	Folder *fl;
	Message *msg;
	int row;
} msg_info;


int folder_add_new (char *name, unsigned long size);
int folder_create (char *name, char *parent);
Folder *folder_first ();
Folder *folder_seek (int id);
int folder_seek_by_name (char *name);
void folder_free (Folder *seek);
void folder_delete (int id);
int message_add_new (Folder *fl, unsigned long seek, unsigned long size,
                     unsigned long hsize, unsigned long flags,
                     unsigned long time, char *folder);
Message *message_seek (Folder *fl, int id);
void message_free (Message *msg);
void message_delete (Folder *fl, int id);
void message_list_destroy (Folder *fl);
void folders_read ();
unsigned long assemble_long (unsigned char *snum);
char *disassemble_long (unsigned long num);
void index_write (pfile *pf, Folder *fl);
void index_read (pfile *pf, Folder *fl);
void folders_write_index ();
void folders_read_index ();
int message_hsize (char *message);
int message_save (char *folder, char *message);
char *message_load (Folder *fl, Message *msg);
char *message_load_head (Folder *fl, Message *msg);
char *message_load_body (Folder *fl, Message *msg);
int folder_reindex (Folder *fl);
int folders_compact ();


#endif /* _MAIL_LOADSAVE_H_ */
