/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: pipe.h,v 1.11 2001/03/20 20:12:35 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _PIPE_H_
#define _PIPE_H_

#include "puma.h"

/* data pipe types */
#define PMESSAGE			0x00     /* for now its the only one */
#define QUEUED                          0x7F     /* Indicate that we've queued a message */

/* status pipe types */

/* commands parent -> child */
#define STOP			1 /* stop everthing, go figure */
#define SKIP			2 /* skip the current account */
#define PACKETRESEND		3 /* request packet again */
#define SKIPACCOUNT		4 /* skips the current account */
#define NEWPASSWORD		5 /* sending a new password */

/* status parent -> child */
#define PACKETOK		6 /* child should free the memory used by first non-freed packet sent */



/* commands child -> parent */

/* status child -> parent */
#define DONE			10 /* process is finished and exiting */
#define CURACCOUNT		11 /* sends the id of the current account being processed (pop3/smtp)*/
#define NEWMESSAGES		12 /* number of new messages on server (4 byte long) */
#define PIPEERROR		13 /* error, sends the error number recived from the function */
#define MSGSIZE			14 /* sends the size of the message */
#define TOTALMESSAGES           15 /* Sends the total number of messages detected on current account. */
#define CURPID                  16 /* So we can send the PID that's assocaited with this account. */
#define EMESSAGE	        55 /* Sends error text as reported by an SMTP server for status display */
#define OOOREAD                 90 /* Force an out of order read.  This will force the status_pipe_read
				      to be processed before the results of the get_message_timeout.  Basically
				      this is needed for effective password changes so that they take effect
				      in a single loop and not two. */

/* get errors/warnings returned from pipe_*_get */
#define NOERR			0 /* nothing is wrong */
#define NODATA			1 /* nothing in the pipe */
#define BADDATA			2 /* wrong char was read (data in pip is bad */
#define READERR			3 /* something went wrong on the read */
#define PACKETERR		4 /* packet was not assembled correctly */

/* this could go in forking.h, but thats it */
typedef struct QueList_ {
  struct QueList_ *next ;
  char *data ;
  long length ;
} QueList ;


void pipe_data_put (int type, char *data, unsigned long size);
char *pipe_data_get (int *type, unsigned long *size, int *error);
void pipe_parent_stat_put (int stat, char *data, int size);
char *pipe_parent_stat_get (unsigned long *stat, unsigned long *size, int *error);
void pipe_child_stat_put (int stat, char *data, int size);
char *pipe_child_stat_get (unsigned long *stat, unsigned long *size, int *error);



#endif /* _PIPE_H_ */
