/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: tcal.h,v 1.5 2001/03/20 20:12:35 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _GTK_TCAL_H_
#define _GTK_TCAL_H_

#ifdef SOLARIS25
#define snprintf          g_snprintf
#endif


#include <gtk/gtkwidget.h>
#include <gtk/gtkmenu.h>
#include <gtk/gtkmenuitem.h>
#include <gtk/gtklabel.h>
#include <gtk/gtksignal.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

typedef struct _GtkTCal GtkTCal;
typedef struct _GtkTCalClass GtkTCalClass;
typedef struct _GtkTCalEvent GtkTCalEvent;
typedef struct _GtkTCalDay GtkTCalDay;
typedef struct _GtkTCalMonth GtkTCalMonth;

#define GTK_TCAL(obj)         GTK_CHECK_CAST (obj, gtk_tcal_get_type (), GtkTCal)
#define GTK_TCAL_CLASS(klass) GTK_CHECK_CLASS_CAST (klass, gtk_tcal_get_type (), GtkTCalClass)
#define GTK_TCAL_IS_TCAL(obj) GTK_CHECK_TYPE (obj, gtk_tcal_get_type ())

typedef enum GtkTCalType {
	GTK_TCAL_MONTH,
	GTK_TCAL_YEAR
} GtkTCalType;

typedef enum GtkTCalHasEvent {
	GTK_TCAL_DONT_KNOW,
	GTK_TCAL_HAS_EVENTS,
	GTK_TCAL_HAS_NOT_EVENTS
} GtkTCalHasEvent;

struct _GtkTCal {
	GtkWidget widget;
	
	GList *event_store;
	GList *months;
	
	GtkStyle *bold;
	GtkStyle *small;
	
	guint width, height;
	guint month, year;
	GtkTCalType type;
	GtkWidget *popup;
};

struct _GtkTCalClass {
	GtkWidgetClass parent_class;
	void (* event_selected) (GtkTCal *tcal, gpointer user_data);	
};

struct _GtkTCalEvent {
	char *subject, *summary;
	guint year, month, date;
	time_t start, end;
	gpointer user_data;
};

typedef struct GtkTCalEventCombo {
	GtkTCalEvent *event;
	GtkTCal *tcal;
} GtkTCalEventCombo;

struct _GtkTCalDay {
	GdkWindow *window;
	guint year, month, date;
	guint width, height;
	GtkTCalHasEvent evnt;
	int x, y;
	int rx, ry; // yeah yeah, inconsistency
};

struct _GtkTCalMonth {
	GList *days;
	GdkWindow *window;
	char *name;
	gint x, y;
	guint width, height;
	gint rx, ry;
	guint month;
};

guint gtk_tcal_get_type ();
GtkWidget *gtk_tcal_new (GtkTCalType type, guint year, guint month);
void gtk_tcal_set (GtkTCal *tcal, guint year, guint month);
void gtk_tcal_set_event_store (GtkTCal *tcal, GList *event_store);
void gtk_tcal_event_store_updated (GtkTCal *tcal);
void gtk_tcal_paint (GtkWidget *widget, GdkRectangle *area);

#endif /* _GTK_TCAL_H_ */
