/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: abook-opts.c,v 1.13 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "abook-main.h"

GList *abooks=NULL;
QuestionFunc question_func=NULL;
GtkSignalFunc about_func=NULL;

void abook_set_question_func (QuestionFunc func) {
  question_func=func;
}

void abook_set_about_func (GtkSignalFunc func) {
  about_func=func;
}

void abook_do_nothing () {}

GtkSignalFunc abook_get_about_func () {
  if (about_func) 
    return about_func;
  return GTK_SIGNAL_FUNC (abook_do_nothing);
}

void abook_question (char *question, char *explanation, GtkSignalFunc func, gpointer data) {
  if (question_func) {
    GtkWidget *widget=question_func (question, explanation, func, data);
    gtk_widget_show (widget);
  }	
}

GList *abook_first () {
  return abooks;
}

Abook *abook_new () { 
  Abook *abook=(Abook *)calloc (1, sizeof (Abook));
  abooks=g_list_append (abooks, abook);
  return abook;
}

void abook_destroy (Abook *abook) { 
  GList *eseek, *gseek, *bseek;
  Abook_entry *entry;
  Abook_group *group;
  g_return_if_fail (abook != NULL);
	
  for (eseek=abook->entrys;eseek;eseek=eseek->next) {
    entry=(Abook_entry *)eseek->data;
    for (bseek=entry->email;bseek;bseek=bseek->next) {
      free (bseek->data);
    }
    g_list_free (entry->email);
    entry->email=NULL;
    abook_entry_free (entry);
    free (entry);		
  }
	
  g_list_free (abook->entrys);
  abook->entrys=NULL;
	
  for (gseek=abook->groups;gseek;gseek=gseek->next) {
    group=(Abook_group *)gseek->data;
    if (group->name) free (group->name);
    free (group);
  }
	
  g_list_free (abook->groups);
	
  if (abook->name) free (abook->name);
	
  abooks=g_list_remove (abooks, abook);
  free (abook);
}

void abook_destroy_all () {
  GList *aseek, *eseek, *gseek, *bseek;
  Abook *abook;
  Abook_entry *entry;
  Abook_group *group;
  for (aseek=abooks;aseek;aseek=aseek->next) {
    abook=(Abook *)aseek->data;
    for (eseek=abook->entrys;eseek;eseek=eseek->next) {
      entry=(Abook_entry *)eseek->data;
      for (bseek=entry->email;bseek;bseek=bseek->next) {
	free (bseek->data);
      }
      g_list_free (entry->email);
      entry->email=NULL;
      abook_entry_free (entry);
      free (entry);		
    }
    g_list_free (abook->entrys);
    abook->entrys=NULL;
    for (gseek=abook->groups;gseek;gseek=gseek->next) {
      group=(Abook_group *)gseek->data;
      if (group->name) free (group->name);
      if (group) free (group);
    }
    g_list_free (abook->groups);
    if (abook->name) free (abook->name);
    free (abook);		
  }
  g_list_free (abooks);
}

Abook *abook_seek_by_name (char *name) { 
  GList *seek;
  Abook *abook;
  for (seek=abooks;seek;seek=seek->next) {
    abook=(Abook *)seek->data;
    if (abook)
      if (abook->name) 
	if (strcmp (abook->name, name)==0) 
	  return abook;
  }
  return NULL;
}

void abook_destroy_entries (Abook *abook) {
  while (abook->entrys) {
    abook_entry_destroy (abook, abook->entrys->data);
  }	
}

void
abook_destroy_groups (Abook *abook)
{
  while (abook->groups)
    {
      abook_group_destroy (abook, abook->groups->data);
    }
}

void abook_entry_free (Abook_entry *entry) {
  g_return_if_fail (entry != NULL);
  if (entry->first) { 
    free (entry->first);
    entry->first=NULL;
  }
  if (entry->last) { 
    free (entry->last);
    entry->last=NULL;
  }
  if (entry->middle) { 
    free (entry->middle);
    entry->middle=NULL;
  }
  if (entry->title) { 
    free (entry->title);
    entry->title=NULL;
  }
  if (entry->nickname) { 
    free (entry->nickname);
    entry->nickname=NULL;
  }
	if (entry->sex){
		free (entry->sex);
		entry->sex=NULL;
	}
  if (entry->hphone) { 
    free (entry->hphone);
    entry->hphone=NULL;
  }
  if (entry->wphone) { 
    free (entry->wphone);
    entry->wphone=NULL;
  }
  if (entry->mphone) { 
    free (entry->mphone);
    entry->mphone=NULL;
  }
  if (entry->pager) { 
    free (entry->pager);
    entry->pager=NULL;
  }
  if (entry->fax) { 
    free (entry->fax);
    entry->fax=NULL;
  }
  if (entry->street) { 
    free (entry->street);
    entry->street=NULL;
  }
  if (entry->webpage) { 
    free (entry->webpage);
    entry->webpage=NULL;
  }
  if (entry->notes) { 
    free (entry->notes);
    entry->notes=NULL;
  }
  if (entry->company) { 
    free (entry->company);
    entry->company=NULL;
  }
  if (entry->position) { 
    free (entry->position);
    entry->position=NULL;
  }
  if (entry->cstreet) { 
    free (entry->cstreet);
    entry->cstreet=NULL;
  }
  if (entry->ccity) { 
    free (entry->ccity);
    entry->ccity=NULL;
  }
  if (entry->cstate) { 
    free (entry->cstate);
    entry->cstate=NULL;
  }
  if (entry->czip) { 
    free (entry->czip);
    entry->czip=NULL;
  }
  if (entry->ccountry) { 
    free (entry->ccountry);
    entry->ccountry=NULL;
  }
  if (entry->coffice) { 
    free (entry->coffice);
    entry->coffice=NULL;
  }
  if (entry->cphone1) { 
    free (entry->cphone1);
    entry->cphone1=NULL;
  }
  if (entry->cphone2) { 
    free (entry->cphone2);
    entry->cphone2=NULL;
  }
  if (entry->cfax) { 
    free (entry->cfax);
    entry->cfax=NULL;
  }
  if (entry->cpager) { 
    free (entry->cpager);
    entry->cpager=NULL;
  }
  if (entry->cwebpage) { 
    free (entry->cwebpage);
    entry->cwebpage=NULL;
  }
  if (entry->cassistant) { 
    free (entry->cassistant);
    entry->cassistant=NULL;
  }
  if (entry->cmanager) {
    free (entry->cmanager);
    entry->cmanager=NULL;
  }
}

Abook_entry *abook_entry_new (Abook *abook) { 
  Abook_entry *entry;
  g_return_val_if_fail (abook != NULL, NULL);
	
  entry=(Abook_entry *)calloc (1, sizeof (Abook_entry));
  abook->entrys=g_list_append (abook->entrys, entry);
  return entry;
}

Abook_entry *abook_entry_seek_in_part (char *partial) {
  GList *aseek;
  GList *eseek;
  GList *mseek;
  Abook *abook;
  Abook_entry *entry;
  g_return_val_if_fail (partial != NULL, NULL);
  for (aseek=abooks;aseek;aseek=aseek->next) {
    abook=aseek->data;
    for (eseek=abook->entrys;eseek;eseek=eseek->next) {
      entry=eseek->data;
      for (mseek=entry->email;mseek;mseek=mseek->next) {
	if (strncasecmp ((char *)mseek->data, partial, strlen (partial))==0) {
	  return entry;
	}
      }
    }
  }
  return NULL;
}

char *abook_entry_seek_in_part2 (char *partial) {
  GList *aseek;
  GList *eseek;
  GList *mseek;
  Abook *abook;
  Abook_entry *entry;
  g_return_val_if_fail (partial != NULL, NULL);
  if (strlen (partial)<=0) 
    return NULL;
  for (aseek=abooks;aseek;aseek=aseek->next) {
    abook=aseek->data;
    for (eseek=abook->entrys;eseek;eseek=eseek->next) {
      entry=eseek->data;
      for (mseek=entry->email;mseek;mseek=mseek->next) {
	if (strncasecmp ((char *)mseek->data, partial, strlen (partial))==0) {
	  return (char *)mseek->data;
	}
      }
    }
  }
  return NULL;
}

Abook_entry *abook_entry_seek_name_in_part (Abook *abook, char *partial) {
  GList *eseek;
  Abook_entry *entry;
	char *tmp ;

	g_return_val_if_fail (partial != NULL, NULL);
	g_return_val_if_fail (abook != NULL, NULL);

	tmp = (char *)calloc (1024, sizeof (char));

  if (strlen (partial)<=0) 
    return NULL;
  for (eseek=abook->entrys;eseek;eseek=eseek->next) {
    entry=eseek->data;
#if 0
    format_name (tmp, entry->first, entry->last);
#else
    format_name (1024, tmp, entry->first, entry->last, entry->def_email);
#endif
    if (strncasecmp (tmp, partial, strlen (partial))==0) {
      free (tmp);
      return entry;
    }
  }
	free( tmp ) ;
  return NULL;
}

void abook_entry_edit (Abook_entry *entry, char *first, char *middle, char *last,
                       char *title, char *nickname, char *sex, char *hphone, char *wphone,
		       char *mphone, char *pager, char *fax, char *street,
		       char *webpage,
		       char *company, char *position,
		       char *cstreet, char *ccity, char *cstate, char *czip, 
		       char *ccountry, char *coffice, char *cphone1, 
		       char *cphone2, char *cfax, char *cpager, char *cwebpage,
		       char *cmanager, char *cassistant, char *notes) {
  g_return_if_fail (entry != NULL);
  abook_entry_free (entry);
  if (first) entry->first=strdup (first); 
  if (middle) entry->middle=strdup (middle);
  if (last) entry->last=strdup (last);
  if (title) entry->title=strdup (title);
  if (nickname) entry->nickname=strdup (nickname);
  if (sex) entry->sex=strdup (sex);
  if (hphone) entry->hphone=strdup (hphone);
  if (wphone) entry->wphone=strdup (wphone);
  if (mphone) entry->mphone=strdup (mphone);
  if (pager) entry->pager=strdup (pager);
  if (fax) entry->fax=strdup (fax);
  if (street) entry->street=strdup (street);
  if (webpage) entry->webpage=strdup (webpage);
  if (notes) entry->notes=strdup (notes);
  if (company) entry->company=strdup (company);
  if (position) entry->position=strdup (position);
  if (cstreet) entry->cstreet=strdup (cstreet);
  if (ccity) entry->ccity=strdup (ccity);
  if (cstate) entry->cstate=strdup (cstate);
  if (czip) entry->czip=strdup (czip);
  if (ccountry) entry->ccountry=strdup (ccountry);
  if (coffice) entry->coffice=strdup (coffice);
  if (cphone1) entry->cphone1=strdup (cphone1);
  if (cphone2) entry->cphone2=strdup (cphone2);
  if (cfax) entry->cfax=strdup (cfax);
  if (cpager) entry->cpager=strdup (cpager);
  if (cwebpage) entry->cwebpage=strdup (cwebpage);
  if (cmanager) entry->cmanager=strdup (cmanager);
  if (cassistant) entry->cassistant=strdup (cassistant);
  abook_entry_dump (entry);
}

void abook_entry_destroy (Abook *abook, Abook_entry *entry) { 
  GList *bseek;
  g_return_if_fail (entry != NULL);
  if (!abook) {
    abook=abook_entry_find_book (entry);
  }
  g_return_if_fail (abook != NULL);
	
  abook->entrys=g_list_remove  (abook->entrys, entry);
  for (bseek=entry->email;bseek;bseek=bseek->next) {
    free (bseek->data);
  }
  g_list_free (entry->email);
  entry->email=NULL;

  abook_entry_free (entry);
  free (entry);
}



/*
 * Remember, we may be needing to find a substring of name.
 * In other words, the email address we are looking for may be
 * part of name, so strstr() must be used.
 */
Abook_entry *abook_entry_seek_by_name (Abook *abook, char *name) { 
  /* Make sure we have a name to start with!!! */
  g_return_val_if_fail (name != NULL, NULL);
  if( !strlen(name) ) return NULL ;

  if (abook) {
    GList *seek;
    Abook_entry *entry;
    char *tmp=(char *)calloc (1024, sizeof (char));
    for (seek=abook->entrys;seek;seek=seek->next) {
      entry=(Abook_entry *)seek->data;
#if 0
      format_name (tmp, entry->first, entry->last);
#else
      format_name (1024, tmp, entry->first, entry->last, entry->def_email);
#endif
      /* Commented out by Greg for testing..I think strstr is really what we want!
      if (strcasecmp (tmp, name)==0) {
      */
      if( strcasecmp( tmp, name ) == 0 ) {
	free (tmp);
	return entry;
      }
    }
    free (tmp);
  } else {
    GList *seek;
    Abook_entry *entry;
    for (seek=abooks;seek;seek=seek->next) {
      abook=(Abook *)seek->data;
      if (abook) {
	entry=abook_entry_seek_by_name (abook, name);
	if (entry)
	  return entry;
      }
    }
  }
  return NULL;
}

Abook_entry *abook_entry_seek_by_email (Abook *abook, char *email) { 
  g_return_val_if_fail (email != NULL, NULL);
  if (abook) {
    GList *seek;
    GList *eseek;
    Abook_entry *entry;
    for (seek=abook->entrys;seek;seek=seek->next) {
      entry=(Abook_entry *)seek->data;
      for (eseek=entry->email;eseek;eseek=eseek->next) {
#if 0
	if (strcasecmp ((char *)eseek->data, email)==0) return entry ;
#else
	if( strstr( email, (char *)eseek->data ) ) return entry ;
#endif
      }
    }
  } else {
    GList *seek;
    Abook_entry *entry;
    for (seek=abooks;seek;seek=seek->next) {
      abook=seek->data;
      entry=abook_entry_seek_by_email (abook, email);
      if (entry)
	return entry;
    }
  }
  return NULL;
}

Abook_entry *abook_entry_seek_by_dname (Abook *abook, char *dname) { 
  g_return_val_if_fail (dname != NULL, NULL);
  if (abook) {
    GList *seek;
    Abook_entry *entry;
    for (seek=abook->entrys;seek;seek=seek->next) {
      entry=(Abook_entry *)seek->data;
      if( entry -> dn ) {
	if (strcasecmp (entry->dn, dname)==0)
	  return entry;
      }
    }
  } else {
    GList *seek;
    Abook_entry *entry;
    for (seek=abooks;seek;seek=seek->next) {
      abook=seek->data;
      entry=abook_entry_seek_by_dname (abook, dname);
      if (entry)
	return entry;
    }
  }
  return NULL;
}

Abook_entry *abook_entry_seek_by_nickname (Abook *abook, char *nickname) {
  g_return_val_if_fail (nickname != NULL, NULL);
  if (abook) {
    GList *seek;
    Abook_entry *entry;
    for (seek=abook->entrys;seek;seek=seek->next) {
      entry=(Abook_entry *)seek->data;
      if (strcasecmp (entry->nickname, nickname)==0) {
        return entry;
      }
    }
  } else {
    GList *seek;
    Abook_entry *entry;
    for (seek=abooks;seek;seek=seek->next) {
      abook=(Abook *)seek->data;
      if (abook) {
        entry=abook_entry_seek_by_nickname (abook, nickname);
        if (entry)
          return entry;
      }
    }
  }
  return NULL;
}


Abook *abook_entry_find_book (Abook_entry *entry) {
  GList *seek;
  for (seek=abooks;seek;seek=seek->next) {
    Abook *abook=seek->data;
    if (g_list_find (abook->entrys, entry))
      return abook;
  }
  return NULL;
}

char *abook_entry_add_email (Abook_entry *entry, char *email) { 
  char *tmp;
  g_return_val_if_fail (entry != NULL, NULL);
  g_return_val_if_fail (email != NULL, NULL);
  tmp=strdup (email);
  entry->email=g_list_append (entry->email, tmp);
  return tmp;
}

void abook_entry_remove_email (Abook_entry *entry, char *email) {
  GList *seek;
  char *data=NULL;
  g_return_if_fail (entry != NULL);
  g_return_if_fail (email != NULL);
	
  for (seek=entry->email;seek;seek=seek->next) {
    if (strcasecmp ((char *)seek->data, email)==0) {
      if (strcasecmp (entry->def_email, email)==0) {
	if (seek->next) {
	  entry->def_email=seek->next->data;
	} else {
	  if (seek->prev) {
	    entry->def_email=seek->prev->data;
	  } else {
	    entry->def_email=NULL;
	  }
	}
      }
      data=seek->data;
    }
  }
  if (data)
    entry->email=g_list_remove (entry->email, data);
}

char *abook_entry_edit_email (Abook_entry *entry, char *oldmail, char *newmail) {
  GList *seek;	
  g_return_val_if_fail (entry != NULL, NULL);
  for (seek=entry->email;seek;seek=seek->next) {
    if (strcasecmp (oldmail, (char *)seek->data)==0) {
      free (seek->data);
      seek->data=strdup (newmail);
      if (strcasecmp (oldmail, entry->def_email)==0) {
	entry->def_email=seek->data;
      }
      return (char *)seek->data;
    }
  }
  return NULL;
}

char *abook_entry_has_email (Abook_entry *entry, char *email) { 
  GList *seek;
  g_return_val_if_fail (entry != NULL, FALSE);
  g_return_val_if_fail (email != NULL, FALSE);
	
  for (seek=entry->email;seek;seek=seek->next) {
    if (strcasecmp ((char *)seek->data, email)==0)
      return seek->data;
  }
  return NULL;
}

void abook_entry_set_default (Abook_entry *entry, char *email) {
  char *tmp;
  g_return_if_fail (entry != NULL);
  g_return_if_fail (email != NULL);
	
  if (!abook_entry_has_email (entry, email)) {
    tmp=abook_entry_add_email (entry, email);
  } else {
    tmp=abook_entry_has_email (entry, email);
  }
  entry->def_email=tmp;
}

Abook_group *abook_group_new (Abook *abook) { 
  Abook_group *group;
  g_return_val_if_fail (abook != NULL, NULL);
	
  group=(Abook_group *)calloc (1, sizeof (Abook_group));
	
  abook->groups=g_list_append (abook->groups, group);
	
  return group;
}

void abook_group_destroy (Abook *abook, Abook_group *group) { 
  g_return_if_fail (abook != NULL);
  g_return_if_fail (group != NULL);
	
  abook->groups=g_list_remove (abook->groups, group);
	
  if (group->name) free (group->name);
  free (group);
}

Abook_group *abook_group_seek (Abook *abook, char *name) { 
  g_return_val_if_fail (abook != NULL, NULL);
  g_return_val_if_fail (name != NULL, NULL);
  if (abook) {
    GList *seek;
    Abook_group *group;
    for (seek=abook->groups;seek;seek=seek->next) {
      group=(Abook_group *)seek->data;
      if (strcasecmp (group->name, name)==0) 
	return group;
    }
  }
  return NULL;
}

void abook_group_add_member (Abook_group *group, Abook_entry *entry) { 
  g_return_if_fail (group != NULL);
  g_return_if_fail (entry != NULL);
	
  entry->groups=g_list_append (entry->groups, group);
}

void abook_group_remove_member (Abook_group *group, Abook_entry *entry) { 
  g_return_if_fail (group != NULL);
  g_return_if_fail (entry != NULL);
	
  entry->groups=g_list_remove (entry->groups, group);
}

GList *abook_group_get_members (Abook_group *group) {
  GList *seek;
  GList *ret=NULL;
  GList *eseek;
  for (seek=abooks;seek;seek=seek->next) {
    Abook *abook=seek->data;
    GList *tmp=g_list_find (abook->groups, group);
    if (tmp) {
      for (eseek=abook->entrys;eseek;eseek=eseek->next) {
	Abook_entry *entry=eseek->data;
	if (g_list_find (entry->groups, group)) 
	  ret=g_list_append (ret, entry);
      }
    }
  }
  return ret;
}



void 
abook_resolve_dnames (Abook *abook) {
  GList *dseek, *gseek;
  Abook_group *group;
  Abook_entry *entry;


  g_return_if_fail (abook != NULL);

  for (gseek=abook->groups;gseek;gseek=gseek->next) {
    group=gseek->data;
    for (dseek=group->members;dseek;dseek=dseek->next) {
      entry=abook_entry_seek_by_dname (abook, (char *)dseek->data);
      if (entry) 
	abook_group_add_member (group, entry);
    }
  }
  return ;
}



Abook_entry *abook_create_dummy (char *email) {
  Abook_entry *entry;
  g_return_val_if_fail (abooks != NULL, NULL);
  entry=abook_entry_new (abooks->data);
  entry->def_email=strdup (email);
  entry->dummy=TRUE;
  return entry;
}
 
void abook_destroy_dummys () {
  GList *seek;
  GList *destroy=NULL;
  for (seek=abooks;seek;seek=seek->next) {	
    Abook *abook=seek->data;
    GList *eseek;
    for (eseek=abook->entrys;eseek;eseek=eseek->next) {
      Abook_entry *entry=eseek->data;
#if DEBUG > 6
      printf ("eseek->data: %p\n", eseek);
#endif
      if (entry) {
	if (entry->dummy) {
#if DEBUG > 6
	  printf ("Adding %p to llist\n", entry);
	  printf ("destroy list length: %d\n", g_list_length (destroy));
#endif
	  destroy=g_list_append (destroy, entry);
	}
      } else {
#ifdef DEBUG
	printf ("Entry came out as NULL, not adding to llist\n");
#endif
      }
    }
#ifdef DEBUG
    printf ("destroy list length: %d\n", g_list_length (destroy));
#endif
    for (eseek=destroy;eseek;eseek=eseek->next) {
#if DEBUG > 1
      printf ("eseek->data: %p\n", eseek->data);
#endif
      abook_entry_destroy (abook, eseek->data);
    }
    g_list_free (destroy);
    destroy=NULL;
  }
}



void 
abook_entry_dump (Abook_entry *entry) {
#if DEBUG > 6
  GList *seek;
  g_return_if_fail (entry != NULL);
  printf ("******** ENTRY dump\n");
  printf ("	entry            :          <%p>\n", entry);
  printf ("	entry->first     : '%20.20s'<%p>\n", entry->first, entry->first);
  printf ("	entry->middle    : '%20.20s'<%p>\n", entry->middle, entry->middle);
  printf ("	entry->last      : '%20.20s'<%p>\n", entry->last, entry->last);
  printf ("	entry->title     : '%20.20s'<%p>\n", entry->title, entry->title);
  printf ("	entry->nickname  : '%20.20s'<%p>\n", entry->nickname, entry->nickname);
  printf ("     entry->sex       : '%20.20s'<%p>\n", entry->sex, entry->sex);
  printf ("	entry->hphone    : '%20.20s'<%p>\n", entry->hphone, entry->hphone);
  printf ("	entry->wphone    : '%20.20s'<%p>\n", entry->wphone, entry->wphone);
  printf ("	entry->mphone    : '%20.20s'<%p>\n", entry->mphone, entry->mphone);
  printf ("	entry->pager     : '%20.20s'<%p>\n", entry->pager, entry->pager);
  printf ("	entry->fax       : '%20.20s'<%p>\n", entry->fax, entry->fax);
  printf ("	entry->street    : '%20.20s'<%p>\n", entry->street, entry->street);
  printf ("	entry->webpage   : '%20.20s'<%p>\n", entry->webpage, entry->webpage);
  printf ("	entry->company   : '%20.20s'<%p>\n", entry->company, entry->company);
  printf ("	entry->position  : '%20.20s'<%p>\n", entry->position, entry->position);
  printf ("	entry->cstreet   : '%20.20s'<%p>\n", entry->cstreet, entry->cstreet);
  printf ("	entry->ccity     : '%20.20s'<%p>\n", entry->ccity, entry->ccity);
  printf ("	entry->cstate    : '%20.20s'<%p>\n", entry->cstate, entry->cstate);
  printf ("	entry->czip      : '%20.20s'<%p>\n", entry->czip, entry->czip);
  printf ("	entry->ccountry  : '%20.20s'<%p>\n", entry->ccountry, entry->ccountry);
  printf ("	entry->coffice   : '%20.20s'<%p>\n", entry->coffice, entry->coffice);
  printf ("	entry->cphone1   : '%20.20s'<%p>\n", entry->cphone1, entry->cphone1);
  printf ("	entry->cphone2   : '%20.20s'<%p>\n", entry->cphone2, entry->cphone2);
  printf ("	entry->cfax      : '%20.20s'<%p>\n", entry->cfax, entry->cfax);
  printf ("	entry->cpager    : '%20.20s'<%p>\n", entry->cpager, entry->cpager);
  printf ("	entry->cwebpage  : '%20.20s'<%p>\n", entry->cwebpage, entry->cwebpage);
  printf ("	entry->cmanager  : '%20.20s'<%p>\n", entry->cmanager, entry->cmanager);
  printf ("	entry->cassistant: '%20.20s'<%p>\n", entry->cassistant, entry->cassistant);
  printf ("	entry->def_email : '%20.20s'<%p>\n", entry->def_email, entry->def_email);
  printf ("	entry->notes     : '%20.20s'<%p>\n", entry->notes, entry->notes);
  for (seek=entry->email;seek;seek=seek->next) {
    printf ("%s; ", (char *)seek->data);
  }
  printf ("\n");
#endif
}
