/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: calender-opts.c,v 1.12 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "puma.h"
#include <time.h>
#include "tcal.h"

GList *event_store=NULL;
Llist *event_list=NULL;

GList *event_store_get () {
	return event_store;
}

void event_store_destroy_all () {
	GList *gseek;
	for (gseek=event_store;gseek;gseek=gseek->next) {
		GtkTCalEvent *event=gseek->data;
		free (event);
	}
	g_list_free (event_store);
	event_store=NULL;
}

void event_store_rehash () {
	Llist *lseek;
	GList *gseek;
	GtkTCalEvent *event;
#if 1
	for (gseek=event_store_get();gseek;gseek=gseek->next) {
		event=gseek->data;
		free (event);
	}
	g_list_free (event_store_get());
	event_store=NULL;
#else
	event_store_destroy_all ();
#endif
	for (lseek=event_list;lseek;lseek=lseek->next) {
		CalendarEvent *event=lseek->data;
		GtkTCalEvent *tce=(GtkTCalEvent *)calloc (1, sizeof (GtkTCalEvent));
#if 1
		tce->summary=event->information;
		tce->subject=event->subject;
		tce->year=event->start->year;
		tce->month=event->start->month;
		tce->date=event->start->date;
		tce->user_data=event;
		event_store=g_list_append (event_store, tce);
#else
	event_store_add (event);
#endif
	}
}

void event_store_add (CalendarEvent *event) {
	GtkTCalEvent *tce=(GtkTCalEvent *)calloc (1, sizeof (GtkTCalEvent));
	tce->summary=strdup (event->information);
	tce->subject=strdup (event->subject);
	tce->year=event->start->year;
	tce->month=event->start->month;
	tce->date=event->start->date;
	tce->user_data=event;
	event_store=g_list_append (event_store, tce);	
}

void event_store_delete (CalendarEvent *evnt) {
	GList *seek;
	for (seek=event_store;seek;seek=seek->next) {
		GtkTCalEvent *event=seek->data;
		if (event->user_data==evnt) {
			event_store=g_list_remove (event_store, event);
			free (event);
			return;
		}
	}
}

void event_get_today (guint *year, guint *month, guint *day) {
	time_t t=time (NULL);
	struct tm *tm_time=localtime (&t);
	*year=tm_time->tm_year+1900;
	*month=tm_time->tm_mon;
	*day=tm_time->tm_mday;
}

Llist *event_first () {
	calendar_dump ();
	return event_list;
}

CalendarEvent *event_new () {
  CalendarEvent *retevent=(CalendarEvent *)calloc (1, sizeof (CalendarEvent));


  if (event_list) {
    event_list = llist_append (event_list, retevent);
  } else {
    event_list = llist_append (NULL, retevent);
  }
  return retevent;
}

void event_add (CalendarEvent *event) {
  g_return_if_fail (event != NULL);
  g_return_if_fail (event->start != NULL);
  g_return_if_fail (event->stop != NULL);

  if (event_list) {
    event_list = llist_append (event_list, event);
  } else {
    event_list = llist_append (NULL, event);
  }
}


void event_destroy (CalendarEvent *event) {
	if (llist_seek (event_list, event)) {
		event_list=llist_detach (event_list, event);
	}
}

void calendar_rehash_daily () {
}

void calendar_rehash_weekly () {
}

void event_time_destroy (EventTime *time) {
	if (!time)
		return;
	free (time);
}

time_t event_time_to_time_t (EventTime *time) {
	struct tm *thetime=(struct tm *)calloc (1, sizeof (struct tm));
	time_t newtime;
	if (!time)
		return (time_t)0;
	thetime->tm_min=time->minute;
	thetime->tm_hour=time->hour;
	thetime->tm_mon=time->month;
	thetime->tm_mday=time->date;
	thetime->tm_year=time->year-1900;
	thetime->tm_isdst=-1;
	newtime=mktime (thetime);
	free (thetime);
	return newtime;
}

EventTime *time_t_to_event_time (time_t time) {
	struct tm *thetime=localtime (&time);
	EventTime *ev_time=(EventTime *)calloc (1, sizeof (EventTime));
	ev_time->year=thetime->tm_year+1900;
	ev_time->month=thetime->tm_mon;
	ev_time->date=thetime->tm_mday;
	ev_time->hour=thetime->tm_hour;
	ev_time->minute=thetime->tm_min;
	ev_time->time=time;
	return ev_time;
}

EventTime *gtk_to_event_time (GtkEntry *yeare, GtkEntry *monthe, GtkEntry *datee, GtkEntry *timee) {
	char *year=(gtk_entry_get_text (yeare));
	char *month=(gtk_entry_get_text (monthe));
	char *date= (gtk_entry_get_text (datee));
	char *time= (gtk_entry_get_text (timee));
	char *tmp;
	EventTime *etime=(EventTime *)calloc (1, sizeof (EventTime));
	etime->year=atoi (year);
	etime->date=atoi (date);
	etime->month=month_string_to_int (month);
	tmp=(char *)calloc (25, sizeof (char));
	if (sscanf (time, "%d:%d %s", &etime->hour, &etime->minute, tmp)<2) {
		free (tmp);
		free (etime);
		return NULL;
	}
	if (etime->hour<=12) {
		if (tmp)
			if (strcasecmp (tmp, "pm")==0 && etime->hour!=12) {	
				etime->hour+=12;
			}
			if (strcasecmp (tmp, "am")==0 && etime->hour==12) {
				etime->hour=0;
			}
	}
	if (etime->hour>23) {
		insert_message (_("Invalid Time: hour greater than 23\n"));
		free (etime);
		free (tmp);
		return NULL;
	}
	if (etime->minute>59) {
		insert_message (_("Invalid Time: minute greater than 59\n"));
		free (etime);
		free (tmp);
		return NULL;
	}
	free (tmp);
	return etime;
}

CalendarEvent *event_find_conflict (time_t start, time_t end) {
	Llist *seek=event_list;
	CalendarEvent *event;
	while (seek) {
		event=(CalendarEvent *)seek->data;
		if (event) {
			if (event->start && event->stop) {
				if (start > event->start->time && start < event->stop->time)
					return event;
				if (end > event->start->time && end < event->stop->time)
					return event;
				if (event->start->time > start && event->start->time < end)
					return event;
				if (event->stop->time > start && event->stop->time < end)
					return event;
				if (event->stop->time == start && event->start->time == end)
					return event;
			}
		}
		seek=seek->next;
	}
	return NULL;
}

void event_display (CalendarEvent *event) {
	GtkWidget *cal=calendar_get_widget ();
	guint year, month, day;
	GtkCalendar *daily=GTK_CALENDAR (lookup_widget (cal, "daily_calendar"));
	gtk_calendar_freeze (daily);
	
	gtk_calendar_get_date (daily, &year, &month, &day);
	
	gtk_calendar_select_month (daily, event->start->month, event->start->year);
	gtk_calendar_mark_day (daily, event->start->date);
	
	gtk_calendar_select_month (daily, month, year);
	gtk_calendar_select_day (daily, day);
	
	gtk_calendar_thaw (daily);
}

char *event_int_to_month (gint month) {
	switch (month) {
		case 0:
			return strdup ("January");
		case 1:
			return strdup ("February");
		case 2:
			return strdup ("March");
		case 3:
			return strdup ("April");
		case 4:
			return strdup ("May");
		case 5:
			return strdup ("June");
		case 6:
			return strdup ("July");
		case 7:
			return strdup ("August");
		case 8:
			return strdup ("September");
		case 9:
			return strdup ("October");
		case 10:
			return strdup ("November");
		case 11:
			return strdup ("December");
	}
	return NULL;
}

char *event_int_to_year (int year) {
	static char tmp[7];
	snprintf (tmp, 5, "%d", year);
	return tmp;
}

char *event_int_to_day (int day) {
	static char tmp[7];
	snprintf (tmp, 5, "%d", day);
	return tmp;
}

char *event_time_to_char (int hour, int min) {
	struct tm time;
	static char tmp[40];
	memset (&time, 0, sizeof (struct tm));
	time.tm_hour=hour;
	time.tm_min=min;
	strftime (tmp, 20, "%I:%M %p", &time);
	return tmp;
}

char *event_date_to_char (EventTime *time) {
	static char tmp[60];
	g_return_val_if_fail (time != NULL, NULL);
	snprintf (tmp, 59, "Week Starting %d/%d/%d", time->month+1, time->date+1, time->year);
	return tmp;
}

void calendar_rehash () {
	GtkWidget *widget=calendar_get_widget ();
	GtkCalendar *cal=GTK_CALENDAR (lookup_widget (widget, "daily_calendar"));
	Schedule *schedule=SCHEDULE (lookup_widget (widget, "schedule"));
	CalendarEvent *event;
	guint year, month, date;
	Llist *seek;
	EventTime *time=(EventTime *)calloc (1, sizeof (EventTime));
	time_t upper;
	schedule_destroy_children (SCHEDULE (lookup_widget (widget, "schedule")));
	g_return_if_fail (widget != NULL);
	gtk_calendar_get_date (cal, &year, &month, &date);
	time->year=year;
	time->month=month;
	time->date=date;
	upper=event_time_to_time_t (time);
	free (time);
	schedule_set_upper (schedule, upper);
	for (seek=event_first ();seek;seek=seek->next) {
		event=seek->data;
		if (event->start && event->stop) {
			if (event->start->year == year && event->start->month == month) {
				gtk_calendar_mark_day (GTK_CALENDAR (lookup_widget (widget, "daily_calendar")),
				                       event->start->date);
				if (event->start->date == date) {
					schedule_add_child (SCHEDULE (lookup_widget (widget, "schedule")), 
					                    event->subject, event->start->time, event->stop->time, event);
				}
			}
		}
	}
}

int month_string_to_int (char *string) {
	if (!string)
		return -1;
	if (strcasecmp (string, "january")==0 || strcasecmp (string, "jan")==0)
		return 0;
	if (strcasecmp (string, "february")==0 || strcasecmp (string, "feb")==0)
		return 1;
	if (strcasecmp (string, "march")==0 || strcasecmp (string, "mar")==0)
		return 2;
	if (strcasecmp (string, "april")==0 || strcasecmp (string, "apr")==0)
		return 3;
	if (strcasecmp (string, "may")==0)
		return 4;
	if (strcasecmp (string, "june")==0 || strcasecmp (string, "jun")==0)
		return 5;
	if (strcasecmp (string, "july")==0 || strcasecmp (string, "jul")==0)
		return 6;
	if (strcasecmp (string, "august")==0 || strcasecmp (string, "aug")==0)
		return 7;
	if (strcasecmp (string, "september")==0 || strcasecmp (string, "sep")==0)
		return 8;
	if (strcasecmp (string, "october")==0 || strcasecmp (string, "oct")==0)
		return 9;
	if (strcasecmp (string, "november")==0 || strcasecmp (string, "nov")==0)
		return 10;
	if (strcasecmp (string, "december")==0 || strcasecmp (string, "dec")==0)
		return 11;	
	return -1;
}

int year_string_to_int (char *string) {
	if (!string)
		return -1;
	return atoi (string);
}


void calendar_dump () {
	Llist *seek, *tmp;
	CalendarEvent *event;
	for (seek=event_list;seek;seek=seek->next) {
		event=seek->data;
		if (!event_validate (event)) {
			tmp=seek->next;
			if (tmp) {
				event_list=llist_remove_by_node (tmp->prev);
				return;
			}
		}
	}
}

int event_validate (CalendarEvent *event) {
	if (!event)
		return FALSE;
	if (!event->start)
		return FALSE;
	if (!event->stop)
		return FALSE;

	/* Make sure it doesn't start after it's already ended. */
	if( (event -> start -> time) > (event -> stop -> time) )
	  return FALSE ;
	return TRUE;
}

void event_dump (CalendarEvent *event) {
#if DEBUG > 7
	Llist *att;
	EventAttendee *attend;
	if (!event_validate (event)) {
		printf ("Cannot validate this event.  It is null\n");
	}
	printf ("----------------\n");
	printf ("event: %p\n", event);
	printf ("event->start: %p event->stop: %p\n", event->start, event->stop);
	if (event->start)
		printf ("starttime: %lu\n", event->start->time);
	if (event->stop)
		printf ("stoptime: %lu\n", event->stop->time);
	printf ("event->information: %s <%p>\n", event->information, event->information);
	printf ("event->location: %s <%p>\n", event->location, event->location);
	printf ("event->uid: %s <%p>\n", event->uid, event->uid);	
	for (att=event->attendees;att;att=att->next) {
		attend=att->data;
		if (attend->name)
			printf ("Attendee: %s\n", attend->name);
	}
#endif
}

void event_clear (CalendarEvent *event) {
	Llist *att;
	EventAttendee *attend;
	event_dump (event);
	if (event) {
		if (event->start)
			free (event->start);
		if (event->stop)
			free (event->stop);
		if (event->created)
			free (event->created);
		if (event->subject)
			free (event->subject);
		if (event->location)
			free (event->location);
		if (event->information)
			free (event->information);
		if (event->uid)
			free (event->uid);
		if (event->from)
			free (event->from);
		if (event->program)
			free (event->program);
		llist_blow_away_and_destroy (event->categories);
		for (att=event->attendees;att;att=att->next) {
			attend=att->data;
			if (attend->name)
				free (attend->name);
		}
		llist_blow_away_and_destroy (event->attendees);
		free( event ) ;
	}
}


void event_destroy_all () {
//	CalendarEvent *event;
//	EventAttendee *attend;
	Llist *seek;
	for (seek=event_first ();seek;seek=seek->next) {
//		event=seek->data;
		event_clear (seek->data);
/*		event_dump (event);
		if (event) {		
			if (event->start)
				free (event->start);
			if (event->stop)
				free (event->stop);
			if (event->created)
				free (event->created);
			if (event->subject)
				free (event->subject);
			if (event->information)
				free (event->information);
			if (event->location)
				free (event->location);
			if (event->uid)
				free (event->uid);
			llist_blow_away_and_destroy (event->categories);
			for (att=event->attendees;att;att=att->next) {
				attend=att->data;
				if (attend->name)
					free (attend->name);
			}
			llist_blow_away_and_destroy (event->attendees);
		}*/
		seek->data=NULL;
	}
	llist_blow_away_and_destroy (event_list);
	event_list=NULL;
}


Llist *categories=NULL;

Llist *event_get_categories () {
	if (!categories) {
		int i;
		char **cat=event_get_master_categories ();
		for (i=0;i<NUM_CATEGORIES-1;i++) {
			categories=llist_append (categories, strdup (cat[i]));
		}
	}
	return categories;
}

void event_add_category (char *category) {
	if (!category)
		return;
	categories=llist_append (categories, strdup (category));
}

void event_remove_category (char *category) {
	Llist *seek;
	for (seek=categories;seek;seek=seek->next) {
	}
}

char *event_categories_to_char (Llist *cats) {
	Llist *seek;
	char *tmpc;
	int len=0;
	for (seek=cats;seek;seek=seek->next) {
		len+=strlen ((char *)seek->data)+2;
	}
	len+=5;
	tmpc=(char *)calloc (len, sizeof (char));
	for (seek=cats;seek;seek=seek->next) {
		if (seek->prev)
			sprintf (tmpc, "%s; %s", tmpc, (char *)seek->data);
		else
			sprintf (tmpc, "%s", (char *)seek->data);
	}
	return tmpc;
}

void event_time_dump (EventTime *time) {
#if DEBUG > 6
	g_return_if_fail (time != NULL);
	printf ("EventTime Dump\n**************\n");
	printf ("	year       : %d\n", time->year);
	printf ("	month      : %d\n", time->month+1);  //
	printf ("	date       : %d\n", time->date+1);    //
	printf ("	hour       : %d\n", time->hour);
	printf ("	minute     : %d\n", time->minute);
	printf ("	time_t time: %lu\n", (unsigned long)time->time);
	printf ("EventTime Dump End\n");
#endif
}

GList *todo_list=NULL;
GList *todo_first () { return todo_list; }

Todo *todo_new () {
	Todo *todo=(Todo *)calloc (1, sizeof (Todo));
	todo_list=g_list_append (todo_list, todo);
	return todo;
};

void todo_delete (Todo *todo) {
	if (todo->text) free (todo->text);
	if (todo->due) free (todo->due);
	todo_list=g_list_remove (todo_list, todo);
	free (todo);
}

Todo *todo_find_from_tree (GtkWidget *treeitem) {
	GList *seek;
	for (seek=todo_list;seek;seek=seek->next) {
		Todo *todo=seek->data;
		if (todo->treeitem==treeitem) {
			return todo;
		}
	}
	return NULL;
}

void todo_destroy_all () {
	GList *seek;
	for (seek=todo_list;seek;seek=seek->next) {
		Todo *todo=seek->data;
		if (todo->text) free (todo->text);
		if (todo->due) free (todo->due);
		free (todo);
	}
	g_list_free (todo_list);
	todo_list=NULL;
}
