/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: clipboard.c,v 1.15 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "clipboard.h"
#ifdef DMALLOC
#include <dmalloc.h>
#endif
#define CLIPBOARD
GtkWidget *focus_widget = NULL;
char *clipboard = NULL;
GtkWidget *popup = NULL;
GList *undo_list = NULL;
GList *undo_widgets = NULL;
guint undo_level = 100;
int ignore_next = FALSE;

gboolean find_by_widget (UndoNode * node1, GtkWidget * widget)
{
  if (node1->relative == widget)
    return FALSE;
  return TRUE;
}

GList *
g_list_find_custom_filo (GList * list, gpointer data, GCompareFunc func)
{
  GList *last;

  if (!func)
    return list;
  for (last = g_list_last (list); last; last = g_list_previous (last))
    {
      if (!func (last->data, data))
	return last;
    }
  return NULL;
}

void
clip_focus_in_event (GtkWidget * widget)
{
  focus_widget = widget;
}

int selection_start = -1;
int selection_end = -1;

void
clip_button_press_event (GtkWidget * widget, GdkEventButton * button)
{
  if (button->button == 3)
    {
      if (GTK_IS_EDITABLE (widget))
	{
	  focus_widget = widget;
	  gtk_widget_grab_focus (widget);
	  selection_start = GTK_EDITABLE (widget)->selection_start_pos;
	  selection_end = GTK_EDITABLE (widget)->selection_end_pos;
	}
      else
	{
	  selection_start = -1;
	  selection_end = -1;
	}
      if (GTK_IS_WIDGET (popup))
	{
	  gtk_widget_destroy (popup);
	}
      popup = clip_create_menu (NULL, NULL);
      gtk_menu_popup (GTK_MENU (popup), NULL, NULL, NULL, NULL, 3, 0);
    }
}

void
clip_button_press_event_after (GtkWidget * widget, GdkEventButton * button)
{
  if (selection_start != -1 && selection_end != -1)
    {
      gtk_editable_select_region (GTK_EDITABLE (widget), selection_start,
				  selection_end);
      selection_start = -1;
      selection_end = -1;
    }
}

void
clip_cut (GtkWidget * widget)
{
  if (focus_widget)
    {
      if (GTK_WIDGET_HAS_FOCUS (focus_widget)
	  && GTK_IS_EDITABLE (focus_widget))
	{
	  GtkEditable *edit = GTK_EDITABLE (focus_widget);

	  if (clipboard)
	    free (clipboard);
	  clipboard = gtk_editable_get_chars (edit, edit->selection_start_pos,
					      edit->selection_end_pos);
	  gtk_editable_delete_selection (edit);
	}
    }
  return;
}

void
clip_copy (GtkWidget * widget)
{
  if (!focus_widget)
    return;
  if (GTK_WIDGET_HAS_FOCUS (focus_widget) && GTK_IS_EDITABLE (focus_widget))
    {
      GtkEditable *edit = GTK_EDITABLE (focus_widget);

      if (clipboard)
	free (clipboard);
      clipboard = gtk_editable_get_chars (edit, edit->selection_start_pos,
					  edit->selection_end_pos);
      gtk_editable_copy_clipboard (GTK_EDITABLE (focus_widget));
    }
}

void
clip_paste (GtkWidget * widget)
{
  if (!focus_widget)
    return;
  if (GTK_WIDGET_HAS_FOCUS (focus_widget) && GTK_IS_EDITABLE (focus_widget))
    {
      GtkEditable *edit = GTK_EDITABLE (focus_widget);
      int pos = edit->current_pos;

      if (clipboard)
	gtk_editable_insert_text (edit, clipboard, strlen (clipboard), &pos);
    }
}

void
clip_clear (GtkWidget * widget)
{
  if (!focus_widget)
    return;
  if (GTK_WIDGET_HAS_FOCUS (focus_widget) && GTK_IS_EDITABLE (focus_widget))
    {
      gtk_editable_delete_selection (GTK_EDITABLE (focus_widget));
    }
}

void
clip_select_all (GtkWidget * widget)
{
  if (!focus_widget)
    return;
  if (GTK_WIDGET_HAS_FOCUS (focus_widget) && GTK_IS_EDITABLE (focus_widget))
    {
      gtk_editable_select_region (GTK_EDITABLE (focus_widget), 0, -1);
    }
}

void
clip_undo (GtkWidget * widget)
{
  undo_undo ();
}

void
clear_undo_list ()
{
  GList *seek = NULL;
  UndoNode *node;

  for (seek = undo_list; seek; seek = g_list_next (seek))
  {
    node = seek->data;
    undo_list = g_list_remove (undo_list, seek->data);

    if (node) free (node);
  }

  undo_list = NULL;
  undo_undo ();
}

void
undo_destroy_notify (GtkWidget * widget, GdkEventAny * event)
{
  GList *seek, *tmp = NULL;

  for (seek = undo_list; seek; seek = seek->next)
    {
      UndoNode *node = seek->data;

      if (node->relative == widget)
	tmp = g_list_append (tmp, node);
    }
  for (seek = tmp; seek; seek = seek->next)
    {
      UndoNode *node = seek->data;

#if 0
      if (node->last)
	free (node->last);
#endif
      undo_list = g_list_remove (undo_list, seek->data);
      free (node);
    }
}

void
add_node (UndoNode * node)
{
  g_return_if_fail (node != NULL);
  if (g_list_length (undo_list) > undo_level)
    {
      GList *seek;
      GtkWidget *widget;
      UndoNode *undonode;

      widget = node->relative;
      seek = g_list_find_custom (undo_list, widget,
				 (GCompareFunc) find_by_widget);
      if (seek)
	{
	  seek = g_list_last (seek);
	  undonode = seek->data;
	  if (undonode->last)
	    free (undonode->last);
	  undonode->last = NULL;
	  if (undonode->charecter)
	    free (undonode->charecter);
	  undo_list = g_list_remove (undo_list, undonode);
	  free (undonode);
	}
    }
  undo_list = g_list_prepend (undo_list, node);
}

gint
find_offset (char *text1, char *text2)
{
  guint len1 = strlen (text1);
  guint len2 = strlen (text2);
  guint count = 0;

  if (!text1)
    return -1;
  if (!text2)
    return -1;
  if (strcmp (text1, text2) == 0)
    return -1;
  while (count < len1 && count < len2 && text1[count] == text2[count])
    count++;
  return count;
}

void
undo_undo ()
{
  UndoNode *node;
  UndoNode *last;
  time_t timestamp;

  if (undo_list)
    {
      GList *gtmp;
      GtkEditable *edit;
      gint position;
      int dum = 0;
      int cont = TRUE;

      node = undo_list->data;
      timestamp = node->timestamp;
      while (cont && undo_list)
	{
	  node = undo_list->data;
	  if (!node)
	    return;
	  if (node->timestamp != timestamp)
	    break;
	  gtmp = g_list_find_custom (undo_list->next, node->relative,
				     (GCompareFunc) find_by_widget);
	  if (gtmp)
	    last = gtmp->data;
	  edit = GTK_EDITABLE (node->relative);
	  position = gtk_editable_get_position (edit);
	  switch (node->delta)
	    {
	    case DELTA_DELETE_BLOCK:
	      ignore_next = TRUE;
	      gtk_editable_delete_text (edit, node->offset,
					node->offset + node->length);
	      break;
	    case DELTA_ADD_BLOCK:
	      ignore_next = TRUE;
	      gtk_editable_set_position (edit, node->offset);
	      dum = node->offset;
	      ignore_next = TRUE;
	      gtk_editable_insert_text (edit, node->charecter, node->length,
					&dum);
	      break;
	    case DELTA_DELETE_ALL:
	      ignore_next = TRUE;
	      gtk_editable_delete_text (edit, 0, -1);
	      break;
	    }
	  if (gtmp)
	    {
	      char *tmp = gtk_editable_get_chars (edit, 0, -1);

	      last = gtmp->data;
	      last->last = strdup (tmp);
	      g_free (tmp);
	    }
	  if (node->charecter)
	    free (node->charecter);
	  node->last = NULL;
	  undo_list = g_list_remove (undo_list, node);
	  free (node);
	}
    }
  if (undo_list)
    {
      GList *seek;

      for (seek = undo_widgets; seek; seek = seek->next)
	{
	  gtk_widget_set_sensitive (seek->data, TRUE);
	}
    }
  else
    {
      GList *seek;

      for (seek = undo_widgets; seek; seek = seek->next)
	{
	  gtk_widget_set_sensitive (seek->data, FALSE);
	}
    }
}

void
undo_editable_changed (GtkEditable * editable)
{
  GList *seek;
  char *current;

  if (ignore_next == TRUE)
    {
      ignore_next = FALSE;
      return;
    }
  current = gtk_editable_get_chars (editable, 0, -1);
  seek =
    g_list_find_custom (undo_list, GTK_WIDGET (editable),
			(GCompareFunc) find_by_widget);
  if (seek)
    {
      UndoNode *last = seek->data;

      if (!last)
	{
	  g_free (current);
	  return;
	}
      if (!last->last)
	{
	  g_free (current);
	  return;
	}
      if (strlen (current) > strlen (last->last))
	{
	  UndoNode *node = (UndoNode *) calloc (1, sizeof (UndoNode));

	  node->relative = GTK_WIDGET (editable);
	  node->offset = find_offset (current, last->last);
	  g_return_if_fail (node->offset != -1);
	  node->length = strlen (current) - strlen (last->last);
	  node->delta = DELTA_DELETE_BLOCK;
	  node->last = current;
	  node->timestamp = time (NULL);
	  add_node (node);
	}
      else
	{
	  UndoNode *node = (UndoNode *) calloc (1, sizeof (UndoNode));

	  node->relative = GTK_WIDGET (editable);
	  node->offset = find_offset (current, last->last);
	  node->length = strlen (last->last) - strlen (current);
	  node->charecter = (char *) calloc (node->length + 5, sizeof (char));

	  memmove (node->charecter, last->last + node->offset, node->length);
	  node->delta = DELTA_ADD_BLOCK;
	  node->last = current;
	  node->timestamp = time (NULL);
	  add_node (node);
	}
      last->last = NULL;
    }
  else
    {
      UndoNode *node = (UndoNode *) calloc (1, sizeof (UndoNode));

      node->relative = GTK_WIDGET (editable);
      node->delta = DELTA_DELETE_ALL;
      node->last = current;
      node->timestamp = time (NULL);
      add_node (node);
    }
  if (undo_list)
    {
      GList *seek;

      for (seek = undo_widgets; seek; seek = seek->next)
	{
	  gtk_widget_set_sensitive (seek->data, TRUE);
	}
    }
  else
    {
      GList *seek;

      for (seek = undo_widgets; seek; seek = seek->next)
	{
	  gtk_widget_set_sensitive (seek->data, FALSE);
	}
    }
#ifdef DMALLOC
  if (current)
    dmalloc_verify (current);
#endif
  return;
}

void
clip_handle_editable (GtkWidget * editable)
{
#ifdef CLIPBOARD
  gtk_signal_connect (GTK_OBJECT (editable), "focus_in_event",
		      GTK_SIGNAL_FUNC (clip_focus_in_event), NULL);
  gtk_signal_connect (GTK_OBJECT (editable), "button_press_event",
		      GTK_SIGNAL_FUNC (clip_button_press_event), NULL);
  gtk_signal_connect_after (GTK_OBJECT (editable), "button_press_event",
			    GTK_SIGNAL_FUNC (clip_button_press_event_after),
			    NULL);
  if (GTK_EDITABLE (editable)->editable)
    gtk_signal_connect (GTK_OBJECT (editable), "changed",
			GTK_SIGNAL_FUNC (undo_editable_changed), NULL);
  gtk_signal_connect (GTK_OBJECT (editable), "destroy",
		      GTK_SIGNAL_FUNC (undo_destroy_notify), NULL);
#endif
}

void
undo_widget_destroy_notify (GtkObject * object)
{
  GtkWidget *widget = GTK_WIDGET (object);

  undo_widgets = g_list_remove (undo_widgets, widget);
}

GtkWidget *
clip_create_menu (GtkWidget ** undo, GtkAccelGroup * accel_group)
{
  GtkWidget *menu;
  GtkAccelGroup *menu_accels;
  GtkWidget *menuitem, *tearoff;

  menu = gtk_menu_new ();
  gtk_widget_set_name (menu, "menu");
  gtk_object_set_data (GTK_OBJECT (menu), "menu", menu);
  menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (menu));
  tearoff = gtk_tearoff_menu_item_new ();
  gtk_widget_show (tearoff);
  gtk_container_add (GTK_CONTAINER (menu), tearoff);
  menuitem = gtk_menu_item_new_with_label (_("Cut"));
  gtk_widget_set_name (menuitem, "menuitem");
  gtk_widget_ref (menuitem);
  gtk_object_set_data_full (GTK_OBJECT (menu), "menuitem", menuitem,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu), menuitem);
  if (accel_group)
    gtk_widget_add_accelerator (menuitem, "activate", accel_group,
				GDK_x, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
		      GTK_SIGNAL_FUNC (clip_cut), NULL);
  menuitem = gtk_menu_item_new_with_label (_("Copy"));
  gtk_widget_set_name (menuitem, "menuitem");
  gtk_widget_ref (menuitem);
  gtk_object_set_data_full (GTK_OBJECT (menu), "menuitem", menuitem,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu), menuitem);
  if (accel_group)
    gtk_widget_add_accelerator (menuitem, "activate", accel_group,
				GDK_c, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
		      GTK_SIGNAL_FUNC (clip_copy), NULL);
  menuitem = gtk_menu_item_new_with_label (_("Paste"));
  gtk_widget_set_name (menuitem, "menuitem");
  gtk_widget_ref (menuitem);
  gtk_object_set_data_full (GTK_OBJECT (menu), "menuitem", menuitem,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu), menuitem);
  if (accel_group)
    gtk_widget_add_accelerator (menuitem, "activate", accel_group,
				GDK_v, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
		      GTK_SIGNAL_FUNC (clip_paste), NULL);
  menuitem = gtk_menu_item_new_with_label (_("Select All"));
  gtk_widget_set_name (menuitem, "menuitem");
  gtk_widget_ref (menuitem);
  gtk_object_set_data_full (GTK_OBJECT (menu), "menuitem", menuitem,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu), menuitem);
  if (accel_group)
    gtk_widget_add_accelerator (menuitem, "activate", accel_group,
				GDK_a, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
		      GTK_SIGNAL_FUNC (clip_select_all), NULL);
  menuitem = gtk_menu_item_new_with_label (_("Clear"));
  gtk_widget_set_name (menuitem, "menuitem");
  gtk_widget_ref (menuitem);
  gtk_object_set_data_full (GTK_OBJECT (menu), "menuitem", menuitem,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu), menuitem);
  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
		      GTK_SIGNAL_FUNC (clip_clear), NULL);
  menuitem = gtk_menu_item_new ();
  gtk_widget_set_name (menuitem, "menuitem");
  gtk_widget_ref (menuitem);
  gtk_object_set_data_full (GTK_OBJECT (menu), "menuitem", menuitem,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu), menuitem);
  gtk_widget_set_sensitive (menuitem, FALSE);
  menuitem = gtk_menu_item_new_with_label (_("Undo"));
  gtk_widget_set_name (menuitem, "undo");
  gtk_widget_ref (menuitem);
  gtk_object_set_data_full (GTK_OBJECT (menu), "undo", menuitem,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menu), menuitem);
  if (accel_group)
    gtk_widget_add_accelerator (menuitem, "activate", accel_group,
				GDK_z, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
		      GTK_SIGNAL_FUNC (clip_undo), NULL);
  if (undo)
    *undo = menuitem;
  if (!undo_list)
    gtk_widget_set_sensitive (menuitem, FALSE);
  return menu;
}

GtkWidget *
clip_create_menuitem (GtkWidget * parent, GtkAccelGroup * group)
{
  GtkWidget *menu, *menuitem;
  GtkWidget *undo;

  menuitem = gtk_menu_item_new_with_label (_("Edit"));
  menu = clip_create_menu (&undo, group);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
  gtk_widget_set_name (menuitem, "edit_base");
  gtk_widget_ref (menuitem);
  gtk_object_set_data_full (GTK_OBJECT (menu), "edit_base", menuitem,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (parent), menuitem);
  gtk_signal_connect (GTK_OBJECT (undo), "destroy",
		      GTK_SIGNAL_FUNC (undo_widget_destroy_notify), NULL);
  undo_widgets = g_list_append (undo_widgets, undo);
  return menuitem;
}
