/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: conv.c,v 1.4 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "conv.h"
#ifdef DMALLOC
#include <dmalloc.h>
#endif

/* configuration routines */
void st (GtkWidget *parent, char *object, char *text) {
	int dummy=0;
	g_return_if_fail (GTK_IS_EDITABLE (lw (parent, object)));
	if (!text)
		return;
	gtk_editable_delete_text (GTK_EDITABLE (lw (parent, object)), 0, -1);
	gtk_editable_insert_text (GTK_EDITABLE (lw (parent, object)), text, strlen(text), &dummy);
}

void deflt (GtkWidget *widget) {
  GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_DEFAULT);
  gtk_widget_grab_focus (widget);
  gtk_widget_grab_default (widget);
}

/* widget creation routines */
void seper_new (GtkWidget *menu) {
	GtkWidget *s;
	s = gtk_menu_item_new ();
  ws (s);
  ca (menu, s);
  prot (s);
}

GtkWidget *menubar_new (GtkWidget *parent) {	
	GtkWidget *bar;
  bar = gtk_menu_bar_new ();
  gtk_widget_show (bar);
  gtk_container_add (GTK_CONTAINER (parent), bar);
	return bar;
}

GtkWidget *menuitem_new (GtkWidget *menu, GtkWidget *parent, char *name, char *label, 
                         GtkAccelGroup *group, guint key, guint mask) {
	GtkWidget *i;
	i = gtk_menu_item_new_with_label (label);
	if (name) {
		gtk_widget_set_name (i, name);
	  gtk_widget_ref (i);
	  gtk_object_set_data_full (GTK_OBJECT (parent), name, i,
  	                          (GtkDestroyNotify) gtk_widget_unref);
	}
	if (key > 0 && mask > 0 && group) {
	  gtk_widget_add_accelerator (i, "activate", group, key, mask, GTK_ACCEL_VISIBLE);
	}
  ws (i);
  ca (menu, i);
	return i;
}

GtkWidget *menuitem2_new (GtkWidget *menu, char *label) {
	GtkWidget *i;
	i = gtk_menu_item_new_with_label (label);
  ws (i);
  ca (menu, i);
	return i;
}

GtkWidget *menu_new (GtkWidget *mitem, GtkWidget *p, GtkAccelGroup **g) {
	GtkWidget *m;
  m = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (mitem), m);
	if (g)
  	*g = gtk_menu_ensure_uline_accel_group (GTK_MENU (m));
	return m;
}

GtkWidget *button_new (GtkWidget *parent, char *name, char *label) {	
	GtkWidget *but;
  but = gtk_button_new_with_label (label);
  gtk_widget_set_name (but, name);
  gtk_widget_ref (but);
  gtk_object_set_data_full (GTK_OBJECT (parent), name, but,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (but);
	return but;
}

GtkWidget *button2_new (GtkWidget *parent, char *name, GtkWidget *child) {
	GtkWidget *but;
  but = gtk_button_new ();
  gtk_widget_set_name (but, name);
  gtk_widget_ref (but);
  gtk_object_set_data_full (GTK_OBJECT (parent), name, but,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (but);
	return but;
}

GtkWidget *vb_new (GtkWidget *parent, gboolean one, gint two) {
	GtkWidget *vbox;
  vbox = gtk_vbox_new (one, two);
  ws (vbox);
  ca (GTK_CONTAINER (parent), vbox);
	return vbox;
}

GtkWidget *hb_new (GtkWidget *parent, gboolean one, gint two) {
	GtkWidget *hbox;
  hbox = gtk_hbox_new (one, two);
  ws (hbox);
  ca (GTK_CONTAINER (parent), hbox);
	return hbox;
}

GtkWidget *window_new (char *name, char *title, int x, int y, int w, int h) {
	GtkWidget *win;
  win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (win, name);
  gtk_object_set_data (GTK_OBJECT (win), name, win);
  gtk_widget_set_uposition (win, x, y);
	gtk_widget_set_usize (win, w, h);
  gtk_window_set_title (GTK_WINDOW (win), title);
	return win;
}

GtkWidget *dialog_new (char *name, char *title, int x, int y, int w, int h) {
  GtkWidget *dialog;
  GtkWidget *dialog_action_area;
  GtkWidget *ok_button;
  GtkWidget *cancel_button;

  dialog = gtk_dialog_new ();
  gtk_widget_set_name (dialog, name);
  gtk_object_set_data (GTK_OBJECT (dialog), name, dialog);
  gtk_window_set_title (GTK_WINDOW (dialog), title);
  gtk_window_set_default_size (GTK_WINDOW (dialog), 240, -1);
  gtk_window_set_policy (GTK_WINDOW (dialog), TRUE, TRUE, TRUE);

  dialog_action_area = GTK_DIALOG (dialog)->action_area;
  gtk_widget_set_name (dialog_action_area, "dialog_action_area");
  gtk_object_set_data (GTK_OBJECT (dialog), "dialog_action_area", dialog_action_area);
  gtk_widget_show (dialog_action_area);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area), 10);

  ok_button = gtk_button_new_with_label ("Ok");
  gtk_widget_set_name (ok_button, "ok_button");
  gtk_widget_ref (ok_button);
  gtk_object_set_data_full (GTK_OBJECT (dialog), "ok_button", ok_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ok_button);
  gtk_box_pack_start (GTK_BOX (dialog_action_area), ok_button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (ok_button, GTK_CAN_DEFAULT);

  cancel_button = gtk_button_new_with_label ("Cancel");
  gtk_widget_set_name (cancel_button, "cancel_button");
  gtk_widget_ref (cancel_button);
  gtk_object_set_data_full (GTK_OBJECT (dialog), "cancel_button", cancel_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cancel_button);
  gtk_box_pack_start (GTK_BOX (dialog_action_area), cancel_button, TRUE, TRUE, 0);

  gtk_widget_grab_focus (ok_button);
  gtk_widget_grab_default (ok_button);
  return dialog;
}


GtkWidget *frame_new (GtkWidget *parent, char *name, char *label, guint border) {
	GtkWidget *frame=gtk_frame_new (label);
	if (name) {
		gtk_widget_set_name (frame,  name);
	  gtk_widget_ref (frame);
	  gtk_object_set_data_full (GTK_OBJECT (parent), name, frame,
	                            (GtkDestroyNotify) gtk_widget_unref);
	}
  ws (frame);
  gtk_container_set_border_width (GTK_CONTAINER (frame), border);
	return frame;
}

GtkWidget *entry_new (GtkWidget *parent, char *name) {
	GtkWidget *entry = gtk_entry_new ();
  gtk_widget_set_name (entry, name);
  gtk_widget_ref (entry);
  gtk_object_set_data_full (GTK_OBJECT (parent), name, entry,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry);
	return entry;
}
