/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: forking.c,v 1.77 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "puma.h"
#include "ipc.h"


/*
 * Notice that our queue implementation is actually a FIFO!
 */
static QueList *quelist = NULL ;
static int queSafe = -1 ;
 

/*
 * Init the mutex used by the queue here
 */
void
que_sem_init( void )
{
  queSafe = initSem() ;
  return ;
}


/*
 * Release the system resource held by the IPC semaphore
 */
void
que_sem_deinit( void )
{
  deInitSem( queSafe ) ;
  return ;
}



/* These functions are now thread/process safe.  It seems
 * Alex and Andrew had no concept of concurrent/parallel programming.
 * As such, everything they touched is broken from a concurrent/SMP
 * perspective.  On top of that, it's poorly written!!!  We shouldn't
 * be making yet ANOTHER copy of the message.  We should be storing
 * a pointer to it, whereby, we become the owners of it.
 */
void que_message (char *msg) {
  QueList *queseek ;
#if DEBUG > 4
  int depth = 0 ;
#endif

  lockSem( queSafe ) ;
  queseek = quelist ;

#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  if (queseek==NULL) {
    quelist=(QueList *)calloc (1, sizeof(QueList));
    quelist -> data = msg ;
    quelist -> length = strlen( msg ) ;
#if DEBUG > 4
  depth++ ;
#endif
  } else {
    while( queseek -> next ) {
#if DEBUG > 4
      depth++ ;
#endif
      queseek = queseek -> next ;
    }
    queseek -> next = (QueList *)calloc ( 1, sizeof(QueList) ) ;
    queseek -> next -> data = msg ;
    queseek -> length = strlen( msg ) ;
  }
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
#if DEBUG > 4
  printf( "********************************> Message queue depth of %d.\n", depth ) ;
#endif
  unlockSem( queSafe ) ;

  return ;
}



/* These functions are now thread/process safe.  It seems
 * Alex and Andrew had no concept of concurrent/parallel programming.
 * As such, everything they touched is broken from a concurrent/SMP
 * perspective.  On top of that, it's poorly written!!!  We shouldn't
 * be making yet ANOTHER copy of the message.  We should be storing
 * a pointer to it, whereby, we become the owners of it.
 * This function takes the message on the head of the list, removes
 * it from the queue and returns a pointer to it.  At this point in
 * time, the caller becomes the owner of the message.
 */
char *
deque_message( void ) {
  QueList *q ;
  char *ptr ;


  lockSem( queSafe ) ;
  q = quelist ;

  if( q ) {
    quelist = q -> next ;
  }
  unlockSem( queSafe ) ;

#if DEBUG > 4
  printf( "Dequeing a message of %ld bytes long...\n", q -> length ) ;
  printf( "Dequeing a message of %ld bytes long...\n", q -> length ) ;
  printf( "Dequeing a message of %ld bytes long...\n", q -> length ) ;
  printf( "Dequeing a message of %ld bytes long...\n", q -> length ) ;
  printf( "Dequeing a message of %ld bytes long...\n", q -> length ) ;
  printf( "Dequeing a message of %ld bytes long...\n", q -> length ) ;
#endif

  ptr = q -> data ;
  free( q ) ;

#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  return ptr ;
}


/* These functions are now thread/process safe.  It seems
 * Alex and Andrew had no concept of concurrent/parallel programming.
 * As such, everything they touched is broken from a concurrent/SMP
 * perspective.  On top of that, it's poorly written!!!  We shouldn't
 * be making yet ANOTHER copy of the message.  We should be storing
 * a pointer to it, whereby, we become the owners of it.
 */
void
que_remove( void ) {
  long cnt ;
  QueList *qtemp;


  cnt = 0 ;
  lockSem( queSafe ) ;

#ifdef DEBUG
  if( quelist ) printf( "We have at least one message to remove.\n" ) ;
  else printf( "No messages to remove!\n" ) ;
#endif
  while( quelist ) {
    qtemp = quelist ;
    quelist = quelist -> next ;
    free( qtemp -> data ) ;
    free( qtemp ) ;
    cnt++ ;
  }
  unlockSem( queSafe ) ;

#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
#ifdef DEBUG
  printf( "que_remove() purged %ld messages.\n", cnt ) ;
#endif
  return ;
}


#ifdef DEBUG
#define DEBUGFORK 0
#endif

/* Loop through all of the accounts and take action if needed.
 * Notice that we will not be having a duplicated effort of what
 * fork_receive_message1() does (duh...that would be **VERY** stupid).
 * In fact, we will make use of it to keep it simple!!!!
 */
void 
fork_receive_messages( int id )
{
  int pid ;
#if DEBUGFORK > 0
  int debugger ;
#endif
  char *snum ;
  int childStat ;
  PopAccount *curpop ;

  /* See if we need to process any OOOREAD requests */
  get_message_timeout( NULL ) ;

  /* Synch fix? */
  if( tm_globs -> pid != -1 ) return ;

  /* Now, create a child process to get junk for all accounts */
  if( (pid = fork()) == 0 ) {
#if DEBUGFORK > 0
    /* Here for debugging only - this requires the debugger to change the value!!!! */
    debugger = 1 ;
    while( debugger ) 
      {
	printf( "Waiting on the debugger to save me on pid #%d in fork_receive_messages().\n", getpid() ) ;
	sleep( 1 ) ;
      }
#endif
    /* Get the first account record */
    curpop = (id ? seek_popaccount (id) : first_popaccount());

    /* Now, walk through the list and fetch any mail on accounts that are requesting it */
    while( curpop ) {
      //if( (curpop -> flags & CHECK_WITH_ALL) && (curpop->type != IMAP) )
      if( (curpop -> flags & CHECK_WITH_ALL) )
	{
		if (curpop->type == POP3) {
	  /* If we are supposed to process this account, do so, forking
	   * a child process.  Once we return here, having forked, wait
	   * for it to finish before we continue.  Otherwise, we'll wind
	   * up with lots of concurrent account activity.
	   */
#ifdef DEBUG
	  printf( "****>Forking receiving on account #%d from %d.\n", curpop -> id, getpid() ) ;
#endif

	  /* Make sure everyone knows what account we are on */
	  snum = disassemble_long( (unsigned long)curpop -> id ) ;
#ifdef DEBUG
	  printf( "****>ANNOUNCING ACCOUNT #%d from %d.\n", curpop -> id, getpid() ) ;
#endif
	  pipe_parent_stat_put( CURACCOUNT, snum, 4 ) ;
	  free( snum ) ;

	  /* Now, got get the messages */
	  pid = fork_receive_message1( curpop -> id, 0 ) ;

#ifdef DEBUG
	  printf( "****>Waiting on child process (forked %d) on account #%d.\n", pid, curpop -> id ) ;
#endif
	  /* Wait for the child to finish.  Once it does, reset the value and let everyone know */
	  waitpid( pid, &childStat, 0 ) ;
	  if( WIFSIGNALED( childStat ) ) {
	    printf( "**********Child died because of %d.\n", WTERMSIG(childStat) ) ;

	    /*
	     * Test to see if this is a SIGKILL.  If it is, someone doesn't want us to continue!
	     * We'll exit with a 1 to let everyone know we've been harshly talked to.  ;P~
	     */
	    if( WTERMSIG(childStat) == 9 ) _exit( 1 ) ;
	    else {
	      /*
	       * If we are here, something bad happened which caused the child to die.
	       * As such, we'll notify the parent that this happened via a DONE message.  This
	       * will prevent the parent from hanging on child that would otherwise never report
	       * in.
	       */
#ifdef DEBUG
	      printf( "[Dead Child Process]****>ANNOUNCING DONE\n" ) ;
#endif
	    }
	  }
	  
#ifdef DEBUG
	  else printf( "Child on account #%d exited normally.\n", curpop -> id ) ;
#endif
	}
	}
      if (id) {
        curpop = NULL;
      } else {
      curpop = curpop -> next ;
      }
    }
#ifdef DEBUG
    printf( "****>ANNOUNCING DONE\n" ) ;
#endif
    pipe_parent_stat_put( DONE, "", 0 ) ;

    /* Okay, since all accounts have been serviced, let's kill exit this one */
#ifdef DEBUG
    printf( "Returning from fork_receive_messages() since all accounts have been processed on pid %d\n",
	    getpid() ) ;
#endif
    _exit( 0 ) ;

  } else {
    /* Since we are the parent, let's wait on child (the above fork) to finish elsewhere, this 
     * way, we don't get bunches of zombies...brains....
     */
    tm_globs -> pid = pid ;
  }
#ifdef DEBUG
  printf( "Child process has been created to monitor background receiving...\n" ) ;
#endif
  return ;
}


#ifdef DEBUG
#undef DEBUGFORK
#define DEBUGFORK 0
#endif

int 
fork_receive_message1( int popid, int announce ) {
  int pid=0, newmsgs=0, count=0, error=0;
  char *snum, *msg;
  PopAccount *curpop ;
  MAILSTREAM *stream = NULL ;
  int loggedIn ;
  int index ;
#if DEBUGFORK > 0
  int debugger ;
#endif


  /* Get the current account record */
  curpop = seek_popaccount( popid ) ;

  /* Now, make sure we only have a single connection per account */
  if( curpop -> pid < 1 ) {

    /* Now, fork and let the messaging session run in the backgorund */
    if( ( pid = fork() ) == 0 ) {
#if DEBUGFORK > 0
      /* Here for debugging only - this requires the debugger to change the value!!!! */
      debugger = 1 ;
      while( debugger ) 
	{
	  printf( "Waiting on the debugger to save me on pid #%d in fork_receive_message1().\n", getpid() ) ;
	  sleep( 1 ) ;
	}
#endif


      /* Locate the current account info and create a MAIL stream for it */
      loggedIn = 0 ;

      /* Now that we have the correct account, send it to the parent process */
      if( announce ) {
#if DEBUG > 5
	printf( "Sending 'CURACCOUNT' to parent with id = %d.\n", curpop -> id ) ;
#endif
	snum = disassemble_long( (unsigned long)curpop -> id ) ;
	if( announce ) pipe_parent_stat_put( CURACCOUNT, snum, 4 ) ;
	free( snum ) ;
      }
    
      pid = getpid() ;
#ifdef DEBUG
      printf( "****>ANNOUNCING CURPID %d from %d.\n", pid, curpop -> id ) ;
#endif
      snum = disassemble_long( (unsigned long)pid ) ;
      pipe_parent_stat_put( CURPID, snum, 4 ) ;
      free( snum ) ;
      
      /* Now, figure out how many new messages we have while we are logging into the account */
      newmsgs = new_pop3_login( &stream, curpop -> id ) ;
#if DEBUG > 3
      printf( "Looks like we logged in and have %d new messages that need attention.\n", newmsgs ) ;
#endif

      /* Make sure we have messages to process, if not, report the error */
      snum = disassemble_long( (unsigned long)newmsgs ) ;
      if( newmsgs < 0 ) {
	pipe_parent_stat_put( PIPEERROR, snum, 4 ) ;
	free( snum ) ;

	pipe_data_put( OOOREAD, "", 0 ) ;
#if DEBUG > 4
	printf( "Requesting an Out Of Order Read.\n" ) ;
#endif

	/* Log out if we have an odd error except when the connection
	 * has already been closed, we don't know the host, or CONNREF
	 */
	if( (newmsgs != CONNREF) && (newmsgs != CONNCLOSED) &&
	    (newmsgs != UNKNOWNHOST) ) {
#ifdef DEBUG
	  printf( "calling logout in %d.\n", getpid() ) ;
#endif
	  if( loggedIn ) {
	    new_pop3_logout( curpop -> id, stream ) ;
	    loggedIn = 0 ;
	  }
	}
      } else {
	/* Okay, this means we have messages to process */
	pipe_parent_stat_put( NEWMESSAGES, snum, 4 ) ;
	free( snum ) ;

	/* Now that we know we have messages, let's fetch them! */
	loggedIn = 1 ;
	index = curpop -> popcmsg ;

	/* Remember, msg numbers are base 1 and not base 0 */
	for( count = 1 ; count <= newmsgs ; count++ ) {

#ifdef DMALLOC
	  dmalloc_verify( 0 ) ;
#endif
	  /* If we are deleting messages, we need to fetch all of them */
	  if( curpop -> flags & DELETEFROMSERVER ) {
	    msg = new_pop3_get_message( curpop -> id, count, &error, stream ) ;
	  } else {
	    msg = new_pop3_get_message( curpop -> id, (index + count), &error, stream ) ;
	  }

#ifdef DMALLOC
	  dmalloc_verify( 0 ) ;
#endif

	  /* Now, regardless of either method we did to fetch messages, check for errors */
	  if( error ) {
#ifdef DEBUG
	    printf( "new_pop3_get_message has returned an error of %d.\n", error ) ;
#endif
	    switch( error ) {
	    case PARENTSKIP:
	      if( msg ) free( msg ) ;
	      snum = disassemble_long( (unsigned long)abs(PARENTSKIP) ) ;
	      printf( "Got PARENTSKIP error back.\n" ) ;
	      break ;
	      
	    case PARENTSTOP:
	      if( msg ) free( msg ) ;
	      snum = disassemble_long( (unsigned long)abs(PARENTSTOP) ) ;
	      printf( "Got PARENTSTOP error back.\n" ) ;
	      break ;
	      
	    default:
#ifdef DEBUG
	      printf( "default error hit in fork_receive_message1().\n" ) ;
#endif
	      break ;
	    }

	    /* Now, report the error to the parent. */
	    pipe_parent_stat_put( PIPEERROR, snum, 4 ) ;
	    free( snum ) ;
	  } else {
	    /* We didn't have any errors, so process the message */
	    if( msg ) {
#if DEBUG > 5
	      printf( "Posting a 'PMESSAGE' message.\n" ) ;
#endif
	      pipe_data_put( PMESSAGE, msg, strlen(msg) ) ;
#if DEBUG > 5
	      printf( "Posted a 'PMESSAGE' message.\n" ) ;
#endif
#ifdef DMALLOC
	      dmalloc_verify( 0 ) ;
#endif
	    }
	  }
	}
	/*
	 * That is, we've looped through all of the messages, so I
	 * think this means that were are no more messages
	 */
#if DEBUG > 4
	printf( "no more messages on account %d, logging off account %d on pid %d\n", 
		curpop -> id, popid, getpid() ) ;
#endif
      }
	
      /* Now, log out!!! */
#ifdef DMALLOC
      dmalloc_verify( 0 ) ;
#endif
      if( loggedIn ) {
	new_pop3_logout( curpop -> id, stream ) ;
	loggedIn = 0 ;
      }


      /* We'll use this to reset the curpid value since the parent is waiting on us. */
#ifdef DEBUG
      printf( "****>ANNOUNCING CURPID %d from account #%d because we are done with it.\n", -1, curpop -> id ) ;
#endif
      snum = disassemble_long( (unsigned long)curpop -> pid ) ;
      pipe_parent_stat_put( CURPID, snum, 4 ) ;
      free( snum ) ;
      
      /* Tell everyone that we're done */
      if( announce ) {
#if DEBUG > 5
	printf( "ANNOUNCING 'DONE' on account #%d.\n", curpop -> id ) ;
#endif
	pipe_parent_stat_put( DONE, "", 0 ) ;
      }

#if DEBUG > 4
      printf( "*************************Exiting...\n" ) ;
      printf( "Exiting pid fork_message1 %d.\n", getpid() ) ;
      if( !announce ) printf( "Expect to see another account follow...\n" ) ;
      printf( "*************************Exiting...\n" ) ;
#endif
#ifdef DMALLOC
      dmalloc_verify( 0 ) ;
#endif
      _exit( 0 ) ;
    } /* End of message processing */
  } else {
    /*
     * He hit here because we are the parent process, noticed
     * that we said the pid of this child process so the the
     * parent can make sure he doesn't have any mutant zombie
     * kids running around.  Must eat brains.....duooh!
     */
#ifdef DEBUG
    printf( "****>Assigned account PID and returning control to monitor process on %d\n", pid ) ;
#endif
  }
return pid ;
}


#ifdef DEBUG
#undef DEBUGFORK
#define DEBUGFORK 0
#endif

void new_fork_send_message (PopAccount *pop, MailMessage *mmessage, char *recipline) {
  int pid, serror, i;
  char *snum;
  int recip_count=count_delimiters (recipline, ',')+1;
  char *recip[recip_count];
  char *smtpsender, *domain, *tmp, *tmp2;
#if DEBUGFORK > 0
  int debugger ;
#endif


  if (recip_count>1) {
    recip[0]=cut_string (recipline, ',', 1, 1);
    for (i=1;i<recip_count;i++) {
      tmp=cut_string (recipline, ',', 2, i);
      strip_lead_spaces( tmp ) ;
     if (i<(recip_count-1)) {
	tmp2=cut_string (tmp, ',', 1, 1);
	recip[i]=rm_crlf (tmp2);
	free (tmp2);
      } else {
	recip[i]=rm_crlf (tmp);
      }
      if (recip[i][0]==' ') recip[i]++;// bad bad, we cant free that byte now
      free (tmp);
    }
  } else recip[0]=rm_crlf (recipline);


#ifdef DEBUG
  printf( "Forking send for background operation for %s\n", pop->emailaddr ) ;
#endif
  tm_globs -> curaccount = pop ;
  if( ( pid = fork() ) == 0 ) {
#if DEBUGFORK > 0
      /* Here for debugging only - this requires the debugger to change the value!!!! */
      debugger = 1 ;
      while( debugger ) 
	{
	  printf( "Waiting on the debugger to save me on pid #%d in new_fork_send_message().\n", getpid() ) ;
	  sleep( 1 ) ;
	}
#endif
    smtpsender=strdup(pop->emailaddr);
    serror=0;
    if (serror<0) {
#ifdef DEBUG
      printf ("Child puts PIPEERROR\n");
#endif
      snum=disassemble_long (abs(serror));
      pipe_parent_stat_put (PIPEERROR, snum, 4);
      free (snum);
      goto sdone1;
    }

#ifdef DMALLOC
    dmalloc_verify( 0 ) ;
#endif
    domain=cut_string (pop->emailaddr, '@', 2, 1);
#ifdef DEBUG
    printf("		trying to send msg\n");
#endif		
    serror = new_sendmessage( mmessage ) ;
#ifdef DEBUG
    printf("		sendmsg DONE\n");
#endif		
    for (i=0; i<recip_count; i++) {
      free (recip[i]);
    }
    if (serror<0) {
#ifdef DEBUG
      printf ("Child puts PIPEERROR\n");
#endif
      snum=disassemble_long (abs(serror));
      pipe_parent_stat_put (PIPEERROR, snum, 4);
      free (snum);
    }

  sdone1:
    free (smtpsender);

#ifdef DEBUG
    printf("Child is 'DONE'\n");
#ifdef DMALLOC
    dmalloc_verify( 0 ) ;
#endif
#endif
    pipe_parent_stat_put( DONE, "", 0 ) ;
#ifdef DEBUG
    printf( "Child process %d is dying.\n", getpid() ) ;
#endif
    /*
     * Here, we exit the child process in our own process (the child), so that
     * we can gracefully die.  If we exit() as we should, GTK freaks out!
     */
    _exit( 0 ) ;
  } else {
    pop -> spid = pid ;
  }
#ifdef DMALLOC
    dmalloc_verify( 0 ) ;
#endif
  return ;
}




int get_message_timeout (gpointer data) {
  int type=0, error=0, hsize;
  unsigned long size;
  Message *msg=NULL;
  char *message;
  Folder *fl=NULL;
  Header_Info *hinfo;
  FilterAction *fact;
  int serror;
  unsigned long stype, ssize;
  char *stats = NULL ;
  Attachment *att;
  char *msgbody;

#if DEBUG > 4
  printf ("---> get_message_timeout() has been called! <---\n");
#endif

  /*
   * See if the child process has finished.  If so, it means all of the grand
   * kids have finished playing.  At any rate, we need to check from time to
   * time to make sure they've done their thing.  Notice that we're not waiting
   * around because we have better things to do.  If it's done, we reset the
   * value of the global pid.
   */

  message=pipe_data_get (&type, &size, &error);
#if DEBUG > 6
  printf( "pipe_data_get() has returned with type = %d, size = %d and error = %d.\n",
	  type, size, error ) ;
#endif

  switch (error) {
  case READERR:
#ifdef DEBUG
    printf ("For some reason, the actual file descriptor is messed up and won't let me read\n");
#endif
    break;

  case PACKETERR:
#ifdef DEBUG
    printf ("Packet error is when uh, the different parts don't come in, in the right order\n");
#endif
    break;

  default:
    switch( type ) {
  case OOOREAD:
#if DEBUG > 4
    printf( "'OOOREAD' message is being processed.\n" ) ;
#endif
    status_pipe_timeout( NULL ) ;
    break ;

    case QUEUED:
    case PMESSAGE:
#if DEBUG > 5
      printf( "'QUEUED' or 'PMESSAGE' message is being processed.\n" ) ;
#endif

#ifdef DMALLOC
      dmalloc_verify( 0 ) ;
#endif
      hsize = message_hsize( message ) ;
#if DEBUG > 7
      printf( "got hsize.\n" ) ;
#endif
#ifdef DMALLOC
      dmalloc_verify( 0 ) ;
#endif
      hinfo = parse_header( message ) ;
      att=(Attachment *)calloc (1, sizeof (Attachment));
      att->mtype=strdup ("dummyatt");
      msgbody = mime_process( message, att, 0 );
      free(att->mtype);
      free(att);
#if DEBUG > 7
      printf( "got hinfo.\n" ) ;
#endif
#ifdef DMALLOC
      dmalloc_verify( 0 ) ;
#endif
      stats = pipe_child_stat_get( &stype, &ssize, &serror ) ;
#if DEBUG > 7
      printf( "got child stat.\n" ) ;
#endif
#ifdef DMALLOC
      dmalloc_verify( 0 ) ;
#endif

      /* This way, we only increment the msg count that we have submitted on our queue!!! */
      tm_globs -> msg_count++ ;
      if (tm_globs->curaccount)
        tm_globs -> curaccount -> popcmsg++ ;
      
      /* Now, process the message filters */
      if (tm_globs->msg_count>tm_globs->msg_total) tm_globs->msg_count=tm_globs->msg_total;
      set_receiving (tm_globs->msg_count, tm_globs->msg_total);

	if ((hinfo->uidl)&&(index_find_uidl (hinfo->uidl, NULL)))
	{
#if DEBUG > 4
		printf("FOUND DUPLICATE UIDL (%s)\n",hinfo->uidl);
#endif
		msg = NULL;
		fl = NULL;
	} else {
/* XXX: trey: add message-id checking */
      
#if DEBUG > 6
      printf( "applying filters to message...\n" ) ;
#endif
      fact = filter_message( hinfo -> subject, hinfo -> from, hinfo -> to, hinfo->cc, hinfo->bcc, msgbody, hinfo->keywords, hinfo->replyto) ;

      if( fact ) {
	switch( fact -> action ) {
	case F_ACTION_DELETE:
	  /* The filter says we are to delete it */
#if DEBUG > 6
	  printf( "deleting message from: '%s' subject: '%s'\n", hinfo->from, hinfo->subject ) ;
#endif
	  msg = NULL ;
	  fl = NULL ;
	  break ;
	  
	case F_ACTION_MOVE:
	  /* The filter says we are to move it */
#if DEBUG > 6
	  printf( "Moving message from: '%s' subject: '%s'\n", hinfo->from, hinfo->subject ) ;
#endif
	  fl = folder_seek( folder_seek_by_name( fact->folder ) ) ;
	  msg = message_seek( fl, message_save( fl->name, message ) ) ;
	  break ;
	  
	default:
	  /* The filter didn't say what to do with it, so file it! */
	  if( tm_globs -> curaccount != NULL ) {
	    fl = folder_seek( folder_seek_by_name( ( (PopAccount *)tm_globs -> curaccount ) -> folder ) ) ;
	    msg = message_seek( fl, message_save( ( (PopAccount *)tm_globs -> curaccount ) -> folder, message ) ) ;
	  } else {
	    fl = folder_first();
	    msg = message_seek( fl, message_save( fl->name, message ) ) ;
	  }
	  break ;
	}
      }
      /* Be sure to free the memory used on the message filter, if any! */
      if( fact ) filter_action_destroy( fact ) ;
    }
      destroy_hinfo( hinfo ) ;
      free (message);
      if(msgbody) free (msgbody);

      if( msg ) {
	msg -> flags |= MESSAGE_UNREAD ;
	msg -> rfolder = strdup( fl -> name ) ;
      }
      
      if( fl ) fl -> unread++ ;
      if( fl && msg ) create_new_message( fl, msg ) ;
      break ;
    }
  }

  /*
   * Now, check the status of everything that's going on.
   * This was previously on it's own timer, but it doesn't really make sense to do
   * that since, to some extent they really need to be serialized.
   */
  return status_pipe_timeout( NULL ) ;
}




int status_pipe_timeout(gpointer data) {
  unsigned long type=0, size=0;
  int forkStat ;
  int error=0;
  char *stat;
  unsigned long tmpl;
  int tmpi=0;
  GtkWidget *pwd_diag;
  PopAccount *acct=tm_globs->curaccount;


#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif

  stat=pipe_parent_stat_get (&type, &size, &error);
#if DEBUG > 6
  if( stat ) printf( "pipe_parent_stat_get returned with %s and type = %ld, size=%ld, error=%d,\n", stat,
		     type, size, error ) ;
  else printf( "stat = NULL, type = %ld, size=%ld, error=%d,\n",
	       type, size, error ) ;
#endif

  
  switch( type ) {
  case -1:
    break ;
    
  case DONE:
#ifdef DEBUG
    printf( "'DONE' has been received, waiting on the child process to terminate.\n" ) ;
#endif
    /*
     * Remember, we are waiting on our child!!  In turn, our child is waiting
     * on it's children.  So, once our child is done, all of the grandchildren are done.
     * In either case, we will not be hanging waiting for them to finish.  We might need
     * to rethink this in the future, but for now, it seems a-okay!
     */
    if((acct)&&(acct -> pid != -1 ))
      {
#ifdef DEBUG
	printf( "Waiting on child process %d in DONE section on account #%d.\n", acct -> pid, acct -> id ) ;
#endif
	waitpid( acct -> pid, &forkStat, 0 ) ;

#ifdef DEBUG
	printf( "Waiting on child process %d in DONE has finished account #%d.\n", acct -> pid, acct -> id ) ;
#endif
	/* Now, start timed background fetches again! */
//TREY	tm_globs -> getMsgTimeout = gtk_timeout_add( 950, (GtkFunction)message_event_loop, NULL ) ;

	/*
	 * Since we are setup for deleting, we need to reset our values
	 * after the transfer is completed.
	 */
	if( tm_globs -> curaccount -> flags & DELETEFROMSERVER ) {
	  tm_globs -> curaccount -> popcmsg = 0 ;
	  tm_globs -> curaccount -> poptmsg = 0 ;
#if DEBUG > 4
	  printf( "Message counters have been reset because we are in delete mode for this account.\n" ) ;
#endif
	}
	    
	/* Okay, reset the pid values */
	acct -> pid = -1 ;
      }
	
    /* No matter what, we need to see if the monitor process has finished on this DONE too. */
    if( tm_globs -> pid != -1 ) {
      forkStat = 0 ;
      
#ifdef DEBUG
      printf( "Waiting on child monitor process %d in DONE section.\n", tm_globs -> pid ) ;
#endif
      if( tm_globs -> pid == waitpid( tm_globs -> pid, &forkStat, WNOHANG ) ) {
#ifdef DEBUG
	printf( "Waiting on child monitor process %d in DONE section has completed.\n", tm_globs -> pid ) ;
#endif
//TREY	tm_globs -> getMsgTimeout = gtk_timeout_add( 950, (GtkFunction)message_event_loop, NULL ) ;
	
	/* Now, start timed background fetches again! */
	tm_globs -> pid = -1 ;
      }
    }
	
    /* Re-enable the GUI */
#ifdef DMALLOC
    dmalloc_verify( 0 ) ;
#endif
    //TREYgtk_timeout_remove (tm_globs->getMsgTimeout);
    gtk_widget_set_sensitive (lookup_widget (tm_globs->main_window, "get_message"), TRUE);
    gtk_widget_set_sensitive (lookup_widget (tm_globs->status, "stopthat"), FALSE);
    gtk_widget_set_sensitive (lookup_widget (tm_globs->status, "skip"), FALSE);
    set_progress (1, 1);
    set_receiving (0, 0);
    folders_write_index ();
    set_account_name (_("Done"));
    insert_message (_("Finished.\n"));
#ifdef DMALLOC
    dmalloc_verify( 0 ) ;
#endif

    if ((tm_globs->flags & HIDE_STATUS_WHEN_DONE)) {
      destroy_status (tm_globs->status, NULL);
    }
    if (stat) free (stat);

    return FALSE;
    break;

  case CURACCOUNT:
    tmpl=assemble_long (stat);
    acct=tm_globs->curaccount=seek_popaccount ((int)tmpl);
    set_account_name (acct->name);
    insert_message (_("Logging onto account %s.\n"), acct->name);
    set_receiving (0, 0);
    set_progress (0, 0);
    break;

  case CURPID:
    tmpl=assemble_long( stat ) ;
    tm_globs -> curaccount -> pid = tmpl ;
#ifdef DEBUG
    printf( "Cur piD = %ld on account #%d.\n", tmpl, tm_globs -> curaccount -> id ) ;
#endif
    break ;

  case TOTALMESSAGES:
    tmpl = assemble_long( stat ) ;
    tm_globs -> curaccount -> poptmsg = tmpl ;

    if( (tm_globs -> curaccount -> flags & FORCE_FETCH) || ((tm_globs -> curaccount -> popcmsg) > tmpl) ) {
      if( acct -> flags & FORCE_FETCH ) tm_globs -> curaccount -> flags ^= FORCE_FETCH ;
      tm_globs -> curaccount -> popcmsg = 0 ;
    }

    /* Reset the force bit if it's set */
    if( tm_globs -> curaccount -> flags & FORCE_FETCH ) tm_globs -> curaccount -> flags ^= FORCE_FETCH ;

#ifdef DEBUG
    printf( "Got a 'TOTALMESSAGES' message with %ld total messages.\n", tmpl ) ;
#endif
    break ;

  case NEWMESSAGES:
    tmpl=assemble_long (stat);
    tm_globs->msg_count=1;
    tm_globs->msg_total=tmpl;

    set_receiving (tm_globs->msg_count, tm_globs->msg_total);
    insert_message (_("Connected, %ld new messages.\n"), tmpl);
#ifdef DEBUG
    printf( "Got a 'NEWMESSAGES' message with %ld new messages.\n", tmpl ) ;
#endif
    break;

  case MSGSIZE:
    tmpl=assemble_long (stat);
#ifdef DEBUG
    printf ("Receiving message of size '%d'\n", (int)tmpl);
#endif
    tm_globs->msgtotal=tmpl;
    tm_globs->msgblocks=0;
    set_progress (tm_globs->msg_count, tm_globs->msg_total);
    break;

  case PIPEERROR:
    tmpl = assemble_long( stat ) ;

    switch (tmpl) {
    case 0:
      break ;

    case NDEFACCOUNT:
      insert_error (_("Bug encountered. Tried to access undefined account\n"));
      break;
      
    case NDEFHOST:
      insert_error (_("Undefined host on account %s\n"), acct->name); 
      break;
      
    case UNKNOWNHOST:
      insert_error (_("Unknown Host on account %s\n"), acct->name);
      pipe_child_stat_put (SKIP, "", 0);
      break;
      
    case NDEFUSER:
      insert_error (_("Undefined user on account %s\n"), acct->name);
      break;
      
    case NDEFPASSWORD:
      insert_error (_("Password Needed\n"));
      pwd_diag=create_enter_password (tm_globs->curaccount);
      gtk_widget_show (pwd_diag);
      break;

    case UNKNOWNUSER:
      insert_error (_("Server says: Unknown user '%s' on account %s\n"), acct->puser, acct->name);
      break;

    case INCORRECTPW:
      insert_error (_("Server said: Incorrect password\n"));
      pwd_diag=create_enter_password (tm_globs->curaccount);
      gtk_widget_show (pwd_diag);
      break;

    case PARENTSTOP:
      insert_warning (_("Stopping transaction\n"));
      insert_warning (_("Received %d messages successfully.\n"),tm_globs->msg_count-1);
      break;

    case PARENTSKIP:
      insert_warning (_("Skipping Account\n"));
      insert_warning (_("Received %d messages successfully.\n"),tm_globs->msg_count-1);
      break;

    default:
      insert_error (_("Unknown error type %d received\n"),tmpi);
    }
    break;

  case NEWPASSWORD:
#if DEBUG > 4
    printf( "New password message has been received and password is being set w/length = %ld...\n", size ) ;
#endif
    if( acct -> ppasswd ) free( acct -> ppasswd ) ;
    acct -> ppasswd = malloc( size*sizeof(char)+2 ) ;
    strncpy( acct -> ppasswd, stat, size ) ;
    acct -> ppasswd[size] = 0x00 ;
    break ;
  }

if (stat) free (stat);
#ifdef DMALLOC
dmalloc_verify( 0 ) ;
#endif

return TRUE;
}



int 
send_message_timeout (StupidFunc *func) {
  unsigned long type=0, size=0;
  int error=0;
  char *stat;
  unsigned long tmpl;
  int tmpi=0;
  PopAccount *pop ;



#if DEBUG > 3
  printf( "---*******=======>>>>>>>>send_message_timeout has been hit\n" ) ;
#endif

#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  pop = tm_globs -> curaccount ;
  if( !pop ) {
#ifdef DEBUG
  printf( "There isn't a pop account assigned, so this isn't going to work...exit FALSE!!!\n" ) ;
#endif
    return FALSE ;
  }
  stat=pipe_parent_stat_get (&type, &size, &error);

  if (error!=0) {

    switch (error) {
    case NODATA:
      break;

    case BADDATA:
#ifdef DEBUG
      printf ("Something really messed up stat\n");
#endif
      break;

    case READERR:
#ifdef DEBUG
      printf ("For some reason, the actual file descriptor is messed up and won't let me read stat\n");
#endif
      break;

    case PACKETERR:
#ifdef DEBUG
      printf ("Packet error is when uh, the different parts don't come in, in the right order stato\n");
#endif
      break;
    }
  } else {

    switch (type) {

    case -1:
      break;

    case DONE:
#ifdef DEBUG
      printf( "Waiting for child to finish in 'DONE' logic...\n" ) ;
#endif
      if( pop -> spid != -1 )
	{
	  if( waitpid( pop -> spid, NULL, WNOHANG ) ) {
	    pop -> spid = -1 ;
	  }
#ifdef DEBUG
	  printf( "Child is done, resetting GUI...\n" ) ;
#endif
	  gtk_timeout_remove( tm_globs -> send_timeout ) ;
	  gtk_widget_set_sensitive (lookup_widget (tm_globs->main_window, "get_message"), TRUE);
	  gtk_widget_set_sensitive (lookup_widget (tm_globs->status, "stopthat"), FALSE);
	  gtk_widget_set_sensitive (lookup_widget (tm_globs->status, "skip"), FALSE);
	  set_progress (1, 1);
	  set_receiving (0, 0);
	  
	  folders_write_index ();
	  set_account_name (_("Done"));
	  insert_message (_("Finished.\n"));
	  
	  if ((tm_globs->flags & HIDE_STATUS_WHEN_DONE)) {
	    destroy_status (tm_globs->status, NULL);
	  }
	  
#ifdef DEBUG
	  printf( "Doing status window junk...\n" ) ;
#endif
	  if (stat) free (stat);
	  
	  if (func) {
	    if (func->func)
	      func->func (func->messagebody, func->widget);
	  }
	}
      return FALSE;
      break;

    case EMESSAGE:
#ifdef DEBUG
      printf( "'EMESSAGE' error message is being processed.\n" ) ;
#endif
      insert_message( _("The SMTP server reported:\n") ) ;
      insert_warning( "%s\n", stat ) ;

      if( pop -> spid != -1 ) {
	if( waitpid( pop -> spid, NULL, 0 ) ) {
	  pop -> spid = -1 ;
	}
      }
#ifdef DEBUG
      printf( "Child is done on error, resetting GUI...\n" ) ;
#endif
      gtk_timeout_remove( tm_globs -> send_timeout ) ;
      gtk_widget_set_sensitive (lookup_widget (tm_globs->main_window, "get_message"), TRUE);
      gtk_widget_set_sensitive (lookup_widget (tm_globs->status, "stopthat"), FALSE);
      gtk_widget_set_sensitive (lookup_widget (tm_globs->status, "skip"), FALSE);
      set_progress (1, 1);
      set_receiving (0, 0);
      
      folders_write_index ();
      
      set_account_name (_("Done"));
      insert_message (_("Error reported.\n"));
      
      /* So we can properly report an error. */
      if (func) {
#ifdef DEBUG
	printf( "Firing failure function\n" ) ;
#endif
	func->failure( func->widget ) ;
      }

      if ((tm_globs->flags & HIDE_STATUS_WHEN_DONE)) {
	destroy_status (tm_globs->status, NULL);
      }
      if (stat) free (stat);
      
      return FALSE ;
      break ;

    case PIPEERROR:
      tmpl=assemble_long (stat);
      tmpi=-(tmpl);
      switch (tmpi) {
      case NDEFACCOUNT:
	insert_error (_("Bug encountered. Tried to access undefined account\n"));
	break;

      case NDEFHOST:
	insert_error (_("Bug encountered. Undefined host on account %s\n"), pop->name); 
	break;

      case UNKNOWNHOST:
	insert_error (_("Unknown Host on account %s\n"), pop->name);
	break;

      case CONNREF:
	insert_warning (_("Connection to the server was refused.  Please contact your system's administrator\n")); 
	break;

      case UNKNOWNERR:
	insert_warning (_("An unknown error occurred while connecting to the server\n")); 
	break;

      case NORELAY:
	insert_error (_("Server says that it's not not configured for relaying.  Use a different host.\n") ) ;
	break;

      default:
	insert_error (_("Unknown error type %d received\n"),tmpi);
	break;
      }

      if (func) {
#ifdef DEBUG
	printf( "Firing failure function\n" ) ;
#endif
	func->failure (func->widget);
      }

      if( pop -> spid != -1 ) {
#ifdef DEBUG
      printf( "Waiting for child that reported 'DONE' to finished...\n" ) ;
#endif
	if( waitpid( pop -> spid, NULL, WNOHANG ) ) {
	  pop -> spid = -1 ;
#ifdef DEBUG
	  printf( "Child has finished.\n" ) ;
#endif
	}
      }

#ifdef DMALLOC
      dmalloc_verify( 0 ) ;
#endif
      return FALSE;
    }
  }

#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  return TRUE;
}
