/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: imap-window-callbacks.c,v 1.7 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gtk/gtk.h>

#include "imap-window-callbacks.h"


gboolean
on_imap_window_delete_event (GtkWidget * widget,
			     GdkEvent * event, gpointer user_data)
{
  return FALSE;
}


void
on_imap_window_destroy (GtkObject * object, gpointer user_data)
{
  ImapInfo *imapinfo = user_data;
  char *mailbox;

  if (imapinfo->mailbox)
    {
      mailbox = strdup (imapinfo->mailbox);
    }
  else
    {
      mailbox = (char *) calloc (4096, sizeof (char));
      snprintf (mailbox, 4094, "{%s:%d/imap/user=\"%s\"}%s",
		imapinfo->acct->phost, imapinfo->acct->pport,
		imapinfo->acct->puser, "INBOX");
    }

  mmpopid = imapinfo->acct->id;
  tm_globs->imap_node = imapinfo->node;
  mail_expunge (tm_globs->mailstream);
  mail_ping (tm_globs->mailstream);
  mail_status (tm_globs->mailstream, mailbox, SA_UNSEEN | SA_MESSAGES);
  free (mailbox);
  tm_globs->imap_window = NULL;
}


void
on_print_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *clist = lookup_widget (GTK_WIDGET (menuitem), "imap_clist");
  char *header, *body, *msg;
  int length;
  int row;
  ImapMsg *imapmsg = NULL;
  GList *seek;


  if (!GTK_CLIST (clist)->selection)
    return;

  for (seek = GTK_CLIST (clist)->selection; seek; seek = seek->next)
    {

      row = (int) seek->data;
      imapmsg = gtk_clist_get_row_data (GTK_CLIST (clist), row);
      if (!imapmsg)
	return;

      header = mail_fetchheader (tm_globs->mailstream, imapmsg->msgno);
      body = mail_fetchtext (tm_globs->mailstream, imapmsg->msgno);

      length = strlen (header) + strlen (body) + 2;
      msg = (char *) malloc (length * sizeof (char));
      msg = strncpy (msg, header, length);
      msg = strncat (msg, body, length);
      if (!msg)
	break;

      insert_message ("Printing Message... ");
      message_print_stream (msg);
      free (msg);
    }
  insert_message ("Done!\n");
}


void
on_close_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *window = tm_globs->imap_window;

  tm_globs->imap_window = NULL;
  gtk_widget_destroy (window);
}


void
on_exit_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  puma_die (tm_globs->imap_window, user_data);
}


void
on_reply_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *clist = user_data;
  int row;
  ImapMsg *imapmsg = NULL;
  char *header, *body, *msg;
  int length;
  GList *seek;

  if (!GTK_CLIST (clist)->selection)
    return;

  for (seek = GTK_CLIST (clist)->selection; seek; seek = seek->next)
    {

      row = (int) seek->data;

      imapmsg = gtk_clist_get_row_data (GTK_CLIST (clist), row);
      if (!imapmsg)
	return;

      header = mail_fetchheader (tm_globs->mailstream, imapmsg->msgno);
      body = mail_fetchtext (tm_globs->mailstream, imapmsg->msgno);

      length = strlen (header) + strlen (body) + 2;
      msg = (char *) malloc (length * sizeof (char));
      msg = strncpy (msg, header, length);
      msg = strncat (msg, body, length);

      send_on_its_merry_way_stream (msg, "Re:");
    }
}


void
on_forward_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *clist = user_data;
  int row;
  ImapMsg *imapmsg = NULL;
  char *header, *body, *msg;
  int length;
  GList *seek;

  if (!GTK_CLIST (clist)->selection)
    return;

  for (seek = GTK_CLIST (clist)->selection; seek; seek = seek->next)
    {

      row = (int) seek->data;

      imapmsg = gtk_clist_get_row_data (GTK_CLIST (clist), row);
      if (!imapmsg)
	return;

      header = mail_fetchheader (tm_globs->mailstream, imapmsg->msgno);
      body = mail_fetchtext (tm_globs->mailstream, imapmsg->msgno);

      length = strlen (header) + strlen (body) + 2;
      msg = (char *) malloc (length * sizeof (char));
      msg = strncpy (msg, header, length);
      msg = strncat (msg, body, length);

      send_on_its_merry_way_stream (msg, "Fwd:");
    }
}


void
on_delete_activate (GtkMenuItem * menuitem, gpointer user_data)
{
  GtkWidget *clist = user_data;
  GList *del;
  int row;
  ImapMsg *imapmsg = NULL;
  char *msg;
  GList *dellist = NULL;

  if (!GTK_CLIST (clist)->selection)
    return;

  gtk_clist_freeze (GTK_CLIST (clist));
  for (del = GTK_CLIST (clist)->selection; del; del = del->next)
    {
      row = (int) del->data;
      imapmsg = gtk_clist_get_row_data (GTK_CLIST (clist), row);
      dellist = g_list_append (dellist, imapmsg);
    }

  for (del = dellist; del; del = del->next)
    {
      imapmsg = del->data;

      msg = (char *) calloc (5, sizeof (char));
      snprintf (msg, 5, "%d", abs (imapmsg->msgno));
      mail_setflag (tm_globs->mailstream, msg, "\\DELETED");
      free (msg);

      row = gtk_clist_find_row_from_data (GTK_CLIST (clist),
					  (gpointer) imapmsg);
      gtk_clist_remove (GTK_CLIST (clist), row);
    }

  gtk_clist_thaw (GTK_CLIST (clist));
}


void
on_b_reply_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *clist = lookup_widget (GTK_WIDGET (button), "imap_clist");
  int row;
  ImapMsg *imapmsg = NULL;
  char *header, *body, *msg;
  int length;
  GList *seek;

  if (!GTK_CLIST (clist)->selection)
    return;

  for (seek = GTK_CLIST (clist)->selection; seek; seek = seek->next)
    {

      row = (int) seek->data;
      imapmsg = gtk_clist_get_row_data (GTK_CLIST (clist), row);
      if (!imapmsg)
	return;

      header = mail_fetchheader (tm_globs->mailstream, imapmsg->msgno);
      body = mail_fetchtext (tm_globs->mailstream, imapmsg->msgno);

      length = strlen (header) + strlen (body) + 2;
      msg = (char *) malloc (length * sizeof (char));
      msg = strncpy (msg, header, length);
      msg = strncat (msg, body, length);

      send_on_its_merry_way_stream (msg, "Re:");
    }
}


void
on_b_forward_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *clist = lookup_widget (GTK_WIDGET (button), "imap_clist");
  int row;
  ImapMsg *imapmsg = NULL;
  char *header, *body, *msg;
  int length;
  GList *seek;

  if (!GTK_CLIST (clist)->selection)
    return;

  for (seek = GTK_CLIST (clist)->selection; seek; seek = seek->next)
    {

      row = (int) seek->data;
      imapmsg = gtk_clist_get_row_data (GTK_CLIST (clist), row);
      if (!imapmsg)
	return;

      header = mail_fetchheader (tm_globs->mailstream, imapmsg->msgno);
      body = mail_fetchtext (tm_globs->mailstream, imapmsg->msgno);

      length = strlen (header) + strlen (body) + 2;
      msg = (char *) malloc (length * sizeof (char));
      msg = strncpy (msg, header, length);
      msg = strncat (msg, body, length);

      send_on_its_merry_way_stream (msg, "Fwd:");
    }
}


void
on_b_delete_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *clist = lookup_widget (GTK_WIDGET (button), "imap_clist");
  GList *del;
  int row;
  ImapMsg *imapmsg = NULL;
  char *msg;
  GList *dellist = NULL;

  if (!GTK_CLIST (clist)->selection)
    return;

  gtk_clist_freeze (GTK_CLIST (clist));
  for (del = GTK_CLIST (clist)->selection; del; del = del->next)
    {
      row = (int) del->data;
      imapmsg = gtk_clist_get_row_data (GTK_CLIST (clist), row);
      dellist = g_list_append (dellist, imapmsg);
    }

  for (del = dellist; del; del = del->next)
    {
      imapmsg = del->data;

      msg = (char *) calloc (5, sizeof (char));
      snprintf (msg, 5, "%d", abs (imapmsg->msgno));
      mail_setflag (tm_globs->mailstream, msg, "\\DELETED");
      free (msg);

      row = gtk_clist_find_row_from_data (GTK_CLIST (clist),
					  (gpointer) imapmsg);
      gtk_clist_remove (GTK_CLIST (clist), row);
    }

  gtk_clist_thaw (GTK_CLIST (clist));
}


void
on_imap_clist_click_column (GtkCList * clist, gint column, gpointer user_data)
{
  if (GTK_CLIST (clist)->sort_type == GTK_SORT_ASCENDING)
    {
      GTK_CLIST (clist)->sort_type = GTK_SORT_DESCENDING;
    }
  else
    {
      GTK_CLIST (clist)->sort_type = GTK_SORT_ASCENDING;
    }

  gtk_clist_freeze (GTK_CLIST (clist));
  gtk_clist_set_sort_column (GTK_CLIST (clist), column);
  if (column == 2)
    {
      gtk_clist_set_compare_func (GTK_CLIST (clist),
				  (GtkCListCompareFunc) new_clist_sort_func);
    }
  else
    {
      gtk_clist_set_compare_func (GTK_CLIST (clist),
				  (GtkCListCompareFunc) NULL);
    }

  gtk_clist_sort (GTK_CLIST (clist));
  gtk_clist_thaw (GTK_CLIST (clist));
}


void
on_imap_clist_select_row (GtkWidget * widget, GdkEventButton * event,
			  gpointer data)
{
  ImapMsg *imapmsg;
  int row, column;
  char *header = NULL, *body = NULL;
  int length;
  char *msg = NULL;
  int popup = 0;
  GtkWidget *popmenu;
  GList *msel = NULL;

  if (event->button == 3)
    {
      msel = GTK_CLIST (widget)->selection;
      if (!msel)
	return;
      popmenu = imap_window_popup_menu (widget, msel);
      gtk_menu_popup (GTK_MENU (popmenu), NULL, NULL, NULL, NULL, 3, 0);
    }
  else
    {
      if (event->button == 2)
	{
	  popup = 1;
	}
      else
	{
	  if (event->button != 1)
	    {
	      return;
	    }
	}
    }

  if (!event)
    {
      popup = 0;
    }
  else
    {
      if (event->type == 5)
	{
	  popup = 1;
	}
    }

  gtk_clist_get_selection_info (GTK_CLIST (widget), event->x, event->y, &row,
				&column);
  if (event->button == 2)
    {
      gtk_clist_unselect_all (GTK_CLIST (widget));
      gtk_clist_select_row (GTK_CLIST (widget), row, column);
    }
  imapmsg = gtk_clist_get_row_data (GTK_CLIST (widget), row);
  if (!imapmsg)
    return;

  if (popup)
    {
      header = mail_fetchheader (tm_globs->mailstream, imapmsg->msgno);
      body = mail_fetchtext (tm_globs->mailstream, imapmsg->msgno);

      length = strlen (header) + strlen (body) + 2;
      msg = (char *) malloc (length * sizeof (char));
      msg = strncpy (msg, header, length);
      msg = strncat (msg, body, length);

      gtk_clist_set_row_style (GTK_CLIST (widget), row, tm_globs->read_odd);
      gtk_widget_show (message_window_new_stream (msg));
    }
}


void
on_imap_delete_clicked (GtkButton * button, gpointer user_data)
{
  ImapInfo *imapinfo;
  GtkWidget *ctree = lookup_widget (tm_globs->main_window, "folder_ntree");
  GtkCTreeNode *node = user_data;
  char *mbox;
  extern GList *nf_imap_folder_list;

  imapinfo = gtk_ctree_node_get_row_data (GTK_CTREE (ctree), node);
  mbox = cut_string (imapinfo->mailbox, '}', 2, 1);

  mmpopid = imapinfo->acct->id;
  if (mail_delete (tm_globs->mailstream, imapinfo->mailbox))
    {
      mail_unsubscribe (tm_globs->mailstream, imapinfo->mailbox);
      nf_imap_folder_list =
	g_list_remove (nf_imap_folder_list, (gpointer) imapinfo->mailbox);
    }
  else
    {
      insert_error ("Unable to delete mailbox \"%s\" in account \"%s\"!\n",
		    mbox, imapinfo->acct->name);
    }

  free (mbox);

  gtk_ctree_remove_node (GTK_CTREE (ctree), node);
  gtk_widget_destroy (lookup_widget (GTK_WIDGET (button), "yesno"));
}


void
on_imap_unsubscribe_clicked (GtkButton * button, gpointer user_data)
{
  ImapInfo *imapinfo;
  GtkWidget *ctree = lookup_widget (tm_globs->main_window, "folder_ntree");
  GtkCTreeNode *node = user_data;
  char *mbox;
  extern GList *nf_imap_folder_list;

  imapinfo = gtk_ctree_node_get_row_data (GTK_CTREE (ctree), node);
  mbox = cut_string (imapinfo->mailbox, '}', 2, 1);

  mmpopid = imapinfo->acct->id;
  mail_unsubscribe (tm_globs->mailstream, imapinfo->mailbox);
  nf_imap_folder_list =
    g_list_remove (nf_imap_folder_list, (gpointer) imapinfo->mailbox);

  free (mbox);

  gtk_ctree_remove_node (GTK_CTREE (ctree), node);
  gtk_widget_destroy (lookup_widget (GTK_WIDGET (button), "yesno"));
}

void
on_imap_delete_cancel_clicked (GtkButton * button, gpointer user_data)
{
}



void
on_imap_new_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *ac = lookup_widget (GTK_WIDGET (button), "combo_entry2");
  GtkWidget *mb = lookup_widget (GTK_WIDGET (button), "entry1");
  PopAccount *acct = seek_popaccount (seek_popaccount_by_name
				      (gtk_entry_get_text (GTK_ENTRY (ac))));
  char *mbox = gtk_entry_get_text (GTK_ENTRY (mb));
  char *mailbox;

  if ((mbox) && (strlen (mbox)))
    {
      mailbox = (char *) calloc (4096, sizeof (char));
      snprintf (mailbox, 4094, "{%s:%d/imap/user=\"%s\"}%s",
		acct->phost, acct->pport, acct->puser, mbox);
      mmpopid = acct->id;
      tm_globs->mailstream =
	mail_open (tm_globs->mailstream, mailbox, OP_HALFOPEN);
      if (!tm_globs->mailstream)
	return;
      if (mail_create (tm_globs->mailstream, mailbox))
	{
	  mail_subscribe (tm_globs->mailstream, mailbox);
	}
      else
	{
	  insert_error
	    ("Unable to create mailbox \"%s\" in account \"%s\"!\n", mbox,
	     acct->name);
	}

      free (mailbox);

    }
  on_imap_new_cancel_clicked (button, NULL);
}


void
on_imap_new_cancel_clicked (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (lookup_widget (GTK_WIDGET (button), "window2"));
}


GtkWidget *
imap_window_popup_menu (GtkWidget * widget, GList * msel)
{
  GtkWidget *imapm_popup;
  GtkWidget *imapm_title;
  GtkWidget *m_sep0, *m_sep1;
  GtkWidget *replyto, *forward, *delete;
  GtkWidget *moveto, *move_menu;
  GtkWidget *copylocal, *copylocal_menu, *copy_folder;
  GList *seek;
  GtkWidget *m_mbox;
  extern GList *nf_imap_folder_list;
  char *mainmailbox, *mailbox, *tmp;
  ImapInfo *imapinfo;
  int length;
  Folder *fseek = folder_first ();

  imapm_popup = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (imapm_popup), "imapm_popup", imapm_popup);
  gtk_widget_show (imapm_popup);

  imapm_title = gtk_menu_item_new_with_label ("Message Actions");
  gtk_widget_ref (imapm_title);
  gtk_object_set_data_full (GTK_OBJECT (imapm_popup), "imapm_title",
			    imapm_title, (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (imapm_popup), imapm_title);
  gtk_widget_show (imapm_title);

  m_sep0 = gtk_menu_item_new ();
  gtk_widget_ref (m_sep0);
  gtk_object_set_data_full (GTK_OBJECT (imapm_popup), "m_sep0",
			    m_sep0, (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (imapm_popup), m_sep0);
  gtk_widget_show (m_sep0);

  replyto = gtk_menu_item_new_with_label ("Reply To");
  gtk_widget_ref (replyto);
  gtk_object_set_data_full (GTK_OBJECT (imapm_popup), "replyto",
			    replyto, (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (imapm_popup), replyto);
  gtk_widget_show (replyto);

  forward = gtk_menu_item_new_with_label ("Forward");
  gtk_widget_ref (forward);
  gtk_object_set_data_full (GTK_OBJECT (imapm_popup), "forward",
			    forward, (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (imapm_popup), forward);
  gtk_widget_show (forward);

  delete = gtk_menu_item_new_with_label ("Delete");
  gtk_widget_ref (delete);
  gtk_object_set_data_full (GTK_OBJECT (imapm_popup), "delete",
			    delete, (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (imapm_popup), delete);
  gtk_widget_show (delete);

  m_sep1 = gtk_menu_item_new ();
  gtk_widget_ref (m_sep1);
  gtk_object_set_data_full (GTK_OBJECT (imapm_popup), "m_sep1",
			    m_sep1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (imapm_popup), m_sep1);
  gtk_widget_show (m_sep1);

  moveto = gtk_menu_item_new_with_label ("Move to Folder");
  gtk_widget_ref (moveto);
  gtk_object_set_data_full (GTK_OBJECT (imapm_popup), "moveto",
			    moveto, (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (imapm_popup), moveto);
  gtk_widget_show (moveto);

  move_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (imapm_popup), "move_menu", move_menu);
  gtk_widget_show (move_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (moveto), move_menu);

  imapinfo =
    gtk_ctree_node_get_row_data (GTK_CTREE
				 (lookup_widget
				  (tm_globs->main_window, "folder_ntree")),
				 GTK_CTREE_NODE (tm_globs->imap_node));
  if (imapinfo->mailbox)
    {
      mainmailbox = strdup (imapinfo->mailbox);
    }
  else
    {
      mainmailbox = (char *) calloc (4096, sizeof (char));
      snprintf (mainmailbox, 4094, "{%s:%d/imap/user=\"%s\"}%s",
		imapinfo->acct->phost, imapinfo->acct->pport,
		imapinfo->acct->puser, "INBOX");
    }

  m_mbox = gtk_menu_item_new_with_label ("INBOX");
  gtk_widget_show (m_mbox);
  gtk_container_add (GTK_CONTAINER (move_menu), m_mbox);
  gtk_signal_connect (GTK_OBJECT (m_mbox), "activate",
		      GTK_SIGNAL_FUNC (nf_imap_moveto), (gpointer) "INBOX");

  for (seek = nf_imap_folder_list; seek; seek = seek->next)
    {
      mailbox = seek->data;
      tmp = cut_string (mailbox, '}', 2, 1);
      length = strlen (mailbox) - strlen (tmp);
      if (strncmp (mainmailbox, mailbox, length) == 0)
	{
	  m_mbox = gtk_menu_item_new_with_label (tmp);
	  gtk_widget_show (m_mbox);
	  gtk_container_add (GTK_CONTAINER (move_menu), m_mbox);
	  gtk_signal_connect (GTK_OBJECT (m_mbox), "activate",
			      GTK_SIGNAL_FUNC (nf_imap_moveto),
			      (gpointer) strdup (tmp));
	}
      free (tmp);
    }
  free (mainmailbox);

  copylocal = gtk_menu_item_new_with_label ("Make Copy in");
  gtk_widget_ref (copylocal);
  gtk_object_set_data_full (GTK_OBJECT (imapm_popup), "copylocal",
			    copylocal, (GtkDestroyNotify) gtk_widget_unref);
  gtk_container_add (GTK_CONTAINER (imapm_popup), copylocal);
  gtk_widget_show (copylocal);

  copylocal_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (imapm_popup), "copylocal_menu",
		       copylocal_menu);
  gtk_widget_show (copylocal_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (copylocal), copylocal_menu);

  while (fseek)
    {
      if ((strcmp (fseek->name, "Outbox") != 0)
	  && (strcmp (fseek->name, "Sent-Items") != 0)
	  && (strcmp (fseek->name, "Trash") != 0))
	{
	  copy_folder = gtk_menu_item_new_with_label (fseek->name);
	  gtk_widget_show (copy_folder);
	  gtk_container_add (GTK_CONTAINER (copylocal_menu), copy_folder);
	  gtk_signal_connect (GTK_OBJECT (copy_folder), "activate",
			      GTK_SIGNAL_FUNC (nf_imap_copyto),
			      (gpointer) fseek);
	}
      fseek = fseek->next;
    }

  gtk_signal_connect (GTK_OBJECT (replyto), "activate",
		      GTK_SIGNAL_FUNC (on_reply_activate), widget);
  gtk_signal_connect (GTK_OBJECT (forward), "activate",
		      GTK_SIGNAL_FUNC (on_forward_activate), widget);
  gtk_signal_connect (GTK_OBJECT (delete), "activate",
		      GTK_SIGNAL_FUNC (on_delete_activate), widget);

  return imapm_popup;
}


void
on_imap_subscribe_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *ctree = lookup_widget (GTK_WIDGET (button), "ctree1");
  GList *seek;
  GtkWidget *node;
  ImapInfo *imapinfo;
  char *mailbox;

  if (GTK_CLIST (ctree)->selection) {
    for (seek = GTK_CLIST (ctree)->selection; seek; seek = seek->next) {
      node = seek->data;
      imapinfo = gtk_ctree_node_get_row_data (GTK_CTREE (ctree),
			GTK_CTREE_NODE (node));
      if (imapinfo->mailbox) {
        mailbox = (char *) calloc (4096, sizeof (char));
        snprintf (mailbox, 4094, "{%s:%d/imap/user=\"%s\"}%s",
                  imapinfo->acct->phost, imapinfo->acct->pport,
                  imapinfo->acct->puser, imapinfo->mailbox);

        mmpopid = imapinfo->acct->id;
        tm_globs->mailstream =
          mail_open (tm_globs->mailstream, mailbox, OP_HALFOPEN);
        if (!tm_globs->mailstream)
          return;

        mail_subscribe (tm_globs->mailstream, mailbox);

        free (mailbox);
      }
    }
    gtk_widget_destroy (lookup_widget (GTK_WIDGET (button), "window1"));
  } else {
    insert_message ("Please select a folder to subscribe to.\n");
  }
}


void
on_imap_subscribe_cancel_clicked (GtkButton * button, gpointer user_data)
{
  gtk_widget_destroy (lookup_widget (GTK_WIDGET (button), "window1"));
}
