/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: imap-window.c,v 1.5 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "imap-window.h"
#include "imap-window-callbacks.h"

GtkWidget *
create_imap_window (PopAccount * acct, char *mailbox, ImapInfo * imapinfo)
{
  GtkWidget *imap_window;
  GtkWidget *vbox3;
  GtkWidget *handlebox5;
  GtkWidget *menubar3;
  GtkWidget *file3;
  GtkWidget *file3_menu;
  GtkAccelGroup *file3_menu_accels;
  GtkWidget *m_print;
  GtkWidget *m_close;
  GtkWidget *separator6;
  GtkWidget *m_exit;
  GtkWidget *actions2;
  GtkWidget *actions2_menu;
  GtkAccelGroup *actions2_menu_accels;
  GtkWidget *m_reply;
  GtkWidget *m_forward;
  GtkWidget *m_separator;
  GtkWidget *m_delete;
  GtkWidget *handlebox6;
  GtkWidget *toolbar4;
  GtkWidget *b_reply;
  GtkWidget *b_reply_icon;
  GtkWidget *b_forward;
  GtkWidget *b_forward_icon;
  GtkWidget *vseparator1;
  GtkWidget *b_delete;
  GtkWidget *b_delete_icon;
  GtkWidget *scrolledwindow3;
  GtkWidget *imap_clist;
  GtkWidget *label9;
  GtkWidget *label10;
  GtkWidget *label11;
  GtkAccelGroup *accel_group;
  char *title;
  char *mbox;

  accel_group = gtk_accel_group_new ();

  imap_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (imap_window, "imap_window");
  gtk_object_set_data (GTK_OBJECT (imap_window), "imap_window", imap_window);
  mbox = cut_string (mailbox, '}', 2, 1);
  title = (char *) malloc (1024 * sizeof (char));
  snprintf (title, 1024, "IMAP - %s:%s", acct->name, mbox);
  if (mbox)
    free (mbox);
  gtk_window_set_title (GTK_WINDOW (imap_window), title);
  free (title);
  gtk_window_set_default_size (GTK_WINDOW (imap_window), 600, 400);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox3, "vbox3");
  gtk_widget_ref (vbox3);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "vbox3", vbox3,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (imap_window), vbox3);

  handlebox5 = gtk_handle_box_new ();
  gtk_widget_set_name (handlebox5, "handlebox5");
  gtk_widget_ref (handlebox5);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "handlebox5",
			    handlebox5, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (handlebox5);
  gtk_box_pack_start (GTK_BOX (vbox3), handlebox5, FALSE, TRUE, 0);

  menubar3 = gtk_menu_bar_new ();
  gtk_widget_set_name (menubar3, "menubar3");
  gtk_widget_ref (menubar3);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "menubar3", menubar3,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menubar3);
  gtk_container_add (GTK_CONTAINER (handlebox5), menubar3);

  file3 = gtk_menu_item_new_with_label ("File");
  gtk_widget_set_name (file3, "file3");
  gtk_widget_ref (file3);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "file3", file3,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (file3);
  gtk_container_add (GTK_CONTAINER (menubar3), file3);

  file3_menu = gtk_menu_new ();
  gtk_widget_set_name (file3_menu, "file3_menu");
  gtk_widget_ref (file3_menu);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "file3_menu",
			    file3_menu, (GtkDestroyNotify) gtk_widget_unref);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (file3), file3_menu);
  file3_menu_accels =
    gtk_menu_ensure_uline_accel_group (GTK_MENU (file3_menu));

  m_print = gtk_menu_item_new_with_label ("Print");
  gtk_widget_set_name (m_print, "m_print");
  gtk_widget_ref (m_print);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "m_print", m_print,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (m_print);
  gtk_container_add (GTK_CONTAINER (file3_menu), m_print);
  gtk_widget_add_accelerator (m_print, "activate", accel_group,
			      GDK_P, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);

  m_close = gtk_menu_item_new_with_label ("Close");
  gtk_widget_set_name (m_close, "m_close");
  gtk_widget_ref (m_close);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "m_close", m_close,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (m_close);
  gtk_container_add (GTK_CONTAINER (file3_menu), m_close);
  gtk_widget_add_accelerator (m_close, "activate", accel_group,
			      GDK_W, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);

  separator6 = gtk_menu_item_new ();
  gtk_widget_set_name (separator6, "separator6");
  gtk_widget_ref (separator6);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "separator6",
			    separator6, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (separator6);
  gtk_container_add (GTK_CONTAINER (file3_menu), separator6);
  gtk_widget_set_sensitive (separator6, FALSE);

  m_exit = gtk_menu_item_new_with_label ("Exit TradeClient");
  gtk_widget_set_name (m_exit, "m_exit");
  gtk_widget_ref (m_exit);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "m_exit", m_exit,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (m_exit);
  gtk_container_add (GTK_CONTAINER (file3_menu), m_exit);

  actions2 = gtk_menu_item_new_with_label ("Actions");
  gtk_widget_set_name (actions2, "actions2");
  gtk_widget_ref (actions2);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "actions2", actions2,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (actions2);
  gtk_container_add (GTK_CONTAINER (menubar3), actions2);

  actions2_menu = gtk_menu_new ();
  gtk_widget_set_name (actions2_menu, "actions2_menu");
  gtk_widget_ref (actions2_menu);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "actions2_menu",
			    actions2_menu,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (actions2), actions2_menu);
  actions2_menu_accels =
    gtk_menu_ensure_uline_accel_group (GTK_MENU (actions2_menu));

  m_reply = gtk_menu_item_new_with_label ("Reply");
  gtk_widget_set_name (m_reply, "m_reply");
  gtk_widget_ref (m_reply);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "m_reply", m_reply,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (m_reply);
  gtk_container_add (GTK_CONTAINER (actions2_menu), m_reply);
  gtk_widget_add_accelerator (m_reply, "activate", accel_group,
			      GDK_R, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);

  m_forward = gtk_menu_item_new_with_label ("Forward");
  gtk_widget_set_name (m_forward, "m_forward");
  gtk_widget_ref (m_forward);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "m_forward", m_forward,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (m_forward);
  gtk_container_add (GTK_CONTAINER (actions2_menu), m_forward);
  gtk_widget_add_accelerator (m_forward, "activate", accel_group,
			      GDK_F, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);

  m_separator = gtk_menu_item_new ();
  gtk_widget_set_name (m_separator, "m_separator");
  gtk_widget_ref (m_separator);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "m_separator",
			    m_separator, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (m_separator);
  gtk_container_add (GTK_CONTAINER (actions2_menu), m_separator);
  gtk_widget_set_sensitive (m_separator, FALSE);

  m_delete = gtk_menu_item_new_with_label ("Delete");
  gtk_widget_set_name (m_delete, "m_delete");
  gtk_widget_ref (m_delete);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "m_delete", m_delete,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (m_delete);
  gtk_container_add (GTK_CONTAINER (actions2_menu), m_delete);
  gtk_widget_add_accelerator (m_delete, "activate", accel_group,
			      GDK_D, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);

  handlebox6 = gtk_handle_box_new ();
  gtk_widget_set_name (handlebox6, "handlebox6");
  gtk_widget_ref (handlebox6);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "handlebox6",
			    handlebox6, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (handlebox6);
  gtk_box_pack_start (GTK_BOX (vbox3), handlebox6, FALSE, TRUE, 0);

  toolbar4 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_widget_set_name (toolbar4, "toolbar4");
  gtk_widget_ref (toolbar4);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "toolbar4", toolbar4,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (toolbar4);
  gtk_container_add (GTK_CONTAINER (handlebox6), toolbar4);

  b_reply_icon = pixmap_widget_create (imap_window, PIXMAP_REPLY);
  gtk_widget_set_name (b_reply_icon, "b_reply_icon");
  gtk_widget_ref (b_reply_icon);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "b_reply_icon",
			    b_reply_icon,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (b_reply_icon);

  b_reply = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar4),
					GTK_TOOLBAR_CHILD_BUTTON,
					NULL,
					"Reply",
					"Reply to selected messages", NULL,
					b_reply_icon, NULL, NULL);
  gtk_widget_set_name (b_reply, "b_reply");
  gtk_widget_ref (b_reply);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "b_reply", b_reply,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (b_reply);

  b_forward_icon = pixmap_widget_create (imap_window, PIXMAP_FORWARD);
  gtk_widget_set_name (b_forward_icon, "b_forward_icon");
  gtk_widget_ref (b_forward_icon);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "b_forward_icon",
			    b_forward_icon,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (b_forward_icon);

  b_forward = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar4),
					  GTK_TOOLBAR_CHILD_BUTTON,
					  NULL, "Forward",
					  "Forward selected messages", NULL,
					  b_forward_icon, NULL, NULL);
  gtk_widget_set_name (b_forward, "b_forward");
  gtk_widget_ref (b_forward);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "b_forward", b_forward,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (b_forward);

  vseparator1 = gtk_vseparator_new ();
  gtk_widget_set_name (vseparator1, "vseparator1");
  gtk_widget_ref (vseparator1);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "vseparator1",
			    vseparator1, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vseparator1);
  gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar4), vseparator1, NULL, NULL);
  gtk_widget_set_usize (vseparator1, 10, 15);

  b_delete_icon = pixmap_widget_create (imap_window, PIXMAP_AX);
  gtk_widget_set_name (b_delete_icon, "b_delete_icon");
  gtk_widget_ref (b_delete_icon);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "b_delete_icon",
			    b_delete_icon,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (b_delete_icon);

  b_delete = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar4),
					 GTK_TOOLBAR_CHILD_BUTTON,
					 NULL,
					 "Delete",
					 "Delete selected messages", NULL,
					 b_delete_icon, NULL, NULL);
  gtk_widget_set_name (b_delete, "b_delete");
  gtk_widget_ref (b_delete);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "b_delete", b_delete,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (b_delete);

  scrolledwindow3 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scrolledwindow3, "scrolledwindow3");
  gtk_widget_ref (scrolledwindow3);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "scrolledwindow3",
			    scrolledwindow3,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow3);
  gtk_box_pack_start (GTK_BOX (vbox3), scrolledwindow3, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow3),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  imap_clist = gtk_clist_new (3);
  gtk_widget_set_name (imap_clist, "imap_clist");
  gtk_widget_ref (imap_clist);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "imap_clist",
			    imap_clist, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (imap_clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow3), imap_clist);
  gtk_clist_set_column_width (GTK_CLIST (imap_clist), 0, 200);
  gtk_clist_set_column_width (GTK_CLIST (imap_clist), 1, 200);
  gtk_clist_column_titles_show (GTK_CLIST (imap_clist));
  gtk_clist_set_selection_mode (GTK_CLIST (imap_clist),
				GTK_SELECTION_EXTENDED);

  label9 = gtk_label_new ("Subject");
  gtk_widget_set_name (label9, "label9");
  gtk_widget_ref (label9);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "label9", label9,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label9);
  gtk_clist_set_column_widget (GTK_CLIST (imap_clist), 0, label9);

  label10 = gtk_label_new ("From");
  gtk_widget_set_name (label10, "label10");
  gtk_widget_ref (label10);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "label10", label10,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label10);
  gtk_clist_set_column_widget (GTK_CLIST (imap_clist), 1, label10);

  label11 = gtk_label_new ("Date");
  gtk_widget_set_name (label11, "label11");
  gtk_widget_ref (label11);
  gtk_object_set_data_full (GTK_OBJECT (imap_window), "label11", label11,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label11);
  gtk_clist_set_column_widget (GTK_CLIST (imap_clist), 2, label11);

  gtk_signal_connect (GTK_OBJECT (imap_window), "delete_event",
		      GTK_SIGNAL_FUNC (on_imap_window_delete_event),
		      (gpointer) imapinfo);
  gtk_signal_connect (GTK_OBJECT (imap_window), "destroy",
		      GTK_SIGNAL_FUNC (on_imap_window_destroy),
		      (gpointer) imapinfo);
  gtk_signal_connect (GTK_OBJECT (m_print), "activate",
		      GTK_SIGNAL_FUNC (on_print_activate), NULL);
  gtk_signal_connect (GTK_OBJECT (m_close), "activate",
		      GTK_SIGNAL_FUNC (on_close_activate), NULL);
  gtk_signal_connect (GTK_OBJECT (m_exit), "activate",
		      GTK_SIGNAL_FUNC (on_exit_activate), NULL);
  gtk_signal_connect (GTK_OBJECT (m_reply), "activate",
		      GTK_SIGNAL_FUNC (on_reply_activate), imap_clist);
  gtk_signal_connect (GTK_OBJECT (m_forward), "activate",
		      GTK_SIGNAL_FUNC (on_forward_activate), imap_clist);
  gtk_signal_connect (GTK_OBJECT (m_delete), "activate",
		      GTK_SIGNAL_FUNC (on_delete_activate), imap_clist);
  gtk_signal_connect (GTK_OBJECT (b_reply), "clicked",
		      GTK_SIGNAL_FUNC (on_b_reply_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (b_forward), "clicked",
		      GTK_SIGNAL_FUNC (on_b_forward_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (b_delete), "clicked",
		      GTK_SIGNAL_FUNC (on_b_delete_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (imap_clist), "click_column",
		      GTK_SIGNAL_FUNC (on_imap_clist_click_column), NULL);
  gtk_signal_connect (GTK_OBJECT (imap_clist), "button_press_event",
		      GTK_SIGNAL_FUNC (on_imap_clist_select_row), NULL);

  gtk_window_add_accel_group (GTK_WINDOW (imap_window), accel_group);

  return imap_window;
}


GtkWidget *
imap_new_folder (void)
{
  GtkWidget *window2;
  GtkWidget *vbox2;
  GtkWidget *frame3;
  GtkWidget *combo2;
  GtkWidget *combo_entry2;
  GtkWidget *frame4;
  GtkWidget *eventbox1;
  GtkWidget *entry1;
  GtkWidget *hbuttonbox2;
  GtkWidget *button3;
  GtkWidget *button4;
  ImapInfo *imapinfo;
  PopAccount *acct;
  extern GList *nf_imap_list;
  GList *seek, *imap_accounts = NULL;

  window2 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (window2, "window2");
  gtk_object_set_data (GTK_OBJECT (window2), "window2", window2);
  gtk_window_set_title (GTK_WINDOW (window2), "Create Folder");

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox2, "vbox2");
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (window2), "vbox2", vbox2,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (window2), vbox2);

  frame3 = gtk_frame_new ("Create in what account?");
  gtk_widget_set_name (frame3, "frame3");
  gtk_widget_ref (frame3);
  gtk_object_set_data_full (GTK_OBJECT (window2), "frame3", frame3,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox2), frame3, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame3), 3);

  combo2 = gtk_combo_new ();
  gtk_widget_set_name (combo2, "combo2");
  gtk_widget_ref (combo2);
  gtk_object_set_data_full (GTK_OBJECT (window2), "combo2", combo2,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo2);
  gtk_container_add (GTK_CONTAINER (frame3), combo2);
  gtk_container_set_border_width (GTK_CONTAINER (combo2), 3);

  for (seek = nf_imap_list; seek; seek = seek->next)
    {
      imapinfo = seek->data;
      acct = imapinfo->acct;
      imap_accounts = g_list_append (imap_accounts, acct->name);
    }
  gtk_combo_set_popdown_strings (GTK_COMBO (combo2), imap_accounts);

  combo_entry2 = GTK_COMBO (combo2)->entry;
  gtk_widget_set_name (combo_entry2, "combo_entry2");
  gtk_widget_ref (combo_entry2);
  gtk_object_set_data_full (GTK_OBJECT (window2), "combo_entry2",
			    combo_entry2,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo_entry2);
  gtk_entry_set_editable (GTK_ENTRY (combo_entry2), FALSE);

  frame4 = gtk_frame_new ("Type in folder name");
  gtk_widget_set_name (frame4, "frame4");
  gtk_widget_ref (frame4);
  gtk_object_set_data_full (GTK_OBJECT (window2), "frame4", frame4,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame4);
  gtk_box_pack_start (GTK_BOX (vbox2), frame4, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame4), 3);

  eventbox1 = gtk_event_box_new ();
  gtk_widget_set_name (eventbox1, "eventbox1");
  gtk_widget_ref (eventbox1);
  gtk_object_set_data_full (GTK_OBJECT (window2), "eventbox1", eventbox1,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (eventbox1);
  gtk_container_add (GTK_CONTAINER (frame4), eventbox1);
  gtk_container_set_border_width (GTK_CONTAINER (eventbox1), 3);

  entry1 = gtk_entry_new ();
  gtk_widget_set_name (entry1, "entry1");
  gtk_widget_ref (entry1);
  gtk_object_set_data_full (GTK_OBJECT (window2), "entry1", entry1,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry1);
  gtk_container_add (GTK_CONTAINER (eventbox1), entry1);

  hbuttonbox2 = gtk_hbutton_box_new ();
  gtk_widget_set_name (hbuttonbox2, "hbuttonbox2");
  gtk_widget_ref (hbuttonbox2);
  gtk_object_set_data_full (GTK_OBJECT (window2), "hbuttonbox2", hbuttonbox2,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox2);
  gtk_box_pack_start (GTK_BOX (vbox2), hbuttonbox2, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox2),
			     GTK_BUTTONBOX_SPREAD);

  button3 = gtk_button_new_with_label ("Create");
  gtk_widget_set_name (button3, "button3");
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (window2), "button3", button3,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button3);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), button3);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

  button4 = gtk_button_new_with_label ("Cancel");
  gtk_widget_set_name (button4, "button4");
  gtk_widget_ref (button4);
  gtk_object_set_data_full (GTK_OBJECT (window2), "button4", button4,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button4);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), button4);
  GTK_WIDGET_SET_FLAGS (button4, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (window2), "delete_event",
		      GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);
  gtk_signal_connect (GTK_OBJECT (button3), "clicked",
		      GTK_SIGNAL_FUNC (on_imap_new_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (button4), "clicked",
		      GTK_SIGNAL_FUNC (on_imap_new_cancel_clicked), NULL);

  return window2;
}


GtkWidget *
imap_win_subscribe (void)
{
  GtkWidget *window1;
  GtkWidget *vbox1;
  GtkWidget *frame1;
  GtkWidget *scrolledwindow1;
  GtkWidget *ctree1;
  GtkWidget *hbuttonbox1;
  GtkWidget *button1;
  GtkWidget *button2;

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (window1, "window1");
  gtk_object_set_data (GTK_OBJECT (window1), "window1", window1);
  gtk_window_set_title (GTK_WINDOW (window1), "Subscribe to...");
  gtk_widget_set_usize (window1, 200, 300);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox1, "vbox1");
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window1), vbox1);

  frame1 = gtk_frame_new ("Select folder to subscribe to");
  gtk_widget_set_name (frame1, "frame1");
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 3);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scrolledwindow1, "scrolledwindow1");
  gtk_widget_ref (scrolledwindow1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "scrolledwindow1", scrolledwindow1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow1);
  gtk_container_add (GTK_CONTAINER (frame1), scrolledwindow1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  ctree1 = gtk_ctree_new (1, 0);
  gtk_widget_set_name (ctree1, "ctree1");
  gtk_widget_ref (ctree1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "ctree1", ctree1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ctree1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), ctree1);
  gtk_clist_column_titles_hide (GTK_CLIST (ctree1));
  gtk_clist_set_selection_mode (GTK_CLIST (ctree1),
                                GTK_SELECTION_EXTENDED);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_set_name (hbuttonbox1, "hbuttonbox1");
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "hbuttonbox1", hbuttonbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_SPREAD);

  button1 = gtk_button_new_with_label ("Subscribe");
  gtk_widget_set_name (button1, "button1");
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  button2 = gtk_button_new_with_label ("Cancel");
  gtk_widget_set_name (button2, "button2");
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (window1), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button2);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button2);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (window1), "delete_event",
                      GTK_SIGNAL_FUNC (gtk_widget_destroy),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (on_imap_subscribe_clicked),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (button2), "clicked",
                      GTK_SIGNAL_FUNC (on_imap_subscribe_cancel_clicked),
                      NULL);

  return window1;
}
