/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: ipc-proxy.c,v 1.3 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "puma.h"
#include "pop3smtp.h"


/*
 * Proxy function to have an account locked or unlocked.
 * Lock logic allows for recursive locks.
 */
int 
proxy_account_lock( int id, int lock )
{
#ifdef NEW_IPC
  return account_lock( id, lock ) ;
#else
  return 0 ;
#endif
}



/*
 * Find out which account owns this pid.  Since every pid is unique,
 * we should be able to walk the list of application pids and find out
 * which account (sending or receiving) actually owns this pid.  Then,
 * return the account number that owns it.
 */
int
proxy_seek_account_by_pid( pid_t pid )
{
#ifdef NEW_IPC
  return account_seek_account_by_pid( pid ) ;
#else
  return -1 ;
#endif
}



/*
 * Add a new node which allows for pid <-> account number
 * look ups!
 */
void
proxyNewAccountPidNodeInsert( int id, pid_t pid )
{
#ifdef NEW_IPC
  newAccountPidNodeInsert( id, pid ) ;
#endif

  return ;
}



/*
 * Look up a node by it's pid and remove it from the
 * linked list.
 */
void
proxyAccountPidNodeDestroy( pid_t pid )
{
#ifdef NEW_IPC
  accountPidNodeDestroy( pid ) ;
#endif

  return ;
}



/*
 * Set the account value for the total number of messages
 */
int 
proxySetAccountTotalMessages( int id, int numMsgs )
{
#ifdef NEW_IPC
  return setAccountTotalMessages( id, numMsgs ) ;
#else
  return 0 ;
#endif
}
