/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: mail-loadsave.c,v 1.32 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "puma.h"

#define EOW	'\04'
#define IO_CHUNK (64*1024)
#define fw( v, n, f ) fwrite( &v, sizeof(v), n, f ) ;
#define fws( v, n, f ) fwrite( v, (strlen(v)+1), n, f ) ;

Folder *folder_list=NULL;

int folder_add_new (char *name, unsigned long size) {
	Folder *newfolder, *flist=folder_list;
	if (flist==NULL) {
		flist=(Folder *)calloc (1, sizeof(Folder));
		folder_list=flist;
		flist->prev=NULL;
		flist->next=NULL;

		flist->id=0;
		flist->flags=0;
		flist->asize=size;
		flist->name=strdup(name);
		flist->parent=NULL;
		flist->msglst=NULL;

		return flist->id;
	} else {
		newfolder=(Folder *)calloc (1, sizeof(Folder));
		while (flist->next!=NULL) { flist=flist->next; } 
		newfolder->prev=flist;
		flist->next=newfolder;
		newfolder->next=NULL;

		newfolder->id=flist->id+1;
		newfolder->flags=0;
		newfolder->asize=size;
		newfolder->name=strdup(name);
		newfolder->parent=NULL;
		newfolder->msglst=NULL;

		return newfolder->id;
	}

	return -1;
}

int folder_create (char *name, char *parent) {
	char *home;
	char *tmp;
	int id;
	int fd;
	Folder *fl;

	home=(char *)calloc(1024, sizeof(char));
	snprintf (home, 1024, "$(HOME)");
	env_srch_rep(home);

	tmp=(char *)calloc(1024, sizeof(char));

	snprintf (tmp, 1024, "%s/.tradeclient/%s", home, name);
	fd=open (tmp, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);
	close (fd);

	id=folder_add_new (name, 0);
	fl=folder_seek (id);

	fl->parent=parent;
	fl->unread=0;
	fl->total=0;
	fl->flags=CAN_STORE_NEW_MAIL | CAN_HAVE_SUBFOLDERS;
	free (home);
	free (tmp);
	return fl->id;
}

Folder *folder_first () {
	return folder_list;
}

Folder *folder_seek (int id) {
	Folder *seek=folder_list;
	if( id != -1 ) {
	  while (seek) {
	    if (seek->id==id) return seek;
	    seek=seek->next;
	  }
	}
	return NULL;
}

int folder_seek_by_name (char *name) {
	Folder *seek=folder_list;
	if (!name) return -1;
	while (seek) {
		if((seek->name)&&(name)){
			if (strcmp(seek->name,name)==0) return seek->id;
			seek=seek->next;
		}
	}
	return -1;
}

void folder_free (Folder *seek) {
	free (seek->name);
	free (seek->parent);
	message_list_destroy (seek);
	seek->prev=NULL;
	seek->next=NULL;
	free (seek);
}

void folder_delete (int id) {
	Folder *seek=folder_seek (id);
	Folder *ntmp, *ptmp;
	if (seek!=NULL) {
		if (seek==folder_list) 
			folder_list=folder_list->next;
		ntmp=seek->next;
		ptmp=seek->prev;
		if (ntmp) ntmp->prev=ptmp;
		if (ptmp) ptmp->next=ntmp;
		folder_free (seek);
	}
}

int message_add_new (Folder *fl, unsigned long seek, unsigned long size,
                     unsigned long hsize, unsigned long flags,
                     unsigned long time, char *folder) {
	Message *newmsg, *mlist=fl->msglst;
	Folder *vfl;
	if (mlist==NULL) {
		mlist=(Message *)calloc (1, sizeof(Message));
		fl->msglst=mlist;
		mlist->prev=NULL;
		mlist->next=NULL;

		mlist->id=0;
		mlist->seek=seek;
		mlist->size=size;
		mlist->hsize=hsize;
		mlist->flags=flags;
		mlist->folder=strdup (folder);
		mlist->from=NULL;
		mlist->subject=NULL;
		mlist->date=NULL;
		mlist->time=time;
		mlist->text=NULL;
		mlist->texttime=0;
		vfl=folder_seek (folder_seek_by_name (mlist->folder));
		if (!vfl) {
			printf ("Error! Unable to find folder '%s'\n", mlist->folder);
		} else {
			vfl->total++;
			if (flags & MESSAGE_UNREAD) vfl->unread++;
		}
		return mlist->id;
	} else {
		newmsg=(Message *)calloc (1, sizeof(Message));
		while (mlist->next!=NULL) { mlist=mlist->next; }
		newmsg->prev=mlist;
		mlist->next=newmsg;
		newmsg->next=NULL;

		newmsg->id=mlist->id+1;
		newmsg->seek=seek;
		newmsg->size=size;
		newmsg->hsize=hsize;
		newmsg->flags=flags;
		newmsg->folder=strdup (folder);
		newmsg->from=NULL;
		newmsg->subject=NULL;
		newmsg->date=NULL;
		newmsg->time=time;
		newmsg->text=NULL;
		newmsg->texttime=0;
		vfl=folder_seek (folder_seek_by_name (newmsg->folder));
		if (!vfl) {
			printf ("Error! Unable to find folder '%s'\n", newmsg->folder);
		} else {
			vfl->total++;
			if (flags & MESSAGE_UNREAD) vfl->unread++;
		}

		return newmsg->id;
	}

	return -1;
}

Message *message_seek (Folder *fl, int id) {
  Message *seek ;

  if( (fl != NULL) || (id != -1) ) {
    seek = fl->msglst ;
    while (seek) {
      if (seek->id==id) return seek;
      seek=seek->next;
    }
  }
  return NULL;
}

void message_free (Message *msg) {
	if (msg!=NULL) {
		msg->prev=NULL;
		msg->next=NULL;
		msg->id=0;
		msg->seek=0;
		msg->size=0;
		msg->hsize=0;
		msg->flags=0;
		if (msg->folder!=NULL) free (msg->folder);
		if (msg->rfolder!=NULL) free (msg->rfolder);
		if (msg->from!=NULL) free (msg->from);
		if (msg->subject!=NULL) free (msg->subject);
		if (msg->priority!=NULL) free (msg->priority);
		if (msg->replyto!=NULL) free (msg->replyto);
		if (msg->sensitivity!=NULL) free (msg->sensitivity);
		if (msg->replyby!=NULL) free (msg->replyby);
		if (msg->expiry!=NULL) free (msg->expiry);
		if (msg->keywords!=NULL) free (msg->keywords);
		if (msg->date!=NULL) free (msg->date);
		if (msg->text) free (msg->text);
		if (msg->uidl) {
			free (msg->uidl);
		}
		free (msg);
	}
}

void message_delete (Folder *fl, int id) {
	Message *seek=message_seek (fl, id);
	Message *tmp;
	if (seek!=NULL) {
		if (seek==fl->msglst) {
			fl->msglst=seek->next;
			if (fl->msglst) fl->msglst->prev=NULL;
		}
		if (seek->next!=NULL) {
			tmp=seek->next;
			tmp->prev=seek->prev;
		}

		if (seek->prev!=NULL) {
			tmp=seek->prev;
			tmp->next=seek->next;
		}
		message_free (seek);
	}
}

void message_list_destroy (Folder *fl) {
	Message *seek=fl->msglst;
	Message *tmp;
	
	if (seek!=NULL) {
		while (seek->next) {
			tmp=seek->next;
			message_free (seek);
			seek=tmp;
		}
	}
}

void folders_read () {
	DIR *tdir;
	struct dirent *de;
	struct stat *st=(struct stat *)calloc(1, sizeof(struct stat));
	char *dir;
	char *tmps=(char *)calloc(1024, sizeof(char));

	dir=(char *)calloc(1024, sizeof(char));
	snprintf (dir, 1024, "$(HOME)/.tradeclient");
	env_srch_rep (dir);

	tdir=opendir (dir);

	if (tdir==NULL) {
		mkdir(dir, S_IREAD | S_IWRITE | S_IEXEC);
		tdir=opendir (dir);
	}

	while ((de=readdir(tdir))) {
		if (de->d_name[0]!='.') {
			snprintf (tmps, 1024, "%s/%s", dir, de->d_name);
			stat (tmps, st);
			if (S_ISREG(st->st_mode)) {
				folder_add_new (de->d_name, st->st_size);
			}
		}
	}

	closedir (tdir);
	free (st);
	free (dir);
	free (tmps);
}

unsigned long assemble_long (unsigned char *snum) {
	unsigned long ret=0;
	unsigned long tmpi=0;
	tmpi=snum[0];
	ret=ret|((tmpi<<24)&0xFF000000);
	tmpi=snum[1];
	ret=ret|((tmpi<<16)&0x00FF0000);
	tmpi=snum[2];
	ret=ret|((tmpi<<8) &0x0000FF00);
	tmpi=snum[3];
	ret=ret| (tmpi     &0x000000FF);
	return ret;
}

char *disassemble_long (unsigned long num) {
	char *ret=(char *)calloc(5, sizeof(char));

#if DEBUG > 7
	char *bits=bits32(num);
	printf ("bits befor=%s\n", bits);
	free (bits);
#endif

	ret[0]=(num&0xff000000)>>24;
	ret[1]=(num&0x00ff0000)>>16;
	ret[2]=(num&0x0000ff00)>>8;
	ret[3]=(num&0x000000ff);

#if DEBUG > 7
	bits=bits8(((int)ret[0]));
	printf ("bits after=%s", bits);
	free (bits);
	bits=bits8(((int)ret[1]));
	printf ("%s", bits);
	free (bits);
	bits=bits8(((int)ret[2]));
	printf ("%s", bits);
	free (bits);
	bits=bits8(((int)ret[3]));
	printf ("%s\n", bits);
	free (bits);
#endif

	return ret;
}


/*
 * Completely new index functions to replace the completely lame index code
 * that was previously in place.  I can't for the life of me imagine why
 * anyone would do byte at a time I/O and not expect to get slammed for it!!!
 * Additionally, some dummy decided that we'll have distinct indecies, but
 * maintain everything in memory in a single list.  Duh!
 *
 * At any rate, this will be a new implemention of the current logic (highly
 * optimized) with only minor departures.  Once the current implementation
 * is better understood, and give allows, the optimizations will be complete,
 * which should yield MUCH better results.  The next thing to do here is to
 * optimize the folder opening code.  Between these two, things should be
 * TONS faster when dealing with large folders.
 *
 * This function returns the number of index nodes that were written out.
 */
#if 0
int
new_index_write( pfile *pf, Folder *fl )
{
  int ret ;
  FILE *f ;
  Message *msg ;


  ret = 0 ;
  f = pf -> fs ;
  msg = (Message *)NULL ;

  /*
   * Write the index version that we are using.  Versions < 2 are crap.
   * Version 2 is ALL good.  Version 3 should be full implementation of a good index scheme by then.
   * Notice that we null terminate and make it 5-bytes long now.
   */
  fws( "0002\x00", 1, f ) ;

  /* Okay, write the name of the parent folder, if there is a parent.  Notice the NULL is also written. */
  if( fl -> parent ) 
    {
      fws( fl -> parent, 1, f ) ;
    }
  else fws( "", 1, f ) ;

  /*
   * Now, write the current value of FLAGS.  I personally don't care if different
   * platforms can share index files.  In theory, they can be rebuilt for each message
   * store, so it's a totally lame idea that simply wastes more CPU.
   */
  fw( fl -> flags, 1, f ) ;

  msg = fl -> msglst ;
  while( msg )
    {
      /* Write out various msg attributes */
      fw( msg -> seek, 1, f ) ;
      fw( msg -> size, 1, f ) ;
      fw( msg -> hsize, 1, f ) ;
      fw( msg -> flags, 1, f ) ;
      fw( msg -> time, 1, f ) ;

      /* Folder name that this message belongs to --hhmmmm */
      fws( msg -> folder, 1, f ) ;
      fws( msg -> subject, 1, f ) ;
      fws( msg -> date, 1, f ) ;
      fws( msg -> uidl, 1, f ) ;             /* Not really sure why the UIDL is indexed at the moment. */

      ret++ ;
      msg = msg -> next ;
    }

  return ret ;
}
#endif


void index_write (pfile *pf, Folder *fl) {
	char *sep=(char *)calloc(2, sizeof(char));
	char *snum;
	Message *mseek;
	sep[0]=EOW;
	if (fl->parent) fwrite (fl->parent, strlen(fl->parent), 1, pf->fs);
	fwrite (sep, 1, 1, pf->fs);

	fwrite ("0006", 4, 1, pf->fs);

	snum=disassemble_long (fl->flags);
	fwrite (snum, 4, 1, pf->fs);
	free (snum);
	mseek=fl->msglst;
	while (mseek) {
		snum=disassemble_long (mseek->seek);
		fwrite (snum, 4, sizeof (char), pf->fs);
		free (snum);

		snum=disassemble_long (mseek->size);
		fwrite (snum, 4, sizeof (char), pf->fs);
		free (snum);

		snum=disassemble_long (mseek->hsize);
		fwrite (snum, 4, sizeof (char), pf->fs);
		free (snum);

		snum=disassemble_long (mseek->flags);
		fwrite (snum, 4, sizeof (char), pf->fs);
		free (snum);

		snum=disassemble_long (mseek->time);
		fwrite (snum, 4, sizeof (char), pf->fs);
		free (snum);

		fwrite (mseek->folder, strlen(mseek->folder), 1, pf->fs);
		fwrite (sep, 1, 1, pf->fs);

		if (mseek->from) fwrite (mseek->from, strlen(mseek->from), 1, pf->fs);
		fwrite (sep, 1, 1, pf->fs);

		if (mseek->subject) fwrite (mseek->subject, strlen(mseek->subject), 1, pf->fs);
		fwrite (sep, 1, 1, pf->fs);

		if (mseek->priority) fwrite (mseek->priority, strlen(mseek->priority), 1, pf->fs);
		fwrite (sep, 1, 1, pf->fs);

		if (mseek->replyto) fwrite (mseek->replyto, strlen(mseek->replyto), 1, pf->fs);
		fwrite (sep, 1, 1, pf->fs);

		if (mseek->sensitivity) fwrite (mseek->sensitivity, strlen(mseek->sensitivity), 1, pf->fs);
		fwrite (sep, 1, 1, pf->fs);

                if (mseek->expiry) fwrite (mseek->expiry, strlen(mseek->expiry), 1, pf->fs);
                fwrite (sep, 1, 1, pf->fs);

                if (mseek->replyby) fwrite (mseek->replyby, strlen(mseek->replyby), 1, pf->fs);
                fwrite (sep, 1, 1, pf->fs);

		if (mseek->keywords) fwrite (mseek->keywords, strlen(mseek->keywords), 1, pf->fs);
		fwrite (sep, 1, 1, pf->fs);

		if (mseek->date) fwrite (mseek->date, strlen(mseek->date), 1, pf->fs);
		fwrite (sep, 1, 1, pf->fs);

		if (mseek->uidl) fwrite (mseek->uidl, strlen(mseek->uidl), 1, pf->fs);
		fwrite (sep, 1, 1, pf->fs);

		mseek=mseek->next;
	}
	free (sep);
}




/*
 * Completely new index functions to replace the completely lame index code
 * that was previously in place.  I can't for the life of me imagine why
 * anyone would do byte at a time I/O and not expect to get slammed for it!!!
 * Additionally, some dummy decided that we'll have distinct indecies, but
 * maintain everything in memory in a single list.  Duh!
 *
 * At any rate, this will be a new implemention of the current logic (highly
 * optimized) with only minor departures.  Once the current implementation
 * is better understood, and give allows, the optimizations will be complete,
 * which should yield MUCH better results.  The next thing to do here is to
 * optimize the folder opening code.  Between these two, things should be
 * TONS faster when dealing with large folders.
 *
 * This function returns the number of index nodes that were read out.
 * Also, note that we use 64K blocks for our I/O.
 *
 * This code is NOT complete and will have to wait until some other code
 * can be re-written (folder logic) to properly make use of indexes per
 * foler, whereby, the folder index/contents are actually maintained
 * properly instead of trying to cramp everthing into a single linked
 * list.   Grrrr...
 */
#if 0
int
new_index_read( pfile *pf, Folder *fl )
{
  int ret ; 
  FILE *f ;
  char *buf ;
  int index ;
  int length ;


  ret = 0 ;
  index = 0 ;
  length = 0 ;
  f = pf -> fs ;
  buf = malloc( IO_CHUNK ) ;

  /* Fetch our first block chunk */
  length = fread( buf, IO_CHUNK, 1, f ) ;

  /* Make sure we have fetched something, otherwise, report the error */
  if( length > 0 )
    {
      /*
       * First, fetch the version number.  If the version
       * number is an older number, we'll error for now.  In the future, I'll
       * try to automatically reindex.
       */
      if( atoi(buf) >= 2 )
	{
	  /* Jump past the version */
	  index += 4 + 1 ;  /* NULL value */

	  fl -> parent = strdup( &buf[index] ) ;
	  index += strlen(&buf[index]) ;
	  index++ ;  /* NULL value */

	  fl -> flags = (int *)&buf[index] ;
	  index += sizeof( int ) ;

	  fl -> seek = (int *)&buf[index] ;
	  index += sizeof( int ) ;

	  fl -> size = (int *)&buf[index] ;
	  index += sizeof( int ) ;

	  fl -> hsize = (int *)&buf[index] ;
	  index += sizeof( int ) ;

	  fl -> flags = (int *)&buf[index] ;
	  index += sizeof( int ) ;

	  fl -> time = (int *)&buf[index] ;
	  index += sizeof( int ) ;

	  fl -> folder = strdup( &buf[index] ) ;
	  index += strlen(&buf[index]) ;
	  index++ ;  /* NULL value */

	  fl -> subject = strdup( &buf[index] ) ;
	  index += strlen(&buf[index]) ;
	  index++ ;  /* NULL value */

	  fl -> date = strdup( &buf[index] ) ;
	  index += strlen(&buf[index]) ;
	  index++ ;  /* NULL value */

	  fl -> uidl = strdup( &buf[index] ) ;
	  index += strlen(&buf[index]) ;
	  index++ ;  /* NULL value */
	}
    }
  return ret ;
}
#endif

void index_read (pfile *pf, Folder *fl) {
	char *tmps=(char *)calloc(1025, sizeof(char));
	unsigned long seek, size, hsize, flags, time;
	int iversion;
	char *folder=NULL, *from=NULL, *subject=NULL, *date=NULL, *uidl=NULL;
	char *priority=NULL;
	char *replyto=NULL;
	char *sensitivity=NULL;
	char *expiry=NULL;
	char *replyby=NULL;
	char *keywords=NULL;
	char snum[4];
	char ch;
	int i=0, id;
	Message *msgs;

	ch=getch (pf);
	while ((ch!=EOW)&&(i<1024)) {
		tmps[i]=ch;
		ch=getch (pf);
		i++;
		if (pf->T_EOF==1) goto errorhead;
	}
	if (tmps[i]==EOW) tmps[i]='\0';

	fl->parent=strdup (tmps);

	snum[0]=getch(pf);
	if (pf->T_EOF==1) goto errorhead;
	snum[1]=getch(pf);
	if (pf->T_EOF==1) goto errorhead;
	snum[2]=getch(pf);
	if (pf->T_EOF==1) goto errorhead;
	snum[3]=getch(pf);

	iversion=atoi (snum);
#if DEBUG > 4
	printf( "Reading index file version %d.\n", iversion ) ;
#endif

	if (iversion==0) {
		fl->flags=assemble_long (snum);
	} else {
		snum[0]=getch(pf);
		if (pf->T_EOF==1) goto errorhead;
		snum[1]=getch(pf);
		if (pf->T_EOF==1) goto errorhead;
		snum[2]=getch(pf);
		if (pf->T_EOF==1) goto errorhead;
		snum[3]=getch(pf);
		fl->flags=assemble_long (snum);
	}

	ch=getch (pf);
	if (ch!=-1) ungetch(pf);

	while (pf->T_EOF==0) {
		snum[0]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[1]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[2]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[3]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		seek=assemble_long (snum);

		snum[0]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[1]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[2]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[3]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		size=assemble_long (snum);

		snum[0]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[1]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[2]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[3]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		hsize=assemble_long (snum);

		snum[0]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[1]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[2]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[3]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		flags=assemble_long (snum);

		snum[0]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[1]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[2]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		snum[3]=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
		time=assemble_long (snum);

		ch=getch(pf);
		i=0;
		memset (tmps, 0, 1024);
		while ((ch!=EOW)&&(pf->T_EOF!=-1)) {
			tmps[i]=ch;
			ch=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
			i++;
		}
		folder=strdup(tmps);

		ch=getch(pf);
		i=0;
		memset (tmps, 0, 1024);
		while ((ch!=EOW)&&(pf->T_EOF!=-1)) {
			tmps[i]=ch;
			ch=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
			i++;
		}
		from=strdup(tmps);

		ch=getch(pf);
		i=0;
		memset (tmps, 0, 1024);
		while ((ch!=EOW)&&(pf->T_EOF!=-1)) {
			tmps[i]=ch;
			ch=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
			i++;
		}
		subject=strdup(tmps);

		/* Trey added priority support */
		if( iversion > 1 ) {
		  ch=getch(pf);
		  i=0;
		  memset (tmps, 0, 1024);
		  while ((ch!=EOW)&&(pf->T_EOF!=-1)) {
		    tmps[i]=ch;
		    ch=getch(pf);
		    if (pf->T_EOF==1) goto errormsg;
		    i++;
		  }
		  priority=strdup(tmps);
		}
		else priority = strdup( "3" ) ;

                if( iversion > 3 ) {
                  ch=getch(pf);
                  i=0;
                  memset (tmps, 0, 1024);
                  while ((ch!=EOW)&&(pf->T_EOF!=-1)) {
                    tmps[i]=ch;
                    ch=getch(pf);
                    if (pf->T_EOF==1) goto errormsg;
                    i++;
                  }
                  replyto=strdup(tmps);
                }

                if( iversion > 4 ) {
                  ch=getch(pf);
                  i=0;
                  memset (tmps, 0, 1024);
                  while ((ch!=EOW)&&(pf->T_EOF!=-1)) {
                    tmps[i]=ch;
                    ch=getch(pf);
                    if (pf->T_EOF==1) goto errormsg;
                    i++;
                  }
                  sensitivity=strdup(tmps);
                }

                if( iversion > 5 ) {
                  ch=getch(pf);
                  i=0;
                  memset (tmps, 0, 1024);
                  while ((ch!=EOW)&&(pf->T_EOF!=-1)) {
                    tmps[i]=ch;
                    ch=getch(pf);
                    if (pf->T_EOF==1) goto errormsg;
                    i++;
                  }
                  expiry=strdup(tmps);
                }

                if( iversion > 5 ) {
                  ch=getch(pf);
                  i=0;
                  memset (tmps, 0, 1024);
                  while ((ch!=EOW)&&(pf->T_EOF!=-1)) {
                    tmps[i]=ch;
                    ch=getch(pf);
                    if (pf->T_EOF==1) goto errormsg;
                    i++;
                  }
                  replyby=strdup(tmps);
                }

                if( iversion > 2 ) {
                  ch=getch(pf);
                  i=0;
                  memset (tmps, 0, 1024);
                  while ((ch!=EOW)&&(pf->T_EOF!=-1)) {
                    tmps[i]=ch;
                    ch=getch(pf);
                    if (pf->T_EOF==1) goto errormsg;
                    i++;
                  }
                  keywords=strdup(tmps);
                }

		ch=getch(pf);
		i=0;
		memset (tmps, 0, 1024);
		while ((ch!=EOW)&&(pf->T_EOF!=-1)) {
			tmps[i]=ch;
			ch=getch(pf);
			if (pf->T_EOF==1) goto errormsg;
			i++;
		}
		date=strdup(tmps);

		if ( iversion > 0 ) {
			ch=getch(pf);
			i=0;
			memset (tmps, 0, 1024);
			while ((ch!=EOW)&&(pf->T_EOF!=-1)) {
				tmps[i]=ch;
				ch=getch(pf);
				if (pf->T_EOF==1) goto errormsg;
				i++;
			}
			uidl=strdup(tmps);
		}

		id=message_add_new (fl, seek, size, hsize, flags, time, folder);
		
		msgs=message_seek (fl, id);
		msgs->rfolder=strdup (fl->name);
		msgs->from=strdup (from);
		msgs->subject=strdup (subject);

		/* Priority support added by trey */
		if( iversion > 1 ) {
		  msgs->priority=strdup(priority);
		} else msgs->priority=strdup("3");

		if (iversion > 3)
		{
			if (replyto) msgs->replyto = strdup (replyto);
		}

		if (iversion > 3)
		{
			if (sensitivity) msgs->sensitivity = strdup (sensitivity);
		}

		if (iversion > 5)
                {
                        if (expiry) msgs->expiry = strdup (expiry);
                }

		if (iversion > 5)
                {
                        if (replyby) msgs->replyby = strdup (replyby);
                }

		if ( iversion > 2 ) {
		  msgs->keywords=strdup(keywords);
		}

		msgs->date=strdup (date);

		if ( iversion > 0 ) {
			msgs->uidl=strdup (uidl);
		}

		ch=getch (pf);
		if (ch!=-1) ungetch(pf);

		if (folder!=NULL) free (folder);
		if (from!=NULL) free (from);
		if (subject!=NULL) free (subject);
		if (date!=NULL) free (date);
		if (uidl!=NULL) free (uidl);
		if (keywords!=NULL) free (keywords);
		if (sensitivity!=NULL) free (sensitivity);
		if (replyby!=NULL) free (replyby);
		if (expiry!=NULL) free (expiry);
		if (replyto!=NULL) free (replyto);
		if (priority!=NULL) free (priority);
	}

	goto ending;

errorhead:
	printf ("index file header crouption, discarding\n");
	fl->flags=F_INDEX_HEAD_CORRUPT;
	goto ending;

errormsg:
	printf ("error occured while reading message index, destroying list\n");
	message_list_destroy (fl);
	if (folder!=NULL) free (folder);
	if (from!=NULL) free (from);
	if (subject!=NULL) free (subject);
	if (date!=NULL) free (date);

ending:
	if (tmps!=NULL) free (tmps);
}

void folders_write_index () {
	Folder *flist=folder_list;
	pfile *pf;
	char *tmps;
	char *home;

	home=(char *)calloc(1024, sizeof(char));
	snprintf (home, 1024, "$(HOME)/.tradeclient");
	env_srch_rep (home);

	tmps=(char *)calloc(1024, sizeof(char));
	if (flist!=NULL) {
		while (flist) {
			if ((flist->flags&F_INDEX_MISSING)==0) {
	 			pf=new_pfile();
				snprintf (tmps, 1024, "%s/.%s.smi", home, flist->name);
				strcpy( pf->name, tmps ) ;
				pf->fs=fopen (tmps, "w+");
#if 1
				index_write (pf, flist);
#else
				new_index_write( pf, flist ) ;
#endif
				fclose (pf->fs);
				delete_pfile( pf ) ;
			}
			flist=flist->next;
		}
	}
	free (home);
	free (tmps);
}

void folders_read_index () {
  pfile *pf;
  char *home;
  char *tmps;
  Folder *flist=folder_list;


  home=(char *)calloc(1024, sizeof(char));
  snprintf (home, 1024, "$(HOME)/.tradeclient");
  env_srch_rep (home);

  tmps=(char *)calloc(1024, sizeof(char));
  if (flist!=NULL) {
    while (flist) {
      pf=new_pfile();
      snprintf (tmps, 1024, "%s/.%s.smi", home, flist->name);
      strcpy( pf->name, tmps ) ;
      pf->fs=fopen (tmps, "r");
      if (pf->fs!=NULL) {
#if 1
	index_read (pf, flist);
#else
	new_index_read( pf, flist ) ;
#endif
	fclose (pf->fs);
	flist->flags&=~F_INDEX_MISSING;
	delete_pfile( pf ) ;
      } else {
	flist->flags|=F_INDEX_MISSING;
	delete_pfile( pf ) ;
      }
      flist=flist->next;
    }
  }
  free (tmps);
  free (home);

  return ;
}

int message_hsize (char *message) {
	int i=0, len=strlen(message);


#ifdef DMALLOC
	dmalloc_verify( 0 ) ;
#endif
	while (i<len) {
		if ((message[i+1]=='\n')&&(message[i]=='\r')) {
			i++;i++;
			return i;
		} else if (message[i]=='\n') {
			i++;
			return i;
		} else {
			while ((message[i]!='\n')&&(i<len)) { i++; }
		}
		i++;
	}
#ifdef DMALLOC
	dmalloc_verify( 0 ) ;
#endif
	return i;
}

/* Modified by Dieter Fiebelkorn <dieter@fiebelkorn.net>
 * to support MBOX mailboxes, while keeping compatability with old TC format.
 */
int message_save (char *folder, char *message) {
	int fid=folder_seek_by_name (folder);
	unsigned long size, hsize;
	unsigned long seek, ttime;
	Folder *fseek;
	FILE *fs;
	char *home=NULL;
	char *tmps=NULL;
	char *filename, *s;
	int id, slen;
	Message *msglst;
	Header_Info *headinf;
	struct stat st;


#if DEBUG > 4
	printf( "Inside of message_save().\n" ) ;
#endif

	/* If we don't have a valid folder, use the default Inbox folder */
	if (fid==-1) {
		fid=folder_seek_by_name ("Inbox");
	}

	fseek=folder_seek (fid);
	size=strlen (message);
	if (message[size]!='\n') {
		message[size]='\n';
	}

#if DEBUG > 6
	printf ("message:\n%s", message);
#endif

	hsize=message_hsize (message);

	tmps=(char *)calloc (hsize+1, sizeof(char));

	memmove (tmps, message, hsize);
	headinf=parse_header (tmps);
	free (tmps);

#if DEBUG > 8
	printf( "Saved data...\n" ) ;
	printf ("headinf->from='%s'\n", headinf->from);
	printf ("headinf->to='%s'\n", headinf->to);
	printf ("headinf->cc='%s'\n", headinf->cc);
	printf ("headinf->subject='%s'\n", headinf->subject);
	printf ("headinf->priority='%s'\n",headinf->priority);
	printf ("headinf->replyto='%s'\n",headinf->replyto);
	printf ("headinf->sensitivity='%s'\n",headinf->sensitivity);
	printf ("headinf->expiry='%s'\n",headinf->expiry);
	printf ("headinf->replyby='%s'\n",headinf->replyby);
	printf ("headinf->keywords='%s'\n",headinf->keywords);
	printf ("headinf->date='%s'\n", headinf->date);
	printf ("headinf->content_type='%s'\n", headinf->content_type);
	printf ("headinf->content_t_e='%s'\n", headinf->content_t_e);
	printf ("headinf->content_d='%s'\n", headinf->content_d);
	printf ("headinf->uidl='%s'\n", headinf->uidl);
#endif

#if DEBUG > 8
	printf ("size=%d hsize=%d\n", (int)size, (int)hsize);
#endif

	home=(char *)calloc (4096, sizeof(char));
	snprintf (home, 1024, "$(HOME)");
	env_srch_rep (home);

	tmps=(char *)calloc (4096, sizeof(char));
	filename=(char *)calloc (4096, sizeof(char));
	snprintf (filename, 4096, "%s/.tradeclient/%s",home, fseek->name);
	fs=fopen(filename, "a");

	if (fs!=NULL) {

		ttime=time(NULL);
#ifdef SUPP_RETRIEVED
		snprintf (tmps, 4096, "Retrieved -- %lu\n", ttime);
		fwrite (tmps, strlen(tmps), sizeof(char), fs);
#else
		s = ctime (&ttime);
		slen=strlen(s) - 1;
		snprintf (tmps, 4096, "From %s %*.*s\n", "TradeClient", slen,slen,s);
		fwrite (tmps, strlen(tmps), sizeof(char), fs);
#endif
		seek=ftell (fs);
		fwrite (message, strlen(message), sizeof(char), fs);
		id=message_add_new (fseek, seek, size, hsize, 0, ttime, fseek->name);
		msglst=message_seek (fseek, id);

		fclose (fs);

		if (headinf->from) msglst->from=strdup(headinf->from);
		if (headinf->subject) msglst->subject=strdup(headinf->subject);
		if (headinf->date) msglst->date=strdup(headinf->date);
		if (headinf->uidl) msglst->uidl=strdup(headinf->uidl);
		if (headinf->priority) msglst->priority=strdup(headinf->priority);
		if (headinf->replyto) msglst->replyto=strdup(headinf->replyto);
		if (headinf->sensitivity) msglst->sensitivity=strdup(headinf->sensitivity);
		if (headinf->replyby) msglst->replyby=strdup(headinf->replyby);
		if (headinf->expiry) msglst->expiry=strdup(headinf->expiry);
		if (headinf->keywords) msglst->keywords=strdup(headinf->keywords);

		destroy_hinfo (headinf);

		stat (filename, &st);
		fseek->asize=st.st_size;
		free (filename);
		free (tmps);
		free (home);

#if DEBUG > 4
	printf( "Finished message_save() when !NULL.\n" ) ;
#endif

		return msglst->id;
	} else {
		printf ("Could not append message to folder; probably permission is denied\n");
	}

	fclose(fs);

	destroy_hinfo (headinf);
	free (home);
	free (tmps);

#if DEBUG > 4
	printf( "Finished message_save().\n" ) ;
#endif

	return -1;
}

char *message_load (Folder *fl, Message *msg) {
	pfile *pf=new_pfile();
	char *ret=(char *)calloc(msg->size+5, sizeof(char));
	char *tmps=(char *)calloc(1024, sizeof(char));
	snprintf (tmps, 1024, "$(HOME)/.tradeclient/%s", fl->name);
	env_srch_rep (tmps);

	if ((msg->text) && (!fl->flags&F_NO_CACHE)) {
		memmove (ret, msg->text, msg->size);
		msg->texttime=time(NULL);
		free (tmps);
		return ret;
	}

	strcpy( pf -> name, fl -> name ) ;
	pf->fs=fopen (tmps, "r");
	free (tmps);
	if (pf->fs!=NULL) {
		fseek (pf->fs,msg->seek,SEEK_SET);
		fread (ret, msg->size, sizeof(char), pf->fs);	
		fclose (pf->fs);
		delete_pfile( pf ) ;
		if (!fl->flags&F_NO_CACHE) {
			msg->text=(char *)calloc(msg->size+5, sizeof(char));
			memmove (msg->text, ret, msg->size);
			msg->texttime=time(NULL);
		} else if (msg->text) {
			free (msg->text);
			msg->text=NULL;
		}
		return ret;
	} else {
		printf ("Unable to load message, file permission denied.\n");
	}
	if (pf) delete_pfile( pf ) ;

	free (ret);

	return NULL;
}

char *message_load_head (Folder *fl, Message *msg) {
	char *tmpret=message_load (fl, msg);
	char *ret=NULL;

	if (tmpret) {
		ret=(char *)calloc (msg->hsize+1, sizeof(char));
		memmove (ret, tmpret, msg->hsize);
		free (tmpret);
	}
	return ret;
}


char *message_load_body (Folder *fl, Message *msg) {
	char *tmpret=message_load (fl, msg);
	char *ret=NULL;

	if (tmpret) {
		ret=(char *)calloc ((msg->size-msg->hsize)+1, sizeof(char));
		memmove (ret, tmpret+msg->hsize, msg->size-msg->hsize);
		free (tmpret);
	}
	return ret;
}

/* Added by Dieter Fiebelkorn <dieter@fiebelkorn.net>
 * to support MBOX mailboxes, while keeping compatability with old TC format.
 */
static int get_wday(char *wday)
{
	if (strncmp (wday, "Sun", 3)==0) return 0;
	if (strncmp (wday, "Mon", 3)==0) return 1;
	if (strncmp (wday, "Tue", 3)==0) return 2;
	if (strncmp (wday, "Wed", 3)==0) return 3;
	if (strncmp (wday, "Thu", 3)==0) return 4;
	if (strncmp (wday, "Fri", 3)==0) return 5;
	if (strncmp (wday, "Sat", 3)==0) return 6;
	return -1;
}

/* Added by Dieter Fiebelkorn <dieter@fiebelkorn.net>
 * to support MBOX mailboxes, while keeping compatability with old TC format.
 */
static int get_month(char *mons)
{
	if (strncmp (mons, "Jan", 3)==0) return  0;
	if (strncmp (mons, "Feb", 3)==0) return  1;
	if (strncmp (mons, "Mar", 3)==0) return  2;
	if (strncmp (mons, "Apr", 3)==0) return  3;
	if (strncmp (mons, "May", 3)==0) return  4;
	if (strncmp (mons, "Jun", 3)==0) return  5;
	if (strncmp (mons, "Jul", 3)==0) return  6;
	if (strncmp (mons, "Aug", 3)==0) return  7;
	if (strncmp (mons, "Sep", 3)==0) return  8;
	if (strncmp (mons, "Oct", 3)==0) return  9;
	if (strncmp (mons, "Nov", 3)==0) return 10;
	if (strncmp (mons, "Dec", 3)==0) return 11;
	return -1;
}

/* Added by Dieter Fiebelkorn <dieter@fiebelkorn.net>
 * to support MBOX mailboxes, while keeping compatability with old TC format.
 */
static int isMailHeader(char *tmps, int elns, unsigned long *tmptime)
{
	if (tmps[0]=='R' && strncmp (tmps, "Retrieved --", 12)==0) {
		sscanf (tmps, "Retrieved -- %lu", tmptime);
		return TRUE;
	}

	/* Examples for mbox-Headers:
	 * From uucphost!xx.yy.dom!user Wed Nov 10 11:11:50 1993
	 * From user@xx.yy.dom Wed Feb 17 08:52:51 1999	+0200			*/
	if (elns > 0 && tmps[0]=='F' && strncmp (tmps, "From ", 5)==0) {
		char email[1024], wdays[8], mons[8];
		int day, hour, min, sec, year, mon, wday;
		struct tm timep;

		sscanf (tmps, "From %s %3s %3s %2u %2u:%2u:%2u %4u", email,
				wdays, mons, &day, &hour, &min, &sec, &year);
		if ((wday=get_wday(wdays))>=0) {
			if ((mon=get_month(mons))>=0) {
				timep.tm_mday=day;
				timep.tm_mon=mon;
				timep.tm_year=year-1900;
				timep.tm_hour=hour;
				timep.tm_min=min;
				timep.tm_sec=sec;
				timep.tm_wday=wday;
				timep.tm_yday=0;
				timep.tm_isdst=-1;
				*tmptime=mktime(&timep);
				if (*tmptime>=0) {
					return TRUE;
				}
			}
		}
	}
	return FALSE;
}

/* Modified by Dieter Fiebelkorn <dieter@fiebelkorn.net>
 * to support MBOX mailboxes, while keeping compatability with old TC format.
 */
int folder_reindex (Folder *fl) {
	pfile *pf=(pfile *)calloc(1, sizeof(pfile));
	char *tmps1, *tmps2, *dir, *head;
	unsigned long tmpoff=0, tmpsize=0, tmphsize=0, tmptime=0;
	Header_Info *tmphinfo;
	Message *msg;
	int mid, emptylines;
#if DEBUG > 8
	struct timeval tv1, tv2;
	float tf1,tf2;
	int i;
#endif

	if (!fl) { free (pf); return FALSE; }

	dir=(char *)calloc(1024, sizeof(char));
	snprintf (dir, 1024, "$(HOME)/.tradeclient");
	env_srch_rep (dir);

	tmps1=(char *)calloc(strlen(fl->name)+strlen(dir)+10, sizeof(char));
	snprintf (tmps1, strlen(fl->name)+strlen(dir)+10, "%s/%s", dir, fl->name);
	free (dir);

	pf->fs=fopen (tmps1, "r");
	free (tmps1);

/* This is going to be slow and very painfull, VERY painfull */
/* im making this slower than i need to, but for now lets be pedantic */

	emptylines=1;
	tmps2=(char *)calloc(1028, sizeof(char));
	while (pf->T_EOF!=1) {
		getline_ (pf, tmps2);

		if (isMailHeader(tmps2, emptylines, &tmptime)) {
			emptylines = 0;
#if DEBUG > 7
			printf ("Next Message start, offset: %lu\n", pf->bboff+pf->bpos);
#endif
			tmpoff=pf->bboff+pf->bpos;

			tmps1=(char *)calloc(1028, sizeof(char));
			while (pf->T_EOF!=1) {
				getline_ (pf, tmps1);
				if (!tmps1) break;
				if ((tmps1[0]=='\r' && tmps1[1]=='\n') || tmps1[0]=='\n') {
					emptylines++;
					break;
				}
				else
					emptylines=0;
			}
			if (tmps1) free (tmps1);
			if (pf->T_EOF!=1)
				ungetline (pf);

			tmphsize=(pf->bboff+pf->bpos)-tmpoff;
			fseek (pf->fs, tmpoff, SEEK_SET);
			head=(char *)calloc(tmphsize+2, sizeof(char));
			fread (head, sizeof(char), tmphsize, pf->fs);
			tmphinfo=parse_header (head);
			free (head);

			tmps1=(char *)calloc(1028, sizeof(char));

#if DEBUG > 8
			gettimeofday (&tv1, NULL);
			i=0;
#endif
			while (pf->T_EOF!=1) {
				getline_ (pf, tmps1);
				if (!tmps1) break;
				if ((tmps1[0]=='\r' && tmps1[1]=='\n') || tmps1[0]=='\n')
					emptylines++;
				else {
					if (isMailHeader(tmps1, emptylines, &tmptime)) {
						break;
					}
					emptylines=0;
				}
#if DEBUG > 8
				i++;
				printf (".");
				fflush (stdout);
#endif
			}

#if DEBUG > 8
			printf ("\n");
			gettimeofday (&tv2, NULL);
			tf1=(tv1.tv_sec&0x000000ff)+((float)tv1.tv_usec/(float)1000000);
			tf2=(tv2.tv_sec&0x000000ff)+((float)tv2.tv_usec/(float)1000000);
			printf ("Processed %d in %f seconds at %f lines/second\n", i, (tf2-tf1), ((float)i)/(tf2-tf1));
#endif

			if (tmps1) free (tmps1);
			if (pf->T_EOF!=1)
				ungetline (pf);

			tmpsize=(pf->bboff+pf->bpos)-tmpoff;
			mid=message_add_new (fl, tmpoff, tmpsize, tmphsize, 0, tmptime, fl->name);
			msg=message_seek (fl, mid);

			msg->rfolder=strdup (fl->name);
 			if (tmphinfo->from) msg->from=strdup(tmphinfo->from);
 			if (tmphinfo->subject) msg->subject=strdup(tmphinfo->subject);
 			if (tmphinfo->date) msg->date=strdup(tmphinfo->date);
 			if (tmphinfo->uidl) msg->uidl=strdup(tmphinfo->uidl);
			if (tmphinfo->priority) msg->priority=strdup(tmphinfo->priority);
			if (tmphinfo->sensitivity) msg->sensitivity=strdup(tmphinfo->sensitivity);
			if (tmphinfo->replyby) msg->replyby=strdup(tmphinfo->replyby);
			if (tmphinfo->expiry) msg->expiry=strdup(tmphinfo->expiry);
			if (tmphinfo->replyto) msg->replyto=strdup(tmphinfo->replyto);
			if (tmphinfo->keywords) msg->keywords=strdup(tmphinfo->keywords);

			destroy_hinfo (tmphinfo);
		}
	}
	fl->flags&=~F_INDEX_MISSING;
	free (pf);
	free (tmps2);
	return TRUE;
}


/* Modified by Dieter Fiebelkorn <dieter@fiebelkorn.net>
 * to support MBOX mailboxes, while keeping compatability with old TC format.
 */
int folders_compact () {
	Folder *fl=folder_first ();
	Folder *flbackup=fl;
	Message *msg;
	struct stat st;
	FILE *fd;
	int slen;
	char *tmps, *tmps2, *tmps3, *dir, *s;

	if (!fl) return FALSE;//should indicate no folders defined

	dir=(char *)calloc(1024, sizeof(char));
	snprintf (dir, 1024, "$(HOME)/.tradeclient");
	env_srch_rep (dir);

	tmps=(char *)calloc(strlen(dir)+258, sizeof(char));
	tmps2=(char *)calloc(1028, sizeof(char));

	while (fl) {
		msg=fl->msglst;
		fl->flags|=F_NO_CACHE;

		tmps=memset (tmps, 0, strlen(dir)+258);
		snprintf (tmps, strlen(dir)+strlen(fl->name)+4, "%s/%s~",
		          dir, fl->name);
		fd=fopen (tmps, "a");
		fclose (fd);

		while (msg) {
			tmps=memset (tmps, 0, strlen(dir)+258);
			snprintf (tmps, strlen(dir)+strlen(msg->folder)+4, "%s/%s~",
			          dir, msg->folder);
			fd=fopen (tmps, "a");

			tmps2=memset (tmps2, 0, 1028);
#ifdef SUPP_RETRIEVED
			snprintf (tmps2, 1024, "Retrieved -- %lu\n", msg->time);
			fwrite (tmps2, strlen(tmps2), sizeof(char), fd);
#else
			s = ctime (&(msg->time));
			slen=strlen(s) - 1;
			snprintf (tmps2, 1024, "From %s %*.*s\n", "TradeClient", slen,slen,s);
			fwrite (tmps2, strlen(tmps2), sizeof(char), fd);
#endif

			tmps3=message_load (fl, msg);
			fwrite (tmps3, strlen(tmps3), sizeof(char), fd);
			free (tmps3);
			fclose (fd);

			msg=msg->next;
		}
		fl=fl->next;
	}
	free (tmps);
	free (tmps2);
	fl=flbackup;
	while (fl) {
		fl->flags&=~F_NO_CACHE;
		tmps=(char *)calloc(strlen(dir)+strlen(fl->name)+4, sizeof(char));
		tmps2=(char *)calloc(strlen(dir)+strlen(fl->name)+4, sizeof(char));
		snprintf (tmps, strlen(dir)+strlen(fl->name)+3, "%s/%s~",
		          dir, fl->name);
		snprintf (tmps2, strlen(dir)+strlen(fl->name)+2, "%s/%s",
		          dir, fl->name);
		rename (tmps, tmps2);
		chmod (tmps2, S_IRUSR | S_IWUSR);
		stat (tmps2, &st);
		fl->asize=st.st_size;
		free (tmps);
		free (tmps2);

		fl=fl->next;
	}
	free (dir);

	return TRUE;
}
