/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: main.c,v 1.47 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "puma.h"
#include "ipc.h"
#ifdef WANTLDAP
#include "tcldap.h"
#endif


TradeClientGlobs *tm_globs=NULL;
Folders *folders=NULL;

time_t works_before () {
	struct tm time;
	time.tm_sec=0;
	time.tm_min=0;
	time.tm_hour=0;
	time.tm_mday=0;
	time.tm_mon=2;
	time.tm_year=100;
	time.tm_wday=0;
	time.tm_yday=0;
	time.tm_isdst=-1;
	return mktime (&time);
}

char *stringdup (char *str) {
	char *ret=NULL;
	unsigned long len=0;
	if (str) {
		len=strlen(str);
		ret=(char *)calloc(len+1, sizeof(char));
		memmove (ret, str, len);
	}
	return ret;
}

char *unique_name (unsigned int seed, int length) {
	char *ret=(char *)calloc(length+2, sizeof(char));
	int i=0;

	if (!length) {
		free (ret);
		return NULL;
	}
	srandom (seed);
	for (i=0; i<length; i++) {
		ret[i]=gethex ((int)(random()%16));
	}
	return ret;
}

char *bits32 (unsigned long x) {
	char *ret=(char *)calloc (40, sizeof(char));
	ret[0 ]=((x&0x80000000)>>31) == 1 ? '1' : '0';
	ret[1 ]=((x&0x40000000)>>30) == 1 ? '1' : '0';
	ret[2 ]=((x&0x20000000)>>28) == 1 ? '1' : '0';
	ret[3 ]=((x&0x10000000)>>28) == 1 ? '1' : '0';
	ret[4 ]=((x&0x08000000)>>27) == 1 ? '1' : '0';
	ret[5 ]=((x&0x04000000)>>26) == 1 ? '1' : '0';
	ret[6 ]=((x&0x02000000)>>25) == 1 ? '1' : '0';
	ret[7 ]=((x&0x01000000)>>24) == 1 ? '1' : '0';
	ret[8 ]=((x&0x00800000)>>23) == 1 ? '1' : '0';
	ret[9 ]=((x&0x00400000)>>22) == 1 ? '1' : '0';
	ret[10]=((x&0x00200000)>>21) == 1 ? '1' : '0';
	ret[11]=((x&0x00100000)>>20) == 1 ? '1' : '0';
	ret[12]=((x&0x00080000)>>19) == 1 ? '1' : '0';
	ret[13]=((x&0x00040000)>>18) == 1 ? '1' : '0';
	ret[14]=((x&0x00020000)>>17) == 1 ? '1' : '0';
	ret[15]=((x&0x00010000)>>16) == 1 ? '1' : '0';

	ret[16]=((x&0x00008000)>>15) == 1 ? '1' : '0';
	ret[17]=((x&0x00004000)>>14) == 1 ? '1' : '0';
	ret[18]=((x&0x00002000)>>13) == 1 ? '1' : '0';
	ret[19]=((x&0x00001000)>>12) == 1 ? '1' : '0';
	ret[20]=((x&0x00000800)>>11) == 1 ? '1' : '0';
	ret[21]=((x&0x00000400)>>10) == 1 ? '1' : '0';
	ret[22]=((x&0x00000200)>>9 ) == 1 ? '1' : '0';
	ret[23]=((x&0x00000100)>>8 ) == 1 ? '1' : '0';
	ret[24]=((x&0x00000080)>>7 ) == 1 ? '1' : '0';
	ret[25]=((x&0x00000040)>>6 ) == 1 ? '1' : '0';
	ret[26]=((x&0x00000020)>>5 ) == 1 ? '1' : '0';
	ret[27]=((x&0x00000010)>>4 ) == 1 ? '1' : '0';
	ret[28]=((x&0x00000008)>>3 ) == 1 ? '1' : '0';
	ret[29]=((x&0x00000004)>>2 ) == 1 ? '1' : '0';
	ret[30]=((x&0x00000002)>>1 ) == 1 ? '1' : '0';
	ret[31]=((x&0x00000001)    ) == 1 ? '1' : '0';
	return ret;
}

char *bits8 (int x) {
	char *ret=(char *)calloc (20, sizeof(char));
	ret[0 ]=((x&0x80)>>7) == 1 ? '1' : '0';
	ret[1 ]=((x&0x40)>>6) == 1 ? '1' : '0';
	ret[2 ]=((x&0x20)>>5) == 1 ? '1' : '0';
	ret[3 ]=((x&0x10)>>4) == 1 ? '1' : '0';
	ret[4 ]=((x&0x08)>>3) == 1 ? '1' : '0';
	ret[5 ]=((x&0x04)>>2) == 1 ? '1' : '0';
	ret[6 ]=((x&0x02)>>1) == 1 ? '1' : '0';
	ret[7 ]=((x&0x01)   ) == 1 ? '1' : '0';
	return ret;
}

int count_delimiters (char *stream, char delim) {
	char *tmps=stream;
	int num=0;
	if (tmps) {
		while ( (tmps=strchr (tmps, (int)delim))!=NULL ) {
			tmps++;
			num++;
		}
	}
	return num;
}

char *remove_charecter (char *stream, char delim) {
	int num;
	char *ret=NULL;
	int rpos=0, spos=0, len=0;
	if (stream) {
		num=count_delimiters (stream, delim);
		len=strlen (stream);
		ret=(char *)calloc (len-num+2, sizeof(char));
		if (num>0) {
			for (spos=0;spos<len;spos++) {
				if (stream[spos]!=delim) {
					ret[rpos]=stream[spos];
					rpos++;
				}	
			}
		} else {
			ret=strdup (stream);
		}
	}
	return ret;
}

char *rm_cr (char *string) {
	char *ret=NULL, *tmps=NULL;
	unsigned long length=0, i=0;
	if (string) {
		if (string[0]=='\0') {
			ret=(char *)calloc (2, sizeof(char));
			return ret;
		}
		length=strlen (string);
		ret=(char *)calloc (length+2, sizeof(char));
		tmps=ret;
		for (i=0;i<length;i++) {
			if (string[i]!='\r') {
				*ret=string[i];
				ret++;
			}
		}
	}
	ret=tmps;
	return ret;
}


/* Modified to removed any leading spaces too */
char *rm_crlf (char *string) {
	char *ret=NULL, *tmps=NULL;
 	unsigned long length=0, i=0;
	if (string) {
		if (string[0]=='\0') {
			ret=(char *)calloc (2, sizeof(char));
			return ret;
		}

		length = strlen( string ) ;
		ret=(char *)calloc(length+2, sizeof(char));
		tmps=ret;
		for (i=0; i<length; i++) {
			if ((string[i]!='\r')&&(string[i]!='\n')) {
				*ret=string[i];
				ret++;
			}
		}
	}
	ret=tmps;
	return ret;
}

char *add_cr (char *string) {
	char *ret=NULL; char *tmps1=NULL; char *tmps2=NULL;
	unsigned long length=0, i=0;
	if (string) {
		if (string[0]=='\0') {
			ret=(char *)calloc (2, sizeof(char));
			return ret;
		}
		length=strlen (string);
		ret=(char *)calloc (length*2, sizeof(char));
		tmps1=ret;
		for (i=0; i<length; i++) {
			if (i>0) {
				if ((string[i]=='\n') && (string[i-1]!='\r')) {
					*ret='\r';
					ret++;
				}
			} else {
				if (string[i]=='\n') {
					*ret='\r';
					ret++;
				}
			}
			*ret=string[i];
			ret++;
		}
		tmps2=strdup (tmps1);
		free (tmps1);
	}
	ret=tmps2;
	return ret;
}

char *insert_reply_indicator (char *body) {
	int len=strlen (body), i;
	char *tmpret=(char *)calloc( len*2 + 2, sizeof(char)), *ret;
	char *tmpretb=tmpret;


	*tmpret='>';
	tmpret++;

	for (i=0; i<len; i++) {
		*tmpret=body[i];
		if (*tmpret=='\n') {
			tmpret++;
			*tmpret='>';
		}
		tmpret++;
	}
	ret=(char *)calloc (strlen(tmpretb)+2, sizeof(char));
	memmove (ret, tmpretb, strlen(tmpretb));
	free (tmpretb);
	return ret;
}



int
strip_lead_spaces( char *string )
{
  int i ;
  int length ;


  /* This clips any leading spaces */
  length = strlen( string ) ;

  i = 0 ;
  while( (i < length) && string[i] == ' ' ) i++ ;

  if( i ) {
    memcpy( string, &string[i], length - i ) ;
    string[length-i] = 0x00 ;
  }

  return i ;
}


char *cut_string(char *string, char delimiter, int side, int occurrence) {
	char *ret;
	char *errret;
	int length ;
	int charpos=0, i=0, j=0;
	if (!string) {
			errret=(char *)calloc (2, sizeof(char));
			return errret;
	}
	length = strlen( string ) ;
	for (j=0; j<occurrence; j++) {
		while (((unsigned)i<length) && (string[i]!=delimiter)) {
			i++;
		}
		i++;
		if ((unsigned)i>=length+1) {
			errret=(char *)calloc (2, sizeof(char));
			return errret;
		}
	}
	i--;
	charpos=i;
	if (side<=1) {
		ret=(char *)calloc(i+2, sizeof(char));
		for (i=0; i<charpos; i++) {
			ret[i]=string[i];
		}
	} else {
		ret=(char *)calloc((length-i)+2, sizeof(char));
		for (i=charpos+1; (unsigned)i<=length; i++) {
			ret[i-charpos-1]=string[i];
		}
	}
	return ret;
}

//static char valids[]={ "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ[]-_,.{}+=@#$%^&*()" };

char *validate_file_string (char *data) {
//	int count, vpos, cpos;
	int spos, len=strlen (data);
//	char *returnable=(char *)calloc (1, len);
	for (spos=0;spos<len;spos++) {
//		for (vpos=0;vpos<strlen (valids)
	}
	return NULL;
}

void puma_error ( char *fmt, ...) {
/*	GtkWidget *window;
	va_list ap;
	char outstr[1024];
	va_start (ap, fmt);
	vsnprintf (outstr, 1024, fmt, ap);
	va_end (ap);
	window=create_ewmd_dialog (ERROR_DIALOG, "Fatal Error Occured", outstr);
	gtk_widget_show (window);*/
}

void puma_warning (int level, char *fmt, ...) {
#ifdef DEBUG
	if ((level==0) || (level<=DEBUG)) {
#else
	if (level==0) {
#endif
		va_list ap;
		va_start (ap, fmt);
		insert_warning (fmt, ap);
	}
}

void puma_message (int level, char *fmt, ...) {
#ifdef DEBUG
	if ((level==0) || (level<=DEBUG)) {
#else
	if (level==0) {
#endif
		va_list ap;
		va_start (ap, fmt);
		insert_message (fmt, ap);
	}
}

void puma_debug (int level, char *fmt, ...) {
#ifdef DEBUG
	if ((level==0) || (level<=DEBUG)) {
#else
	if (level==0) {
#endif
		va_list ap;
		va_start (ap, fmt);
		insert_debug (fmt, ap);
	}
}

void startup (int argc, char **argv) {
	GtkWidget *clist;
	int i;
	gtk_init (&argc, &argv);
/*	add_pixmap_directory ("./pics");
	add_pixmap_directory ("/usr/local/share/puma/pics"); */
	tm_globs=(TradeClientGlobs *)calloc (1, sizeof(TradeClientGlobs));
	for (i=0;i<20;i++) tm_globs->new_messages[i]=NULL;
	tm_globs->status=create_status_window ();
	tm_globs->main_window=create_main_window ();	
	tm_globs->acct_selected=-1;
	tm_globs->num_accounts=0;
	tm_globs->newserver_type=NULL;
	tm_globs->visable=NULL;
	tm_globs->message_popup=NULL;
	tm_globs->folder_popup=create_folder_popup ();
	tm_globs->create_folder=create_create_folder ();
	tm_globs->msg_row=-1;
	clist=lookup_widget (tm_globs->main_window, "message_clist");

	tm_globs->read_odd=gtk_style_copy (clist->style);
	tm_globs->read_even=gtk_style_copy (clist->style);
	tm_globs->unread_odd=gtk_style_copy (clist->style);
	tm_globs->unread_even=gtk_style_copy (clist->style);
#if 1
	tm_globs->unread_odd->font =
	tm_globs->unread_even->font = 
		gdk_font_load ("-*-helvetica-bold-r-*--*-*-*-*-*-*-*-1");
	tm_globs->read_odd->font = 
	tm_globs->read_even->font = 
		gdk_font_load ("-*-helvetica-medium-r-*--*-*-*-*-*-*-*-1");
#endif
	tm_globs->read_even->base[GTK_STATE_NORMAL].red=(0xe0 << 8);
	tm_globs->read_even->base[GTK_STATE_NORMAL].green=(0xe0 << 8);
	tm_globs->read_even->base[GTK_STATE_NORMAL].blue=(0xe0 << 8);

	tm_globs->read_odd->base[GTK_STATE_NORMAL].red=(0xff << 8);
	tm_globs->read_odd->base[GTK_STATE_NORMAL].green=(0xff << 8);
	tm_globs->read_odd->base[GTK_STATE_NORMAL].blue=(0xff << 8);

	tm_globs->unread_even->base[GTK_STATE_NORMAL].red=(0xe0 << 8);
	tm_globs->unread_even->base[GTK_STATE_NORMAL].green=(0xe0 << 8);
	tm_globs->unread_even->base[GTK_STATE_NORMAL].blue=(0xe0 << 8);

	tm_globs->unread_odd->base[GTK_STATE_NORMAL].red=(0xff << 8);
	tm_globs->unread_odd->base[GTK_STATE_NORMAL].green=(0xff << 8);
	tm_globs->unread_odd->base[GTK_STATE_NORMAL].blue=(0xff << 8);

#if 1 /* trey */

	tm_globs->sens_u1 = gtk_style_copy (tm_globs->unread_odd);
	tm_globs->sens_r1 = gtk_style_copy (tm_globs->read_odd);
	tm_globs->sens_u1->base[GTK_STATE_NORMAL].red = tm_globs->sens_r1->base[GTK_STATE_NORMAL].red = (0xe0 << 8);
	tm_globs->sens_u1->base[GTK_STATE_NORMAL].green = tm_globs->sens_r1->base[GTK_STATE_NORMAL].green = (0xa0 << 8);
	tm_globs->sens_u1->base[GTK_STATE_NORMAL].blue = tm_globs->sens_r1->base[GTK_STATE_NORMAL].blue = (0xa0 << 8);

	tm_globs->sens_u2 = gtk_style_copy (tm_globs->unread_odd);
	tm_globs->sens_r2 = gtk_style_copy (tm_globs->read_odd);
        tm_globs->sens_u2->base[GTK_STATE_NORMAL].red = tm_globs->sens_r2->base[GTK_STATE_NORMAL].red = (0xa0 << 8);
        tm_globs->sens_u2->base[GTK_STATE_NORMAL].green = tm_globs->sens_r2->base[GTK_STATE_NORMAL].green = (0xa0 << 8);
        tm_globs->sens_u2->base[GTK_STATE_NORMAL].blue = tm_globs->sens_r2->base[GTK_STATE_NORMAL].blue = (0xe0 << 8);

        tm_globs->sens_u3 = gtk_style_copy (tm_globs->unread_odd);
        tm_globs->sens_r3 = gtk_style_copy (tm_globs->read_odd);
        tm_globs->sens_u3->base[GTK_STATE_NORMAL].red = tm_globs->sens_r3->base[GTK_STATE_NORMAL].red = ((0xa0-40) << 8);
        tm_globs->sens_u3->base[GTK_STATE_NORMAL].green = tm_globs->sens_r3->base[GTK_STATE_NORMAL].green = ((0xa0-40) << 8);
        tm_globs->sens_u3->base[GTK_STATE_NORMAL].blue = tm_globs->sens_r3->base[GTK_STATE_NORMAL].blue = (0xe0 << 8);

        tm_globs->sens_u4 = gtk_style_copy (tm_globs->unread_odd);
        tm_globs->sens_r4 = gtk_style_copy (tm_globs->read_odd);
        tm_globs->sens_u4->base[GTK_STATE_NORMAL].red = tm_globs->sens_r4->base[GTK_STATE_NORMAL].red = (0xe0 << 8);
        tm_globs->sens_u4->base[GTK_STATE_NORMAL].green = tm_globs->sens_r4->base[GTK_STATE_NORMAL].green = ((0xa0-40) << 8);
        tm_globs->sens_u4->base[GTK_STATE_NORMAL].blue = tm_globs->sens_r4->base[GTK_STATE_NORMAL].blue = ((0xa0-40) << 8);

        tm_globs->sens_u5 = gtk_style_copy (tm_globs->unread_odd);
        tm_globs->sens_r5 = gtk_style_copy (tm_globs->read_odd);
        tm_globs->sens_u5->base[GTK_STATE_NORMAL].red = tm_globs->sens_r5->base[GTK_STATE_NORMAL].red = (0xe0 << 8);
        tm_globs->sens_u5->base[GTK_STATE_NORMAL].green = tm_globs->sens_r5->base[GTK_STATE_NORMAL].green = (0xa0 << 8);
        tm_globs->sens_u5->base[GTK_STATE_NORMAL].blue = tm_globs->sens_r5->base[GTK_STATE_NORMAL].blue = (0xa0 << 8);

#endif


	tm_globs->pid=-1;
	tm_globs->rlist=NULL;
	tm_globs->rsel=NULL;
	tm_globs->vsel=NULL;
	pipe(tm_globs->dpipe);
	pipe(tm_globs->pspipe);
	pipe(tm_globs->cspipe);
	if (fcntl (tm_globs->dpipe[0], F_SETFL, O_NONBLOCK)==-1) {
#ifdef DEBUG
		insert_error ("error while setting non-blocking on pipe\n");
#endif
	}
	if (fcntl (tm_globs->pspipe[0], F_SETFL, O_NONBLOCK)==-1) {
#ifdef DEBUG
		insert_error ("error while setting non-blocking on pipe\n");
#endif
	}
	if (fcntl (tm_globs->cspipe[0], F_SETFL, O_NONBLOCK)==-1) {
#ifdef DEBUG
		insert_error ("error while setting non-blocking on pipe\n");
#endif
	}
	tm_globs->debug.red=0; tm_globs->debug.blue=255; tm_globs->debug.green=0;
	tm_globs->warning.red=255; tm_globs->warning.blue=0; tm_globs->warning.green=255;
	tm_globs->error.red=255; tm_globs->error.blue=0; tm_globs->error.green=0;
	tm_globs->message.red=0; tm_globs->message.blue=0; tm_globs->message.green=255;
	
	gtk_timeout_add (60000, (GtkFunction)message_anti_leak, NULL);

	calendar_load ();
}

int puma_sigsegv (int blah) {
	return -1;
}

int pop_count_accounts () {
	PopAccount *account=first_popaccount ();
	int count=0;
	for (;account;account=account->next) {
		count++;
	}
	return count;
}

void grey_out_stuff () {
	GtkWidget *get_message=lookup_widget (tm_globs->main_window, "get_message");
	GtkWidget *new_message=lookup_widget (tm_globs->main_window, "new_message");
	int pc=pop_count_accounts ();
	if (pc==0) {
		gtk_widget_set_sensitive (get_message, FALSE);
		gtk_widget_set_sensitive (new_message, FALSE);
	} else {
		gtk_widget_set_sensitive (get_message, TRUE);
		gtk_widget_set_sensitive (new_message, TRUE);
	}
}

void self_check () {
	{ // storage verification
		PopAccount *seek=first_popaccount ();
		while (seek) {
			if (folder_seek_by_name (seek->folder)==-1) {
				free (seek->folder);
				seek->folder=strdup (_("Inbox"));
			}
			seek=seek->next;
		} 
		seek=first_popaccount ();
		if (seek) {
			if (tm_globs->default_account) {
				if (seek_popaccount_by_name (tm_globs->default_account)==-1) {
					insert_warning (_("Default account self check failed.  Resetting to first account\n"));
					seek=seek_popaccount (0);
					tm_globs->default_account=strdup (seek->name);
				}
			} else {
				insert_warning (_("Default account self check failed.  Resetting to first account\n"));
				tm_globs->default_account=strdup (seek->name);
			} 
		}
	}
	{ // Filter rule verification 
		GList *seek, *d=NULL;
		for (seek=filter_first ();seek;seek=seek->next) {
			FilterRule *rule=seek->data;
			if (rule->folder) {
				if (folder_seek_by_name (rule->folder)==-1) {
					d=g_list_append (d, rule);
				}
			}
		}
		for (seek=d;seek;seek=seek->next) {
			filter_delete_rule (seek->data);
		}
		if (d) g_list_free (d);
	}
	{// Folder storeable verification
		Folder *fl;
		for (fl=folder_first ();fl;fl=fl->next) {
			if (strcasecmp (fl->name, "outbox")     != 0 &&
			    strcasecmp (fl->name, "sent-items") != 0 &&
					strcasecmp (fl->name, "trash")      != 0 &&
					(fl->flags & CAN_STORE_NEW_MAIL)    == 0) {
				fl->flags |= CAN_STORE_NEW_MAIL;
			}
		}
	}
	{// Event verification
		Llist *seek;
		GList *destroy=NULL, *gseek;
		for (seek=event_first();seek;seek=seek->next) {
			CalendarEvent *event=seek->data;
			if (!event) {
				insert_error (_("Calendar Data is apparently corrupt, attempting to continue\n"));
			} else {
				if (!event->start) {
					insert_error (_("Event '%s' is corrupt, deleting\n"), event->subject);
					destroy=g_list_append (destroy, event);
					goto done;
				}
				if (!event->stop) {
					insert_error (_("Event '%s' is corrupt, deleting\n"), event->subject);
					destroy=g_list_append (destroy, event);
					goto done;
				}
				if (event->start->time == event->stop->time) {
					insert_error (_("Event '%s' is corrupt, deleting\n"), event->subject);
					destroy=g_list_append (destroy, event);
				}
done:
			}
		}
		for (gseek=destroy;gseek;gseek=gseek->next) {
			event_destroy (gseek->data);
		}
	}
}



int main (int argc, char **argv) {
	Folder *fl;
	PopAccount *acct;
#ifdef LANG_DEBUG
	lang_load_external ("lang.db");
	lang_startup ();
#else
	lang_generate_list ();
#endif

	abook_set_question_func ((QuestionFunc)create_yesno);

	startup (argc, argv);
	make_default_prefrances ();
	window_sizes_rehash();
	paned_pos_rehash();
	clist_pos_rehash();

	folders_read ();
	folders_read_index ();
	
	fl=folder_first ();
	
	while (fl) {create_new_folder (fl);	fl=fl->next;}

	if (load_preferences ()==-1) {
		insert_warning (_("Unable to load Preferences.  New preferences file has been created.\n"));
		grey_out_stuff ();
	}

        /* go through list of mail accounts and pick out the imap ones */
        acct = first_popaccount ();
        while (acct) {
		if (acct->type == IMAP) {
			nf_add_imap_acct (acct);
		}
		acct = acct->next;
        }

	mime_type_add_defaults ();
	gtk_widget_show (tm_globs->main_window);
	if (!tm_globs->printer) tm_globs->printer = strdup("lpr");

	abooks_read ();
	abooks_load_all ();

	abooks_save_all ();

	mime_types_rehash_clist ();
	self_check ();

	/* I'm pretty sure that the wasted space values don't need to be
	 * updated every second.  Once per 10 seconds should be fine!
	 */
	prefs_show(0);
	tm_globs->wsflag = 1;
	prefs_setup_info() ;
	gtk_timeout_add( 10000, (GtkFunction)prefs_setup_info, NULL ) ;

	/* This actually does more that I thought.  We need it to check
	 * very often since this is where completion events are checked.  In
	 * other words, when we are finished receiving messages, the GUI isn't
	 * notified until this timer fires and has a chance to be updated.  As
	 * such, 60-seconds is a VERY bad idea.  Now, we check every ~1/2 second.
	 */
	tm_globs -> getMsgTimeout = gtk_timeout_add( 950, (GtkFunction)message_event_loop, NULL ) ;

	/* Add a timer to maintain our event notifications */
	calendar_event_loop() ;
	tm_globs -> eventLoopTimeout = gtk_timeout_add( 60*60*1000, (GtkFunction)calendar_event_loop, NULL ) ;

	/* Now, set up global configuration for c-client code */
#include "linkage.c"	
#ifdef NEWMAILBACKEND
	mail_parameters( NULL, SET_RFC822OUTPUT, (void *)post_rfc822_output) ;
#endif

#ifdef WANTLDAP
	/* This builds and populates the GAB initially */
	gab_update_wrapper( NULL ) ;
#endif

	/* clear the undo list to get ready */
	clear_undo_list ();

	/* Go into the GTK main event loop */
	gtk_main() ;

#ifdef LANG_DEBUG
	lang_save_internal ("include/lang-database.h");
	lang_save_external ("lang.db");
#endif
	lang_shutdown ();
	return 0;
}


#ifdef WANTLDAP
int
gab_update_search (gpointer data, char *target)
{
  int retval = 0;
  char *base = NULL;
  char *filter = NULL;
  char *name = NULL;
  char *pass = NULL;

  if (tm_globs->ldap_port == 0)
    tm_globs->ldap_port = 389;
#ifdef WANTSSL
  if (tm_globs->ldap_ssl)
    tm_globs->ldap_ssl = 0;
#endif

  if (tm_globs->ldap_base)
    base = strdup (tm_globs->ldap_base);
  else
    base = strdup ("");

  if (target)
      filter = strdup (target);

  if (tm_globs->ldap_username)
    {
       name = malloc ((strlen (tm_globs->ldap_username) + 4) * sizeof (char));
       sprintf (name, "cn=%s", tm_globs->ldap_username);
    }
  else
    name = strdup ("");

  if (tm_globs->ldap_password)
    pass = strdup (tm_globs->ldap_password);
  else
    pass = strdup ("");

  if (tm_globs->ldap_host)
    update_gab (tm_globs->ldap_host, tm_globs->ldap_port, base, name, pass, filter);
  else
    retval = 1;

  if (base) free (base);
  if (name) free (name);
  if (pass) free (pass);

  return retval;
}


void 
gab_update_wrapper (gpointer data) {
  char *tmp ;
  int baseal ;
  char *base ;
  char *ptmp ;


  /* Make sure we have a default port correctly configured. */
  if( tm_globs -> ldap_port == 0 ) tm_globs -> ldap_port = 389 ;
#ifdef WANTSSL
  if (!tm_globs->ldap_ssl) tm_globs->ldap_ssl=0;
#endif

  /* Make sure we always have a base dn */
  if( !tm_globs -> ldap_base) {
    base = strdup( "" ) ;
    baseal = 1 ;
  } else {
    base = tm_globs -> ldap_base ;
    baseal = 0 ;
  }

  /* Now, make sure we have a username to auth against */
  if( tm_globs -> ldap_username ) {
    /* We allloc +4 for the "cn=" (3) plus a null. */
    tmp = malloc( (strlen(tm_globs -> ldap_username)+4)*sizeof(char) ) ;
    sprintf( tmp, "cn=%s", tm_globs -> ldap_username ) ;
  } else {
    tmp = strdup( "" ) ;
  }

  /* See if we are configed to log on */
  if( (tm_globs -> ldap_host) && (tm_globs -> ldap_username) &&
      (tm_globs -> ldap_password) ) {
    update_gab( tm_globs -> ldap_host, tm_globs -> ldap_port,
		base, tmp, tm_globs -> ldap_password, NULL ) ;
  } else {
    /* This is for anonymous access */
    if( tm_globs -> ldap_host ) {
      ptmp = strdup( "" ) ;
      update_gab( tm_globs -> ldap_host, tm_globs -> ldap_port,
		  base, tmp, ptmp, NULL ) ;
      free( ptmp ) ;
    }
  }

  if( baseal ) free( base ) ;
  free( tmp ) ;
  return ;
 }
#endif
