/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: message-fun.c,v 1.27 2001/03/21 00:45:22 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "puma.h"

/*
   A whole bunch of subrutines for processing rfc822 messages, some need
   cleaning up.
*/

char *getnline (CNSS *ss) {
	char *ret;
	int i=0;
	while (i<ss->len) {
		if (ss->str[i+ss->pos]=='\n') {
			ret=(char *)calloc(i+2, sizeof(char));
			memmove (ret, ss->str+ss->pos, i);
			ss->pos+=i+1;
			return ret;
		}
		i++;
	}
	return NULL;
}

char *getnl_rfc822 (CNSS *ss) {
	char *ret;
	int i=0;
	while ((i+ss->pos)<ss->len) {
		if ((ss->str[i+ss->pos]=='\n')&&(isspace(ss->str[i+ss->pos+1])==0)) {
			ret=(char *)calloc(i+2, sizeof(char));
			memmove (ret, ss->str+ss->pos, i+1);
			ss->pos+=i+1;
			return ret;
		}
		i++;
	}
	return NULL;
}

void putnline (CNSS *ss, char *string) {
	int i=strlen (string);
	if (i+ss->pos<ss->len) {
		memmove (ss->str+ss->pos, string, i);
		ss->pos+=i;
	} else {
		i=(ss->pos+i)+ss->len;
		if (i>0) {
			memmove (ss->str+ss->pos, string, i);
			ss->pos+=i;
		}
	}
}

void destroy_hinfo (Header_Info *hinfo) {

  if (hinfo) {
    if (hinfo->from) free (hinfo->from);
    if (hinfo->to) free (hinfo->to);
    if (hinfo->cc) free (hinfo->cc);
    if (hinfo->subject) free (hinfo->subject);
    if (hinfo->date) free (hinfo->date);
    if (hinfo->content_type) free (hinfo->content_type);
    if (hinfo->content_t_e) free (hinfo->content_t_e);
    if (hinfo->content_d) free (hinfo->content_d);
    if (hinfo->disposition_notify) free (hinfo->disposition_notify);
    if (hinfo->uidl) free (hinfo->uidl);
    if (hinfo->mid) free (hinfo->mid);
    if (hinfo->priority) free (hinfo->priority);
    if (hinfo->replyto) free (hinfo->replyto);
    if (hinfo->sensitivity) free (hinfo->sensitivity);
    if (hinfo->replyby) free (hinfo->replyby);
    if (hinfo->expiry) free (hinfo->expiry);
    if (hinfo->keywords) free (hinfo->keywords);
    free (hinfo);
    hinfo = NULL ;
  }
  return ;
}

char *header_displayable (char *head) {
	int len=0;
	CNSS *ss=(CNSS *)calloc(1, sizeof(CNSS));
	char *ret=(char *)calloc (strlen (head)+2, sizeof(char));
	char *tmps;

	ss->str=head;
	ss->len=strlen (head);
	ss->pos=0;

	if ((tm_globs->flags&SHOW_ALL_HEADER)==SHOW_ALL_HEADER) {
		free (ss);
		free (ret);
		return strdup(head);
	}

	if ((tm_globs->flags&SHOW_SOME_HEADER)==SHOW_SOME_HEADER) {
		while ((tmps=getnl_rfc822(ss))!=NULL) {

			if (strncasecmp(tmps, "from: ", 6)==0) {
				memmove (ret+len, tmps, strlen (tmps));
				len+=strlen (tmps);
			}
			if (strncasecmp(tmps, "to: ", 4)==0) {
				memmove (ret+len, tmps, strlen (tmps));
				len+=strlen (tmps);
			}
			if (strncasecmp(tmps, "cc: ", 4)==0) {
				memmove (ret+len, tmps, strlen (tmps));
				len+=strlen (tmps);
			}
			if (strncasecmp(tmps, "subject: ", 9)==0) {
				memmove (ret+len, tmps, strlen (tmps));
				len+=strlen (tmps);
			}
			if (strncasecmp(tmps, "date: ", 6)==0) {
				memmove (ret+len, tmps, strlen (tmps));
				len+=strlen (tmps);
			}

			if (tmps) { free (tmps); tmps=NULL; }
		}
		memmove (ret+len, "\n", 1);
		len++;
	} else {
		free (ss);
		free (ret);
		return (char *)calloc(2, sizeof(char));
	}

	free (ss);
	return ret;
}

Header_Info *parse_header (char *head) {
	Header_Info *headinf=(Header_Info *)calloc(1, sizeof(Header_Info));
	CNSS *ss=(CNSS *)calloc(1, sizeof(CNSS));
	char *tmps;
	char *rm;
	char *tmpp;
	int i;

	ss->str=head;
	ss->len=strlen(head);
	ss->pos=0;

#ifdef DMALLOC
	dmalloc_verify( 0 ) ;
#endif
	while ((tmps=getnl_rfc822(ss))!=NULL) {

		if (strncasecmp(tmps, "from: ", 6)==0) {
			rm = rm_crlf(tmps);
			headinf->from=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
		if (strncasecmp(tmps, "to: ", 4)==0) {
			rm = rm_crlf(tmps);
			headinf->to=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
		if (strncasecmp(tmps, "cc: ", 4)==0) {
			rm = rm_crlf(tmps);
			headinf->cc=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
		if (strncasecmp(tmps, "bcc: ", 4)==0) {
			rm = rm_crlf(tmps);
			headinf->bcc=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
		if (strncasecmp(tmps, "subject: ", 9)==0) {
			rm = rm_crlf(tmps);
			headinf->subject=cut_string (rm, ' ', 2, 1);
			free (rm);
			for(i=0;i<strlen(headinf->subject);i++)
				if(!isprint(*(headinf->subject+i)))
					*(headinf->subject+i)='\0';
					
		}
		if (strncasecmp(tmps, "date: ", 6)==0) {
			rm = rm_crlf(tmps);
			headinf->date=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
		if (strncasecmp(tmps, "content-type: ", 14)==0) {
			rm = rm_crlf(tmps);
			headinf->content_type=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
		if (strncasecmp(tmps, "content-transfer-encoding: ", 27)==0) {
			rm = rm_crlf(tmps);
			headinf->content_t_e=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
		if (strncasecmp(tmps, "content-disposition: ", 21)==0) {
			rm = rm_crlf(tmps);
			headinf->content_d=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
		if (strncasecmp(tmps, "disposition-notification-to: ", 29)==0) {
			rm = rm_crlf(tmps);
			headinf->disposition_notify=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
		if (strncasecmp(tmps, "x-uidl: ", 8)==0) {
			rm = rm_crlf(tmps);
			headinf->uidl=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
		if (strncasecmp(tmps, "keywords:", 9)==0) {
			rm = rm_crlf(tmps);
			headinf->keywords=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
                if (strncasecmp(tmps, "reply-to:", 9)==0) {
                        rm = rm_crlf(tmps);
                        headinf->replyto=cut_string (rm, ' ', 2, 1);
                        free (rm);
                }
                if (strncasecmp(tmps, "sensitivity:", 12)==0) {
                        rm = rm_crlf(tmps);
                        headinf->sensitivity=cut_string (rm, ' ', 2, 1);
                        free (rm);
                }
                if (strncasecmp(tmps, "reply-by:", 9)==0) {
                        rm = rm_crlf(tmps);
                        headinf->replyby=cut_string (rm, ' ', 2, 1);
                        free (rm);
                }
                if (strncasecmp(tmps, "expiry-date:", 12)==0) {
                        rm = rm_crlf(tmps);
                        headinf->expiry=cut_string (rm, ' ', 2, 1);
                        free (rm);
                }
                if (strncasecmp(tmps, "x-priority: ", 12)==0) {
			rm = rm_crlf(tmps);
			tmpp=cut_string (rm, ' ', 2, 1);
			free (rm);
			switch (*(tmpp+0)) {
				case '1':
					headinf->priority=strdup("1");
					break;
				case '2':
					headinf->priority=strdup("2");
					break;
				case '3':
					headinf->priority=strdup("3");
					break;
				case '4':
					headinf->priority=strdup("4");
					break;
				case '5':
					headinf->priority=strdup("5");
					break;
				default:
					headinf->priority=strdup("3");
			}
			if (tmpp) { free (tmpp); tmpp=NULL; }
                }

		if (strncasecmp(tmps, "message-id: ", 12)==0) {
			rm = rm_crlf(tmps);
			headinf->mid=cut_string (rm, ' ', 2, 1);
			free (rm);
		}
		if (tmps) { free (tmps); tmps=NULL; }

	}
	if (headinf->priority==NULL) headinf->priority=strdup("3");

	if (headinf->content_type==NULL || headinf->content_type[0]=='\0') {
		headinf->content_type=strdup ("text/plain");
	}

	free (ss);
#ifdef DMALLOC
	dmalloc_verify( 0 ) ;
#endif
	return headinf;
}

char *message_get_reply_quoted (Message *msg) {
	int headinfo ;
	char *newMsg ;
	int newMsgLen ;
	Attachment *att ;
	Header_Info *info ;
	char *email, *first, *last ;
	char *body, *body_processed, *body_ret ;
	const char qvar[] = "\n\nOn %s, %s wrote:\n%s" ;


#if DEBUG > 5
	printf( "in mesage_get_reply_quoted()\n" ) ;
#endif
	headinfo = 0 ;
	body=message_load (folder_seek (folder_seek_by_name
	                               (msg->rfolder)), msg);

	if (tm_globs->flags&SHOW_SOME_HEADER) {
		headinfo=SHOW_SOME_HEADER;
	} else if (tm_globs->flags&SHOW_ALL_HEADER) {
		headinfo=SHOW_ALL_HEADER;
	}

	tm_globs->flags&=~SHOW_SOME_HEADER;
	tm_globs->flags&=~SHOW_ALL_HEADER;

	att=(Attachment *)calloc (1, sizeof (Attachment));
	att->name=strdup ("dummyatt");

	/* Before we MIME process it, pull out some header values */
	info = parse_header( body ) ;
	body_processed=mime_process (body, att, 0);
	free( body ) ;

	/* Now, parse the fromm field to get it's parts */
	parse_from_line( info -> from, &email, &first, &last ) ;

	/* Insert our he said, she said string. */
	body_ret = insert_reply_indicator( body_processed ) ;
	free( body_processed ) ;

	/* Now, place in a fancy retort for the quoted reply */

	if (!body_ret) body_ret = strdup("");
	if (!first) first = strdup("");
	if (!last) last = strdup("");
	if (!email) email = strdup("");
	if (!info->subject) info->subject = strdup("");

        newMsgLen = strlen(body_ret) + strlen(qvar)
                + strlen (info->date) + strlen (info->from) + 1;

	newMsg = malloc( newMsgLen ) ;
        snprintf( newMsg, newMsgLen-1, qvar, info->date, info->from, body_ret ) ;
	free( body_ret ) ;
	body_ret = newMsg ;
	if( first ) free( first ) ;
	if( last ) free( last ) ;
	if( email ) free( email ) ;
	destroy_hinfo( info ) ;

	tm_globs->flags|=headinfo;

	return body_ret;
}

char *message_get_reply_quoted_stream (char *body) {
	int headinfo ;
	char *newMsg ;
	int newMsgLen ;
	Attachment *att ;
	Header_Info *info ;
	char *email, *first, *last ;
	char *body_processed, *body_ret ;
	const char qvar[] = "\n\nOn %s, %s wrote:\n%s" ;


	headinfo = 0 ;
#if DEBUG > 5
	printf( "in mesage_get_reply_quoted_stream()\n" ) ;
#endif
	if (tm_globs->flags&SHOW_SOME_HEADER) {
		headinfo=SHOW_SOME_HEADER;
	} else if (tm_globs->flags&SHOW_ALL_HEADER) {
		headinfo=SHOW_ALL_HEADER;
	}

	tm_globs->flags&=~SHOW_SOME_HEADER;
	tm_globs->flags&=~SHOW_ALL_HEADER;

	att=(Attachment *)calloc (1, sizeof (Attachment));
	att->name=strdup ("dummyatt");

	/* Before we MIME process it, pull out some header values */
	info = parse_header( body ) ;
	body_processed=mime_process (body, att, 0);

	/* Insert our he said, she said string. */
	body_ret = insert_reply_indicator( body_processed ) ;
	free( body_processed ) ;

	/* Now, parse the fromm field to get it's parts */
	parse_from_line( info -> from, &email, &first, &last ) ;

	/* Now, place in a fancy retort for the quoted reply */

        if (!body_ret) body_ret = strdup("");
        if (!first) first = strdup("");
        if (!last) last = strdup("");
        if (!email) email = strdup("");
        if (!info->subject) info->subject = strdup("");

	newMsgLen = strlen(body_ret) + strlen(qvar)
		+ strlen (info->date) + strlen (info->from) + 1;

	newMsg = malloc( newMsgLen ) ;
	snprintf( newMsg, newMsgLen-1, qvar, info->date, info->from, body_ret ) ;
	free( body_ret ) ;
	body_ret = newMsg ;
	if( first ) free( first ) ;
	if( last ) free( last ) ;
	if( email ) free( email ) ;
	destroy_hinfo( info ) ;

	tm_globs->flags|=headinfo;

	return body_ret;
}


void message_transpose_folders (Folder *fl) {
	GList *seek=message_get_selection ();
	GtkWidget *clist=lookup_widget (tm_globs->main_window, "message_clist");
	GtkCTree *folder_ctree=GTK_CTREE (lookup_widget (tm_globs->main_window, "folder_tree"));
	Folder *rfolder;
	msg_info *info;
	Message *msg;
	char *number=(char *)calloc (20, sizeof (char));
	int row;
	if (!seek)
		return;
	if (fl) {
		gtk_clist_freeze (GTK_CLIST (clist));
		while (seek) {
			info=seek->data;
			if (info) {
				msg=info->msg;
				if (msg->folder) {				
					rfolder=folder_seek (folder_seek_by_name (msg->folder));
				} else {
					rfolder=folder_seek (folder_seek_by_name (msg->rfolder));
				}
				rfolder->total--;
				fl->total++;
				if (msg->flags & MESSAGE_UNREAD) {
					rfolder->unread--;
					fl->unread++;
				}
				snprintf (number, 5, "%d", fl->unread);
				gtk_ctree_node_set_text (folder_ctree, fl->tree_item, 1, number);
				snprintf (number, 5, "%d", fl->total);
				gtk_ctree_node_set_text (folder_ctree, fl->tree_item, 2, number);

				snprintf (number, 5, "%d", rfolder->unread);
				gtk_ctree_node_set_text (folder_ctree, rfolder->tree_item, 1, number);
				snprintf (number, 5, "%d", rfolder->total);
				gtk_ctree_node_set_text (folder_ctree, rfolder->tree_item, 2, number);

				if (msg->folder) free (msg->folder);
				msg->folder=strdup (fl->name);
			}
			seek=seek->next;
		}
		seek=message_get_selection ();
		while (seek) {
			info=seek->data;
			row=gtk_clist_find_row_from_data (GTK_CLIST (clist), seek->data);
			seek=seek->next;
			gtk_clist_remove (GTK_CLIST (clist), row);
		}
		gtk_clist_thaw (GTK_CLIST (clist));
		message_clist_reset_shading (clist);
	}
	message_selection_null ();
	free (number);
}

void hinfo_destroy (Header_Info *hinfo) {
	if (hinfo) {
	  if (hinfo->from) free (hinfo->from);
	  if (hinfo->to) free (hinfo->to);
	  if (hinfo->cc) free (hinfo->cc);
	  if (hinfo->subject) free (hinfo->subject);
	  if (hinfo->date) free (hinfo->date);
	  if (hinfo->priority) free (hinfo->priority);
	  if (hinfo->replyto) free (hinfo->replyto);
	  if (hinfo->sensitivity) free (hinfo->sensitivity);
	  if (hinfo->expiry) free (hinfo->expiry);
	  if (hinfo->replyby) free (hinfo->replyby);
	  if (hinfo->keywords) free (hinfo->keywords);
	  if (hinfo->content_type) free (hinfo->content_type);
	  if (hinfo->content_t_e) free (hinfo->content_t_e);
	  if (hinfo->content_d) free (hinfo->content_d);
		if (hinfo->uidl) free (hinfo->uidl);
		if (hinfo->mid) free (hinfo->mid);
		free (hinfo);
	}
}

char *message_fix_for_send (char *string) {
	char *ret=(char *)calloc (strlen (string)*2, sizeof(char));
	char *tmps1=NULL;	char *tmps2=NULL; char *tmps3=NULL;
	CNSS *ss=(CNSS *)calloc(1, sizeof(CNSS));
	CNSS *ss2=(CNSS *)calloc(1, sizeof(CNSS));

	tmps1=add_cr(string);

	ss->str=tmps1;
	ss->len=strlen (tmps1);
	ss->pos=0;

	ss2->str=ret;
	ss2->len=strlen (string)*2;
	ss2->pos=0;

	while ((tmps2=getnl_rfc822(ss))!=NULL) {
		if (tmps2[0]=='.') {
			tmps3=(char *)calloc (strlen (tmps2)+4, sizeof(char));
			tmps3[0]='.';
			memmove (tmps3+1, tmps2, strlen (tmps2));
			free (tmps2);
			tmps2=tmps3;
			tmps3=NULL;
		}
		putnline (ss2, tmps2);
		free (tmps2);
	}

	ss->str=NULL;
	free (ss);
	ss2->str=NULL;
	free (ss2);
	free (tmps1);

	return ret;
}


/*
 * Walk through all messages looking for a matching
 * UIDL and/or msgID.  Remember, msgID doesn't have
 * to be unique.
 * msgID can optionally be NULL as input.
 * Return Zero (0) on no match, 1 if uidl matches, and
 * two if uidl and msgID match.
 */
int
index_find_uidl( const char *uidl, const char *msgID )
{
  int retValue;
  Message *mseek;
  Folder *folder;
  GList *fseek;
  char *msg;
  Header_Info *hinfo;

  retValue = 0;
  /* Make sure we have a UIDL to work with */
  if(uidl)
  {
    /* Since we have a UIDL that we are looking for, walk through all indexes */
    for (fseek = folder_create_glist (); fseek; fseek = fseek->next)
    {
      folder = folder_seek (folder_seek_by_name (fseek->data));
      for (mseek = folder->msglst; mseek; mseek = mseek->next)
      {
        if (mseek->uidl)
        {
          if (strncmp(mseek->uidl,uidl,strlen(uidl)-4)==0)
          {
            retValue = 1;
            if (msgID)
            {
              msg = message_load (folder_seek (folder_seek_by_name
                                               (mseek->rfolder)), mseek);
              hinfo = parse_header (msg);
              if (strcmp(hinfo->mid,msgID)==0)
                retValue = 2;
              free (msg);
              destroy_hinfo (hinfo);
            }
          }
        }
        if (retValue)
          break;
      }
      if (retValue)
        break;
    }
  }
  return retValue;
}

char
*find_message_uidl(char *msg)
{
  char *tmp = NULL;

  if (msg)
  {
    tmp = strstr (msg, "X-UIDL:");
    if (tmp) {
      tmp += 7;
      while (*(++tmp) == ' ');
    }
  }
  return tmp;
}
