/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: new-message-callbacks.c,v 1.62 2001/03/21 00:45:22 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "puma.h"

static char *fetch_string( GtkWidget *parent, const char *string ) ;
static char *get_priority_string (GtkWidget *parent) ;
static char *get_keyword_string( GtkWidget *parent ) ;
static char *get_sensitivity_string( GtkWidget *parent ) ;
static char *get_expire_string( GtkWidget *parent ) ;
static char *get_replyby_string( GtkWidget *parent ) ;
static char *get_messageflag_string( GtkWidget *parent ) ;


/* *************************************
This is the function you will need to manipulate.  All of the data you need
for generating the header without attachments is already there.  I'll show 
you how to grab the information for the attachments
************************************* */

static char *fetch_string( GtkWidget *parent, const char *string )
{
  char *tmp ;


  tmp = gtk_editable_get_chars (GTK_EDITABLE (lookup_widget(parent, string)), 0, -1) ;
  if( !tmp || (tmp && !strlen( tmp )) ) return NULL ;
  else return tmp ;
}


static char *get_priority_string (GtkWidget *parent)
{
  return fetch_string( parent, "priority_to_use" ) ;
}


static char *get_sensitivity_string( GtkWidget *parent )
{
  return fetch_string( parent, "sensitivity_to_use" ) ;
}


static char *get_keyword_string( GtkWidget *parent )
{
  return fetch_string( parent, "entry9" ) ;
}


static char *get_expire_string( GtkWidget *parent )
{
  return fetch_string( parent, "msg_exp_entry" ) ;
}


static char *get_replyby_string( GtkWidget *parent )
{
  return fetch_string( parent, "replyby_entry" ) ;
}


static char *get_messageflag_string( GtkWidget *parent )
{
#if 0
  return fetch_string( parent, "NA" ) ;
#else
  return NULL ;
#endif
}


char *nemail_fix(char *nickname) {
        Abook_entry *entry=abook_entry_seek_by_nickname(NULL,nickname);
	if (entry) {
        	if (entry->def_email) return (entry->def_email);
	}
        return nickname;
}


int is_email (char *addr) {
	int found=0;
	int i;


	if( addr == NULL ) return FALSE ;

	for (i=0;i< strlen (addr);i++) {
		if (addr[i]=='@')
			found++;
	}
	if (found==1)
		return TRUE;
	return FALSE;
}

int is_emailv( char *addr )
{
	int errors = 0 ;
	int i ;


	if( addr == NULL ) return FALSE ;

	for( i = 0 ; i < strlen( addr ) ; i++ )
	{
		if( !isprint( addr[i] ) )
			errors++ ;
	}

	if( errors )
		return FALSE;
	return TRUE;
}

int validate_email (char *addr) {
	char *tmp1=NULL, *tmp2=NULL, *tmp3=NULL;
	int len, i;
	if (tmp2==NULL) goto failed;
	len=tmp2-tmp1;
	tmp3=(char *)calloc (len+1, sizeof(char));
	memmove (tmp3, tmp1+1, len-1);
	len=strlen(tmp3);
	if (strstr (tmp3, "@")==NULL) goto failed;
	for (i=0; i<len; i++) {
		if ( (!isdigit(tmp3[i])) ||
		     (!islower(tmp3[i])) ||
		     (!isupper(tmp3[i])) ||
		     (tmp3[i]!='@') ||
		     (tmp3[i]!='.') ||
		     (tmp3[i]!='_') ||
		     (tmp3[i]!='-') ||
		     (tmp3[i]!='+') ) goto failed;
	}
	return TRUE;
failed:
	if (tmp3) {
		insert_warning ("'%s' is not a valid email address!\n", tmp3);
		free (tmp3);
	} else if (addr) {
		insert_warning ("'%s' is not a valid email address!\n", addr);
	} else {
		insert_warning ("A NULL string was sent to validate_email!\n");
	}
	return FALSE;	
}

GList *new_message_get_recipients (GtkTree *tree) {
	Recipient *recp;
	GList *seek;
	GList *ret=NULL;
	g_return_val_if_fail (tree != NULL, NULL);
	
	for (seek=tree->children;seek;seek=seek->next) {
		GtkWidget *item=seek->data;
		GtkWidget *hbox=GTK_BIN (item)->child;
		GtkBoxChild *typew=GTK_BOX (hbox)->children->next->data;
		GtkBoxChild *add=GTK_BOX (hbox)->children->next->next->data;
		char *type=gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (typew->widget)->entry));
		char *address=gtk_entry_get_text (GTK_ENTRY (add->widget));
		if (strlen (address)>0) {
			recp=(Recipient *)calloc (1, sizeof (Recipient));
			if (is_email (address)) {
				recp->email=strdup (address);
			} else {
				Abook_entry *entry=abook_entry_seek_by_name (NULL, address);
				if (entry) {
					if (entry->def_email)	recp->email=strdup (entry->def_email);
				} else {
					recp->email=strdup (address);
				}
			}
			if (strcasecmp (type, "To:")==0)
				recp->type=RECIPIENT_TO;
			if (strcasecmp (type, "Cc:")==0) 
				recp->type=RECIPIENT_CC;
			if (strcasecmp (type, "Bcc:")==0)
				recp->type=RECIPIENT_BCC;
			ret=g_list_append (ret, recp);
		}
	}
	return ret;
}

void new_message_destroy_reciplist (GList *recip) {
	GList *seek;
	Recipient *recp;
	for (seek=recip;seek;seek=seek->next) {
		recp=seek->data;
		if (recp->email) free (recp->email);
		free (recp);
	}
	g_list_free (recip);
}

void new_message_unprotect (GtkWidget *widget) {
	gtk_widget_set_sensitive (lw (widget, "new_message_window"), TRUE);
	gtk_widget_set_sensitive (lookup_widget (tm_globs->main_window, "get_message"), TRUE);
	gtk_widget_set_sensitive (lookup_widget (tm_globs->status, "stopthat"), FALSE);
	gtk_widget_set_sensitive (lookup_widget (tm_globs->status, "skip"), FALSE);
}



/*
 * Used for text saves, printing, and folding filing of locally generated messages.
 */
char *new_message_generate_full (GtkWidget *parent, char **to, char **cc, char **bcc) {
        char *to_line=NULL, *cc_line=NULL, *bcc_line=NULL;
        char *subject_line=gtk_entry_get_text (GTK_ENTRY(lookup_widget (parent, "subject_entry")));
        char *body=gtk_editable_get_chars (GTK_EDITABLE(lookup_widget (parent, "body_textbox")), 0, -1);
        GtkCList *clist=GTK_CLIST (lookup_widget (parent, "attachment_clist"));
        GtkTree *tree=GTK_TREE (lw (parent, "recip_tree"));
        int popid=seek_popaccount_by_name (gtk_entry_get_text (GTK_ENTRY (lookup_widget (parent, "account_to_use"))));
        PopAccount *pop=seek_popaccount (popid);
        char *timestamp=generate_timestamp ();
        char *full_message=NULL, *tmp_full_message=NULL, *sender=NULL, *org=NULL, *replyto=NULL;
        char *tmp, *tmp2;
        Llist *alist=NULL;
        AFS *afs;
        int i;
        guint tol=1, ccl=1, bccl=1;
        GList *s;

	tmp = word_wrap_body( body, tm_globs->wwrap ) ;
	if( !tmp ) {
	  tmp = strdup( body ) ;
	}

	g_free( body ) ;
	body = tmp ;
	tmp = NULL ;

        if (tm_globs->pid!=-1) {
		insert_error ("Receive in progress.  Wait until Receive is finished\n");
                return
                NULL;
        }

        /* should never be null but just to be sure */
        if (pop->emailaddr)
                sender=strdup (pop->emailaddr);
        else
                sender=strdup ("Defunct");

        if (pop->organization)
                if (pop->organization[0]!='\0')
                        org=strdup(pop->organization);
        if (tm_globs->organization && org==NULL)
                if (tm_globs->organization[0]!='\0')
                        org=strdup(tm_globs->organization);
                else
                        org=(char *)calloc(2, sizeof(char));
        else if (!org)
                org=(char *)calloc(2, sizeof(char));

        if (pop->replyaddr)
                if (pop->replyaddr[0]!='\0')
                        replyto=strdup(pop->replyaddr);
        if (tm_globs->replyto && replyto==NULL)
                if (tm_globs->replyto[0]!='\0')
                        replyto=strdup(tm_globs->replyto);
                else
                        replyto=(char *)calloc(2, sizeof(char));
        else if (!replyto)
                replyto=(char *)calloc(2, sizeof(char));

//      if (strlen (body) > 0) {
                if (pop->flags & USE_THIS_SIG) {
                        if (pop->signature) {
                                tmp=(char *)calloc (strlen (body)+strlen (pop->signature)+4, sizeof (char));
                                sprintf (tmp, "%s\n\n%s", body, pop->signature);
                                free (body);
                                body=tmp;
                        }
                } else {
                        if (pop->flags & USE_THIS_UNIX_SIG) {
                                tmp=(char *)calloc (1024, sizeof (char));
                                snprintf (tmp, 1024, "$(HOME)/.sig");
                                env_srch_rep (tmp);
                                if (!access (tmp, R_OK)) {
                                        struct stat buf;
                                        int fd;
                                        stat (tmp, &buf);
                                        fd=open (tmp, O_RDONLY);
                                        free (tmp);
                                        tmp2=(char *)calloc (buf.st_size+1, sizeof (char));
                                        read (fd, tmp2, buf.st_size+1);
                                        close (fd);
                                        tmp=(char *)calloc (strlen (body)+strlen (tmp2)+4, sizeof (char));
                                        sprintf (tmp, "%s\n\n%s", body, tmp2);
                                        free (tmp2);
                                        free (body);
                                        body=tmp;
                                } else {
                                        perror ("Unable to stat ~/.sig");
                                        free (tmp);
                                }
                        } else {
                                if (tm_globs->flags & USE_SIGNATURE) {
                                        tmp=(char *)calloc (strlen (body)+strlen (tm_globs->signature)+4, sizeof (char));
                                        sprintf (tmp, "%s\n\n%s", body, tm_globs->signature);
                                        free (body);
                                        body=tmp;
                                }
                        }
                }
                afs=(AFS *)calloc (1, sizeof (AFS));
                afs->data=strdup (body);
                afs->type=strdup ("text/plain");
                alist=llist_append (alist, afs);
//      }


        for (i=0;i<clist->rows;i++) {
                afs=gtk_clist_get_row_data (clist, i);
#if DEBUG > 7
                printf ("1afs: <%p>\nafs->filename: <%p> '%s'\n", afs, afs->filename, afs->filename);
#endif
                alist=llist_append (alist, afs);
        }

        /* generate the to, cc, and bcc lines */
        for (s=tree->children;s;s=s->next) {
                GtkWidget *item=s->data;
                GtkWidget *hbox=GTK_BIN (item)->child;
                GtkBoxChild *typew=GTK_BOX (hbox)->children->next->data;
                GtkBoxChild *add=GTK_BOX (hbox)->children->next->next->data;
                char *type=gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (typew->widget)->entry));
                char *address=gtk_entry_get_text (GTK_ENTRY (add->widget));
                if (strcasecmp (type, "To:")==0) {
                        tol+=strlen (address)+3;
                }
                if (strcasecmp (type, "Cc:")==0) {
                        ccl+=strlen (address)+3;
                }
                if (strcasecmp (type, "Bcc:")==0) {
                        bccl+=strlen (address)+3;
                }
        }
        if (tol>0) {
                if (tol==1) {
                        insert_message ("You must supply at least one To: recipient\n");
                        return NULL;
                }
                to_line=(char *)calloc (tol, sizeof (char));
        } else {
                insert_message ("You must supply at least one To: recipient\n");
                return NULL;
        }
        if (ccl>0) {
                cc_line=(char *)calloc (ccl, sizeof (char));
        }
        if (bccl>0) {
                bcc_line=(char *)calloc (bccl, sizeof (char));
        }
        for (s=tree->children;s;s=s->next) {
                GtkWidget *item=s->data;
                GtkWidget *hbox=GTK_BIN (item)->child;
                GtkBoxChild *typew=GTK_BOX (hbox)->children->next->data;
                GtkBoxChild *add=GTK_BOX (hbox)->children->next->next->data;
                char *type=gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (typew->widget)->entry));
                char *address=gtk_entry_get_text (GTK_ENTRY (add->widget));
                if (strcasecmp (type, "To:")==0) {
                        if (strlen (to_line)>0) {
                                sprintf (to_line, "%s,%s", to_line, address);
                        } else {
                                sprintf (to_line, "%s", address);
                        }
                }
                if (strcasecmp (type, "Cc:")==0) {
                        if (strlen (cc_line)>0) {
                                sprintf (cc_line, "%s,%s", cc_line, address);
                        } else {
                                sprintf (cc_line, "%s", address);
                        }
                }
                if (strcasecmp (type, "Bcc:")==0) {
                        if (strlen (bcc_line)>0) {
                                sprintf (bcc_line, "%s,%s", bcc_line, address);
                        } else {
                                sprintf (bcc_line, "%s", address);
                        }
                }
        }
        *to=to_line;
        *cc=cc_line;
        *bcc=bcc_line;
#if DEBUG>=1
        printf ("to_line: %s\n", to_line);
        printf ("cc_line: %s\n", cc_line);
        printf ("bcc_line: %s\n", bcc_line);
        printf ("to: %s\n", *to);
        printf ("cc: %s\n", *cc);
        printf ("bcc: %s\n", *bcc);
        printf ("**********\n");
#endif
        tmp_full_message=mime_encode (sender, replyto, org, to_line, cc_line,
                                      bcc_line, subject_line, timestamp, alist,
	                              GTK_TOGGLE_BUTTON (lookup_widget (parent, "return_receipt"))->active, get_priority_string(parent));
        free (body);
        llist_blow_away (alist);
        full_message=message_fix_for_send (tmp_full_message);
        free (tmp_full_message);
        if (replyto) free (replyto);
        if (org) free (org);
        if (sender) free (sender);
	if( timestamp ) free(timestamp) ;
        return full_message;
}




MailMessage *new_new_message_generate_full (GtkWidget *parent, char **to, char **cc, char **bcc) {
	char *to_line=NULL, *cc_line=NULL, *bcc_line=NULL;
	char *subject_line=gtk_entry_get_text (GTK_ENTRY(lookup_widget (parent, "subject_entry")));
	char *body=gtk_editable_get_chars (GTK_EDITABLE(lookup_widget (parent, "body_textbox")), 0, -1);
	GtkCList *clist=GTK_CLIST (lookup_widget (parent, "attachment_clist"));
	GtkTree *tree=GTK_TREE (lw (parent, "recip_tree"));
	int popid=seek_popaccount_by_name (gtk_entry_get_text (GTK_ENTRY (lookup_widget (parent, "account_to_use"))));
	PopAccount *pop=seek_popaccount (popid);
	MailMessage *mmessage=NULL;
	char *timestamp=generate_timestamp ();
	char *sender=NULL, *org=NULL, *replyto=NULL;
	char *tmp, *tmp2;
	Llist *alist=NULL;
	AFS *afs=NULL;
	int i;

	guint tol=1, ccl=1, bccl=1;
	GList *s;
	{
		char *t=strdup (body);
		g_free (body);
		body=t;
	}
	
	if( (pop -> spid != -1) || (pop -> pid != -1) ) {
	  insert_error ( _("Transmission in progress.  Please wait until transmission is finished\n") );
#if DEBUG > 3
	  printf( "please report these values: spid = %d, pid = %d,\n", pop -> spid, pop -> pid ) ;
#endif
	  return mmessage;
	}

	/* should never be null but just to be sure */
	if (pop->emailaddr) sender=strdup (pop->emailaddr);
	else sender=strdup ("Defunct");

	if (pop->organization) 
	  if (pop->organization[0]!='\0')
	    org=strdup(pop->organization) ;
	if (tm_globs->organization && org==NULL)
	  if (tm_globs->organization[0]!='\0')
	    org=strdup(tm_globs->organization);
	  else
	    org=(char *)calloc(2, sizeof(char));
	else if (!org)
	  org=(char *)calloc(2, sizeof(char));

	if (pop->replyaddr)
		if (pop->replyaddr[0]!='\0')
			replyto=strdup(pop->replyaddr);
	if (tm_globs->replyto && replyto==NULL)
		if (tm_globs->replyto[0]!='\0')
			replyto=strdup(tm_globs->replyto);
		else
			replyto=(char *)calloc(2, sizeof(char));
	else if (!replyto)
		replyto=(char *)calloc(2, sizeof(char));

#if DEBUG > 3
	printf( "Value for word wrapping is %d!\n", tm_globs -> wwrap ) ;
#endif

	/* This value is hard coded for now, but will soon have GUI support */
	
	tmp = word_wrap_body( body, tm_globs->wwrap ) ;
	if( tmp ) {
	  free( body ) ;
	  body = tmp ;
	}

	if (pop->flags & USE_THIS_SIG) {
			if (pop->signature) {
				tmp=(char *)malloc( strlen (body)+strlen (pop->signature)+4 ) ;
				sprintf (tmp, "%s\n\n%s", body, pop->signature);
				free (body);
				body=tmp;
			}
		} else {
			if (pop->flags & USE_THIS_UNIX_SIG) {
				tmp=(char *)calloc (1024, sizeof (char));
				snprintf (tmp, 1024, "$(HOME)/.sig");
				env_srch_rep (tmp);
				if (!access (tmp, R_OK)) {
					struct stat buf;
					int fd;
					stat (tmp, &buf);
					fd=open (tmp, O_RDONLY);
					free (tmp);
					tmp2=(char *)calloc (buf.st_size+1, sizeof (char));
					read (fd, tmp2, buf.st_size+1);
					close (fd);
					tmp=(char *)calloc (strlen (body)+strlen (tmp2)+4, sizeof (char));
					sprintf (tmp, "%s\n\n%s", body, tmp2);
					free (tmp2);
					free (body);
					body=tmp;					
				} else {
					perror ("Unable to stat ~/.sig");
					free (tmp);
				}
			} else {
			  if (tm_globs->flags & USE_SIGNATURE) {
			    tmp=(char *)malloc( strlen (body)+strlen (tm_globs->signature)+4 ) ;
			    sprintf (tmp, "%s\n\n%s", body, tm_globs->signature);
			    free (body);
			    body = tmp ;
			  }
			}
		}

		afs=(AFS *)calloc (1, sizeof (AFS));
		afs->data = body ;
		afs->type=strdup ("text/plain");
		alist=llist_append (alist, afs);
//	}


	for (i=0;i<clist->rows;i++) {
		afs=gtk_clist_get_row_data (clist, i);
#if DEBUG > 7
		printf ("1afs: <%p>\nafs->filename: <%p> '%s'\n", afs, afs->filename, afs->filename);
#endif
		alist=llist_append (alist, afs);
	}

	/* generate the to, cc, and bcc lines */
	for (s=tree->children;s;s=s->next) {
		GtkWidget *item=s->data;
		GtkWidget *hbox=GTK_BIN (item)->child;
		GtkBoxChild *typew=GTK_BOX (hbox)->children->next->data;
		GtkBoxChild *add=GTK_BOX (hbox)->children->next->next->data;
		char *type=gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (typew->widget)->entry));
		char *address=gtk_entry_get_text (GTK_ENTRY (add->widget));
		if (strcasecmp (type, "To:")==0) {
			tol+=strlen (address)+3;
		}
		if (strcasecmp (type, "Cc:")==0) {
			ccl+=strlen (address)+3;
		}
		if (strcasecmp (type, "Bcc:")==0) {
			bccl+=strlen (address)+3;
		}
	}
	if (tol>0) {
		if (tol==1) {
			insert_message ("You must supply at least one To: recipient\n");
			return mmessage;	
		}
		to_line=(char *)calloc (tol, sizeof (char));
	} else {
		insert_message ("You must supply at least one To: recipient\n");
		return mmessage;
	}
	if (ccl>0) {
		cc_line=(char *)calloc (ccl, sizeof (char));
	}
	if (bccl>0) {
		bcc_line=(char *)calloc (bccl, sizeof (char));
	}
	for (s=tree->children;s;s=s->next) {
		GtkWidget *item=s->data;
		GtkWidget *hbox=GTK_BIN (item)->child;
		GtkBoxChild *typew=GTK_BOX (hbox)->children->next->data;
		GtkBoxChild *add=GTK_BOX (hbox)->children->next->next->data;
		char *type=gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (typew->widget)->entry));
		char *address=gtk_entry_get_text (GTK_ENTRY (add->widget));
		if(!strlen(address)) break;
		if (strcasecmp (type, "To:")==0) {
			if (strlen (to_line)>0) {
				sprintf (to_line, "%s,%s", to_line, address);
			} else {
				sprintf (to_line, "%s", address);
			}
		}
		if (strcasecmp (type, "Cc:")==0) {
			if (strlen (cc_line)>0) {
				sprintf (cc_line, "%s,%s", cc_line, address);
			} else {
				sprintf (cc_line, "%s", address);
			}
		}
		if (strcasecmp (type, "Bcc:")==0) {
			if (strlen (bcc_line)>0) {
				sprintf (bcc_line, "%s,%s", bcc_line, address);
			} else {
				sprintf (bcc_line, "%s", address);
			}
		}		
	}	
	*to=to_line;
	*cc=cc_line;
	*bcc=bcc_line;
#if DEBUG>=1
	printf ("to_line: %s\n", to_line);
	printf ("cc_line: %s\n", cc_line);
	printf ("bcc_line: %s\n", bcc_line);
	printf ("to: %s\n", *to);
	printf ("cc: %s\n", *cc);
	printf ("bcc: %s\n", *bcc);
	printf ("**********\n");
#endif

	/* This will return a mmessage with all attachments MIME encoded. */
	mmessage=new_mime_encode (sender, replyto, org, to_line, cc_line,
				  bcc_line, subject_line, timestamp, alist,
				  GTK_TOGGLE_BUTTON (lookup_widget (parent, "return_receipt"))->active,
				  get_priority_string( parent ),
				  get_sensitivity_string( parent ),
				  get_keyword_string( parent ),
				  get_expire_string( parent ),
				  get_replyby_string( parent ),
				  get_messageflag_string( parent ) ) ;

	mmessage->popid=popid;			      
        llist_blow_away (alist);
        if (replyto) free (replyto);
        if (org) free (org);
        if (sender) free (sender);
	if( timestamp ) free( timestamp ) ;
	if( to_line ) free( to_line ) ;
	if( cc_line ) free( cc_line ) ;
	if( bcc_line ) free( bcc_line ) ;
	return mmessage;
}

void new_message_send_success (char *messagebody, GtkWidget *widget) {
	Folder *fl;
	int msgid;
	Message *msg;
	fl=folder_seek (folder_seek_by_name (_("Sent-Items")));
	msgid=message_save (_("Sent-Items"), messagebody);
	msg=message_seek (fl, msgid);
	msg->flags=0;	
	msg->rfolder=strdup (_("Sent-Items"));
	create_new_message (fl, msg);
	fl->total++;
	destroy_new_message_window (widget, NULL);
	free (messagebody);
}

void accept_new_message_window (GtkWidget *widget, gpointer data2) {	
	GtkWidget *parent=lookup_widget (widget, "new_message_window");
	char *to=NULL, *cc=NULL, *bcc=NULL;
	MailMessage *mmessage;
	int popid=seek_popaccount_by_name (gtk_entry_get_text (GTK_ENTRY (lookup_widget (parent, "account_to_use"))));
	PopAccount *pop=seek_popaccount (popid);
	char *recp_line;
#if 0
	Message *msel=data2;
	Folder *fsel;
#endif

#if 1
	Folder *fl;
	int msgid;
	Message *msg;
	char *msgText ;


	msgText = NULL ;
#endif

	tm_globs -> wwrap = 0 ;
	if( TBA( parent, "wwrap_tog" ) )
		tm_globs -> wwrap = gv( parent, "wwrap_val" ) ;

	mmessage = new_new_message_generate_full( parent, &to, &cc, &bcc ) ;
#if 1
	msgText = new_message_generate_full( parent, &to, &cc, &bcc ) ;
#endif
	/* Make sure the message has been accepted for processing */
	if (!to || !cc || !bcc)  return ;


	recp_line = (char *)calloc( (strlen(to) + strlen(cc) + strlen(bcc) )*2, sizeof(char) ) ;
	if( (pop -> spid != -1) || (pop -> pid != -1) ) {
	  insert_error ( _("Transmission in progress.  Please wait until transmission is finished\n") );
	  return ;
	}
	
	sprintf (recp_line, "%s", to);
	if (strlen (cc)>0) sprintf (recp_line, "%s,%s", recp_line, cc);
	if (strlen (bcc)>0) sprintf (recp_line, "%s,%s", recp_line, bcc);

	insert_message( _("Sending message... ") ) ;
	new_fork_send_message( pop, mmessage, recp_line ) ;
	tm_globs -> send_timeout = gtk_timeout_add(750, (GtkFunction)send_message_timeout, NULL ) ;
	destroy_new_message_window (widget, data2);

#if 1
	/* Now, insert the message into the sent-items folder */
	insert_message( _("Sent-Items...") ) ;

	fl=folder_seek (folder_seek_by_name( _("Sent-Items") ) ) ;
	if( fl ) {
	  msgid = message_save( _("Sent-Items"), msgText ) ;
	  msg = message_seek( fl, msgid ) ;
	  msg -> flags = 0 ;
	  msg -> rfolder = strdup( _("Sent-Items") ) ;
	  create_new_message( fl, msg ) ;
	  fl -> total++ ;
	}

#if 0
	if ((msel)&&(msg->subject)) {
		if (strncasecmp (msg->subject, "re:", 3)==0)
			msel->flags |= MESSAGE_REPLY;
		if (strncasecmp (msg->subject, "fwd:", 4)==0)
			msel->flags |= MESSAGE_FORWARD;
		fsel = folder_seek (folder_seek_by_name (msel->folder));
		if (fsel)
               		folder_display (GTK_CLIST (lookup_widget (tm_globs->main_window,
                        	        "message_clist")), fsel);
	}
#endif

#if 1
	/* This needs further investigation to find out why it's not valid to do this */
	free( msgText ) ;
#endif

#endif

	free (recp_line);
	return ;
}

void destroy_new_message_window (GtkWidget *widget, gpointer data) {
	GtkWidget *parent=lookup_widget (widget, "new_message_window");
	GtkCList *clist=GTK_CLIST (lookup_widget (parent, "attachment_clist"));
	AFS *afs=NULL;
	int i=0;
	for (i=0;i<clist->rows;i++) {
		afs=gtk_clist_get_row_data (clist, i);
#if 0 /* XXX */
		if (afs) {
			if (afs->filename) free (afs->filename);
			if (afs->data) free (afs->data);
			if (afs->type) free (afs->type);
			free (afs);
		}
#endif
	}
	gtk_widget_destroy (parent);
	for (i=0;i<20;i++) {
		if (parent==tm_globs->new_messages[i]) tm_globs->new_messages[i]=NULL;
	}

#if 0
	if( data ) {
	  if( ((RecipSignalData *)data) -> email ) free( ((RecipSignalData *)data) -> email ) ;
#if DEBUG > 6
	  printf( "Destroying RecipSignalData %p.\n", data ) ;
#endif
	  free( data ) ;
	}
#endif
}


void new_message_save (GtkWidget *widget, gpointer data) {
	GtkWidget *window=lookup_widget (widget, "new_message_window");
	char *filename;
	char *subject=gtk_entry_get_text (GTK_ENTRY (lookup_widget (window, "subject_entry")));
	char *to, *cc, *bcc;
	char *full_message=new_message_generate_full (window, &to, &cc, &bcc);
	if (strlen (subject)<=0) {
		subject=strdup (_("Untitled"));	
	}	
	filename=(char *)calloc (strlen (subject)+5, sizeof(char));
	sprintf (filename, "%s.eml", subject);
	
	if (full_message) 
		save_string (full_message, strlen (full_message), filename);
	free (filename);
}

void new_message_print (GtkWidget *widget, gpointer data) {
	char *to, *cc, *bcc;
	char *full_message=new_message_generate_full (lookup_widget (widget, "new_message_window"),
	                                              &to, &cc, &bcc);
	insert_message ("Printing Message... ");	
	message_print_stream (full_message);
	free (full_message);
	insert_message ("Done!\n");
}


void new_message_add_attachment (GtkWidget *widget, gpointer data) {
	GtkWidget *message=lookup_widget (widget, "new_message_window");
	GtkWidget *fs=file_selector_new (NULL, NULL, GTK_SIGNAL_FUNC (new_message_accept_add_attachment), GTK_SIGNAL_FUNC (do_something), message);
	gtk_widget_show (fs);
}

void new_message_accept_add_attachment (GtkWidget *fs, char *filename, gpointer data) {
	GtkWidget *message=(GtkWidget *)data;
	GtkCList *clist=GTK_CLIST (lookup_widget (message, "attachment_clist"));
	struct stat buf;
	char *cl_data[3];
	int row;
	AFS *afs=(AFS *)calloc (1, sizeof (AFS));	
	gtk_widget_destroy (fs);
	afs->filename=filename;
//	afs->type=strdup ("message/rfc822");
//	printf ("Got %s\n", filename);
	stat (filename, &buf);
	cl_data[0]=NULL;
	cl_data[1]=filename;
	cl_data[2]=(char *)calloc (20, sizeof (char));
	sprintf (cl_data[2], "%d", (int)buf.st_size);
	row=gtk_clist_append (clist, cl_data);
	free (cl_data[2]);
#if DEBUG > 7
	printf ("0afs: <%p>\nafs->filename: <%p> '%s'\n", afs, afs->filename, afs->filename);
#endif
	gtk_clist_set_row_data (clist, row, (gpointer)afs);
}

void new_message_attachment_selected (GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data) {
//	char *filename=gtk_clist_get_row_data (GTK_CLIST (widget), row);
	gtk_widget_set_sensitive (lookup_widget (widget, "remove_attachment"), TRUE);
//	printf ("Filename: %s\n", filename);
}

void new_message_attachment_unselected (GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data) {
	gtk_widget_set_sensitive (lookup_widget (widget, "remove_attachment"), FALSE);
}

void new_message_delete_attachment (GtkWidget *widget, gpointer data) {
	GtkCList *clist=GTK_CLIST (lookup_widget (widget, "attachment_clist"));
	free (gtk_clist_get_row_data (clist, clist->focus_row));
	gtk_clist_remove (clist, clist->focus_row);
}

void new_message_drag_data_received (GtkWidget *widget, GdkDragContext *context,
                                 gint x, gint y, GtkSelectionData *data,
                                 guint info, guint time) {
//	printf ("Info=%d data->data=%s\n", info, data->data);
}

void new_message_check_names (GtkWidget *widget, gpointer data) {
  char *tmp ;
  GList *list ;
  GList *seek ;
  int count=0;

  /* 
   * Now, walk through the list of recipients to make sure they are valid.
   * Substitute as we go.
   */
  list = new_message_get_recipients (GTK_TREE (lw (widget, "recip_tree"))) ;
  for (seek=list;seek;seek=seek->next) {
    Recipient *recip=seek->data;
    if (strlen (recip->email) > 0) {

      /* If the alias is found, plug in the new name - use that to check */
      if( !is_email( recip -> email ) ) {
	tmp = strdup( nemail_fix( recip -> email ) ) ;
	if( tmp ) {
	  free( recip -> email ) ;
	  recip -> email = tmp ;
	}
      }
	
      if (!abook_entry_seek_by_email (NULL, recip->email)) {
	if (!validate_email (recip->email)) 
	  count++;
      }
    }
  }
  new_message_destroy_reciplist (list);
  if (count==0) insert_message( "All email addresses are valid.\n" ) ;
}

void new_message_recip_check (GtkWidget *entry, gpointer data) {
	char *oldline=gtk_entry_get_text(GTK_ENTRY(entry));
	char *newline;
	if (strlen(oldline)) {
		if(!is_email(oldline)) {
			newline=strdup(nemail_fix(oldline));
			gtk_entry_set_text(GTK_ENTRY(entry),newline);
		}
	}
}

void
new_message_recip_changed (GtkWidget *widget, gpointer data)
{
#if 0
	GtkEntry *entry = GTK_ENTRY (GTK_WIDGET (widget));
	char *original = strdup (gtk_entry_get_text (entry));
	char *possible;
	static int ignore = FALSE;

	if (ignore)
	{
		ignore = FALSE;
	}
	else
	{
		possible = abook_entry_seek_in_part2 (original);
		if (possible)
		{
			ignore = TRUE;
			gtk_entry_set_text (entry, possible);
			gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
		}
	}

	if (original) free (original);
#endif
}

void new_message_select_recipients_accept (GList *recips, GtkTree *tree) {
  /*GtkTree *tree=GTK_TREE (lw (widget, "recip_tree"));*/
  GList *seek;
  Recipient *recp;
  RecipSignalData *recip ;
  gtk_tree_clear_items (tree, 0, g_list_length (tree->children));
  

  /* Notice that there isn't a free for this! */
  recip = calloc( 1, sizeof(RecipSignalData) ) ;
  for (seek=recips;seek;seek=seek->next) {
		recp=seek->data;
		recip -> type = recp -> type ;
		recip -> email = recp -> email ;
		recip -> tree = tree ;
		new_message_window_add_recip (NULL, recip ) ;
  }
  
  /* We must NULL Terminate the recip in the list */
  recip -> type = 0 ;
  recip -> email = NULL ;
  recip -> tree = tree ;
  if (!tree->children) {
    new_message_window_add_recip (NULL, recip ) ;
  }

  return ;
}

void new_message_select_recipients (GtkWidget *widget, gpointer data) {
#if 1
#if 1
	static char *send_types[]={ "To:", "Cc:", "Bcc:", NULL};
#else
	static char *send_types[]={ "Add To:", "Add Cc:", "Add Bcc:", NULL};
#endif
#else
	char *send_types[]= { "To:", "CC:", "BCC:", NULL } ;
#endif


	GtkTree *tree=GTK_TREE (lw (widget, "recip_tree"));
	GList *types=NULL;
	GList *recips=new_message_get_recipients (tree);
	int i;
	for (i=0;send_types[i];i++) {
		types=g_list_append (types, send_types[i]);
	}
	abook_selector_create (types, recips, 
	                       GTK_SIGNAL_FUNC (new_message_select_recipients_accept), 
	                       tree);
	new_message_destroy_reciplist (recips);
}



/*
 * This function takes a char * and word wraps the whole buffer,
 * allocating and returning a new word wrapped buffer.  The caller
 * becomes the owner of this allocated memory.
 */
char *word_wrap_body( char *msg, int wl )
{
  char *ret ;
  int length ;
  int loffset ;
  int moffset ;
  int lloffset ;


  ret = NULL ;

  /* First, let's make sure we have a reasonable value to wrap with! */
  if( (wl >= 20) && (wl <=160) )
    {
      if( msg )
	{
	  if( (length = strlen( msg )) > wl )
	    {
	      /* We'll allocate enough to add a newline for every line
	       * plus a null plus one for rounding up.  Remember, if
	       * everyline isn't going to wrap, then it just means we
	       * wind up with room to spare!  Whew...
	       */
	      if( (ret = malloc( length + (length / wl) + 2 )) )
		{
		  moffset = 0 ;
		  loffset = 0 ;
		  ret[0] = 0x00 ;   /* This is so it can append to the very start! */

		  /* Now, here's were the magic happens */
		  while( moffset < length )
		    {
		      while( (msg[moffset+loffset] != (char)0x00) &&
			     (msg[moffset+loffset] != '\n') &&
			     (loffset < wl) ) loffset++ ;

		      /*
		       * Okay, if we are here, it's because one of three reasons.
		       * 1) We found the end of the message.
		       * 2) We found a new line which means we need to adjust some.
		       * 3) We hit the max word wrap size and need to find the last word boundry.
		       */

		      /* Okay, #1.  We are at the end of the message, so just copy what's left. */
		      if( msg[moffset+loffset] == (char)0x00 )
			{
			  strcat( ret, &msg[moffset] ) ;
			  moffset += loffset ;  /* This let's us drop out of our loop */
			} /* end #1 */

		      /* Okay, #2.  If we found a new line, we need only copy verbatium and adjust our index past. */
		      if ( msg[moffset+loffset] == '\n' )
			{
			  /* Copy only up to the newline to the buffer */
			  strncat( ret, &msg[moffset], loffset + 1 ) ;
			  moffset += loffset + 1 ;  /* Need to get past this new line */
			  loffset = 0 ;
			} /* end #2 */

		      /* Okay, #3.  This means we are forced to find the last word so we can insert the wrap */
		      if( loffset >= wl )
			{
			  lloffset = loffset ;
			  while( lloffset > 0 )
			    {
			      if( msg[moffset+lloffset] == 0x20 )
				{
				  /*
				   * Okay, we found our first word gap, copy up to that point,
				   * add a new line, and result the indexs to start from the
				   * beginning of the word on the right.
				   */
				  strncat( ret, &msg[moffset], lloffset ) ;
				  strcat( ret, "\n" ) ;
				  moffset += lloffset + 1 ;
				  loffset = 0 ;
				  lloffset = 0 ;    /* This is to make us break out from the loop */
				}
			      else 
				{
				  /* Make sure that lines that CAN'T wrap don't hang us up! */
				  if( lloffset == 1 )
				    {
				      strncat( ret, &msg[moffset], wl ) ;
				      strcat( ret, "\n" ) ;
				      moffset += wl ;
				      loffset = 0 ;
				    }
				  lloffset-- ;
				}
			    } /* End of while */
			}
		    }  /* outer while */
		} /* if allocation */
	    } /* if we have enough to worry about wraping */
	} /* if there is even a message to wrap on */
    } /* Make sure was have reasonable values to start with */

  return ret ;
}


void msgexp_cal(GtkWidget *widget, gpointer data){
GtkWidget *nmwindow=lookup_widget(widget, "new_message_window");
create_calendar(1,nmwindow);
}

void replyto_cal(GtkWidget *widget, gpointer data){
GtkWidget *nmwindow=lookup_widget(widget, "new_message_window");
create_calendar(2,nmwindow);
}
