/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: pop3smtp.c,v 1.85 2001/03/21 04:40:55 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "puma.h"
#include <unistd.h>

// MEMWATCH
#ifdef MEMWATCH
#include "memwatch.h"
#endif

// DMALLOC
#ifdef DMALLOC
#include "dmalloc.h"
#endif



PopAccount *popaccount_list=NULL;
static char *pop_errors[]= {
	NULL,
	"Undefined account",
	"Undefined host",
	"Unknown host",
	"Connection refused",
	"Connection already open",
	"Connection reset by peer",
	"Unknown response",
	"Unknown error",
	NULL,
	NULL, 
	"Undefined login",
	"Undefined password",
	"Unknown user",
	"Password incorrect",
	"Unknown message",
	"Relaying denied",
	"Unknown sender domain",
	"Unknown recipient domain",
	"No recipient specified"
};



void popaccounts_dump_em_all (char *identifier) {
#if DEBUG > 8
	PopAccount *seek=popaccount_list;
	printf ("****************************************************\n");
	printf ("Identifier: %s\n", identifier);
	while (seek) {
		printf ("seek->name: %p '%s'\n", seek->name, seek->name);
		printf ("seek->phost: %p '%s'\n", seek->phost, seek->phost);
		printf ("seek->ppasswd: %p '%s'\n", seek->ppasswd, seek->ppasswd);
		printf ("seek->puser: %p '%s'\n", seek->puser, seek->puser);
		printf ("seek->suser: %p '%s'\n", seek->suser, seek->suser);
		printf ("seek->folder: %p '%s'\n", seek->folder, seek->folder);
		printf ("seek->real_name: %p '%s'\n", seek->real_name, seek->real_name);
		printf ("seek->replyaddr: %p '%s'\n", seek->replyaddr, seek->replyaddr);
		printf ("seek->organization: %p '%s'\n", seek->organization, seek->organization);
		printf ("seek->signature: %p '%s'\n", seek->signature, seek->signature);
		printf ("seek->attachdir: %p '%s'\n", seek->attachdir, seek->attachdir);
		printf ("-------------------\n");
		seek=seek->next;
	}
#endif
}


int new_popaccount (const char *name, const char *phost,
                    const char *puser, const char *ppasswd,
                    int pport, const char *shost, const char *suser,
                    int sport, unsigned long flags, int timeout, char *folder,
                    char *real_name, char *emailaddr, char *replyaddr,
                    char *organization, char *signature, char *attachdir, int msgCount, void *extra) {

	PopAccount *seek=popaccount_list, *newpopaccount;

	if (popaccount_list==NULL) {
		seek=(PopAccount *)calloc(1, sizeof(PopAccount));
		popaccount_list=seek;
		seek->next=NULL;

		if (name!=NULL) {
			seek->name=(char *)calloc(strlen(name)+1, sizeof(char));
			memmove(seek->name, name, strlen(name));
		} else seek->name=NULL;

		/***** POP3 *****/
		if (phost!=NULL) {
			seek->phost=(char *)calloc(strlen(phost)+1, sizeof(char));
			memmove(seek->phost, phost, strlen(phost));
		} else seek->phost=NULL;
		if (puser!=NULL) {
			seek->puser=(char *)calloc(strlen(puser)+1, sizeof(char));
			memmove(seek->puser, puser, strlen(puser));
		} else seek->puser=NULL;
		if (ppasswd!=NULL) {
			seek->ppasswd=(char *)calloc(strlen(ppasswd)+1, sizeof(char));
			memmove(seek->ppasswd, ppasswd, strlen(ppasswd));
		} else seek->ppasswd=NULL;
		if (pport>=0) {
			seek->pport=pport;
		} else seek->pport=110;

		/***** SMTP *****/
		if (shost!=NULL) {
			seek->shost=(char *)calloc(strlen(shost)+1, sizeof(char));
			memmove(seek->shost, shost, strlen(shost));
		} else seek->shost=NULL;
		if (suser!=NULL) {
			seek->suser=(char *)calloc(strlen(suser)+1, sizeof(char));
			memmove(seek->suser, suser, strlen(suser));
		} else seek->suser=NULL;
		if (sport>=0) {
			seek->sport=sport;
		} else seek->sport=25;
		seek->smode='S';

		/***** Optional *****/
		if (folder!=NULL) {
			seek->folder=(char *)g_malloc(strlen(folder)+1);
			memmove(seek->folder, folder, strlen(folder));
		} else seek->folder=NULL;
		if (real_name!=NULL) {
			seek->real_name=(char *)calloc(strlen(real_name)+1, sizeof(char));
			memmove(seek->real_name, real_name, strlen(real_name));
		} else seek->real_name=NULL;
		if (emailaddr!=NULL) {
			seek->emailaddr=(char *)calloc(strlen(emailaddr)+1, sizeof(char));
			memmove(seek->emailaddr, emailaddr, strlen(emailaddr));
		} else seek->emailaddr=NULL;
		if (folder!=NULL) {
			seek->folder=(char *)calloc(strlen(folder)+1, sizeof(char));
			memmove(seek->folder, folder, strlen(folder));
		} else seek->folder=NULL;
		if (replyaddr!=NULL) {
			seek->replyaddr=(char *)calloc(strlen(replyaddr)+1, sizeof(char));
			memmove(seek->replyaddr, replyaddr, strlen(replyaddr));
		} else seek->replyaddr=NULL;
		if (organization!=NULL) {
			seek->organization=(char *)calloc(strlen(organization)+1, sizeof(char));
			memmove(seek->organization, organization, strlen(organization));
		} else seek->organization=NULL;
		if (signature!=NULL) {
			seek->signature=(char *)calloc(strlen(signature)+1, sizeof(char));
			memmove(seek->signature, signature, strlen(signature));
		} else seek->signature=NULL;
		if (attachdir!=NULL) {
			seek->attachdir=(char *)calloc(strlen(attachdir)+1, sizeof(char));
			memmove(seek->attachdir, attachdir, strlen(attachdir));
		} else seek->attachdir=NULL;

		if (extra) seek->extra=extra;

		/* glob */
		seek->flags=flags;
		seek->timeout=timeout;
		seek->pid=-1;
		seek->spid=-1;
		seek->id=0;
		seek->busy=0;
		seek->popcmsg = msgCount ;
		seek->poptmsg = msgCount ;
#if DEBUG > 4
		if ((name)&&(msgCount))
			printf( "Account '%s', currently has %d messages on server.\n", name, msgCount ) ;
#endif

		return seek->id;

	} else {
		newpopaccount=(PopAccount *)calloc(1, sizeof(PopAccount));
		while (seek->next!=NULL) { seek=seek->next; }
		seek->next=newpopaccount;
		newpopaccount->next=NULL;

		if (name!=NULL) {
			newpopaccount->name=(char *)calloc(strlen(name)+1, sizeof(char));
			memmove(newpopaccount->name, name, strlen(name));
		} else newpopaccount->name=NULL;

		/***** POP3 *****/
		if (phost!=NULL) {
			newpopaccount->phost=(char *)calloc(strlen(phost)+1, sizeof(char));
			memmove(newpopaccount->phost, phost, strlen(phost));
		} else newpopaccount->phost=NULL;
		if (puser!=NULL) {
			newpopaccount->puser=(char *)calloc(strlen(puser)+1, sizeof(char));
			memmove(newpopaccount->puser, puser, strlen(puser));
		} else newpopaccount->puser=NULL;
		if (ppasswd!=NULL) {
			newpopaccount->ppasswd=(char *)calloc(strlen(ppasswd)+1, sizeof(char));
			memmove(newpopaccount->ppasswd, ppasswd, strlen(ppasswd));
		} else newpopaccount->ppasswd=NULL;
		if (pport>=0) {
			newpopaccount->pport=pport;
		} else newpopaccount->pport=110;

		/***** SMTP *****/
		if (shost!=NULL) {
			newpopaccount->shost=(char *)calloc(strlen(shost)+1, sizeof(char));
			memmove(newpopaccount->shost, shost, strlen(shost));
		} else newpopaccount->shost=NULL;
		if (suser!=NULL) {
			newpopaccount->suser=(char *)calloc(strlen(suser)+1, sizeof(char));
			memmove(newpopaccount->suser, suser, strlen(suser));
		} else newpopaccount->suser=NULL;
		if (sport>=0) {
			newpopaccount->sport=sport;
		} else newpopaccount->sport=25;
		newpopaccount->smode='S';

		/***** Optional *****/
		if (folder!=NULL) {
			newpopaccount->folder=(char *)g_malloc0(strlen(folder)+1);
			memmove(newpopaccount->folder, folder, strlen(folder));
		} else newpopaccount->folder=NULL;
		if (real_name!=NULL) {
			newpopaccount->real_name=(char *)calloc(strlen(real_name)+1, sizeof(char));
			memmove(newpopaccount->real_name, real_name, strlen(real_name));
		} else newpopaccount->real_name=NULL;
		if (emailaddr!=NULL) {
			newpopaccount->emailaddr=(char *)calloc(strlen(emailaddr)+1, sizeof(char));
			memmove(newpopaccount->emailaddr, emailaddr, strlen(emailaddr));
		} else newpopaccount->emailaddr=NULL;
		if (folder!=NULL) {
			newpopaccount->folder=(char *)calloc(strlen(folder)+1, sizeof(char));
			memmove(newpopaccount->folder, folder, strlen(folder));
		} else newpopaccount->folder=NULL;
		if (replyaddr!=NULL) {
			newpopaccount->replyaddr=(char *)calloc(strlen(replyaddr)+1, sizeof(char));
			memmove(newpopaccount->replyaddr, replyaddr, strlen(replyaddr));
		} else newpopaccount->replyaddr=NULL;
		if (organization!=NULL) {
			newpopaccount->organization=(char *)calloc(strlen(organization)+1, sizeof(char));
			memmove(newpopaccount->organization, organization, strlen(organization));			
		} else newpopaccount->organization=NULL;
		if (signature!=NULL) {
			newpopaccount->signature=(char *)calloc(strlen(signature)+1, sizeof(char));
			memmove(newpopaccount->signature, signature, strlen(signature));
		} else newpopaccount->signature=NULL;

		if (extra) newpopaccount->extra=extra;

		if (attachdir!=NULL) {
			newpopaccount->attachdir=(char *)calloc(strlen(attachdir)+1, sizeof(char));
			memmove(newpopaccount->attachdir, attachdir, strlen(attachdir));
		} else newpopaccount->attachdir=NULL;
		/* glob */
		newpopaccount->flags=flags;
		newpopaccount->timeout=timeout;
		newpopaccount->pid=-1 ;
		newpopaccount->spid=-1 ;
		newpopaccount->busy=0 ;
		newpopaccount->id=seek->id+1;
		newpopaccount->popcmsg = msgCount ;
		newpopaccount->poptmsg = msgCount ;
#if DEBUG > 4
		printf( "Account '%s', currently has %d messages on server.\n", name, msgCount ) ;
#endif
		return newpopaccount->id;
	}
	return -1;
}

PopAccount *seek_popaccount (int id) {
	PopAccount *seek ;
	PopAccount *ret ;

	seek = popaccount_list;
	ret = (PopAccount *)NULL ;
	while (seek) {
		if (seek->id==id) ret = seek ;
		seek=(PopAccount *)seek->next;
	}
	return ret ;
}

PopAccount *popaccount_is_my_address (char *address) {
	PopAccount *seek;
	for (seek=popaccount_list;seek;seek=seek->next) {
		if (seek->emailaddr) {
			if (strcasecmp (address, seek->emailaddr)==0) {
				return seek;
			}
		}
		if (seek->replyaddr) {
			if (strcasecmp (address, seek->replyaddr)==0) {
				return seek;
			}
		}
	}
	return NULL;
}


PopAccount *first_popaccount (void) {
	return popaccount_list;
}

int seek_popaccount_by_flag (unsigned long flag) {
	PopAccount *seek=popaccount_list;

	if (seek==NULL)
		return NDEFACCOUNT;

	while (seek) {
		if ((seek->flags&flag)==flag) return seek->id;
		seek=(PopAccount *)seek->next;
	}
	return NDEFACCOUNT;
}

int seek_popaccount_by_name (char *name) {
	PopAccount *seek=popaccount_list;
	while (seek) {
		if (strcasecmp (seek->name, name)==0) return seek->id;
		seek=seek->next;
	}
	return NDEFACCOUNT;
}



int destroy_popaccount (int id) {
  int ret ;
  int cnt ;
  PopAccount *ntmp, *prev ;
  PopAccount *seek=seek_popaccount(id);

  if( seek ) {
    if( popaccount_list == seek ) popaccount_list = seek -> next ;
    else {
      prev = seek_popaccount( id - 1 ) ;
      prev -> next = seek -> next ;
    }

    if (seek->name) free (seek->name);
    if (seek->phost) free (seek->phost);
    if (seek->ppasswd) free (seek->ppasswd);
    if (seek->puser) free (seek->puser);
    if (seek->shost) free (seek->shost);
    if (seek->suser) free (seek->suser);

    if (seek->folder) free (seek->folder);
    if (seek->real_name) free (seek->real_name);
    if (seek->emailaddr) free (seek->emailaddr);
    if (seek->replyaddr) free (seek->replyaddr);

    if (seek->organization) free (seek->organization);
    if (seek->signature) free (seek->signature);
    if (seek->attachdir) free (seek->attachdir);
    if (seek->extra) free (seek->extra);

    free (seek);

    /*
     * Now, we need to renumber every body so that we to renumber everyone
     * so that the counts stay ordered.  I think that some logic may not
     * like finding non-linear account numbers.
     */
    ntmp = first_popaccount() ;
    for( cnt = 0, ntmp = first_popaccount() ; ntmp ; cnt++ ) {
      ntmp -> id = cnt ;
      ntmp = ntmp -> next ;
    }

    ret = 0 ;
  } else {
    ret = NDEFACCOUNT ;
  }
  return ret ;
}



int edit_popaccount (int id, const char *name, const char *phost,
                    const char *puser, const char *ppasswd,
                    int pport, const char *shost, const char *suser,
                    int sport, unsigned long flags, int timeout, char *folder,
                    char *real_name, char *emailaddr, char *replyaddr,
                    char *organization, char *signature, char *attachdir, void *extra) {

	PopAccount *seek=seek_popaccount (id);

	if (seek==NULL)
		return NDEFACCOUNT;

	if (name!=NULL) {
		if (seek->name) free (seek->name);
		seek->name=strdup (name);
	}

	/***** POP3 *****/
	if (phost!=NULL) {
		if (seek->phost) free (seek->phost);
		seek->phost=strdup (phost);
	}
	if (puser!=NULL) {
		if (seek->puser) free (seek->puser);
		seek->puser=strdup (puser);
	}

	if (ppasswd!=NULL) {
		if (seek->ppasswd) free (seek->ppasswd);
		seek->ppasswd=strdup (ppasswd);
	}
	if (pport>0) {
		seek->pport=pport;
	}

	/***** SMTP *****/
	if (shost!=NULL) {
		if (seek->shost) free (seek->shost);
		seek->shost=strdup (shost);
	}
	if (suser!=NULL) {
		if (seek->suser) free (seek->suser);
		seek->suser=strdup (suser);
	}
	if (sport>0) {
		seek->sport=sport;
	}

	/***** Globs *****/
	if (flags>=0) {
		seek->flags=flags;
	}
	seek->timeout=timeout;

	/***** Optional *****/
	if (folder!=NULL) {
		if (seek->folder) free (seek->folder);
		seek->folder=strdup (folder);
	}
	if (real_name!=NULL) {
		if (seek->real_name) free (seek->real_name);
		seek->real_name=strdup (real_name);
	}
	if (emailaddr!=NULL) {
		if (seek->emailaddr) free (seek->emailaddr);
		seek->emailaddr=strdup (emailaddr);
	}
	if (replyaddr!=NULL) {
		if (seek->replyaddr) free (seek->replyaddr);
		seek->replyaddr=strdup (replyaddr);
	}
	if (organization!=NULL) {
		if (seek->organization) free (seek->organization);
		seek->organization=strdup (organization);
	}
	if (signature!=NULL) {
		if (seek->signature) free (seek->signature);
		seek->signature=strdup (signature);
	}
	if (attachdir!=NULL) {
		if (seek->attachdir) free (seek->attachdir);
		seek->attachdir=strdup (attachdir);
	}

	if (extra) {
		free (extra);
		seek->extra=extra;
	}

	return 0;
}

int is_popaccount_password_set (int id) {
	PopAccount *curpop=seek_popaccount(id);

	if (curpop==NULL)
		return NDEFACCOUNT;

	if (curpop->ppasswd==NULL)
		return FALSE;
	return TRUE;
}

int destroy_popaccount_password (int id) {
	PopAccount *curpop=seek_popaccount(id);

	if (curpop==NULL)
		return NDEFACCOUNT;

	if (curpop->ppasswd!=NULL) {
		memset (curpop->ppasswd, 0, strlen(curpop->ppasswd));
		free(curpop->ppasswd);
		curpop->ppasswd=NULL;
	}
	return 0;
}

unsigned long get_popaccount_flags (int id) {
	PopAccount *curpop=seek_popaccount(id);
	if (curpop==NULL)
		return NDEFACCOUNT;
	return curpop->flags;
}


int new_smtpconnect(int id) {
	return 0;
}
int new_smtpehlo (int id, char *domain) {
	return 0;
}	
int splitit(char *to,Nto *nto[256]) {
        char *tmp = NULL;
	int i,j;
	j=0;	
	for(i=0;i<strlen(to);i++) {
		if(to[i]!=',') {
		  tmp[j]=to[i];
		  j++;
		} else {
		  //we should check if it has a personal name
		  fprintf(stderr,"TO match found\n");
		  j=0;
		}
      }
	return 0;
}


/*
 * This function is influenced by PINE's function of a like name.
 */
char *
generate_msg_id( void ) {
  char r ;
  int val ;
  time_t t ;
  char *tmp ;
  char *ret ;
  char *host ;
  char *os_info ;
  struct tm *t_tm ;


  t = time( NULL ) ;
  t_tm = localtime( &t ) ;
  ret = (char *)malloc( 2048*sizeof(char) ) ;

  if( t_tm -> tm_sec != 0 ) {
    srand( t_tm -> tm_sec + t ) ;
    val = rand() + t_tm -> tm_sec ;
    r = ((char)rand()%64)+ 32 ;
  } else {
    r = ((char)rand()%64)+ 32 ;
    val = rand() + t_tm -> tm_sec + t_tm -> tm_min ;
  }

  /* Get the os information and then replace any spaces with '-''s */
  tmp = os_info = generate_os_info() ;
  while( *tmp ) {
    if( *tmp == 0x20 ) *tmp = '-' ;
    tmp++ ;
  }

  /* Make sure we always have a hostname value defined! */
  host = getenv( "HOSTNAME" ) ;
  if( !host ) host = "localhost" ;

  sprintf( ret, "<TradeClient.%.20s.%.20s.%02d%02d%02d%02d%02d%02d%X.%d@%.50s>",
	   VERSION, os_info, (t_tm -> tm_year) % 100, t_tm -> tm_mon + 1,
	   t_tm -> tm_mday, t_tm -> tm_hour, t_tm -> tm_min, t_tm -> tm_sec, 
	   r, getpid(), host ) ;
  
#if DEBUG > 8
  printf( "\nos info = '%s'\n", os_info ) ;
  printf( "Generated message id = '%s'\n\n", ret ) ;
#endif

  return ret ;
}



int new_sendmessage (MailMessage *message) {
  PopAccount *curpop=seek_popaccount(message->popid);
  SENDSTREAM *stream=NULL;
  char from[1024];
  char *from2 = NULL;
  ENVELOPE *msg = mail_newenvelope ();
  BODY *body = mail_newbody ();
  PART *part = mail_newbody_part() ;
  char **hostlist ;
  int smtpport,i;
  char *type,*subtype;
  char *curhost = NULL ;
  char *reply ;
  int hdrCntr ;
  
  
#if DEBUG > 6
  fprintf(stderr,"sendmessage INIT\n");
#endif	  
  
  /* Properly create the hostlist */
  hostlist = (char **)calloc( 2, sizeof(char *) ) ;
  hostlist[0]=strdup(curpop->shost);
  hostlist[1] = (char *)NULL ;
  
  /* Now, set the port! */
  if(curpop->sport) {
    smtpport=curpop->sport;
  } else {
    smtpport=25;
  }

#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  curhost=cut_string (curpop->emailaddr, '@', 2, 1);
  if(message->to) {
#if DEBUG > 3
    printf("Sendmsg: sending to: %s\n",message->to);
#endif	  
    rfc822_parse_adrlist (&msg->to,message->to,curhost);
  }  
  if(message->cc) {
#if DEBUG > 3
    printf("Sendmsg: sending cc: %s\n",message->cc);
#endif	  
    rfc822_parse_adrlist (&msg->cc,message->cc,curhost);
  }  
  if(message->bcc) {
#if DEBUG > 3
    printf("Sendmsg: sending bcc: %s\n",message->bcc);
#endif	  
    rfc822_parse_adrlist (&msg->bcc,message->bcc,curhost);
  }  
  if(message->from) {
    if(curpop->real_name && curpop->emailaddr) {
      sprintf(from,"\"%s\"<%s>",curpop->real_name,curpop->emailaddr);
    } else if(curpop->emailaddr) {
      sprintf(from,"%s",curpop->emailaddr);
    } else {
#if DEBUG > 3
      printf("pop3smtp: No valid address");
#endif	  
      return NOSENDERDOM;
    }
#if DEBUG > 3
    printf("Sendmsg: sending from: %s\n",from);
#endif	  

    /* Check to see if we need generate a rr header for this email. */
#if DEBUG > 5
    printf( "sendmsg: message has rr turned '%s'.\n", message -> rr?"ON":"OFF" ) ;
#endif
    if( message -> rr ) {
      reply = from ;
      
#if DEBUG > 5
      printf( "Reply has been set to '%s'.\n", reply ) ;
#endif
      
      /* Now, find the last item in the extra header list */
      hdrCntr = 0 ;
      while( message -> extraHeader[hdrCntr++] ) ;
      message -> extraHeader[hdrCntr-1] = strdup( "Disposition-Notification-To" ) ;
      message -> extraValue[hdrCntr-1] = strdup( reply ) ;
      message -> extraHeader[hdrCntr] = (char *)NULL ;
      message -> extraValue[hdrCntr] = (char *)NULL ;
    }

    /* Charles Hornberger <hornberger@tradeclient.org> found the problem with
     * this, it seems rfc822_parse_adrlist() is mangling ``from'', we need to
     * use a copy.
     */
    from2 = strdup (from);
    rfc822_parse_adrlist (&msg->from, from, curhost);
    /* use from2, if we have it, otherwise use the probably mangled ``from'' */
    rfc822_parse_adrlist (&msg->return_path,
				((from2 != NULL) ? from2 : from), curhost);
    if (from2) free (from2);
  }  

  from[0] = (char)0x00 ;
  if(curpop->replyaddr) {
    sprintf(from,"%s",curpop->replyaddr);
    rfc822_parse_adrlist (&msg->reply_to,from,curhost);
  } else {
    /* Use the global value if one exists */
    if( (tm_globs -> replyto) && strlen(tm_globs -> replyto) )
      {
	sprintf( from, "%s", tm_globs -> replyto ) ;
	rfc822_parse_adrlist( &msg -> reply_to, from, curhost ) ;
      }
  }
#if DEBUG > 3
  if( strlen( from ) ) printf("Sendmsg: replyaddr: %s\n",from);
#endif	  


  /*
   * Now, handle the extra header values.  This is where we pick up things
   * like rr, message priorities, X-Mailers, Organization, etc...
   * Thanks Mahogany for this patch.  It saved me from having to do it.
   */
  rfc822_setextraheaders( (const char **)message -> extraHeader, (const char **)message -> extraValue ) ;

  if(message->body) {
#if DEBUG > 8
    printf("Sendmsg: sending body: (%d attachments) \n%s\nSendmsg: end of body\n",message->attach_count,message->body);
#endif	  
#ifdef DMALLOC
    dmalloc_verify( 0 ) ;
#endif
#if DEBUG > 4
    printf( "We have %d attachments to encode! (body+n-1)\n", message -> attach_count ) ;
#endif
    if(!message->attach_count) {
      body->type=TYPETEXT;
      body->contents.text.data=(unsigned char *) message->body;
      body->contents.text.size=strlen(message->body);
    } else {
      body->type=TYPEMULTIPART;
      body->contents.text.data=(unsigned char *) message->body;
      body->contents.text.size=strlen(message->body);
      if (!body->parameter) {
	char tmp[MAILTMPLEN];
	sprintf( tmp, "---TradeClientMIMEBOUNDRY---%ld-%ld-%ld=:%ld", 
		 gethostid(), random()*time(NULL)/getpid(), time(NULL), (long)getpid ) ;
	body->parameter = mail_newbody_parameter ();
	body->parameter->attribute = cpystr ("BOUNDARY");
	body->parameter->value = cpystr (tmp);
      }
      part=body->nested.part=mail_newbody_part();
      part->body.type=TYPETEXT;
      part->body.contents.text.data=(unsigned char *) message->body;
      part->body.contents.text.size=strlen(message->body);
      for(i=0;i<message->attach_count-1;i++) {
#if DEBUG > 4
	printf( "Adding a new body part #%d of %d to MIME message.\n", i, message->attach_count-1 ) ;
#endif
	part=part->next=mail_newbody_part();
	part->body.contents.text.data=cpystr(message->attach[i]);
	part->body.contents.text.size=strlen(message->attach[i]);
        part->body.parameter=mail_newbody_parameter ();
        part->body.parameter->attribute = cpystr ("name");
        part->body.parameter->value=cpystr(message->attachname[i]);

	/* Now, set the encoding type.  If it's not a message, assume it's BASE64! */
	if( strncasecmp( message->attachtype[i], "message", 7 ) == 0 ) part->body.encoding=ENC7BIT ;
	else part->body.encoding=ENCBASE64;

	part->body.disposition.type=cpystr("attachment");
	part->body.disposition.parameter=mail_newbody_parameter ();
	part->body.disposition.parameter->attribute=cpystr("filename");
	part->body.disposition.parameter->value=cpystr(message->attachname[i]);
	subtype=strchr(message->attachtype[i],'/');
#if DEBUG > 4
	printf("trying to encode the msg\n");
	printf("------------------------------\n%s\n-------------------------------\n",message->attach[i]);
        printf("attachtype: %s - %s\n",message->attachtype[i], subtype ) ;
#endif		  
	if(subtype!=NULL) {
	  subtype++;	
	  type=strdup(message->attachtype[i]);
	  if(strncmp(type,"audio",5) == 0) {
	    part->body.type=TYPEAUDIO;
	    part->body.subtype=strdup(subtype);
	  } else if(strncmp(type,"text",4) == 0) {
	    part->body.type=TYPETEXT;
	    part->body.subtype=strdup(subtype);
	  } else if(strncmp(type,"message",7) == 0) {
	    part->body.type=TYPEMESSAGE;
	    part->body.subtype=strdup(subtype);
#if DEBUG > 4
	    printf( "body part is now '%s'.\n", part->body.subtype ) ;
#endif
	  } else if(strncmp(type,"application",11) == 0) {
	    part->body.type=TYPEAPPLICATION;
	    part->body.subtype=strdup(subtype);
	  } else if(strncmp(type,"image",5) == 0) {
	    part->body.type=TYPEIMAGE;
	    part->body.subtype=strdup(subtype);
	  } else if(strncmp(type,"video",5) == 0) {
	    part->body.type=TYPEVIDEO;
	    part->body.subtype=strdup(subtype);
	  } else if(strncmp(type,"other",5) == 0) {
	    part->body.type=TYPEOTHER;
	    part->body.subtype=strdup(subtype);
	  } else {
	    part->body.type=TYPEAPPLICATION;
	    part->body.subtype=strdup(subtype);
	  }
	}else { 
	  part->body.type=TYPEAPPLICATION;
	  part->body.subtype=("octet-stream");
	}
	
      }
    }
  }
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  if(message->subject) {
#if DEBUG > 4
    printf("Sendmsg: subject: %s\n",message->subject);
#endif	  
    msg->subject=cpystr(message->subject);
  }
  if(message->timestamp) {
#if DEBUG > 4
    printf("Sendmsg: timestamp: %s\n",message->timestamp);
#endif	  
    msg->date=cpystr(message->timestamp);
  }
#if DEBUG > 4
  printf("Try to open smtp connection to %s:%d : ",curpop->shost,smtpport);
#endif
  msg -> message_id = generate_msg_id() ;

  if( !(stream=smtp_open_full (NIL,hostlist,"smtp",smtpport,(long)NULL)) ) {
#ifdef DEBUG
    printf("error opening SMTP connection\n");
#endif	  
    return CONNREF;
  }

  if( smtp_mail(stream,"MAIL",msg,body) ) {
#ifdef DEBUG
    printf( _("Sendmsg: Sending message done\n") ) ;
#endif	  
  }  else  {
#ifdef DEBUG
    printf( "[Failed] Sending message to parent: %s\n", stream -> reply ) ;
#endif	  
    /* Send the text back to the parent */
    pipe_parent_stat_put( EMESSAGE, stream -> reply, strlen(stream -> reply)+1 ) ;
    return 0 ;
  }

  
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  //Cleanup of memory
  if(stream) smtp_close(stream);

  /*
   * Leak alert!!!  This is leaking every structure member that was allocated
   * and assigned to this structure.
   */
#if 1
  if( message ) free( message ) ;
#else
  if( message ) freeMessage( message ) ;
#endif

#if 0
  /* It seems that c-client actually owns this memory! */
  if(body) free(body);
  if(msg) free (msg);
#endif  
  if( hostlist[0] ) free( hostlist[0] ) ;
  if( hostlist ) free( hostlist ) ;
  //TIMESTAMP // MAILER // ORG
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  
  return 0;
}



void new_smtpdisconnect (int id) {
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  return;
}


char *popsmtp_error (int err) {
  return pop_errors [abs (err)];
}



/* POP3 session */
int new_pop3_login (MAILSTREAM **retStream, int id) {
  MAILSTREAM *stream ;
  PopAccount *curpop=seek_popaccount (id);
  char flags[4096] ;
  char *mailbox;
  char *snum ;
  int msgs;


  stream = NULL ;
  mailbox=(char *)calloc(4096,sizeof(char));
  /* check to see if the popaccount struct exists */
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  if (curpop==NULL) {
#ifdef DEBUG
    printf ("pop3smtp: pop3login: Pop account %d not defined\n", id);
#endif
    if(mailbox) free(mailbox);
    return NDEFACCOUNT;
  }
  if (curpop->phost==NULL) {
#ifdef DEBUG
    printf ("pop3smtp: pop3login: Host not defined for account '%s', id=%d\n", curpop->name, id);
#endif
    if(mailbox) free(mailbox);
    return NDEFHOST;
  }
  if (curpop->puser==NULL) {
#ifdef DEBUG
    printf ("pop3smtp: pop3login: User not defined for account '%s', id=%d\n", curpop->name, id);
#endif
    if(mailbox) free(mailbox);
    return NDEFUSER;
  }
  if( (curpop -> ppasswd == NULL) || 
  ( (curpop -> ppasswd) && (!strlen(curpop -> ppasswd)) ) ) {
#ifdef DEBUG
    printf ("pop3smtp: pop3login: Password not defined for account '%s', id=%d\n", curpop->name, id);
#endif 
    if(mailbox) free(mailbox);
    return NDEFPASSWORD;
  }

  flags[0] = 0x00 ;
#if WANTSSL
  /*
   * See if we need to modify the mailbox specification by adjusting
   * for SSL.  Also notice that we support the certification validation
   * request flags too.
   */
  if( curpop -> flags & SSL_CONN ) {
    strcat( flags, "/ssl" ) ;
    if( curpop -> flags & SSL_VERIFY_CERT ) strcat( flags, "/validate-cert" ) ;
    else strcat( flags, "/novalidate-cert" ) ;
  }
#endif
  
  switch( curpop -> type ) {
  case POP3:
    sprintf(mailbox,"{%s:%d/pop3%s/user=\"%s\"}%s",curpop->phost,curpop->pport,
	    flags, curpop->puser, "Inbox" ) ;
    break ;
    
  case IMAP:
    sprintf(mailbox,"{%s:%d/imap%s/user=\"%s\"}%s",curpop->phost,curpop->pport,
	    flags, curpop->puser, "Inbox" ) ;
    break ;
  }
#ifdef DEBUG
  printf("try to open mailbox: %s\n", mailbox);
#endif
  mmpopid=id;
	
#if DEBUG > 4
  if( curpop -> flags & DELETEFROMSERVER ) stream = mail_open( stream, mailbox, OP_EXPUNGE | OP_DEBUG ) ;
  else stream = mail_open( stream, mailbox, OP_DEBUG ) ;
#else
  if( curpop -> flags & DELETEFROMSERVER ) stream = mail_open( stream, mailbox, OP_EXPUNGE ) ;
  else stream = mail_open( stream, mailbox, 0 ) ;
#endif
  if( !stream ) {
    printf("ERROR:  %s mailbox failed to open.\n", mailbox );
    if(mailbox) free(mailbox);
    return UNKNOWNERR;
  }
#if DEBUG > 3
  printf( "$mailbox ok - messages: %lu   - recent: %lu\n", stream->nmsgs, stream->recent ) ;
  printf( "cmsg = %d and tmsg = %d.\n", curpop -> popcmsg, curpop -> poptmsg ) ;
  if( stream -> recent > curpop -> popcmsg ) printf( "Looks like we need to be fetching some mail!\n" ) ;
#endif

  if(mailbox) free(mailbox);


  /*
   * Tell everyone how many TOTAL messages we have on account.
   */
#if DEBUG > 5
  printf( "Sending 'TOTALMESSAGES' to parent with id = %d.\n", curpop -> id ) ;
#endif
  snum = disassemble_long( (unsigned long)stream -> nmsgs ) ;
  pipe_parent_stat_put( TOTALMESSAGES, snum, 4 ) ;
  free( snum ) ;

  /*
   * We need this to guard against multiple fetches!
   * The logic works like this.  As long as we are
   * in non-delete mode, popcmsg should always be equal to
   * or less than the recent messages.  Otherwise, that
   * means that the account has been manipulated else where
   * and/or had some deleted, which means that we need to
   * attempt to fetch all messages since they could all
   * be new messages.
   */
#ifdef DEBUG
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
#endif
  /*
   * If this is an IMAP account, directly report the number of new messages
   * reported by the server.  Otherwise, we need to jump through hoops to
   * deduce the value.  Likewise, if we are not in delete mode, 
   */
  if( curpop -> type == IMAP ) {
#if DEBUG > 4
    printf( "account type is IMAP\n" ) ;
#endif
    msgs = stream -> recent ;
  } else {
#if DEBUG > 4
    printf( "account type is POP3\n" ) ;
#endif
    if( curpop -> flags & DELETEFROMSERVER) {
      if( curpop -> popcmsg > stream -> recent ) curpop -> popcmsg = 0 ;
      else curpop -> popcmsg = stream -> nmsgs ;
      msgs = curpop -> popcmsg ;
    } else {
      msgs = abs(stream -> recent - curpop -> popcmsg) ;
    }
  }

  /*
   * Now, if forced fetch is enable, we're going to override the
   * above computed values.
   */
  if( curpop -> flags & FORCE_FETCH ) {
    msgs = stream -> nmsgs ;
    curpop -> popcmsg = 0 ;
  }

  /* just go ahead and force download whatever is in the mailbox */
    msgs = stream -> nmsgs ;
    curpop -> popcmsg = 0 ;

  /* No matter what, we always do this */
  curpop -> poptmsg = stream -> nmsgs ;

  *retStream = stream ;
  return msgs ;
}



int new_pop3_logout (int id, MAILSTREAM *stream) {
  PopAccount *curpop=seek_popaccount(id);
#if DEBUG > 4	
  printf("**POP3logout: doing logout!\n");
#endif	
  if (((curpop->flags&REMBER_PASSWORD)!=REMBER_PASSWORD)&&
      ((curpop->flags&REMBER_PASSWORD_ALWAYS)!=REMBER_PASSWORD_ALWAYS))
    destroy_popaccount_password (id);
#if DEBUG >4		
  printf("DOING LOGOUT\n");	
#endif	
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  if( curpop -> flags & DELETEFROMSERVER ) {
    /* just to be extra sure */
    mail_expunge( stream ) ;
    mail_close_full( stream, CL_EXPUNGE ) ;
  }
  else mail_close_full( stream, 0 ) ;
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  return 0;
}



char *new_pop3_get_message(int id, int message, int *error, MAILSTREAM *stream ) {
  PopAccount *curpop;
  char *msgbuf = NULL;
  char *snum,*data,*msgbuf2,*ret,*smsg;
  unsigned long stype, ssize;
  int msgsize, hdrsize, bodysize ;
  int index, serror ;
  *error=0;


  ret = NULL ;
  curpop=seek_popaccount(id);

#if DEBUG > 6
  puts( "new_pop3_get_message()\n" ) ;
  printf("*************\nnew pop3msg init: %d    nmsgs: %ld   account:%d\n",message, stream->nmsgs, id ) ;
#endif	
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif
  if (curpop==NULL) {
    *error=NDEFACCOUNT;
    goto error;
  }


  /* make sure we have messages available to fetch, otherwise c-client blows chunks */
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif

  /* Figure out which message to get */
  index = message ;
  msgsize = 0 ;
  
#ifdef DEBUG	
  printf( "fetching header and it's length while using an index value of %d...\n", index ) ;
#endif	

  /* Fetch the data lengths */
  if( (hdrsize = strlen( mail_fetchheader( stream, index ) )) ) {
#ifdef DEBUG	
    printf( "fetching text body and it's length...\n" ) ;
#endif	
    bodysize = strlen( mail_fetchtext( stream, index ) ) ;
    msgsize = hdrsize + bodysize + (1 + 1) ;     // NULL and '\n' stuff
#ifdef DEBUG	
    printf("Counting OK w/%d\n", msgsize ) ;
#endif

#ifdef DEBUG
#ifdef DMALLOC
    dmalloc_verify( 0 ) ;
#endif
    printf( "Getting ready to do a pipe_parent_stat_put with the MSGSIZE...\n" ) ;
#endif

    snum=disassemble_long (((unsigned long)msgsize));
    pipe_parent_stat_put (MSGSIZE, snum, 4);
    free (snum);
#if DEBUG > 4
#ifdef DMALLOC
    dmalloc_verify( 0 ) ;
#endif
    printf("try to setup the msgbuf (size:%d)\n",msgsize);
#endif	
    msgbuf = mail_fetchheader( stream, index ) ;

    msgbuf2 = mail_fetchtext( stream, index ) ;	
    ret = (char *)calloc( msgsize, sizeof(char) ) ;
    sprintf( ret, "%s\n%s", msgbuf, msgbuf2 ) ;
#if DEBUG > 6
#ifdef DMALLOC
    dmalloc_verify( 0 ) ;
#endif
    printf("****we said we put %d of data %d real done!\n",msgsize,strlen(ret));
    printf("do child_stat_get()\n");
#endif	

    /* Get the status from the child */
    data=pipe_child_stat_get( &stype, &ssize, &serror ) ;
#ifdef DMALLOC
    dmalloc_verify( 0 ) ;
#endif

    if (stype==STOP) {
      *error=PARENTSTOP;
      goto error; 
    } else if (stype==SKIP) {
      *error=PARENTSKIP;
      goto error;
    } else if (stype!=-1) {
      pipe_child_stat_put (stype, data, ssize);
    }

    if ((curpop->flags&DELETEFROMSERVER)==DELETEFROMSERVER) {
#ifdef DEBUG     
      printf("Deleting messages from server:\n");
#endif       
      smsg=(char *)calloc(256,sizeof(char));
      sprintf(smsg,"%d",index);
#ifdef DEBUG       
      printf("Flags: %d\n",stream->perm_seen);
#endif       
      mail_setflag (stream,smsg,"\\deleted");
      if(smsg) free(smsg);
    }
    goto done ;

  error:
    if(msgbuf) free(msgbuf);
    ret = NULL ;

  } else {
#ifdef DMALLOC
    dmalloc_verify( 0 ) ;
#endif
#ifdef DEBUG
    printf("Counting FAILED on header and got %d.\n", hdrsize ) ;
#endif	
    *error = 0xfaded ;
  }

 done:
#ifdef DMALLOC
  dmalloc_verify( 0 ) ;
#endif

  return ret ;
}


/*
 * Okay, it works like this:
 * c-client calls this function and so on, and so on...
 * c-client -> post_rfc_output -> tradeclient_rfc822_output ->
 * tradeclient_encode_body; tradeclient_rfc822_header; 
 * tradeclient_rfc822_output_body
 * The end result is that we wind up creating the message
 * within the framework of c-client.
 */
int
post_rfc822_output( char *tmp, ENVELOPE *env, BODY *body, soutr_t f, TCPSTREAM *s ) {
  return( tradeclient_rfc822_output( env, body, f, s ) ) ;
}



int
tradeclient_rfc822_output( ENVELOPE *env, BODY *body, soutr_t f, TCPSTREAM *s ) {
  int retval;


  /* Encode the body and any body parts as needed */
  tradeclient_encode_body( body ) ;

  /* build and output RFC822 header, output body */
  retval = tradeclient_rfc822_header( env, body, f, s ) &&
    (body ? tradeclient_rfc822_output_body( body, f, s ) : 1 ) ;
  
  return( retval ) ;
}



/*
 * This function is heavily influenced by PINE.  Then again, using
 * this model, it's hard to do it much different!
 */
void
tradeclient_encode_body( BODY *body ) {
  int p ;
  PART *bp ;
  time_t t ;
  char tmp[4*1024] ;


  /* See if we have a message body, if so, then we'll do out fancy body encoding */
  if( body ) {
    switch( body -> type ) {
    case TYPEMULTIPART:
      /*
       * if we have a multipart message, create a unique message boundry and then
       * attach the various body parts by walking through the list of nested parts.
       */
      t = time( NULL ) ;
      p = getpid() ;
      if( !body -> parameter ) {
	/* This should always be less than our 4k buffer above! */
	sprintf( tmp, "---TradeClientMIMEBOUNDRY---%ld-%ld-%ld=:%ld", 
		 gethostid(), random()*t/p, t, (long)p ) ;
	body -> parameter = mail_newbody_parameter();
	body -> parameter -> attribute = cpystr( "BOUNDARY" ) ;
	body -> parameter -> value = cpystr( tmp ) ;
      }

      /* Now, encode all of the body parts - nested parts, notice that we are recursive! */
      bp = body -> nested.part ;
      while( bp ) {
	tradeclient_encode_body( &bp -> body ) ;
	bp = bp -> next ;
      }
      break ;

    default:
      /*
       * Like PINE, I don't think we have to do anything at this point because
       * everything is either encoded as MIME (see above) else it's quoted
       * printable?  Need to verify this.
       */
#if DEBUG > 4
      printf( "Got a body type of %d when looking to MIME encode.\n", body -> type ) ;
#endif
      break ;
    }
  }
  return ;
}



int
tradeclient_rfc822_header( ENVELOPE *env, BODY *body, soutr_t f, TCPSTREAM *s ) {

  return 0 ;
}



long
tradeclient_rfc822_output_body( BODY *body, soutr_t f, TCPSTREAM *s ) {
#if 0
  PART *part ;
  char *delim ;
  PARAMETER *parm ;
  char temp[MAILTMPLEN] ;
#endif

#if 0
  if( body -> type == TYPEMULTIPART ) {
    part = body -> nested.part ;

    for( parm = body -> parameter ; parm ; parm = parm -> next )
      if( ) ;
    /*
     * Output verbose text explaining this is MIME and what it is.
     */
    if( f && !(*f)( s, "This message has been encoding in MIME format.  If you are reading this,\015\012then your email client does not support this format.  As such, it is\015\012recommended that you contact your system administrator to find out how to\015\012obtain a MIME compliant email client or contact the message originator and\015\012request this email be resent in a format that is supported by your email client.\015\012\015\012" ) )
      return 0 ;

    do {
      
#endif

  return 0L ;
}


