/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: pref-loadsave.c,v 1.28 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "pref-loadsave.h"
#include "hashtime.h"

// MEMWATCH
#ifdef MEMWATCH
#include "memwatch.h"
#endif

// DMALLOC
#ifdef DMALLOC
#include "dmalloc.h"
#endif

#ifdef write
#undef write
#endif
char sep='\03';
#define EOW	'\04'

void make_default_prefrances () {
	char *home;
	char *tmp;
	char *sep=(char *)calloc(2, sizeof(char));
	char *tmps;
	unsigned long tmpflags;
	int fd;

	sep[0]=EOW;

	home=(char *)calloc(1024, sizeof(char));
	snprintf (home, 1024, "$(HOME)");
	env_srch_rep(home);

	tmp=(char *)calloc(1024, sizeof(char));

	snprintf (tmp, 1024, "%s/.tradeclient", home);
	mkdir (tmp, S_IREAD | S_IWRITE | S_IEXEC);

	snprintf (tmp, 1024, "%s/.tradeclient/Addressbooks", home);
	mkdir (tmp, S_IREAD | S_IWRITE | S_IEXEC);

	snprintf (tmp, 1024, "%s/.tradeclient/Inbox", home);
	if (access (tmp, R_OK)) {
		fd=open (tmp, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);
		close (fd);

		snprintf (tmp, 1024, "%s/.tradeclient/.Inbox.smi", home);
		fd=open (tmp, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);
		write (fd, sep, 1);
		tmpflags=CAN_STORE_NEW_MAIL|CAN_HAVE_SUBFOLDERS;
		tmps=disassemble_long (tmpflags);
		write (fd, tmps, 4);
		free (tmps);
		close (fd);
	}

	snprintf (tmp, 1024, "%s/.tradeclient/Outbox", home);
	if (access (tmp, R_OK)) {
		fd=open (tmp, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);
		close (fd);

		snprintf (tmp, 1024, "%s/.tradeclient/.Outbox.smi", home);
		fd=open (tmp, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);
		write (fd, sep, 1);
		tmpflags=CAN_HAVE_SUBFOLDERS;
		tmps=disassemble_long (tmpflags);
		write (fd, tmps, 4);
		free (tmps);
		close (fd);
	}

	snprintf (tmp, 1024, "%s/.tradeclient/Sent-Items", home);
	if (access (tmp, R_OK)) {
		fd=open (tmp, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);
		close (fd);

		snprintf (tmp, 1024, "%s/.tradeclient/.Sent-Items.smi", home);
		fd=open (tmp, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);
		write (fd, sep, 1);
		tmpflags=CAN_HAVE_SUBFOLDERS;
		tmps=disassemble_long (tmpflags);
		write (fd, tmps, 4);
		free (tmps);
		close (fd);
	}

	snprintf (tmp, 1024, "%s/.tradeclient/Trash", home);
	if (access (tmp, R_OK)) {
		fd=open (tmp, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);
		close (fd);

		snprintf (tmp, 1024, "%s/.tradeclient/.Trash.smi", home);
		fd=open (tmp, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);
		write (fd, sep, 1);
		tmpflags=CAN_HAVE_SUBFOLDERS;
		tmps=disassemble_long (tmpflags);
		write (fd, tmps, 4);
		free (tmps);
		close (fd);
	}

	free (tmp);
	free (home);
	free (sep);
}

void password_hide (char *pw) {
	int i=0, lenght=0;
#if DEBUG > 6
	char *bits;
#endif
	if (pw!=NULL) {
		lenght=strlen(pw);
#if DEBUG > 6
		printf ("before hide pw=%s\n", pw);
#endif
		for (i=0; i<lenght; i++) {
#if DEBUG > 6
			bits=bits8(pw[i]);
			printf ("%d: %s:%d\t<<\t", i, bits, (unsigned int)pw[i]);
			free (bits);
#endif
			if (pw[i]!='\0') {
				pw[i]=(char)((pw[i])<<1);
			}
#if DEBUG > 6
			bits=bits8(pw[i]);
			printf ("%s:%d\n", bits, (unsigned int)pw[i]);
			free (bits);
#endif
		}
#if DEBUG > 6
		printf ("after hide pw=%s\n", pw);
#endif
	}
}

void password_unhide (char *pw) {
	int i=0, lenght=0;
#if DEBUG > 6
	char *bits;
#endif
	if (pw!=NULL) {
		lenght=strlen(pw);
#if DEBUG > 6
		printf ("before unhide pw=%s\n", pw);
#endif
		for (i=0; i<lenght; i++) {
#if DEBUG > 6
			bits=bits8(pw[i]);
			printf ("%d: %s:%d\t>>\t", i, bits, (unsigned int)pw[i]);
			free (bits);
#endif
			if (pw[i]!='\0') {
				pw[i]=(char)((pw[i])>>1)&0x7f;
			}
#if DEBUG > 6
			bits=bits8(pw[i]);
			printf ("%s:%d\n", bits, (unsigned int)pw[i]);
			free (bits);
#endif
		}
#if DEBUG > 6
		printf ("after unhide pw=%s\n", pw);
#endif
	}
}


void add_popaccount_char (char *text) {
	char *name=NULL;
	char *phost=NULL;
	char *puser=NULL;
	char *ppasswd=NULL;
	int pport=-1;
	char *shost=NULL;
	char *suser=NULL;
	int sport=-1;
	unsigned long flags=0;
	int timeout=60;
	char *folder=NULL;
	char *real_name=NULL;
	char *emailaddr=NULL;
	char *replyaddr=NULL;
	char *organization=NULL;
	char *signature=NULL;
	char *attachdir;
	char *tmps=NULL, *tmpsi=NULL;
	int popid;
	int lastMsgCount ;
	int thetype;
	PopAccount *account;
	

#if DEBUG > 4
	printf ("Adding an account from global prefs file\n");
#endif

	name=cut_string(text, sep, 1, 1);

	tmps=cut_string(text, sep, 2, 1);
	phost=cut_string(tmps, sep, 1, 1);
	if (tmps) free (tmps);
	
	tmps=cut_string(text, sep, 2, 2);
	puser=cut_string(tmps, sep, 1, 1);
	if (tmps) free(tmps);

	tmps=cut_string(text, sep, 2, 3);
	ppasswd=cut_string(tmps, sep, 1, 1);
	password_unhide(ppasswd);
	if (tmps) free(tmps);

	tmps=cut_string(text, sep, 2, 4);
	tmpsi=cut_string (tmps, sep, 1, 1);
	if (tmpsi!=NULL) {
		pport=atoi(tmpsi);
	}	else {
		pport=110;
	}
	if (tmps) free(tmps);
	if (tmpsi) free(tmpsi);

	tmps=cut_string(text, sep, 2, 5);
	shost=cut_string(tmps, sep, 1, 1);
	if (tmps) free(tmps);

	tmps=cut_string(text, sep, 2, 6);
	suser=cut_string(tmps, sep, 1, 1);
	if (tmps) free(tmps);

	tmps=cut_string(text, sep, 2, 7);
	tmpsi=cut_string (tmps, sep, 1, 1);
	if (tmpsi!=NULL) {
		sport=atoi(tmpsi);
	} else {
		sport=25;
	}
	if (tmpsi) free (tmpsi);
	if (tmps) free(tmps);	

	tmps=cut_string(text, sep, 2, 8);
	tmpsi=cut_string (tmps, sep, 1, 1);
	if (tmps!=NULL) {
		flags=atol(tmpsi);
	} else {
		flags=0;
	}
	if (tmpsi) free (tmpsi);
	if (tmps) free (tmps);

	tmps=cut_string(text, sep, 2, 9);
	tmpsi=cut_string (tmps, sep, 1, 1);
	if (*tmps!='\0') {
		timeout=atoi(tmpsi);
	} else {
		timeout=-1;
	}
	if (tmpsi) free (tmpsi);
	if (tmps) free (tmps);

	tmps=cut_string(text, sep, 2, 10);
	folder=cut_string(tmps, sep, 1, 1);
	if (tmps) free(tmps);

	tmps=cut_string(text, sep, 2, 11);
	real_name=cut_string(tmps, sep, 1, 1);
	if (tmps) free(tmps);

	tmps=cut_string(text, sep, 2, 12);
	emailaddr=cut_string(tmps, sep, 1, 1);
	if (tmps) free(tmps);

	tmps=cut_string(text, sep, 2, 13);
	replyaddr=cut_string(tmps, sep, 1, 1);
	if (tmps) free(tmps);

	tmps=cut_string(text, sep, 2, 14);
	organization=cut_string(tmps, sep, 1, 1);
	if (tmps) free (tmps);

	tmps=cut_string(text, sep, 2, 15);
	signature=cut_string(tmps, sep, 1, 1);
	if (tmps) free (tmps);

	tmps=cut_string( text, sep, 2, 16 ) ;
	attachdir=cut_string(tmps, sep, 1, 1);
	if( tmps ) free( tmps ) ;

	tmps=cut_string(text, sep, 2, 17);
	tmpsi=cut_string (tmps, sep, 1, 1);
	if (tmpsi!=NULL) {
	  lastMsgCount = atoi( tmpsi ) ;
	} else {
	  lastMsgCount = 0 ;
	}
	if (tmpsi) free (tmpsi);
	if (tmps) free(tmps);	

	tmps=cut_string (text, sep, 2, 18);
	tmpsi=cut_string (tmps, sep, 1, 1);
	if (tmpsi != NULL) {
		thetype = atoi (tmpsi);
	} else {
		thetype = POP3;
	}
	if (tmpsi) free (tmpsi);
	if (tmps) free(tmps);

	if (name[0]=='\0') {
		free (name);
		name=NULL;
	}

	if (phost[0]=='\0') {
		free (phost);
		phost=NULL;
	}

	if (puser[0]=='\0') {
		free (puser);
		puser=NULL;
	}

	if (ppasswd[0]=='\0') {
		free (ppasswd);
		ppasswd=NULL;
	}

	if (shost[0]=='\0') {
		free (shost);
		shost=NULL;
	}

	if (suser[0]=='\0') {
		free (suser);
		suser=NULL;
	}

	if (folder[0]=='\0') {
		free (folder);
		folder=NULL;
	}

	if (real_name[0]=='\0') {
		free (real_name);
		real_name=NULL;
	}

	if (emailaddr[0]=='\0') {
		free (emailaddr);
		emailaddr=NULL;
	}

	if (replyaddr[0]=='\0') {
		free (replyaddr);
		replyaddr=NULL;
	}

	if (organization[0]=='\0') {
		free (organization);
		organization=NULL;
	}

	if (signature[0]=='\0') {
		free (signature);
		signature=NULL;
	}
	

	if (attachdir[0]=='\0') {
		free (attachdir);
		attachdir=NULL;
	}

	popid=new_popaccount (name, phost, puser, ppasswd, pport, shost, suser,
	                      sport, flags, timeout, folder, real_name, emailaddr,
	                      replyaddr, organization, signature, attachdir, lastMsgCount, NULL);

	account=seek_popaccount (popid);
	account->type=thetype;
	tm_globs->num_accounts++;

	if (name!=NULL) free (name);
	if (phost!=NULL) free (phost);
	if (puser!=NULL) free (puser);
	if (ppasswd!=NULL) free (ppasswd);
	if (shost!=NULL) free (shost);
	if (suser!=NULL) free (suser);
	if (folder!=NULL) free (folder);
	if (real_name!=NULL) free (real_name);
	if (emailaddr!=NULL) free (emailaddr);
	if (replyaddr!=NULL) free (replyaddr);
	if (organization!=NULL) free (organization);
	if (signature!=NULL) free (signature);
	if (attachdir!=NULL) free (attachdir);
}

void set_global_real_name (char *var) {
	tm_globs->real_name=var;
}

void set_global_email (char *var) {
	tm_globs->email=var;
}

void set_global_replyto (char *var) {
	tm_globs->replyto=var;
}

void set_global_organization (char *var) {
	tm_globs->organization=var;
}

void set_global_signature (char *var) {
	tm_globs->signature=var;
}

void set_default_account (char *var) {
	tm_globs->default_account=var;
}


void
set_printer (char *var)
{
	tm_globs->printer = var;
	return;
}

#ifdef WANTLDAP
void
set_ldap_host( char *var ) {
  tm_globs -> ldap_host = var ;
  return ;
}


void
set_ldap_port( char *var ) {
  tm_globs -> ldap_port = atoi(var) ;
  if( tm_globs -> ldap_port == 0 ) tm_globs -> ldap_port = 389 ;
  return ;
}

void
set_ldap_username( char *var ) {
  tm_globs -> ldap_username = var ;
  return ;
}


/*
 * This one is used when we read from a file.  It fully
 * expects that the password is encrypted.
 */
void
set_ldap_password( char *var ) {
  if( var ) {
    password_unhide( var ) ;
    tm_globs -> ldap_password = var ;
  }
  return ;
}

/*
 * This is the plain text function that is used when the
 * password is update from a dialog box.
 */
void
set_ldap_ppassword( char *var ) {
  if( var ) {
    tm_globs -> ldap_password = var ;
  }
  return ;
}


void
set_ldap_base( char *var ) {
  tm_globs -> ldap_base = var ;
  return ;
}
#endif

void set_wasted_space (int *value) {
	int val = *value;
	tm_globs->wasted = val;
	return;
}

void add_mime_type_char (char *text) {
	char *type, *handle, *ext;
	char *tmps;
	int i, j, len;
	if (text) {

		type=cut_string (text, ';', 1, 1);
		if (type) {
			len=strlen (type);
			for (i=0; i<len; i++) {
				if (type[i]==' ') {
					for (j=0; j<len-i; j++) {
						type[i+j]=type[i+j+1];
					}
					i--;
				}
			}
		}

		tmps=cut_string (text, ';', 2, 1);
		handle=cut_string (tmps, ';', 1, 1);
		if (handle) {
			len=strlen (handle);
			if (handle[0]==' ') {
				for (i=0; i<len; i++) {
					handle[i]=handle[i+1];
				}
			}
		}
		if (tmps) free (tmps); tmps=NULL;

		tmps=cut_string (text, ';', 2, 2);
		ext=cut_string (tmps, ';', 1, 1);
		if (ext) {
			len=strlen (ext);
			for (i=0; i<len; i++) {
				if (ext[i]==' ') {
					for (j=0; j<len-i; j++) {
						ext[i+j]=ext[i+j+1];
					}
					i--;
				}
			}
		}
		if (tmps) free (tmps); tmps=NULL;

		mime_type_add (type, handle, ext);

		if (type) free (type);
		if (handle) free (handle);
		if (ext) free (ext);
	}
}

int load_preferences (void) {
	int i ;
	char *rcfile=(char *)calloc(1024, sizeof(char));
#ifdef WANTLDAP
#ifdef WANTSSL
	int kwn=21;
#else
	int kwn=20;
#endif
#else
	int kwn=15;
#endif
	PopAccount *fseek;
	keyword *kwlist[kwn];

	memmove (rcfile, "$(HOME)/.tradeclient/.tradeclientrc", strlen("$(HOME)/.tradeclient/.tradeclientrc"));

	kwlist[0]=set_option( (const struct keyword) {"popaccount", "char*", NULL, (void *)add_popaccount_char} );
	kwlist[1]=set_option( (const struct keyword) {"mime_type", "char*", NULL, (void *)add_mime_type_char} );
	kwlist[2]=set_option( (const struct keyword) {"default_account", "char*", NULL, (void *)set_default_account} );
	kwlist[3]=set_option( (const struct keyword) {"global_real_name", "char*", NULL, (void *)set_global_real_name} );
	kwlist[4]=set_option( (const struct keyword) {"global_email", "char*", NULL, (void *)set_global_email} );
	kwlist[5]=set_option( (const struct keyword) {"global_replyto", "char*", NULL, (void *)set_global_replyto} );
	kwlist[6]=set_option( (const struct keyword) {"global_organization", "char*", NULL, (void *)set_global_organization} );
	kwlist[7]=set_option( (const struct keyword) {"global_signature", "char*", NULL, (void *)set_global_signature} );
	kwlist[8]=set_option( (const struct keyword) {"global_flags", "int", &tm_globs->flags, NULL} );
	kwlist[9]=set_option( (const struct keyword) {"global_wasted_space", "int", NULL, (void *)set_wasted_space} );
	kwlist[10]=set_option( (const struct keyword) {"filter", "char*", NULL, (void *)filter_add_rule} );
	kwlist[11]=set_option( (const struct keyword) {"wsize", "char*", NULL, (void *)window_sizes_set} );
	kwlist[12]=set_option( (const struct keyword) {"ppos", "char*", NULL, (void *)paned_pos_set} );
	kwlist[13]=set_option( (const struct keyword) {"cpos", "char*", NULL, (void *)clist_pos_set} );
	kwlist[14]=set_option( (const struct keyword) {"printer", "char*", NULL, (void *)set_printer} );
#ifdef WANTLDAP
	kwlist[15]=set_option( (const struct keyword) {"ldap_host", "char*", NULL, (void *)set_ldap_host} );
	kwlist[16]=set_option( (const struct keyword) {"ldap_username", "char*", NULL, (void *)set_ldap_username} );
	kwlist[17]=set_option( (const struct keyword) {"ldap_password", "char*", NULL, (void *)set_ldap_password} );
	kwlist[18]=set_option( (const struct keyword) {"ldap_port", "int", &tm_globs -> ldap_port} ) ;
	kwlist[19]=set_option( (const struct keyword) {"ldap_base", "char*", NULL, (void *)set_ldap_base} );
#ifdef WANTSSL
	kwlist[20]=set_option( (const struct keyword) {"ldap_ssl", "int", &tm_globs -> ldap_ssl} ) ;
#endif
#endif
	env_srch_rep (rcfile);

	if (parse(rcfile, kwlist, kwn)==FALSE) {
		preferences_rehash_interface ();
		free (rcfile);

		/* Free the memory that just got consumed */
		for( i = 0 ; i < kwn ; i++ ) free (kwlist[i]);

		return -1;
	}

	window_sizes_rehash();
	paned_pos_rehash();
	clist_pos_rehash();
	preferences_rehash_interface ();
	fseek=first_popaccount ();

	/* Free the memory that just got consumed */
	for( i = 0 ; i < kwn ; i++ ) free (kwlist[i]);

	free (rcfile);
	return 0;
}

char *popaccounts_to_string (PopAccount *curpop) {
	int length;
	char *ret;
	char *name=NULL;
	char *phost=NULL;
	char *puser=NULL;
	char *ppasswd=NULL;
	char *pport=NULL;
	char *shost=NULL;
	char *suser=NULL;
	char *sport=NULL;
	char *flags=NULL;
	char *timeout=NULL;
	char *folder=NULL;
	char *real_name=NULL;
	char *emailaddr=NULL;
	char *replyaddr=NULL;
	char *organization=NULL;
	char *signature=NULL;
	char *attachdir=NULL;
	char *eport=NULL;
	char *enc_type=NULL;
	char *compress_type=NULL;
	char *auth_type=NULL;
	char *msgCount = NULL ;
	char *thetype=NULL;


	if (curpop->name!=NULL) {
		name=strdup(curpop->name);
	} else {
		name=(char *)calloc(2, sizeof(char));
	}

	if (curpop->phost!=NULL) {
		phost=strdup(curpop->phost);
	} else {
		phost=(char *)calloc(2, sizeof(char));
	}

	if (curpop->puser!=NULL) {
		puser=strdup(curpop->puser);
	} else {
		puser=(char *)calloc(2, sizeof(char));
	}


	if (((curpop->flags&REMBER_PASSWORD_ALWAYS)==REMBER_PASSWORD_ALWAYS)&&
	    (curpop->ppasswd!=NULL)) {
		ppasswd=strdup(curpop->ppasswd);
		password_hide(ppasswd);
	} else {
		ppasswd=(char *)calloc(2, sizeof(char));
	}

	if (curpop->pport>0) {
		pport=(char *)calloc(6, sizeof(char));
		snprintf (pport, 6, "%d", curpop->pport);
	} else {
		pport=(char *)calloc(4, sizeof(char));
		pport[0]='1';
		pport[1]='1';
		pport[2]='0';
	}

	if (curpop->shost!=NULL) {
		shost=strdup(curpop->shost);
	} else {
		shost=(char *)calloc(2, sizeof(char));
	}

	if (curpop->suser!=NULL) {
		suser=strdup(curpop->suser);
	} else {
		suser=(char *)calloc(2, sizeof(char));
	}

	if (curpop->sport>0) {
		sport=(char *)calloc(6, sizeof(char));
		snprintf (sport, 6, "%d", curpop->sport);
	} else {
		sport=(char *)calloc(3, sizeof(char));
		sport[0]='2';
		sport[1]='5';
	}

	if (curpop->flags>0) {
		flags=(char *)calloc(6, sizeof(char));
		snprintf (flags, 6, "%d", (int)curpop->flags);
	} else {
		flags=(char *)calloc(2, sizeof(char));
		*flags='0';
	}

	if (curpop->timeout>=0) {
		timeout=(char *)calloc(6, sizeof(char));
		snprintf (timeout, 6, "%d", curpop->timeout);
	} else {
		timeout=(char *)calloc(3, sizeof(char));
		timeout[0]='6';
		timeout[1]='0';
	}

	if (curpop->folder!=NULL) {
		folder=strdup(curpop->folder);
	} else {
		folder=(char *)calloc(2, sizeof(char));
	}

	if (curpop->real_name!=NULL) {
		real_name=strdup(curpop->real_name);
	} else {
		real_name=(char *)calloc(2, sizeof(char));
	}

	if (curpop->emailaddr!=NULL) {
		emailaddr=strdup(curpop->emailaddr);
	} else {
		emailaddr=(char *)calloc(2, sizeof(char));
	}

	if (curpop->replyaddr!=NULL) {
		replyaddr=strdup(curpop->replyaddr);
	} else {
		replyaddr=(char *)calloc(2, sizeof(char));
	}

	if (curpop->organization!=NULL) {
		organization=strdup(curpop->organization);
	} else {
		organization=(char *)calloc(2, sizeof(char));
	}

	if (curpop->signature!=NULL) {
		signature=strdup(curpop->signature);
	} else {
		signature=(char *)calloc(2, sizeof(char));
	}

	if (curpop->attachdir!=NULL) {
		attachdir=strdup(curpop->attachdir);
	} else {
		attachdir=(char *)calloc(2, sizeof(char));
	}


	/* Save the message counter!!! Very important for non-delete mode accounts! */
	msgCount = (char *)malloc( 8*sizeof(char) ) ;
	snprintf( msgCount, 8, "%07d", curpop -> popcmsg ) ;

        thetype=(char *)calloc(5, sizeof(char));
        snprintf (thetype, 5, "%03d", curpop->type);

#if DEBUG > 6
	printf ("** popaccounts_to_string () **\n");
	printf ("name: %s\n",name);
	printf ("phost: %s\n",phost);
	printf ("puser: %s\n",puser);
	printf ("ppasswd: NOT VISABLE");
	printf ("pport: %s\n",pport);
	printf ("shost: %s\n",shost);
	printf ("suser: %s\n",suser);
	printf ("sport: %s\n",sport);
	printf ("flags: %s\n",flags);
	printf ("folder: %s\n",folder);
	printf ("real_name: %s\n",real_name);
	printf ("emailaddr: %s\n",emailaddr);
	printf ("replyaddr: %s\n",replyaddr);
	printf ("organization: %s\n",organization);
	printf ("signature: %s\n",signature);
	printf ("attachdir: %s\n",attachdir);
	printf ("msgCount: %s\n",msgCount);
	printf ("type: %s\n",thetype);
	printf ("******************************\n");
#endif

	/* I don't think the exta junk is needed anymore!!! */
	if (curpop->extra) {
		length=strlen(name)+strlen(phost)+strlen(puser)+strlen(ppasswd)+
		       strlen(pport)+strlen(shost)+strlen(suser)+strlen(sport)+
		       strlen(flags)+strlen(timeout)+strlen(folder)+strlen(real_name)+
		       strlen(emailaddr)+strlen(replyaddr)+strlen(organization)+
		       strlen(signature)+strlen(attachdir)+strlen(eport)+strlen(enc_type)+
		       strlen(compress_type)+strlen(auth_type)+strlen(msgCount)+25;

		ret=(char *)calloc(length, sizeof(char));

		snprintf (ret, length, "\"%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03\"",
		          name,phost,puser,ppasswd,pport,shost,suser,sport,flags,timeout,
		          folder,real_name,emailaddr,replyaddr,organization,signature,
		          attachdir,msgCount,eport,enc_type,compress_type,auth_type);
	} else {
	/* dont try to understand, just accept */
		length=strlen(name)+strlen(phost)+strlen(puser)+strlen(ppasswd)+
		       strlen(pport)+strlen(shost)+strlen(suser)+strlen(sport)+
		       strlen(flags)+strlen(timeout)+strlen(folder)+strlen(real_name)+
		       strlen(emailaddr)+strlen(replyaddr)+strlen(organization)+
		       strlen(signature)+strlen(attachdir)+strlen(msgCount)+strlen(thetype)+22 ;
		ret=(char *)calloc(length, sizeof(char));
		snprintf (ret, length, "\"%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03%s\03\"",
		          name,phost,puser,ppasswd,pport,shost,suser,sport,flags,timeout,
		          folder,real_name,emailaddr,replyaddr,organization,signature,attachdir,msgCount,thetype);
	}

	free (name);
	free (phost);
	free (puser);
	free (ppasswd);
	free (pport);
	free (shost);
	free (suser);
	free (sport);
	free (flags);
	free (folder);
	free (real_name);
	free (emailaddr);
	free (replyaddr);
	free (organization);
	free (signature);
	free (timeout);
	free (attachdir);
	free( msgCount ) ;
	free (thetype);

	if (eport) free (eport);
	if (enc_type) free (enc_type);
	if (compress_type) free (compress_type);
	if (auth_type) free (auth_type);

	return ret;
}

int save_preferences () {
  PopAccount *poplist=first_popaccount ();
  Llist *mimetypelist=mime_type_first ();
  MimeType *mtype;
  //	FilterRule *rseek;
  GList *seek;
  int fd, len;
  char buffer[256];
  char *dir=(char *)calloc(1024, sizeof(char));
  char *file1=(char *)calloc(1024, sizeof(char));
  char *file2=(char *)calloc(1024, sizeof(char));
  char *tmpstr1;
  char *tmpstr2;

	
  snprintf (dir, 1024, "$(HOME)/.tradeclient");
  env_srch_rep(dir);
  mkdir(dir, S_IREAD | S_IWRITE | S_IEXEC);
  free (dir);

  snprintf (file1,1024, "$(HOME)/.tradeclient/.tradeclientrc~");
  snprintf (file2,1024, "$(HOME)/.tradeclient/.tradeclientrc");
  env_srch_rep(file1);
  env_srch_rep(file2);

  fd=open (file1, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR);

  tmpstr1=(char *)calloc(1024, sizeof(char));

  /* Might need this one day for backward compatibility */
  snprintf (tmpstr1, 1024, "# Generated by TradeClient %s, DO NOT EDIT.\n", VERSION );
  write (fd, tmpstr1, strlen (tmpstr1));

  free (tmpstr1);
  popaccounts_dump_em_all ("Top of save preferences");

  while (poplist) {
    tmpstr1=popaccounts_to_string (poplist);
    tmpstr2=(char *)calloc(strlen(tmpstr1)+13, sizeof(char));
    snprintf (tmpstr2, strlen(tmpstr1)+13,"popaccount=%s\n",tmpstr1);
    write (fd, tmpstr2, strlen(tmpstr2));
    free (tmpstr1);
    free (tmpstr2);
    poplist=(PopAccount *)poplist->next;
  }

  popaccounts_dump_em_all ("Bottom of save preferences");

  write (fd, "\n", 1);
  for (seek=filter_first ();seek;seek=seek->next) {
    write (fd, "filter=\"", 8);
    tmpstr1=filter_to_char (seek->data);
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);
    write (fd, "\"\n", 2);
  }
  write (fd, "\n", 1);

  while (mimetypelist) {
    mtype=((MimeType *)mimetypelist->data);
    write (fd, "mime_type=\"", 11);
    if (mtype->type)
      write (fd, mtype->type, strlen(mtype->type));
    write (fd, "; ", 2);
    if (mtype->handle)
      write (fd, mtype->handle, strlen(mtype->handle));
    write (fd, "; ", 2);
    if (mtype->ext)
      write (fd, mtype->ext, strlen(mtype->ext));
    write (fd, ";\"\n", 3);
    mimetypelist=mimetypelist->next;
  }

  write (fd, "\n", 1);

  if (tm_globs->default_account) {
    len=strlen(tm_globs->default_account)+30;
    tmpstr1=(char *)calloc(len, sizeof(char));
    snprintf (tmpstr1, len,"default_account=\"%s\"\n",tm_globs->default_account);
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);
  }

  if (tm_globs->real_name) {
    len=strlen(tm_globs->real_name)+30;
    tmpstr1=(char *)calloc(len, sizeof(char));
    snprintf (tmpstr1, len,"global_real_name=\"%s\"\n",tm_globs->real_name);
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);
  }

  if (tm_globs->email) {
    len=strlen(tm_globs->email)+30;
    tmpstr1=(char *)calloc(len, sizeof(char));
    snprintf (tmpstr1, len,"global_email=\"%s\"\n",tm_globs->email);
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);
  }

  if (tm_globs->replyto) {
    len=strlen(tm_globs->replyto)+30;
    tmpstr1=(char *)calloc(len, sizeof(char));
    snprintf (tmpstr1, len,"global_replyto=\"%s\"\n",tm_globs->replyto);
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);
  }

  if (tm_globs->organization) {
    len=strlen(tm_globs->organization)+30;
    tmpstr1=(char *)calloc(len, sizeof(char));
    snprintf (tmpstr1, len,"global_organization=\"%s\"\n",tm_globs->organization);
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);
  }

  if (tm_globs->signature) {
    len=strlen(tm_globs->signature)+30;
    tmpstr1=(char *)calloc(len, sizeof(char));
    snprintf (tmpstr1, len,"global_signature=\"%s\"\n",tm_globs->signature);
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);
  }

  len=30;
  tmpstr1=(char *)calloc(len, sizeof(char));
  snprintf (tmpstr1, len,"global_flags=%d\n",(int)tm_globs->flags);
  write (fd, tmpstr1, strlen(tmpstr1));
  free (tmpstr1);

  len=40;
  tmpstr1=(char *)calloc(len, sizeof(char));
  snprintf (tmpstr1, len,"global_wasted_space=%d\n",(int)tm_globs->wasted);
  write (fd, tmpstr1, strlen(tmpstr1));
  free (tmpstr1);

  if (tm_globs->printer)
  {
    len=strlen(tm_globs -> printer) ;
    if( len ) {
      len += 4096 ;
      tmpstr1=(char *)malloc(len*sizeof(char)) ;
      snprintf (tmpstr1, len, "printer=\"%s\"\n", tm_globs -> printer ) ;
      write (fd, tmpstr1, strlen(tmpstr1));
      free (tmpstr1);
    }
  }

#ifdef WANTLDAP
  if( tm_globs -> ldap_host ) {
    len=strlen(tm_globs -> ldap_host) ;
    if( len ) {
      len += 4096 ;
      tmpstr1=(char *)malloc(len*sizeof(char)) ;
      snprintf (tmpstr1, len, "ldap_host=\"%s\"\n", tm_globs -> ldap_host ) ;
      write (fd, tmpstr1, strlen(tmpstr1));
      free (tmpstr1);
    }
  }

  if( tm_globs -> ldap_username ) {
    len=strlen(tm_globs -> ldap_username) ;
    if( len ) {
      len += 4096 ;
      tmpstr1=(char *)malloc(len*sizeof(char)) ;
      snprintf (tmpstr1, len, "ldap_username=\"%s\"\n", tm_globs -> ldap_username ) ;
      write (fd, tmpstr1, strlen(tmpstr1));
      free (tmpstr1);
    }
  }

  if( tm_globs -> ldap_password ) {
    len=strlen(tm_globs->ldap_password) ;
    if( len ) {
      len += 4096 ;
      tmpstr1=(char *)malloc(len*sizeof(char));
      tmpstr2 = strdup( tm_globs -> ldap_password ) ;
      password_hide( tmpstr2 ) ;
      snprintf (tmpstr1, len, "ldap_password=\"%s\"\n", tmpstr2 ) ;
      write (fd, tmpstr1, strlen(tmpstr1));
      free( tmpstr1 ) ;
      free( tmpstr2 ) ;
    }
  }

  /* We'll allow the port to ALWAYS be written out.  No problem. */
  if( tm_globs -> ldap_port ) {
    len=4 + 4096 ;
    tmpstr1=(char *)malloc(len*sizeof(char));
    snprintf (tmpstr1, len, "ldap_port=\"%d\"\n", tm_globs -> ldap_port ) ;
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);
  }

#ifdef WANTSSL
  if( tm_globs -> ldap_ssl ) {
    len=4 + 4096 ;
    tmpstr1=(char *)malloc(len*sizeof(char));
    snprintf (tmpstr1, len, "ldap_ssl=\"%d\"\n", tm_globs -> ldap_ssl ) ;
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);
  }
#endif

  if( tm_globs -> ldap_base ) {
    len=strlen(tm_globs->ldap_base) ;
    if( len ) {
      len += 4096 ;
      tmpstr1=(char *)malloc(len*sizeof(char));
      snprintf (tmpstr1, len, "ldap_base=\"%s\"\n", tm_globs -> ldap_base ) ;
      write (fd, tmpstr1, strlen(tmpstr1));
      free (tmpstr1);
    }
  }
#endif

    snprintf(buffer, sizeof(buffer), "1 %d %d %d %d %d %d %d %d %d %d",
                tm_globs->wsize.main_window.x,
                tm_globs->wsize.main_window.y,
                tm_globs->wsize.new_message_window.x,
                tm_globs->wsize.new_message_window.y,
                tm_globs->wsize.message_window.x,
                tm_globs->wsize.message_window.y,
                tm_globs->wsize.calendar_window.x,
                tm_globs->wsize.calendar_window.y,
                tm_globs->wsize.msearch_window.x,
                tm_globs->wsize.msearch_window.y);
    len=strlen(buffer);
    len += 4096;
    tmpstr1=(char *)malloc(len*sizeof(char));
    snprintf (tmpstr1, len, "wsize=\"%s\"\n", buffer);
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);

    snprintf(buffer, sizeof(buffer), "1 %d %d %d %d",
                tm_globs->ppos.mnw_hp1,
                tm_globs->ppos.mnw_vp1,
                tm_globs->ppos.mnw_vp2,
                tm_globs->ppos.msgw_vp1);
    len=strlen(buffer);
    len += 4096;  
    tmpstr1=(char *)malloc(len*sizeof(char));
    snprintf (tmpstr1, len, "ppos=\"%s\"\n", buffer);
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);

    snprintf(buffer, sizeof(buffer), "1 %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
		tm_globs->cpos.mw_fl[0],
		tm_globs->cpos.mw_fl[1],
		tm_globs->cpos.mw_fl[2],
		tm_globs->cpos.mw_msg[0],
		tm_globs->cpos.mw_msg[1],
		tm_globs->cpos.mw_msg[2],
		tm_globs->cpos.mw_msg[3],
		tm_globs->cpos.mw_msg[4],
		tm_globs->cpos.mw_msg[5],
		tm_globs->cpos.mw_mime[0],
		tm_globs->cpos.mw_mime[1],
		tm_globs->cpos.mw_mime[2],
		tm_globs->cpos.msgw[0],
		tm_globs->cpos.msgw[1],
		tm_globs->cpos.msgw[2],
		tm_globs->cpos.nmsgw[0],
		tm_globs->cpos.nmsgw[1],
		tm_globs->cpos.nmsgw[2],
		tm_globs->cpos.ab[0],
		tm_globs->cpos.ab[1],
                tm_globs->cpos.msw[0],
                tm_globs->cpos.msw[1],
                tm_globs->cpos.msw[2],
                tm_globs->cpos.msw[3],
                tm_globs->cpos.msw[4],
                tm_globs->cpos.msw[5]);
    len=strlen(buffer);
    len += 4096;
    tmpstr1=(char *)malloc(len*sizeof(char));
    snprintf (tmpstr1, len, "cpos=\"%s\"\n", buffer);
    write (fd, tmpstr1, strlen(tmpstr1));
    free (tmpstr1);



  close (fd);
  unlink (file2);
  rename (file1,file2);
  free (file1);
  free (file2);

  return 0;
}


int
save_hashkey( const unsigned char *hash ) {
  int fd ;
  int ret ;
  char tmp[4096] ;
  char hashString[13] ;
  unsigned char rawHash[6] ;


  /* Make our resource directory if needed */
  snprintf( tmp, 4095, "$(HOME)/.tradeclient" ) ;
  env_srch_rep( tmp ) ;
  mkdir( tmp, S_IREAD | S_IWRITE | S_IEXEC ) ;

  strncat( tmp, ".tradehashkey", 4095 ) ;
  fd = open( tmp, O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR ) ;

  snprintf( tmp, 4095, "# Generated by TradeClient %s, DO NOT EDIT.\n" \
	    "# Editing this file will invalidate all account passwords.\n\n", VERSION ) ;
  ret = write( fd, tmp, strlen(tmp) ) ;

  /* If we don't have a hash, create one first! */
  if( hash == (unsigned char *)NULL ) {
    hashTime( rawHash ) ;
    hashTimeString( hashString, rawHash ) ;
    printf( "Creating new cipher key.  Deleting or changing this key will invalidate\n"
	    "all account passwords.  Don't do it!\n" ) ;
  } else {
    hashTimeString( hashString, hash ) ;
  }

  ret = write( fd, hashString, strlen(hashString) ) ;
  close( fd ) ;

  return ret ;
}


void delete_message (GtkWidget *widget, gpointer data) {
	destroy_message (tm_globs->msg_selected);
}



