/* TradeClient <http://tradeclient.sourceforge.net>
 * $Id: schedule-widget.c,v 1.16 2001/03/20 22:19:33 ttabner Exp $
 *
 * Copyright (C) 1999-2000 Bynari Inc.
 * Copyright (C) 2001 Project TradeClient
 *
 * LGPL
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _SCHEDULE_C_
#define _SCHEDULE_C_


#include <gdk/gdkkeysyms.h>
#include <time.h>
#include "schedule-widget.h"


#define HANDLEBAR 0
#define HANDLE_SIZE 8
#define CHILD_HAS_FOCUS (1 << 0)
#define CONFLICT 14


typedef enum ScheduleSignals
{
  CHILD_CHANGED,
  CHILD_SELECTED,
  CHILD_DOUBLE_CLICKED,
  CHILD_RIGHT_CLICKED,
  ROW_SELECTED,
  ROW_UNSELECTED,
  LAST_SIGNAL
}
ScheduleSignals;


static GtkContainerClass *parent_class;
static int schedule_signals[LAST_SIGNAL] = { 0 };
ScheduleDragInfo *info = NULL;
int schedule_count = 0;


static void schedule_class_init (ScheduleClass * schedule_class);
static void schedule_init (Schedule * schedule);
static void schedule_realize (GtkWidget * widget);
static void schedule_size_allocate (GtkWidget * widget,
				    GtkAllocation * allocation);
static void schedule_send_configure (Schedule * schedule);
static void schedule_draw (GtkWidget * widget, GdkRectangle * area);
static gint schedule_event (GtkWidget * widget, GdkEvent * event);
static gint schedule_button_press_event (GtkWidget * widget,
					 GdkEventButton * event);
static gint schedule_button_release_event (GtkWidget * widget,
					   GdkEventButton * event);
static gint schedule_key_press_event (GtkWidget * widget,
				      GdkEventKey * event);
static void schedule_size_request (GtkWidget * widget,
				   GtkRequisition * requisition);
static void schedule_destroy (GtkObject * object);
static void schedule_paint (GtkWidget * widget);
static void schedule_paint_row (GtkWidget * widget, int row);
static void schedule_child_show (Schedule * schedule, ScheduleChild * child);
static void schedule_child_realize (Schedule * schedule,
				    ScheduleChild * child);
static void schedule_child_map (Schedule * schedule, ScheduleChild * child);
static void schedule_child_paint (ScheduleChild * child);
static void schedule_child_handle_text (Schedule * schedule,
					ScheduleChild * child);
static void schedule_child_key_press (GtkWidget * widget, GdkEventKey * event,
				      gpointer data);
static void schedule_child_focus_in (GtkWidget * widget,
				     GdkEventFocus * event, gpointer data);
static void schedule_child_focus_out (GtkWidget * widget,
				      GdkEventFocus * event, gpointer data);
static void child_changed (Schedule * schedule, char *text, gpointer data,
			   time_t start, time_t stop);
static void child_selected (Schedule * schedule, char *text, gpointer data,
			    time_t start, time_t stop);
static void row_selected (Schedule * schedule, int row, time_t index);
static void child_double_clicked (Schedule * schedule, char *text,
				  gpointer data, time_t start, time_t stop);
static void child_right_clicked (Schedule * schedule, char *text,
				 gpointer data, time_t start, time_t stop);
static gint schedule_motion_notify_event (GtkWidget * widget,
					  GdkEventMotion * event);
static void schedule_child_find_best (ScheduleChild * child);
static void schedule_child_unmap (Schedule * schedule, ScheduleChild * child);
static void schedule_child_unrealize (Schedule * schedule,
				      ScheduleChild * child);
static void schedule_remove_child (GtkContainer * container,
				   GtkWidget * widget);
static void schedule_forall (GtkContainer * container,
			     gboolean include_internals, GtkCallback callback,
			     gpointer callback_data);
static void schedule_show_conflict (Schedule * schedule, time_t time);
static int schedule_time_to_row (int first, Schedule * schedule, time_t start, time_t stop);


guint
schedule_get_type (void)
{
  static guint schedule_type = 0;

  if (!schedule_type)
    {
      static const GtkTypeInfo schedule_info = {
	"Schedule",
	sizeof (Schedule),
	sizeof (ScheduleClass),
	(GtkClassInitFunc) schedule_class_init,
	(GtkObjectInitFunc) schedule_init,
	NULL,
	NULL,
	(GtkClassInitFunc) NULL
      };
      schedule_type =
	gtk_type_unique (gtk_container_get_type (), &schedule_info);
    }
  return (schedule_type);
}


GtkWidget *
schedule_new (void)
{
  GtkWidget *widget = GTK_WIDGET (gtk_type_new (schedule_get_type ()));

  return (widget);
}


static void
schedule_class_init (ScheduleClass * schedule_class)
{
  GtkWidgetClass *widget_class;
  GtkObjectClass *object_class;
  GtkContainerClass *container_class;

  parent_class = gtk_type_class (gtk_container_get_type ());

  widget_class = (GtkWidgetClass *) schedule_class;
  object_class = (GtkObjectClass *) schedule_class;
  container_class = (GtkContainerClass *) schedule_class;

  schedule_signals[CHILD_CHANGED] =
    gtk_signal_new ("child_changed", GTK_RUN_LAST, object_class->type,
		    GTK_SIGNAL_OFFSET (ScheduleClass, child_changed),
		    gtk_marshal_NONE__POINTER_POINTER_UINT_UINT,
		    GTK_TYPE_NONE, 3, GTK_TYPE_POINTER, GTK_TYPE_POINTER,
		    GTK_TYPE_UINT, GTK_TYPE_UINT);
  schedule_signals[CHILD_SELECTED] =
    gtk_signal_new ("child_selected", GTK_RUN_LAST, object_class->type,
		    GTK_SIGNAL_OFFSET (ScheduleClass, child_selected),
		    gtk_marshal_NONE__POINTER_POINTER_UINT_UINT,
		    GTK_TYPE_NONE, 3, GTK_TYPE_POINTER, GTK_TYPE_POINTER,
		    GTK_TYPE_ULONG, GTK_TYPE_ULONG);
  schedule_signals[CHILD_DOUBLE_CLICKED] =
    gtk_signal_new ("child_double_clicked", GTK_RUN_LAST, object_class->type,
		    GTK_SIGNAL_OFFSET (ScheduleClass, child_double_clicked),
		    gtk_marshal_NONE__POINTER_POINTER_UINT_UINT,
		    GTK_TYPE_NONE, 3, GTK_TYPE_POINTER, GTK_TYPE_POINTER,
		    GTK_TYPE_ULONG, GTK_TYPE_ULONG);
  schedule_signals[CHILD_RIGHT_CLICKED] =
    gtk_signal_new ("child_right_clicked", GTK_RUN_LAST, object_class->type,
		    GTK_SIGNAL_OFFSET (ScheduleClass, child_right_clicked),
		    gtk_marshal_NONE__POINTER_POINTER_UINT_UINT,
		    GTK_TYPE_NONE, 3, GTK_TYPE_POINTER, GTK_TYPE_POINTER,
		    GTK_TYPE_ULONG, GTK_TYPE_ULONG);
  schedule_signals[ROW_SELECTED] =
    gtk_signal_new ("row_selected", GTK_RUN_LAST, object_class->type,
		    GTK_SIGNAL_OFFSET (ScheduleClass, row_selected),
		    gtk_signal_default_marshaller, GTK_TYPE_NONE,
		    2, GTK_TYPE_UINT, GTK_TYPE_ULONG);
  schedule_signals[ROW_UNSELECTED] =
    gtk_signal_new ("row_unselected", GTK_RUN_LAST, object_class->type,
		    GTK_SIGNAL_OFFSET (ScheduleClass, row_unselected),
		    gtk_signal_default_marshaller, GTK_TYPE_NONE,
		    2, GTK_TYPE_UINT, GTK_TYPE_ULONG);
  gtk_object_class_add_signals (object_class, schedule_signals, LAST_SIGNAL);

  widget_class = (GtkWidgetClass *) schedule_class;
  object_class = (GtkObjectClass *) schedule_class;

  widget_class->realize = schedule_realize;
  widget_class->draw = schedule_draw;
  widget_class->event = schedule_event;
  widget_class->button_press_event = schedule_button_press_event;
  widget_class->button_release_event = schedule_button_release_event;
  widget_class->key_press_event = schedule_key_press_event;
  widget_class->size_request = schedule_size_request;
  widget_class->size_allocate = schedule_size_allocate;
  widget_class->motion_notify_event = schedule_motion_notify_event;

  object_class->destroy = schedule_destroy;

  container_class->forall = schedule_forall;
  container_class->remove = schedule_remove_child;

  schedule_class->child_changed = child_changed;
  schedule_class->child_selected = child_selected;
  schedule_class->child_double_clicked = child_double_clicked;
  schedule_class->child_right_clicked = child_right_clicked;
  schedule_class->row_selected = row_selected;
}


static void
schedule_init (Schedule * schedule)
{
  GTK_WIDGET_UNSET_FLAGS (schedule, GTK_NO_WINDOW);
  GTK_WIDGET_SET_FLAGS (schedule, GTK_CAN_FOCUS);

  schedule->depth_count = 0;
  schedule->child_count = 0;
  schedule->time_style = SCHEDULE_AMPM;
  schedule->granularity = SCHEDULE_30M;
  schedule->selected_row_start = -1;
  schedule->selected_row_stop = -1;
  schedule->row_height = 20;
  schedule->day_start_row = 15;
  schedule->day_end_row = 40;
  schedule->dark = NULL;
  schedule->light = NULL;
}


void
schedule_remove_child (GtkContainer * container, GtkWidget * widget)
{
}


static void
schedule_forall (GtkContainer * container, gboolean include_internals,
		 GtkCallback callback, gpointer callback_data)
{
}


static void
schedule_realize (GtkWidget * widget)
{
  Schedule *schedule;
  GdkWindowAttr attributes;
  guint attributes_mask;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (SCHEDULE_IS_SCHEDULE (widget));

  schedule = SCHEDULE (widget);

  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.x = widget->allocation.x;
  attributes.y = widget->allocation.y;
  attributes.width = widget->allocation.width;
  attributes.height = widget->allocation.height;
  attributes.wclass = GDK_INPUT_OUTPUT;
  attributes.visual = gtk_widget_get_visual (widget);
  attributes.colormap = gtk_widget_get_colormap (widget);
  attributes.event_mask = (GDK_EXPOSURE_MASK |
			   GDK_BUTTON_PRESS_MASK |
			   GDK_BUTTON_RELEASE_MASK |
			   GDK_KEY_PRESS_MASK |
			   GDK_KEY_RELEASE_MASK |
			   GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);

  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
  widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
				   &attributes, attributes_mask);

  gdk_window_set_user_data (widget->window, widget);

  widget->style = gtk_style_attach (widget->style, widget->window);
  gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);

  SCHEDULE (widget)->up_down_cursor = gdk_cursor_new (GDK_DOUBLE_ARROW);

  schedule_send_configure (SCHEDULE (widget));
}


static void
schedule_destroy (GtkObject * object)
{
  ScheduleClass *schedule_class;

  g_return_if_fail (object != NULL);
  g_return_if_fail (SCHEDULE_IS_SCHEDULE (object));

  schedule_destroy_children (SCHEDULE (GTK_WIDGET (object)));

  schedule_class = gtk_type_class (gtk_widget_get_type ());
  if (GTK_OBJECT_CLASS (schedule_class)->destroy)
    (*GTK_OBJECT_CLASS (schedule_class)->destroy) (object);
}


void
schedule_destroy_children (Schedule * schedule)
{
  GList *seek;
  ScheduleChild *child;
  g_return_if_fail (schedule != NULL);
  seek = schedule->children;
  while (seek)
    {
      child = seek->data;
      schedule_child_unmap (schedule, child);
      schedule_child_unrealize (schedule, child);
      gtk_widget_destroy (child->widget);
      g_free (seek->data);
      seek = seek->next;
    }
  g_list_free (schedule->children);
  schedule->children = NULL;
}


static void
schedule_child_unmap (Schedule * schedule, ScheduleChild * child)
{
  if (g_list_find (schedule->children, child))
    {
      gdk_window_hide (child->window);
      if (GTK_WIDGET_MAPPED (child->widget))
	gtk_widget_hide (child->widget);
    }
}

static void
schedule_child_unrealize (Schedule * schedule, ScheduleChild * child)
{
  if (GTK_WIDGET_REALIZED (child->widget))
    gtk_widget_unrealize (child->widget);
  gdk_window_set_user_data (child->window, NULL);
  gdk_window_destroy (child->window);
  child->window = NULL;
}


static void
schedule_draw (GtkWidget * widget, GdkRectangle * area)
{
  g_return_if_fail (widget != NULL);
  g_return_if_fail (SCHEDULE_IS_SCHEDULE (widget));

  schedule_paint (widget);
}


static gint
schedule_event (GtkWidget * widget, GdkEvent * event)
{
  Schedule *schedule;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (SCHEDULE_IS_SCHEDULE (widget), FALSE);

  schedule = SCHEDULE (widget);

  switch (event->type)
    {
    case GDK_EXPOSE:
      break;
    default:
      return FALSE;
    }

#if 1
  gtk_widget_queue_draw (widget);
#else
  schedule_paint (widget);
#endif

  return TRUE;
}


static gint
schedule_button_press_event (GtkWidget * widget, GdkEventButton * event)
{
  Schedule *schedule = SCHEDULE (widget);
  int row = (int) event->y / schedule->row_height;
  ScheduleChild *child = NULL;
  GList *seek;
  int gran;
  char *text = NULL;

  switch (schedule->granularity)
    {
    case SCHEDULE_15M:
      gran = 2.0;
      break;
    case SCHEDULE_30M:
      gran = 1.0;
      break;
    case SCHEDULE_60M:
      gran = 0.5;
      break;
    }

  if (SCHEDULE (widget)->selected_row_start == row)
    {
      SCHEDULE (widget)->selected_row_start = -1;
    }
  else
    {
      SCHEDULE (widget)->selected_row_start = row;
    }
  SCHEDULE (widget)->selected_row_stop = -1;

  if (event->window == widget->window)
    {
      gtk_widget_grab_focus (widget);
    }
  else
    {
      SCHEDULE (widget)->selected_row_start = -1;
      seek = schedule->children;
      for (seek = schedule->children; seek; seek = seek->next)
	{
	  child = (ScheduleChild *) seek->data;
	  if (child->window == event->window)
	    {
	      break;
	    }
	  if (event->window == child->widget->window)
	    {
	      break;
	    }
	}
      if (!seek)
	{
	  return TRUE;
	}
      if (info == NULL)
	{
	  info = (ScheduleDragInfo *) g_malloc0 (sizeof (ScheduleDragInfo));
	}
      info->child = child;
      info->start_x = event->x;
      info->start_y = event->y;
      if (event->y < HANDLE_SIZE)
	{
	  info->type = DRAG_SIZEUP;
	}
      if (event->y > child->height - HANDLE_SIZE)
	{
	  info->type = DRAG_SIZEDOWN;
	}
      if (event->x < HANDLE_SIZE)
	{
	  info->type = DRAG_MOVE;
	}
      gdk_pointer_grab (widget->window, FALSE, GDK_BUTTON_PRESS_MASK |
			GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK,
			NULL, NULL, GDK_CURRENT_TIME);
      if (child)
	text = gtk_editable_get_chars (GTK_EDITABLE (child->widget), 0, -1);
      if (event->button == 1 && event->type == 4 && child)
	{
	  if (!GTK_WIDGET_HAS_FOCUS (child->widget))
	    {
	      gtk_widget_grab_focus (child->widget);
	    }
	}
      if (event->button == 1 && event->type == 5 && child)
	{
	  gtk_signal_emit (GTK_OBJECT (GTK_WIDGET (child->schedule)),
			   schedule_signals[CHILD_DOUBLE_CLICKED], text,
			   child->data, child->start, child->stop);
	}
      if (event->button == 3 && event->type == 4 && child)
	{
	  gtk_signal_emit (GTK_OBJECT (GTK_WIDGET (child->schedule)),
			   schedule_signals[CHILD_RIGHT_CLICKED], text,
			   child->data, child->start, child->stop);
	}
    }
  schedule_paint (widget);

  return TRUE;
}


static gint
schedule_button_release_event (GtkWidget * widget, GdkEventButton * event)
{
  Schedule *schedule = SCHEDULE (widget);
  int row = (int) event->y / schedule->row_height;

  if (widget->window == event->window)
    {
      SCHEDULE (widget)->selected_row_stop = row;
    }
  else
    {
      SCHEDULE (widget)->selected_row_stop = -1;
    }

  gdk_pointer_ungrab (GDK_CURRENT_TIME);
  if (info)
    {
      g_free (info);

      info = NULL;
    }

  schedule_paint (widget);

  return TRUE;
}


static gint
schedule_key_press_event (GtkWidget * widget, GdkEventKey * event)
{
  return FALSE;
}


static void
schedule_paint (GtkWidget * widget)
{
  int i;
  int x, y;
  int width, height;
  int start_row = 0, stop_row = 0;
  GList *seek;
  Schedule *schedule = SCHEDULE (widget);

  g_return_if_fail (widget != NULL);

  x = widget->allocation.x;
  y = widget->allocation.y;
  width = widget->allocation.width;
  height = widget->allocation.height;
  start_row = (y / SCHEDULE (widget)->row_height) - 1;
  /* stop_row = ((y + height) / SCHEDULE (widget)->row_height) + 1; */

  switch (schedule->granularity)
    {
    case SCHEDULE_15M:
      stop_row = 96;
      break;
    case SCHEDULE_30M:
      stop_row = 48;
      break;
    case SCHEDULE_60M:
      stop_row = 24;
      break;
    }

  gtk_draw_shadow (widget->style, widget->window, GTK_STATE_NORMAL,
		   GTK_SHADOW_OUT, 0, 0, SCHEDULE (widget)->width,
		   SCHEDULE (widget)->height);

  for (i = start_row; i < stop_row; i++)
    {
      schedule_paint_row (widget, i);
    }

  seek = SCHEDULE (widget)->children;
  while (seek)
    {
      ScheduleChild *child = seek->data;

      schedule_child_handle_text (schedule, child);
      schedule_child_paint (child);
      gtk_widget_draw (child->widget, NULL);

      seek = g_list_next (seek);
    }
  seek = schedule_find_conflicts (schedule);
  while (seek)
    {
      schedule_show_conflict (schedule, (time_t) seek->data);
      seek = g_list_next (seek);
    }
}


GList *
schedule_find_conflicts (Schedule * schedule)
{
  GList *children = schedule->children;
  GList *seek;
  GList *list = NULL;
  ScheduleChild *child, *test;
  time_t time;

  while (children)
    {
      child = children->data;
      seek = schedule->children;
      while (seek)
	{
	  test = seek->data;
	  time = test->start;

	  if (child->data != test->data)
	    {
	      if ((time >= child->start) && (time <= child->stop))
		{
		  if ((time != child->stop))
		    list = g_list_append (list, (gpointer) time);
		}
	    }
	  seek = seek->next;
	}
      children = children->next;
    }

  return list;
}


static void
schedule_show_conflict (Schedule * schedule, time_t time)
{
  GtkWidget *widget = GTK_WIDGET (schedule);
  GdkWindow *window = widget->window;
  int row = schedule_time_to_row (1, schedule, time, time);
  GtkStyle *custom;
  GdkGC *gc;
  GdkColor cfcolor = {0, 0xff00, 0xff00, 0x5500};
  GdkGC *dark_grey, *light_grey;
  int start = 0, stop = 0;

  if (schedule->dark)
    dark_grey = schedule->dark;
  else
    dark_grey = widget->style->base_gc[GTK_STATE_INSENSITIVE];
  if (schedule->light)
    light_grey = schedule->light;
  else
    light_grey = widget->style->base_gc[GTK_STATE_NORMAL];

  switch (schedule->granularity)
    {
    case SCHEDULE_15M:
      start = schedule->day_start_row * 2 + 1;
      stop = schedule->day_end_row * 2;
      break;
    case SCHEDULE_30M:
      start = schedule->day_start_row;
      stop = schedule->day_end_row;
      break;
    case SCHEDULE_60M:
      start = schedule->day_start_row / 2;
      stop = schedule->day_end_row / 2;
      break;
    }

  if ((SCHEDULE (widget)->selected_row_start) >= 0)
    {
      int start, stop, diff, paint;

      start = SCHEDULE (widget)->selected_row_start;
      stop = SCHEDULE (widget)->selected_row_stop;
      diff = stop - start;
      paint = 0;

      if (row == start)
        {
          paint = 1;
        }
      else
        {
          if (stop == -1)
            diff = 0;
          if (diff > 0)
            {
              if ((row > start) && (row <= (start + diff)))
                paint = 1;
            }
          else if (diff < 0)
            {
              if ((row < start) && (row >= (start + diff)))
                paint = 1;
            }
        }

      if (paint == 1)
        dark_grey = light_grey = widget->style->bg_gc[GTK_STATE_SELECTED];
    }

  if ((row > start) && (row < stop))
    {
      gdk_draw_rectangle (widget->window, light_grey, TRUE, 64,
                          row * SCHEDULE (widget)->row_height + 1,
                          SCHEDULE (widget)->width - 66, 19);
    }
  else
    {
      gdk_draw_rectangle (widget->window, dark_grey, TRUE, 64,
                          row * SCHEDULE (widget)->row_height + 1,
                          SCHEDULE (widget)->width - 66, 19);
    }

  custom = gtk_style_copy (widget->style);
  custom->font =
    gdk_font_load ("-*-helvetica-bold-o-*-*-*-180-*-*-*-*-*-1");
  if (!custom->font)
    custom->font = widget->style->font;

  gc = gdk_gc_new (window);
  gdk_color_alloc (gtk_widget_get_colormap (widget), &cfcolor);
  gdk_gc_set_foreground (gc, &cfcolor);
  
  gdk_draw_string (window, custom->font, widget->style->black_gc,
                   64 + 3, row * SCHEDULE (widget)->row_height + 18, "!");

  gdk_draw_string (window, custom->font, gc,
		   64 + 2, row * SCHEDULE (widget)->row_height + 17, "!");

}


static void
schedule_paint_row (GtkWidget * widget, int row)
{
  GdkGC *white, *black, *dark_grey, *light_grey;
  Schedule *schedule = SCHEDULE (widget);
  int count = 0, min = 0, start = 0, stop = 0;
  char buf[40];
  char tmp[40];
  struct tm time;
  GtkStyle *style;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (widget->style != NULL);

  switch (schedule->granularity)
    {
    case SCHEDULE_15M:
      count = 95;
      min = 15;
      start = schedule->day_start_row * 2 + 1;
      stop = schedule->day_end_row * 2;
      break;
    case SCHEDULE_30M:
      count = 47;
      min = 30;
      start = schedule->day_start_row;
      stop = schedule->day_end_row;
      break;
    case SCHEDULE_60M:
      count = 23;
      min = 60;
      start = schedule->day_start_row / 2;
      stop = schedule->day_end_row / 2;
      break;
    }

  if ((row < 0) || (row > count))
    return;

  black = widget->style->bg_gc[GTK_STATE_SELECTED];
  white = widget->style->dark_gc[GTK_STATE_NORMAL];
  if (schedule->dark)
    dark_grey = schedule->dark;
  else
    dark_grey = widget->style->base_gc[GTK_STATE_INSENSITIVE];
  if (schedule->light)
    light_grey = schedule->light;
  else
    light_grey = widget->style->base_gc[GTK_STATE_NORMAL];

  memset (&time, 0, sizeof (struct tm));
  time.tm_min = row * min;
  time.tm_hour = time.tm_min / 60;
  time.tm_min = time.tm_min % 60;
  time.tm_hour = time.tm_hour % 24;
  time.tm_isdst = -1;
  mktime (&time);

  gdk_draw_rectangle (widget->window, black, TRUE, 2,
		      row * SCHEDULE (widget)->row_height + 1, 62, 19);
  if ((row > start) && (row < stop))
    {
      gdk_draw_rectangle (widget->window, light_grey, TRUE, 64,
			  row * SCHEDULE (widget)->row_height + 1,
			  SCHEDULE (widget)->width - 66, 19);
    }
  else
    {
      gdk_draw_rectangle (widget->window, dark_grey, TRUE, 64,
			  row * SCHEDULE (widget)->row_height + 1,
			  SCHEDULE (widget)->width - 66, 19);
    }

  if ((SCHEDULE (widget)->selected_row_start) >= 0)
    {
      int start, stop, diff, paint;

      start = SCHEDULE (widget)->selected_row_start;
      stop = SCHEDULE (widget)->selected_row_stop;
      diff = stop - start;
      paint = 0;

      if (row == start)
	{
	  paint = 1;
	}
      else
	{
	  if (stop == -1)
	    diff = 0;
	  if (diff > 0)
	    {
	      if ((row > start) && (row <= (start + diff)))
		paint = 1;
	    }
	  else if (diff < 0)
	    {
	      if ((row < start) && (row >= (start + diff)))
		paint = 1;
	    }
	}

      if (paint == 1)
	gdk_draw_rectangle (widget->window, black, TRUE, 64,
			    row * SCHEDULE (widget)->row_height + 1,
			    SCHEDULE (widget)->width - 66, 19);
    }

  gdk_draw_line (widget->window, black, 2,
		 row * SCHEDULE (widget)->row_height +
		 SCHEDULE (widget)->row_height, widget->allocation.width - 2,
		 row * SCHEDULE (widget)->row_height +
		 SCHEDULE (widget)->row_height);
  gdk_draw_line (widget->window, white, 64,
		 row * SCHEDULE (widget)->row_height +
		 SCHEDULE (widget)->row_height, widget->allocation.width - 3,
		 row * SCHEDULE (widget)->row_height +
		 SCHEDULE (widget)->row_height);

  if (SCHEDULE (widget)->time_style == SCHEDULE_AMPM)
    strftime (buf, sizeof (buf), "%I:%M %p", &time);
  else
    strftime (buf, sizeof (buf), "   %H:%M", &time);

  black = widget->style->fg_gc[GTK_STATE_SELECTED];

  strftime (tmp, sizeof (tmp), "%M", &time);
  style = gtk_style_copy (widget->style);
  if (strcmp (tmp, "00"))
    {
    style->font =
      gdk_font_load ("-*-helvetica-normal-r-*-*-*-120-*-*-*-*-*-1");
    }
  else
    {
    style->font =
      gdk_font_load ("-*-helvetica-bold-r-*-*-*-120-*-*-*-*-*-1");
    }
  if (!style->font)
    style->font = widget->style->font;


  gdk_draw_string (widget->window, style->font, black, 5,
		   row * SCHEDULE (widget)->row_height + 15, buf);


  gdk_draw_line (widget->window, white, 63,
		 row * SCHEDULE (widget)->row_height + 1, 63,
		 row * SCHEDULE (widget)->row_height +
		 SCHEDULE (widget)->row_height);
}


static void
schedule_size_request (GtkWidget * widget, GtkRequisition * requisition)
{
  Schedule *schedule = SCHEDULE (widget);
  int count = 0;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (SCHEDULE_IS_SCHEDULE (widget));

  switch (schedule->granularity)
    {
    case SCHEDULE_15M:
      count = 96;
      break;
    case SCHEDULE_30M:
      count = 48;
      break;
    case SCHEDULE_60M:
      count = 24;
      break;
    }

  schedule->width = requisition->width = widget->parent->allocation.width - 4;
  schedule->height = requisition->height = (schedule->row_height * count) + 4;
}


static void
schedule_size_allocate (GtkWidget * widget, GtkAllocation * allocation)
{
  g_return_if_fail (widget != NULL);
  g_return_if_fail (SCHEDULE_IS_SCHEDULE (widget));
  g_return_if_fail (allocation != NULL);

  widget->allocation.x = allocation->x;
  widget->allocation.y = allocation->y;
  widget->allocation.width = allocation->width;
  widget->allocation.height = allocation->height;

  if (GTK_WIDGET_REALIZED (widget))
    {
      gdk_window_move_resize (widget->window, allocation->x, allocation->y,
			      allocation->width, allocation->height);
      schedule_send_configure (SCHEDULE (widget));
    }
}


static void
schedule_send_configure (Schedule * schedule)
{
  GtkWidget *widget;
  GdkEventConfigure event;

  widget = GTK_WIDGET (schedule);

  event.type = GDK_CONFIGURE;
  event.window = widget->window;
  event.x = widget->allocation.x;
  event.y = widget->allocation.y;
  event.width = widget->allocation.width;
  event.height = widget->allocation.height;

  gtk_widget_event (widget, (GdkEvent *) & event);
}


ScheduleChild *
schedule_child_new (Schedule * schedule)
{
  ScheduleChild *child = (ScheduleChild *) g_malloc0 (sizeof (ScheduleChild));

  child->schedule = schedule;
  child->start = -1;
  child->stop = -1;
  child->widget = gtk_text_new (NULL, NULL);
  gtk_text_set_word_wrap (GTK_TEXT (child->widget), TRUE);
  gtk_text_set_editable (GTK_TEXT (child->widget), FALSE);

  schedule->children = g_list_append (schedule->children, child);

  gtk_signal_connect_after (GTK_OBJECT (child->widget),
			    "focus_in_event",
			    GTK_SIGNAL_FUNC (schedule_child_focus_in), child);
  gtk_signal_connect_after (GTK_OBJECT (child->widget),
			    "key_press_event",
			    GTK_SIGNAL_FUNC (schedule_child_key_press),
			    child);
  child->focus_out_id =
    gtk_signal_connect_after (GTK_OBJECT (child->widget),
			      "focus_out_event",
			      GTK_SIGNAL_FUNC (schedule_child_focus_out),
			      child);

  return child;
}


static int
schedule_time_to_row (int first, Schedule * schedule, time_t start, time_t stop)
{
  time_t time = (first ? start : stop);
  int start_year, stop_year, start_month, stop_month, start_day, stop_day;
  struct tm *tms;
  int row = 0;

  tms = localtime (&start);
  start_year = tms->tm_year;
  start_month = tms->tm_mon;
  start_day = tms->tm_mday;

  tms = localtime (&stop);
  stop_year = tms->tm_year;
  stop_month = tms->tm_mon;
  stop_day = tms->tm_mday;

  if (!first) {
    if (stop_year > start_year) return 48;
    if (stop_month > start_month) return 48;
    if (stop_day > start_day) return 48;
  }
  
  tms = localtime (&time);

  switch (schedule->granularity)
    {
    case SCHEDULE_15M:
      row = tms->tm_hour * 4;
      break;
    case SCHEDULE_30M:
      row = tms->tm_hour * 2;
      break;
    case SCHEDULE_60M:
      row = tms->tm_hour;
      break;
    }

  if (tms->tm_min >= 30)
    row++;
  return row;
}


void
schedule_add_child (Schedule * schedule, char *text, time_t start,
		    time_t stop, gpointer data)
{
  ScheduleChild *child = schedule_child_new (schedule);
  GtkStyle *style = GTK_WIDGET (schedule)->style;

  child->x = 64 + CONFLICT;
  child->y =
    (schedule_time_to_row (1, schedule, start, stop) * schedule->row_height) + 1;
  child->width = schedule->width - 66 - CONFLICT;
  child->height =
    (schedule_time_to_row (0, schedule, start, stop) * schedule->row_height) - child->y;
  child->data = data;
  child->start = start;
  child->stop = stop;
  child->color = style->bg_gc[GTK_STATE_SELECTED];

  if (text)
    gtk_text_insert (GTK_TEXT (child->widget), NULL, NULL, NULL,
		     text, strlen (text));

  schedule->child_count++;
  schedule_child_show (schedule, child);
}


static void
schedule_child_show (Schedule * schedule, ScheduleChild * child)
{
  if (!child->window)
    schedule_child_realize (schedule, child);
  schedule_child_map (schedule, child);
  gtk_widget_show (child->widget);
  schedule_child_paint (child);
}


static void
schedule_child_realize (Schedule * schedule, ScheduleChild * child)
{
  GdkWindowAttr attributes;
  gint attributes_mask;
  GtkWidget *widget = GTK_WIDGET (schedule);

  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.x = child->x;
  attributes.y = child->y;
  attributes.width = child->width - child->decor_width;
  attributes.height = child->height;
  attributes.wclass = GDK_INPUT_OUTPUT;
  attributes.visual = gtk_widget_get_visual (widget);
  attributes.colormap = gtk_widget_get_colormap (widget);
  attributes.cursor = schedule->up_down_cursor;
  attributes.event_mask = (GDK_EXPOSURE_MASK
			   | GDK_BUTTON_PRESS_MASK
			   | GDK_BUTTON_RELEASE_MASK
			   | GDK_BUTTON_MOTION_MASK
			   | GDK_POINTER_MOTION_HINT_MASK
			   | GDK_KEY_PRESS_MASK);
  attributes_mask =
    GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP | GDK_WA_CURSOR;
  child->window =
    gdk_window_new (widget->window, &attributes, attributes_mask);
  gdk_window_set_user_data (child->window, widget);
  gtk_style_set_background (widget->style, child->window, GTK_STATE_NORMAL);
  gtk_widget_set_parent (child->widget, GTK_WIDGET (schedule));

  schedule_child_handle_text (schedule, child);
}


static void
schedule_child_handle_text (Schedule * schedule, ScheduleChild * child)
{
  GtkAllocation allocation;
  int has_focus;
  int handle_size;

  has_focus = GTK_WIDGET_HAS_FOCUS (child->widget);

  handle_size = HANDLE_SIZE;

  if (child->width > 400)
    child->width = 400;
  allocation.x = child->x + HANDLE_SIZE;
  allocation.y = child->y + has_focus * HANDLE_SIZE;

  allocation.width = child->width - HANDLE_SIZE;
  allocation.height = child->height - 2 * (has_focus * HANDLE_SIZE);

  gtk_widget_size_request (child->widget, NULL);
  gtk_widget_size_allocate (child->widget, &allocation);
}


static void
schedule_child_map (Schedule * schedule, ScheduleChild * child)
{
  gdk_window_show (child->window);
  gtk_widget_show (child->widget);
  gtk_widget_draw (child->widget, NULL);
}


static void
schedule_child_paint (ScheduleChild * child)
{
  GdkRectangle rectangle;
  GtkStyle *style = GTK_WIDGET (child->widget)->style;

  rectangle.x = 0;
  rectangle.y = 0;
  rectangle.width = HANDLE_SIZE - 1;
  rectangle.height = child->height - 1;
#if HANDLEBAR
  gtk_draw_handle (GTK_WIDGET (child->schedule)->style,
		   child->window, GTK_STATE_NORMAL, GTK_SHADOW_OUT,
		   rectangle.x, rectangle.y, rectangle.width,
		   rectangle.height, GTK_ORIENTATION_VERTICAL);
#else
  gdk_draw_rectangle (child->window, child->color,
		      TRUE, rectangle.x,
		      rectangle.y, rectangle.width, rectangle.height);
  gdk_draw_rectangle (child->window, style->black_gc, FALSE,
		      rectangle.x, rectangle.y, rectangle.width,
		      rectangle.height);
#endif


  if (child->flags & CHILD_HAS_FOCUS)
    {
      rectangle.x = HANDLE_SIZE;
      rectangle.y = 0;
      rectangle.width = child->width - HANDLE_SIZE - 1;
      rectangle.height = HANDLE_SIZE - 1;
#if HANDLEBAR
      gtk_draw_handle (GTK_WIDGET (child->schedule)->style,
		       child->window, GTK_STATE_NORMAL,
		       GTK_SHADOW_OUT, rectangle.x, rectangle.y,
		       rectangle.width, rectangle.height,
		       GTK_ORIENTATION_VERTICAL);
#else
      gdk_draw_rectangle (child->window,
			  child->color, TRUE, rectangle.x,
			  rectangle.y, rectangle.width, rectangle.height);
      gdk_draw_rectangle (child->window, style->black_gc, FALSE,
			  rectangle.x, rectangle.y, rectangle.width,
			  rectangle.height);
#endif

      rectangle.x = HANDLE_SIZE;
      rectangle.y = child->height - HANDLE_SIZE;
      rectangle.width = child->width - HANDLE_SIZE - 1;
      rectangle.height = HANDLE_SIZE - 1;
#if HANDLEBAR
      gtk_draw_handle (GTK_WIDGET (child->schedule)->style,
		       child->window, GTK_STATE_NORMAL,
		       GTK_SHADOW_OUT, rectangle.x, rectangle.y,
		       rectangle.width, rectangle.height,
		       GTK_ORIENTATION_VERTICAL);
#else
      gdk_draw_rectangle (child->window,
			  child->color, TRUE, rectangle.x,
			  rectangle.y, rectangle.width, rectangle.height);
      gdk_draw_rectangle (child->window, style->black_gc, FALSE,
			  rectangle.x, rectangle.y, rectangle.width,
			  rectangle.height);
#endif
    }
}

static void
schedule_child_key_press (GtkWidget * widget, GdkEventKey * event,
			  gpointer data)
{
  if (event->keyval != GDK_Escape)
    return;
  gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "key_press_event");
  gtk_widget_grab_focus (widget->parent);
}


static void
schedule_child_focus_in (GtkWidget * widget, GdkEventFocus * event,
			 gpointer data)
{
  ScheduleChild *child = (ScheduleChild *) data;
  char *text;

  if (child)
    {
      text = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
      if (child->window)
	{
	  child->height += HANDLE_SIZE * 2;
	  child->y -= HANDLE_SIZE;
	  gdk_window_move_resize (child->window, child->x,
				  child->y, child->width, child->height);
	  child->flags |= CHILD_HAS_FOCUS;
	  gtk_signal_emit (GTK_OBJECT (GTK_WIDGET (child->schedule)),
			   schedule_signals[CHILD_SELECTED], text,
			   child->data, child->start, child->stop);
	  schedule_child_paint (child);
	}
    }
}


static void
schedule_child_focus_out (GtkWidget * widget, GdkEventFocus * event,
			  gpointer data)
{
  ScheduleChild *child = (ScheduleChild *) data;
  char *text = gtk_editable_get_chars (GTK_EDITABLE (child->widget),
				       0, -1);

  if (child->window)
    {
      child->height -= HANDLE_SIZE * 2;
      child->y += HANDLE_SIZE;
      gdk_window_move_resize (child->window, child->x,
			      child->y, child->width, child->height);
      child->flags &= ~CHILD_HAS_FOCUS;
      schedule_child_paint (child);
      gdk_pointer_ungrab (GDK_CURRENT_TIME);
      if (info)
	{
	  g_free (info);
	  info = NULL;
	}
      gtk_signal_emit (GTK_OBJECT (GTK_WIDGET (child->schedule)),
		       schedule_signals[CHILD_CHANGED],
		       text, child->data, child->start, child->stop);
    }
}


static void
child_changed (Schedule * schedule, char *text, gpointer data, time_t start,
	       time_t stop)
{
}

static void
child_selected (Schedule * schedule, char *text, gpointer data, time_t start,
		time_t stop)
{
}


static void
child_double_clicked (Schedule * schedule, char *text, gpointer data,
		      time_t start, time_t stop)
{
}


static void
child_right_clicked (Schedule * schedule, char *text, gpointer data,
		     time_t start, time_t stop)
{
  GList *seek = schedule->children;
  int count = 0;
  ScheduleChild *child;

  schedule->depth_count++;
  if (schedule->depth_count > schedule->child_count)
    schedule->depth_count = 1;

  while (seek)
  {
    count++;

    if (count == schedule->depth_count)
      {
        child = seek->data;

        gdk_window_hide (child->window);
        gtk_widget_hide (child->widget);

        gdk_window_show (child->window);
        gtk_widget_show (child->widget);
      }

    seek = g_list_next (seek);
  }
}


static void
row_selected (Schedule * schedule, int row, time_t index)
{
}


static gint
schedule_motion_notify_event (GtkWidget * widget, GdkEventMotion * event)
{
  Schedule *schedule = SCHEDULE (widget);
  float gran;

  if (info)
    {
      switch (schedule->granularity)
	{
	case SCHEDULE_15M:
	  gran = 2.0;
	  break;
	case SCHEDULE_30M:
	  gran = 1.0;
	  break;
	case SCHEDULE_60M:
	  gran = 0.5;
	  break;
	}

      if (info->type == DRAG_MOVE)
	{
	  if (event->y - info->start_y > -HANDLE_SIZE)
	    info->child->y = event->y - info->start_y;
	  else
	    info->child->y = -HANDLE_SIZE;
	}
      if (info->type == DRAG_SIZEUP)
	{
	  if (event->y <
	      info->child->y + info->child->height - (schedule->row_height +
						      HANDLE_SIZE * 2))
	    {
	      if (event->y > -HANDLE_SIZE)
		{
		  int tmpy = info->child->y;
		  info->child->y = event->y + info->start_y;
		  schedule_child_find_best (info->child);
		  info->child->height += tmpy - info->child->y;
		}
	      else
		{
		  int tmpy = info->child->y;
		  if (info->child->y > -HANDLE_SIZE)
		    {
		      info->child->y = -HANDLE_SIZE;
		      schedule_child_find_best (info->child);
		      info->child->height += tmpy + HANDLE_SIZE * 2;
		    }
		}
	    }
	  else
	    {
	      int ty = info->child->height + info->child->y;
	      info->child->y = ty - (schedule->row_height + HANDLE_SIZE * 2);
	      schedule_child_find_best (info->child);
	      info->child->height = (schedule->row_height + HANDLE_SIZE * 2);
	    }
	  schedule_child_find_best (info->child);
	  gdk_window_move_resize (info->child->window,
				  info->child->x,
				  info->child->y,
				  info->child->width, info->child->height);
	  schedule_child_handle_text (schedule, info->child);
	  return TRUE;
	}
      if (info->type == DRAG_SIZEDOWN)
	{
	  if (event->y >
	      info->child->y + (schedule->row_height + HANDLE_SIZE * 2))
	    info->child->height = event->y - info->child->y;
	  else
	    info->child->height = schedule->row_height + HANDLE_SIZE * 2;
	}
      schedule_child_find_best (info->child);
      gdk_window_move_resize (info->child->window,
			      info->child->x,
			      info->child->y,
			      info->child->width, info->child->height);
      schedule_child_handle_text (schedule, info->child);
    }
  return TRUE;
}


static void
schedule_child_find_best (ScheduleChild * child)
{
  int tmpy, h = child->schedule->row_height;
  Schedule *schedule = child->schedule;
  float gran = 0;

  g_return_if_fail (child != NULL);

  switch (schedule->granularity)
    {
    case SCHEDULE_15M:
      gran = 15;
      break;
    case SCHEDULE_30M:
      gran = 30;
      break;
    case SCHEDULE_60M:
      gran = 60;
      break;
    }

  tmpy = (child->y + HANDLE_SIZE) / h;
  child->y = (tmpy * h) - HANDLE_SIZE + 1;
  tmpy = (child->y + child->height);
  tmpy = tmpy / h;
  tmpy = tmpy * h;
  child->height = tmpy - child->y + HANDLE_SIZE;
  if (child->y > 2)
    child->start = ((child->y / h) + 1) * (gran * 60) + schedule->upper;
  else
    child->start = schedule->upper;
  child->stop =
    (((child->y + child->height) + 1) / h) * (gran * 60) + schedule->upper;
}


void
schedule_set_upper (Schedule * schedule, time_t upper)
{
  g_return_if_fail (schedule != NULL);
  schedule->upper = upper;
}


void
schedule_child_get_time (Schedule * schedule, gpointer seek_data,
			 time_t * start, time_t * stop)
{
  GList *seek;
  ScheduleChild *child;
  g_return_if_fail (schedule != NULL);
  g_return_if_fail (start != NULL);
  g_return_if_fail (stop != NULL);
  for (seek = schedule->children; seek; seek = seek->next)
    {
      child = seek->data;
      if (child->data == seek_data)
	{
	  *start = child->start;
	  *stop = child->stop;
	}
    }
}


void
schedule_time_set (Schedule * schedule, ScheduleTime format)
{
  schedule->time_style = format;

  schedule_paint (GTK_WIDGET (schedule));
}


void
schedule_time_toggle (Schedule * schedule)
{
  ScheduleTime format = schedule->time_style;

  if (format == SCHEDULE_AMPM)
    {
      schedule->time_style = SCHEDULE_24HR;
    }
  else
    {
      schedule->time_style = SCHEDULE_AMPM;
    }

  schedule_paint (GTK_WIDGET (schedule));
}


void
schedule_granularity_set (Schedule * schedule,
			  ScheduleGranularity granularity)
{
  GtkWidget *widget = GTK_WIDGET (schedule);

  schedule->granularity = granularity;

  gtk_widget_hide (widget);
  gtk_widget_show (widget);
}


void
schedule_set_work_day (Schedule * schedule, int start, int stop)
{
  schedule->day_start_row = start;
  schedule->day_end_row = stop;

  schedule_paint (GTK_WIDGET (schedule));
}


void
schedule_child_set_color (ScheduleChild * child, GdkGC * color)
{
  child->color = color;

  schedule_paint (GTK_WIDGET (child->schedule));
}


void
schedule_set_background_color (Schedule * schedule, GdkGC * dark,
			       GdkGC * light)
{
  schedule->dark = dark;
  schedule->light = light;

  schedule_paint (GTK_WIDGET (schedule));
}


#endif /* _SCHEDULE_C_ */
